/*     */ package org.apache.batik.bridge;
/*     */ 
/*     */ import java.awt.Shape;
/*     */ import java.awt.geom.AffineTransform;
/*     */ import java.awt.geom.Rectangle2D;
/*     */ import java.lang.ref.SoftReference;
/*     */ import org.apache.batik.css.engine.CSSEngine;
/*     */ import org.apache.batik.css.engine.CSSEngineEvent;
/*     */ import org.apache.batik.css.engine.CSSStylableElement;
/*     */ import org.apache.batik.css.engine.SVGCSSEngine;
/*     */ import org.apache.batik.dom.events.AbstractEvent;
/*     */ import org.apache.batik.dom.svg.AbstractSVGTransformList;
/*     */ import org.apache.batik.dom.svg.AnimatedLiveAttributeValue;
/*     */ import org.apache.batik.dom.svg.LiveAttributeException;
/*     */ import org.apache.batik.dom.svg.SVGContext;
/*     */ import org.apache.batik.dom.svg.SVGMotionAnimatableElement;
/*     */ import org.apache.batik.dom.svg.SVGOMAnimatedTransformList;
/*     */ import org.apache.batik.dom.svg.SVGOMElement;
/*     */ import org.apache.batik.ext.awt.geom.SegmentList;
/*     */ import org.apache.batik.gvt.CanvasGraphicsNode;
/*     */ import org.apache.batik.gvt.CompositeGraphicsNode;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.apache.batik.parser.UnitProcessor;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.Node;
/*     */ import org.w3c.dom.events.DocumentEvent;
/*     */ import org.w3c.dom.events.EventTarget;
/*     */ import org.w3c.dom.events.MutationEvent;
/*     */ import org.w3c.dom.svg.SVGTransformable;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class AbstractGraphicsNodeBridge
/*     */   extends AnimatableSVGBridge
/*     */   implements SVGContext, BridgeUpdateHandler, GraphicsNodeBridge, ErrorConstants
/*     */ {
/*     */   protected GraphicsNode node;
/*     */   protected boolean isSVG12;
/*     */   protected UnitProcessor.Context unitContext;
/*     */   
/*     */   public GraphicsNode createGraphicsNode(BridgeContext ctx, Element e) {
/* 105 */     if (!SVGUtilities.matchUserAgent(e, ctx.getUserAgent())) {
/* 106 */       return null;
/*     */     }
/*     */     
/* 109 */     GraphicsNode node = instantiateGraphicsNode();
/*     */ 
/*     */     
/* 112 */     setTransform(node, e, ctx);
/*     */ 
/*     */     
/* 115 */     node.setVisible(CSSUtilities.convertVisibility(e));
/*     */     
/* 117 */     associateSVGContext(ctx, e, node);
/*     */     
/* 119 */     return node;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected abstract GraphicsNode instantiateGraphicsNode();
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void buildGraphicsNode(BridgeContext ctx, Element e, GraphicsNode node) {
/* 140 */     node.setComposite(CSSUtilities.convertOpacity(e));
/*     */     
/* 142 */     node.setFilter(CSSUtilities.convertFilter(e, node, ctx));
/*     */     
/* 144 */     node.setMask(CSSUtilities.convertMask(e, node, ctx));
/*     */     
/* 146 */     node.setClip(CSSUtilities.convertClipPath(e, node, ctx));
/*     */     
/* 148 */     node.setPointerEventType(CSSUtilities.convertPointerEvents(e));
/*     */     
/* 150 */     initializeDynamicSupport(ctx, e, node);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 158 */   public boolean getDisplay(Element e) { return CSSUtilities.convertDisplay(e); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected AffineTransform computeTransform(SVGTransformable te, BridgeContext ctx) {
/*     */     try {
/* 168 */       AffineTransform at = new AffineTransform();
/*     */ 
/*     */       
/* 171 */       SVGOMAnimatedTransformList atl = (SVGOMAnimatedTransformList)te.getTransform();
/*     */       
/* 173 */       if (atl.isSpecified()) {
/* 174 */         atl.check();
/* 175 */         AbstractSVGTransformList tl = (AbstractSVGTransformList)te.getTransform().getAnimVal();
/*     */         
/* 177 */         at.concatenate(tl.getAffineTransform());
/*     */       } 
/*     */ 
/*     */       
/* 181 */       if (this.e instanceof SVGMotionAnimatableElement) {
/* 182 */         SVGMotionAnimatableElement mae = (SVGMotionAnimatableElement)this.e;
/* 183 */         AffineTransform mat = mae.getMotionTransform();
/* 184 */         if (mat != null) {
/* 185 */           at.concatenate(mat);
/*     */         }
/*     */       } 
/*     */       
/* 189 */       return at;
/*     */     } catch (LiveAttributeException ex) {
/* 191 */       throw new BridgeException(ctx, ex);
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 200 */   protected void setTransform(GraphicsNode n, Element e, BridgeContext ctx) { n.setTransform(computeTransform((SVGTransformable)e, ctx)); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void associateSVGContext(BridgeContext ctx, Element e, GraphicsNode node) {
/* 211 */     this.e = e;
/* 212 */     this.node = node;
/* 213 */     this.ctx = ctx;
/* 214 */     this.unitContext = UnitProcessor.createContext(ctx, e);
/* 215 */     this.isSVG12 = ctx.isSVG12();
/* 216 */     ((SVGOMElement)e).setSVGContext(this);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void initializeDynamicSupport(BridgeContext ctx, Element e, GraphicsNode node) {
/* 228 */     if (ctx.isInteractive())
/*     */     {
/* 230 */       ctx.bind(e, node);
/*     */     }
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleDOMAttrModifiedEvent(MutationEvent evt) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void handleGeometryChanged() {
/* 246 */     this.node.setFilter(CSSUtilities.convertFilter(this.e, this.node, this.ctx));
/* 247 */     this.node.setMask(CSSUtilities.convertMask(this.e, this.node, this.ctx));
/* 248 */     this.node.setClip(CSSUtilities.convertClipPath(this.e, this.node, this.ctx));
/* 249 */     if (this.isSVG12) {
/* 250 */       if (!"use".equals(this.e.getLocalName()))
/*     */       {
/* 252 */         fireShapeChangeEvent();
/*     */       }
/* 254 */       fireBBoxChangeEvent();
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void fireShapeChangeEvent() {
/* 262 */     DocumentEvent d = (DocumentEvent)this.e.getOwnerDocument();
/* 263 */     AbstractEvent evt = (AbstractEvent)d.createEvent("SVGEvents");
/* 264 */     evt.initEventNS("http://www.w3.org/2000/svg", "shapechange", true, false);
/*     */ 
/*     */ 
/*     */     
/*     */     try {
/* 269 */       ((EventTarget)this.e).dispatchEvent(evt);
/*     */     } catch (RuntimeException ex) {
/* 271 */       this.ctx.getUserAgent().displayError(ex);
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleDOMNodeInsertedEvent(MutationEvent evt) {
/* 279 */     if (evt.getTarget() instanceof Element) {
/*     */       
/* 281 */       Element e2 = (Element)evt.getTarget();
/* 282 */       Bridge b = this.ctx.getBridge(e2);
/* 283 */       if (b instanceof GenericBridge) {
/* 284 */         ((GenericBridge)b).handleElement(this.ctx, e2);
/*     */       }
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleDOMNodeRemovedEvent(MutationEvent evt) {
/* 293 */     Node parent = this.e.getParentNode();
/* 294 */     if (parent instanceof SVGOMElement) {
/* 295 */       SVGContext bridge = ((SVGOMElement)parent).getSVGContext();
/* 296 */       if (bridge instanceof SVGSwitchElementBridge) {
/* 297 */         ((SVGSwitchElementBridge)bridge).handleChildElementRemoved(this.e);
/*     */         return;
/*     */       } 
/*     */     } 
/* 301 */     CompositeGraphicsNode gn = this.node.getParent();
/* 302 */     gn.remove(this.node);
/* 303 */     disposeTree(this.e);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleDOMCharacterDataModified(MutationEvent evt) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void dispose() {
/* 317 */     SVGOMElement elt = (SVGOMElement)this.e;
/* 318 */     elt.setSVGContext(null);
/* 319 */     this.ctx.unbind(this.e);
/*     */     
/* 321 */     this.bboxShape = null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 328 */   protected void disposeTree(Node node) { disposeTree(node, true); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void disposeTree(Node node, boolean removeContext) {
/* 336 */     if (node instanceof SVGOMElement) {
/* 337 */       SVGOMElement elt = (SVGOMElement)node;
/* 338 */       SVGContext ctx = elt.getSVGContext();
/* 339 */       if (ctx instanceof BridgeUpdateHandler) {
/* 340 */         BridgeUpdateHandler h = (BridgeUpdateHandler)ctx;
/* 341 */         if (removeContext) {
/* 342 */           elt.setSVGContext(null);
/*     */         }
/* 344 */         h.dispose();
/*     */       } 
/*     */     } 
/* 347 */     for (Node n = node.getFirstChild(); n != null; n = n.getNextSibling()) {
/* 348 */       disposeTree(n, removeContext);
/*     */     }
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleCSSEngineEvent(CSSEngineEvent evt) {
/*     */     try {
/* 357 */       SVGCSSEngine eng = (SVGCSSEngine)evt.getSource();
/* 358 */       int[] properties = evt.getProperties();
/* 359 */       for (int i = 0; i < properties.length; i++) {
/* 360 */         int idx = properties[i];
/* 361 */         handleCSSPropertyChanged(idx);
/* 362 */         String pn = eng.getPropertyName(idx);
/* 363 */         fireBaseAttributeListeners(pn);
/*     */       } 
/*     */     } catch (Exception ex) {
/* 366 */       this.ctx.getUserAgent().displayError(ex);
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void handleCSSPropertyChanged(int property) {
/* 374 */     switch (property) {
/*     */       case 57:
/* 376 */         this.node.setVisible(CSSUtilities.convertVisibility(this.e));
/*     */         break;
/*     */       case 38:
/* 379 */         this.node.setComposite(CSSUtilities.convertOpacity(this.e));
/*     */         break;
/*     */       case 18:
/* 382 */         this.node.setFilter(CSSUtilities.convertFilter(this.e, this.node, this.ctx));
/*     */         break;
/*     */       case 37:
/* 385 */         this.node.setMask(CSSUtilities.convertMask(this.e, this.node, this.ctx));
/*     */         break;
/*     */       case 3:
/* 388 */         this.node.setClip(CSSUtilities.convertClipPath(this.e, this.node, this.ctx));
/*     */         break;
/*     */       case 40:
/* 391 */         this.node.setPointerEventType(CSSUtilities.convertPointerEvents(this.e));
/*     */         break;
/*     */       case 12:
/* 394 */         if (!getDisplay(this.e)) {
/*     */           
/* 396 */           CompositeGraphicsNode parent = this.node.getParent();
/* 397 */           parent.remove(this.node);
/* 398 */           disposeTree(this.e, false);
/*     */         } 
/*     */         break;
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleAnimatedAttributeChanged(AnimatedLiveAttributeValue alav) {
/* 409 */     if (alav.getNamespaceURI() == null && alav.getLocalName().equals("transform")) {
/*     */       
/* 411 */       setTransform(this.node, this.e, this.ctx);
/* 412 */       handleGeometryChanged();
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleOtherAnimationChanged(String type) {
/* 420 */     if (type.equals("motion")) {
/* 421 */       setTransform(this.node, this.e, this.ctx);
/* 422 */       handleGeometryChanged();
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void checkBBoxChange() {
/* 431 */     if (this.e != null)
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 440 */       fireBBoxChangeEvent();
/*     */     }
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void fireBBoxChangeEvent() {
/* 450 */     DocumentEvent d = (DocumentEvent)this.e.getOwnerDocument();
/* 451 */     AbstractEvent evt = (AbstractEvent)d.createEvent("SVGEvents");
/* 452 */     evt.initEventNS("http://www.w3.org/2000/svg", "RenderedBBoxChange", true, false);
/*     */ 
/*     */ 
/*     */     
/*     */     try {
/* 457 */       ((EventTarget)this.e).dispatchEvent(evt);
/*     */     } catch (RuntimeException ex) {
/* 459 */       this.ctx.getUserAgent().displayError(ex);
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 469 */   public float getPixelUnitToMillimeter() { return this.ctx.getUserAgent().getPixelUnitToMillimeter(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 478 */   public float getPixelToMM() { return getPixelUnitToMillimeter(); }
/*     */ 
/*     */   
/* 481 */   protected SoftReference bboxShape = null;
/* 482 */   protected Rectangle2D bbox = null;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Rectangle2D getBBox() {
/* 491 */     if (this.node == null) {
/* 492 */       return null;
/*     */     }
/* 494 */     Shape s = this.node.getOutline();
/*     */     
/* 496 */     if (this.bboxShape != null && s == this.bboxShape.get()) return this.bbox; 
/* 497 */     this.bboxShape = new SoftReference(s);
/* 498 */     this.bbox = null;
/* 499 */     if (s == null) return this.bbox;
/*     */ 
/*     */     
/* 502 */     SegmentList sl = new SegmentList(s);
/* 503 */     this.bbox = sl.getBounds2D();
/* 504 */     return this.bbox;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public AffineTransform getCTM() {
/* 513 */     GraphicsNode gn = this.node;
/* 514 */     AffineTransform ctm = new AffineTransform();
/* 515 */     Element elt = this.e;
/* 516 */     while (elt != null) {
/* 517 */       if (elt instanceof org.w3c.dom.svg.SVGFitToViewBox) {
/*     */         AffineTransform affineTransform;
/* 519 */         if (gn instanceof CanvasGraphicsNode) {
/* 520 */           affineTransform = ((CanvasGraphicsNode)gn).getViewingTransform();
/*     */         } else {
/* 522 */           affineTransform = gn.getTransform();
/*     */         } 
/* 524 */         if (affineTransform != null) {
/* 525 */           ctm.preConcatenate(affineTransform);
/*     */         }
/*     */         
/*     */         break;
/*     */       } 
/* 530 */       AffineTransform at = gn.getTransform();
/* 531 */       if (at != null) {
/* 532 */         ctm.preConcatenate(at);
/*     */       }
/* 534 */       CSSStylableElement cSSStylableElement = CSSEngine.getParentCSSStylableElement(elt);
/* 535 */       CompositeGraphicsNode compositeGraphicsNode = gn.getParent();
/*     */     } 
/* 537 */     return ctm;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 544 */   public AffineTransform getScreenTransform() { return this.ctx.getUserAgent().getTransform(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 551 */   public void setScreenTransform(AffineTransform at) { this.ctx.getUserAgent().setTransform(at); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 559 */   public AffineTransform getGlobalTransform() { return this.node.getGlobalTransform(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 567 */   public float getViewportWidth() { return this.ctx.getBlockWidth(this.e); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 575 */   public float getViewportHeight() { return this.ctx.getBlockHeight(this.e); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 582 */   public float getFontSize() { return CSSUtilities.getComputedStyle(this.e, 22).getFloatValue(); }
/*     */   
/*     */   public abstract boolean isComposite();
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\AbstractGraphicsNodeBridge.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */