/*     */ package org.apache.batik.bridge;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.io.InputStreamReader;
/*     */ import java.io.PushbackInputStream;
/*     */ import java.io.Reader;
/*     */ import java.io.StringReader;
/*     */ import java.io.UnsupportedEncodingException;
/*     */ import java.net.MalformedURLException;
/*     */ import java.net.URL;
/*     */ import java.util.HashSet;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Set;
/*     */ import java.util.jar.Manifest;
/*     */ import org.apache.batik.dom.AbstractElement;
/*     */ import org.apache.batik.dom.events.AbstractEvent;
/*     */ import org.apache.batik.dom.events.NodeEventTarget;
/*     */ import org.apache.batik.dom.util.XLinkSupport;
/*     */ import org.apache.batik.script.Interpreter;
/*     */ import org.apache.batik.script.InterpreterException;
/*     */ import org.apache.batik.script.ScriptEventWrapper;
/*     */ import org.apache.batik.script.ScriptHandler;
/*     */ import org.apache.batik.script.Window;
/*     */ import org.apache.batik.util.ParsedURL;
/*     */ import org.w3c.dom.Document;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.Node;
/*     */ import org.w3c.dom.NodeList;
/*     */ import org.w3c.dom.events.DocumentEvent;
/*     */ import org.w3c.dom.events.Event;
/*     */ import org.w3c.dom.events.EventListener;
/*     */ import org.w3c.dom.svg.EventListenerInitializer;
/*     */ import org.w3c.dom.svg.SVGDocument;
/*     */ import org.w3c.dom.svg.SVGSVGElement;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class BaseScriptingEnvironment
/*     */ {
/*     */   public static final String INLINE_SCRIPT_DESCRIPTION = "BaseScriptingEnvironment.constant.inline.script.description";
/*     */   public static final String EVENT_SCRIPT_DESCRIPTION = "BaseScriptingEnvironment.constant.event.script.description";
/*     */   protected static final String EVENT_NAME = "event";
/*     */   protected static final String ALTERNATE_EVENT_NAME = "evt";
/*     */   protected static final String APPLICATION_ECMASCRIPT = "application/ecmascript";
/*     */   protected BridgeContext bridgeContext;
/*     */   protected UserAgent userAgent;
/*     */   protected Document document;
/*     */   protected ParsedURL docPURL;
/*     */   protected Set languages;
/*     */   protected Interpreter interpreter;
/*     */   
/*     */   public static boolean isDynamicDocument(BridgeContext ctx, Document doc) {
/*  93 */     Element elt = doc.getDocumentElement();
/*  94 */     if (elt != null && "http://www.w3.org/2000/svg".equals(elt.getNamespaceURI())) {
/*     */       
/*  96 */       if (elt.getAttributeNS(null, "onabort").length() > 0)
/*     */       {
/*  98 */         return true;
/*     */       }
/* 100 */       if (elt.getAttributeNS(null, "onerror").length() > 0)
/*     */       {
/* 102 */         return true;
/*     */       }
/* 104 */       if (elt.getAttributeNS(null, "onresize").length() > 0)
/*     */       {
/* 106 */         return true;
/*     */       }
/* 108 */       if (elt.getAttributeNS(null, "onunload").length() > 0)
/*     */       {
/* 110 */         return true;
/*     */       }
/* 112 */       if (elt.getAttributeNS(null, "onscroll").length() > 0)
/*     */       {
/* 114 */         return true;
/*     */       }
/* 116 */       if (elt.getAttributeNS(null, "onzoom").length() > 0)
/*     */       {
/* 118 */         return true;
/*     */       }
/* 120 */       return isDynamicElement(ctx, doc.getDocumentElement());
/*     */     } 
/* 122 */     return false;
/*     */   }
/*     */   
/*     */   public static boolean isDynamicElement(BridgeContext ctx, Element elt) {
/* 126 */     List bridgeExtensions = ctx.getBridgeExtensions(elt.getOwnerDocument());
/* 127 */     return isDynamicElement(elt, ctx, bridgeExtensions);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static boolean isDynamicElement(Element elt, BridgeContext ctx, List bridgeExtensions) {
/* 135 */     Iterator i = bridgeExtensions.iterator();
/* 136 */     while (i.hasNext()) {
/* 137 */       BridgeExtension bridgeExtension = (BridgeExtension)i.next();
/* 138 */       if (bridgeExtension.isDynamicElement(elt)) {
/* 139 */         return true;
/*     */       }
/*     */     } 
/* 142 */     if ("http://www.w3.org/2000/svg".equals(elt.getNamespaceURI())) {
/* 143 */       if (elt.getAttributeNS(null, "onkeyup").length() > 0)
/*     */       {
/* 145 */         return true;
/*     */       }
/* 147 */       if (elt.getAttributeNS(null, "onkeydown").length() > 0)
/*     */       {
/* 149 */         return true;
/*     */       }
/* 151 */       if (elt.getAttributeNS(null, "onkeypress").length() > 0)
/*     */       {
/* 153 */         return true;
/*     */       }
/* 155 */       if (elt.getAttributeNS(null, "onload").length() > 0)
/*     */       {
/* 157 */         return true;
/*     */       }
/* 159 */       if (elt.getAttributeNS(null, "onerror").length() > 0)
/*     */       {
/* 161 */         return true;
/*     */       }
/* 163 */       if (elt.getAttributeNS(null, "onactivate").length() > 0)
/*     */       {
/* 165 */         return true;
/*     */       }
/* 167 */       if (elt.getAttributeNS(null, "onclick").length() > 0)
/*     */       {
/* 169 */         return true;
/*     */       }
/* 171 */       if (elt.getAttributeNS(null, "onfocusin").length() > 0)
/*     */       {
/* 173 */         return true;
/*     */       }
/* 175 */       if (elt.getAttributeNS(null, "onfocusout").length() > 0)
/*     */       {
/* 177 */         return true;
/*     */       }
/* 179 */       if (elt.getAttributeNS(null, "onmousedown").length() > 0)
/*     */       {
/* 181 */         return true;
/*     */       }
/* 183 */       if (elt.getAttributeNS(null, "onmousemove").length() > 0)
/*     */       {
/* 185 */         return true;
/*     */       }
/* 187 */       if (elt.getAttributeNS(null, "onmouseout").length() > 0)
/*     */       {
/* 189 */         return true;
/*     */       }
/* 191 */       if (elt.getAttributeNS(null, "onmouseover").length() > 0)
/*     */       {
/* 193 */         return true;
/*     */       }
/* 195 */       if (elt.getAttributeNS(null, "onmouseup").length() > 0)
/*     */       {
/* 197 */         return true;
/*     */       }
/*     */     } 
/*     */     
/* 201 */     Node n = elt.getFirstChild();
/* 202 */     for (; n != null; 
/* 203 */       n = n.getNextSibling()) {
/* 204 */       if (n.getNodeType() == 1 && 
/* 205 */         isDynamicElement(ctx, (Element)n)) {
/* 206 */         return true;
/*     */       }
/*     */     } 
/*     */     
/* 210 */     return false;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public BaseScriptingEnvironment(BridgeContext ctx) {
/* 243 */     this.languages = new HashSet();
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 255 */     this.bridgeContext = ctx;
/* 256 */     this.document = ctx.getDocument();
/* 257 */     this.docPURL = new ParsedURL(((SVGDocument)this.document).getURL());
/* 258 */     this.userAgent = this.bridgeContext.getUserAgent();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 266 */   public Window createWindow(Interpreter interp, String lang) { return new Window(this, interp, lang); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 273 */   public Window createWindow() { return createWindow(null, null); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Interpreter getInterpreter() {
/* 280 */     if (this.interpreter != null) {
/* 281 */       return this.interpreter;
/*     */     }
/* 283 */     SVGSVGElement root = (SVGSVGElement)this.document.getDocumentElement();
/* 284 */     String lang = root.getContentScriptType();
/* 285 */     return getInterpreter(lang);
/*     */   }
/*     */   
/*     */   public Interpreter getInterpreter(String lang) {
/* 289 */     this.interpreter = this.bridgeContext.getInterpreter(lang);
/* 290 */     if (this.interpreter == null) {
/* 291 */       if (this.languages.contains(lang))
/*     */       {
/* 293 */         return null;
/*     */       }
/*     */ 
/*     */       
/* 297 */       this.languages.add(lang);
/* 298 */       return null;
/*     */     } 
/*     */     
/* 301 */     if (!this.languages.contains(lang)) {
/* 302 */       this.languages.add(lang);
/* 303 */       initializeEnvironment(this.interpreter, lang);
/*     */     } 
/* 305 */     return this.interpreter;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 312 */   public void initializeEnvironment(Interpreter interp, String lang) { interp.bindObject("window", createWindow(interp, lang)); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void loadScripts() {
/* 319 */     Window window = null;
/*     */     
/* 321 */     NodeList scripts = this.document.getElementsByTagNameNS("http://www.w3.org/2000/svg", "script");
/*     */     
/* 323 */     int len = scripts.getLength();
/*     */     
/* 325 */     if (len == 0) {
/*     */       return;
/*     */     }
/*     */     
/* 329 */     for (int i = 0; i < len; i++) {
/* 330 */       AbstractElement script = (AbstractElement)scripts.item(i);
/* 331 */       String type = script.getAttributeNS(null, "type");
/*     */ 
/*     */       
/* 334 */       if (type.length() == 0) {
/* 335 */         type = "text/ecmascript";
/*     */       }
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 341 */       if (type.equals("application/java-archive")) {
/*     */         
/* 343 */         try { String href = XLinkSupport.getXLinkHref(script);
/* 344 */           ParsedURL purl = new ParsedURL(script.getBaseURI(), href);
/*     */           
/* 346 */           checkCompatibleScriptURL(type, purl);
/*     */ 
/*     */           
/* 349 */           URL docURL = null;
/*     */           try {
/* 351 */             docURL = new URL(this.docPURL.toString());
/* 352 */           } catch (MalformedURLException mue) {}
/*     */ 
/*     */           
/* 355 */           DocumentJarClassLoader cll = new DocumentJarClassLoader(new URL(purl.toString()), docURL);
/*     */ 
/*     */ 
/*     */           
/* 359 */           URL url = cll.findResource("META-INF/MANIFEST.MF");
/* 360 */           if (url != null) {
/*     */ 
/*     */             
/* 363 */             Manifest man = new Manifest(url.openStream());
/*     */ 
/*     */ 
/*     */             
/* 367 */             String sh = man.getMainAttributes().getValue("Script-Handler");
/* 368 */             if (sh != null) {
/*     */ 
/*     */               
/* 371 */               ScriptHandler h = (ScriptHandler)cll.loadClass(sh).newInstance();
/*     */               
/* 373 */               if (window == null) {
/* 374 */                 window = createWindow();
/*     */               }
/*     */               
/* 377 */               h.run(this.document, window);
/*     */             } 
/*     */             
/* 380 */             sh = man.getMainAttributes().getValue("SVG-Handler-Class");
/* 381 */             if (sh != null) {
/*     */ 
/*     */               
/* 384 */               EventListenerInitializer initializer = (EventListenerInitializer)cll.loadClass(sh).newInstance();
/*     */ 
/*     */               
/* 387 */               if (window == null) {
/* 388 */                 window = createWindow();
/*     */               }
/*     */               
/* 391 */               initializer.initializeEventListeners((SVGDocument)this.document);
/*     */             } 
/*     */           }  }
/* 394 */         catch (Exception e) { if (this.userAgent != null) {
/* 395 */             this.userAgent.displayError(e);
/*     */           } }
/*     */ 
/*     */ 
/*     */         
/*     */         continue;
/*     */       } 
/*     */ 
/*     */       
/* 404 */       Interpreter interpreter = getInterpreter(type);
/* 405 */       if (interpreter == null) {
/*     */         continue;
/*     */       }
/*     */       
/*     */       try {
/* 410 */         String href = XLinkSupport.getXLinkHref(script);
/* 411 */         String desc = null;
/* 412 */         Reader reader = null;
/*     */         
/* 414 */         if (href.length() > 0) {
/* 415 */           desc = href;
/*     */ 
/*     */           
/* 418 */           ParsedURL purl = new ParsedURL(script.getBaseURI(), href);
/*     */           
/* 420 */           checkCompatibleScriptURL(type, purl);
/* 421 */           InputStream is = purl.openStream();
/* 422 */           String mediaType = purl.getContentTypeMediaType();
/* 423 */           String enc = purl.getContentTypeCharset();
/* 424 */           if (enc != null) {
/*     */             try {
/* 426 */               reader = new InputStreamReader(is, enc);
/*     */             } catch (UnsupportedEncodingException uee) {
/* 428 */               enc = null;
/*     */             } 
/*     */           }
/* 431 */           if (reader == null) {
/* 432 */             if ("application/ecmascript".equals(mediaType)) {
/*     */ 
/*     */               
/* 435 */               if (purl.hasContentTypeParameter("version")) {
/*     */                 continue;
/*     */               }
/*     */ 
/*     */ 
/*     */ 
/*     */               
/* 442 */               PushbackInputStream pbis = new PushbackInputStream(is, 8);
/*     */               
/* 444 */               byte[] buf = new byte[4];
/* 445 */               int read = pbis.read(buf);
/* 446 */               if (read > 0) {
/* 447 */                 pbis.unread(buf, 0, read);
/* 448 */                 if (read >= 2) {
/* 449 */                   if (buf[0] == -1 && buf[1] == -2) {
/*     */                     
/* 451 */                     if (read >= 4 && buf[2] == 0 && buf[3] == 0) {
/*     */                       
/* 453 */                       enc = "UTF32-LE";
/* 454 */                       pbis.skip(4L);
/*     */                     } else {
/* 456 */                       enc = "UTF-16LE";
/* 457 */                       pbis.skip(2L);
/*     */                     } 
/* 459 */                   } else if (buf[0] == -2 && buf[1] == -1) {
/*     */                     
/* 461 */                     enc = "UTF-16BE";
/* 462 */                     pbis.skip(2L);
/* 463 */                   } else if (read >= 3 && buf[0] == -17 && buf[1] == -69 && buf[2] == -65) {
/*     */ 
/*     */ 
/*     */                     
/* 467 */                     enc = "UTF-8";
/* 468 */                     pbis.skip(3L);
/* 469 */                   } else if (read >= 4 && buf[0] == 0 && buf[1] == 0 && buf[2] == -2 && buf[3] == -1) {
/*     */ 
/*     */ 
/*     */                     
/* 473 */                     enc = "UTF-32BE";
/* 474 */                     pbis.skip(4L);
/*     */                   } 
/*     */                 }
/* 477 */                 if (enc == null) {
/* 478 */                   enc = "UTF-8";
/*     */                 }
/*     */               } 
/* 481 */               reader = new InputStreamReader(pbis, enc);
/*     */             } else {
/* 483 */               reader = new InputStreamReader(is);
/*     */             } 
/*     */           }
/*     */         } else {
/* 487 */           checkCompatibleScriptURL(type, this.docPURL);
/* 488 */           DocumentLoader dl = this.bridgeContext.getDocumentLoader();
/* 489 */           AbstractElement abstractElement = script;
/* 490 */           SVGDocument d = (SVGDocument)abstractElement.getOwnerDocument();
/* 491 */           int line = dl.getLineNumber(script);
/* 492 */           desc = Messages.formatMessage("BaseScriptingEnvironment.constant.inline.script.description", new Object[] { d.getURL(), "<" + script.getNodeName() + ">", new Integer(line) });
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */           
/* 498 */           Node n = script.getFirstChild();
/* 499 */           if (n != null) {
/* 500 */             StringBuffer sb = new StringBuffer();
/* 501 */             while (n != null) {
/* 502 */               if (n.getNodeType() == 4 || n.getNodeType() == 3)
/*     */               {
/* 504 */                 sb.append(n.getNodeValue()); } 
/* 505 */               n = n.getNextSibling();
/*     */             } 
/* 507 */             reader = new StringReader(sb.toString());
/*     */           } else {
/*     */             continue;
/*     */           } 
/*     */         } 
/*     */         
/* 513 */         interpreter.evaluate(reader, desc);
/*     */         continue;
/*     */       } catch (IOException e) {
/* 516 */         if (this.userAgent != null) {
/* 517 */           this.userAgent.displayError(e);
/*     */         }
/*     */         return;
/*     */       } catch (InterpreterException e) {
/* 521 */         System.err.println("InterpExcept: " + e);
/* 522 */         handleInterpreterException(e);
/*     */         return;
/*     */       } catch (SecurityException e) {
/* 525 */         if (this.userAgent != null) {
/* 526 */           this.userAgent.displayError(e);
/*     */         }
/*     */         continue;
/*     */       } 
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 539 */   protected void checkCompatibleScriptURL(String scriptType, ParsedURL scriptPURL) { this.userAgent.checkLoadScript(scriptType, scriptPURL, this.docPURL); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void dispatchSVGLoadEvent() {
/* 546 */     SVGSVGElement root = (SVGSVGElement)this.document.getDocumentElement();
/* 547 */     String lang = root.getContentScriptType();
/* 548 */     long documentStartTime = System.currentTimeMillis();
/* 549 */     this.bridgeContext.getAnimationEngine().start(documentStartTime);
/* 550 */     dispatchSVGLoad(root, true, lang);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void dispatchSVGLoad(Element elt, boolean checkCanRun, String lang) {
/*     */     String str;
/* 559 */     Node n = elt.getFirstChild();
/* 560 */     for (; n != null; 
/* 561 */       n = n.getNextSibling()) {
/* 562 */       if (n.getNodeType() == 1) {
/* 563 */         dispatchSVGLoad((Element)n, checkCanRun, lang);
/*     */       }
/*     */     } 
/*     */     
/* 567 */     DocumentEvent de = (DocumentEvent)elt.getOwnerDocument();
/* 568 */     AbstractEvent ev = (AbstractEvent)de.createEvent("SVGEvents");
/*     */     
/* 570 */     if (this.bridgeContext.isSVG12()) {
/* 571 */       str = "load";
/*     */     } else {
/* 573 */       str = "SVGLoad";
/*     */     } 
/* 575 */     ev.initEventNS("http://www.w3.org/2001/xml-events", str, false, false);
/*     */ 
/*     */ 
/*     */     
/* 579 */     NodeEventTarget t = (NodeEventTarget)elt;
/*     */     
/* 581 */     String s = elt.getAttributeNS(null, "onload");
/*     */     
/* 583 */     if (s.length() == 0) {
/*     */ 
/*     */       
/* 586 */       t.dispatchEvent(ev);
/*     */       
/*     */       return;
/*     */     } 
/* 590 */     Interpreter interp = getInterpreter();
/* 591 */     if (interp == null) {
/*     */ 
/*     */       
/* 594 */       t.dispatchEvent(ev);
/*     */       
/*     */       return;
/*     */     } 
/* 598 */     if (checkCanRun) {
/*     */       
/* 600 */       checkCompatibleScriptURL(lang, this.docPURL);
/* 601 */       checkCanRun = false;
/*     */     } 
/*     */     
/* 604 */     DocumentLoader dl = this.bridgeContext.getDocumentLoader();
/* 605 */     SVGDocument d = (SVGDocument)elt.getOwnerDocument();
/* 606 */     int line = dl.getLineNumber(elt);
/* 607 */     String desc = Messages.formatMessage("BaseScriptingEnvironment.constant.event.script.description", new Object[] { d.getURL(), "onload", new Integer(line) });
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 613 */     EventListener l = new EventListener(this, interp, s, desc) { private final Interpreter val$interp; private final String val$s;
/*     */         public void handleEvent(Event evt) {
/*     */           
/*     */           try { Event event1;
/* 617 */             if (evt instanceof ScriptEventWrapper) {
/* 618 */               event1 = ((ScriptEventWrapper)evt).getEventObject();
/*     */             } else {
/* 620 */               event1 = evt;
/*     */             } 
/* 622 */             this.val$interp.bindObject("event", event1);
/* 623 */             this.val$interp.bindObject("evt", event1);
/* 624 */             this.val$interp.evaluate(new StringReader(this.val$s), this.val$desc); }
/* 625 */           catch (IOException io) {  }
/*     */           catch (InterpreterException e)
/* 627 */           { this.this$0.handleInterpreterException(e); }
/*     */         
/*     */         } private final String val$desc; private final BaseScriptingEnvironment this$0; }
/*     */       ;
/* 631 */     t.addEventListenerNS("http://www.w3.org/2001/xml-events", str, l, false, null);
/*     */ 
/*     */     
/* 634 */     t.dispatchEvent(ev);
/* 635 */     t.removeEventListenerNS("http://www.w3.org/2001/xml-events", str, l, false);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void dispatchSVGZoomEvent() {
/* 644 */     if (this.bridgeContext.isSVG12()) {
/* 645 */       dispatchSVGDocEvent("zoom");
/*     */     } else {
/* 647 */       dispatchSVGDocEvent("SVGZoom");
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void dispatchSVGScrollEvent() {
/* 655 */     if (this.bridgeContext.isSVG12()) {
/* 656 */       dispatchSVGDocEvent("scroll");
/*     */     } else {
/* 658 */       dispatchSVGDocEvent("SVGScroll");
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void dispatchSVGResizeEvent() {
/* 666 */     if (this.bridgeContext.isSVG12()) {
/* 667 */       dispatchSVGDocEvent("resize");
/*     */     } else {
/* 669 */       dispatchSVGDocEvent("SVGResize");
/*     */     } 
/*     */   }
/*     */   
/*     */   protected void dispatchSVGDocEvent(String eventType) {
/* 674 */     SVGSVGElement root = (SVGSVGElement)this.document.getDocumentElement();
/*     */ 
/*     */     
/* 677 */     SVGSVGElement sVGSVGElement = root;
/*     */     
/* 679 */     DocumentEvent de = (DocumentEvent)this.document;
/* 680 */     AbstractEvent ev = (AbstractEvent)de.createEvent("SVGEvents");
/* 681 */     ev.initEventNS("http://www.w3.org/2001/xml-events", eventType, false, false);
/*     */ 
/*     */ 
/*     */     
/* 685 */     sVGSVGElement.dispatchEvent(ev);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void handleInterpreterException(InterpreterException ie) {
/* 692 */     if (this.userAgent != null) {
/* 693 */       Exception ex = ie.getException();
/* 694 */       this.userAgent.displayError((ex == null) ? ie : ex);
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void handleSecurityException(SecurityException se) {
/* 702 */     if (this.userAgent != null) {
/* 703 */       this.userAgent.displayError(se);
/*     */     }
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected class Window
/*     */     implements Window
/*     */   {
/*     */     protected Interpreter interpreter;
/*     */ 
/*     */ 
/*     */     
/*     */     protected String language;
/*     */ 
/*     */     
/*     */     private final BaseScriptingEnvironment this$0;
/*     */ 
/*     */ 
/*     */     
/*     */     public Window(BaseScriptingEnvironment this$0, Interpreter interp, String lang) {
/* 725 */       this.this$0 = this$0;
/* 726 */       this.interpreter = interp;
/* 727 */       this.language = lang;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 735 */     public Object setInterval(String script, long interval) { return null; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 743 */     public Object setInterval(Runnable r, long interval) { return null; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void clearInterval(Object interval) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 758 */     public Object setTimeout(String script, long timeout) { return null; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 766 */     public Object setTimeout(Runnable r, long timeout) { return null; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void clearTimeout(Object timeout) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 783 */     public Node parseXML(String text, Document doc) { return null; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 792 */     public void getURL(String uri, Window.URLResponseHandler h) { getURL(uri, h, "UTF8"); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void getURL(String uri, Window.URLResponseHandler h, String enc) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 808 */     public void postURL(String uri, String content, Window.URLResponseHandler h) { postURL(uri, content, h, "text/plain", null); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 814 */     public void postURL(String uri, String content, Window.URLResponseHandler h, String mimeType) { postURL(uri, content, h, mimeType, null); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void postURL(String uri, String content, Window.URLResponseHandler h, String mimeType, String fEnc) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void alert(String message) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 836 */     public boolean confirm(String message) { return false; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 843 */     public String prompt(String message) { return null; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 850 */     public String prompt(String message, String defVal) { return null; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 857 */     public BridgeContext getBridgeContext() { return this.this$0.bridgeContext; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 864 */     public Interpreter getInterpreter() { return this.interpreter; }
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\BaseScriptingEnvironment.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */