/*      */ package org.apache.batik.bridge;
/*      */ 
/*      */ import java.awt.Cursor;
/*      */ import java.awt.geom.Dimension2D;
/*      */ import java.io.IOException;
/*      */ import java.io.InterruptedIOException;
/*      */ import java.lang.ref.SoftReference;
/*      */ import java.lang.ref.WeakReference;
/*      */ import java.net.MalformedURLException;
/*      */ import java.util.HashMap;
/*      */ import java.util.HashSet;
/*      */ import java.util.Iterator;
/*      */ import java.util.LinkedList;
/*      */ import java.util.List;
/*      */ import java.util.ListIterator;
/*      */ import java.util.Map;
/*      */ import java.util.Set;
/*      */ import java.util.WeakHashMap;
/*      */ import org.apache.batik.bridge.svg12.SVG12BridgeContext;
/*      */ import org.apache.batik.bridge.svg12.SVG12BridgeExtension;
/*      */ import org.apache.batik.css.engine.CSSContext;
/*      */ import org.apache.batik.css.engine.CSSEngine;
/*      */ import org.apache.batik.css.engine.CSSEngineEvent;
/*      */ import org.apache.batik.css.engine.CSSEngineListener;
/*      */ import org.apache.batik.css.engine.CSSEngineUserAgent;
/*      */ import org.apache.batik.css.engine.SystemColorSupport;
/*      */ import org.apache.batik.css.engine.value.Value;
/*      */ import org.apache.batik.dom.AbstractNode;
/*      */ import org.apache.batik.dom.events.NodeEventTarget;
/*      */ import org.apache.batik.dom.svg.AnimatedAttributeListener;
/*      */ import org.apache.batik.dom.svg.AnimatedLiveAttributeValue;
/*      */ import org.apache.batik.dom.svg.SVGContext;
/*      */ import org.apache.batik.dom.svg.SVGDOMImplementation;
/*      */ import org.apache.batik.dom.svg.SVGOMDocument;
/*      */ import org.apache.batik.dom.svg.SVGOMElement;
/*      */ import org.apache.batik.dom.svg.SVGStylableElement;
/*      */ import org.apache.batik.dom.xbl.XBLManager;
/*      */ import org.apache.batik.gvt.CompositeGraphicsNode;
/*      */ import org.apache.batik.gvt.GraphicsNode;
/*      */ import org.apache.batik.gvt.TextPainter;
/*      */ import org.apache.batik.script.Interpreter;
/*      */ import org.apache.batik.script.InterpreterPool;
/*      */ import org.apache.batik.util.CleanerThread;
/*      */ import org.apache.batik.util.ParsedURL;
/*      */ import org.apache.batik.util.Service;
/*      */ import org.w3c.dom.Document;
/*      */ import org.w3c.dom.Element;
/*      */ import org.w3c.dom.Node;
/*      */ import org.w3c.dom.events.Event;
/*      */ import org.w3c.dom.events.EventListener;
/*      */ import org.w3c.dom.events.EventTarget;
/*      */ import org.w3c.dom.events.MouseEvent;
/*      */ import org.w3c.dom.events.MutationEvent;
/*      */ import org.w3c.dom.svg.SVGDocument;
/*      */ import org.w3c.dom.svg.SVGSVGElement;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class BridgeContext
/*      */   implements ErrorConstants, CSSContext
/*      */ {
/*      */   protected Document document;
/*      */   protected boolean isSVG12;
/*      */   protected GVTBuilder gvtBuilder;
/*  115 */   protected Map interpreterMap = new HashMap(7);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private Map fontFamilyMap;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  129 */   protected Map viewportMap = new WeakHashMap();
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  134 */   protected List viewportStack = new LinkedList();
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected UserAgent userAgent;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Map elementNodeMap;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Map nodeElementMap;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Map namespaceURIMap;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Bridge defaultBridge;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Set reservedNamespaceSet;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Map elementDataMap;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected InterpreterPool interpreterPool;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected DocumentLoader documentLoader;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Dimension2D documentSize;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected TextPainter textPainter;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static final int STATIC = 0;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static final int INTERACTIVE = 1;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static final int DYNAMIC = 2;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  236 */   protected int dynamicStatus = 0;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected UpdateManager updateManager;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected XBLManager xblManager;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected BridgeContext primaryContext;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  257 */   protected HashSet childContexts = new HashSet();
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected SVGAnimationEngine animationEngine;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected int animationLimitingMode;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected float animationLimitingAmount;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  277 */   private static InterpreterPool sharedPool = new InterpreterPool();
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  289 */   public BridgeContext(UserAgent userAgent) { this(userAgent, sharedPool, new DocumentLoader(userAgent)); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  301 */   public BridgeContext(UserAgent userAgent, DocumentLoader loader) { this(userAgent, sharedPool, loader); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public BridgeContext(UserAgent userAgent, InterpreterPool interpreterPool, DocumentLoader documentLoader) {
/*  313 */     this.userAgent = userAgent;
/*  314 */     this.viewportMap.put(userAgent, new UserAgentViewport(userAgent));
/*  315 */     this.interpreterPool = interpreterPool;
/*  316 */     this.documentLoader = documentLoader;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void finalize() {
/*  323 */     if (this.primaryContext != null) {
/*  324 */       dispose();
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public BridgeContext createSubBridgeContext(SVGOMDocument newDoc) {
/*  338 */     CSSEngine eng = newDoc.getCSSEngine();
/*  339 */     if (eng != null) {
/*  340 */       return (BridgeContext)newDoc.getCSSEngine().getCSSContext();
/*      */     }
/*      */ 
/*      */     
/*  344 */     BridgeContext bridgeContext = createBridgeContext(newDoc);
/*  345 */     bridgeContext.primaryContext = (this.primaryContext != null) ? this.primaryContext : this;
/*  346 */     bridgeContext.primaryContext.childContexts.add(new WeakReference(bridgeContext));
/*  347 */     bridgeContext.dynamicStatus = this.dynamicStatus;
/*  348 */     bridgeContext.setGVTBuilder(getGVTBuilder());
/*  349 */     bridgeContext.setTextPainter(getTextPainter());
/*  350 */     bridgeContext.setDocument(newDoc);
/*  351 */     bridgeContext.initializeDocument(newDoc);
/*  352 */     if (isInteractive())
/*  353 */       bridgeContext.addUIEventListeners(newDoc); 
/*  354 */     return bridgeContext;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public BridgeContext createBridgeContext(SVGOMDocument doc) {
/*  363 */     if (doc.isSVG12()) {
/*  364 */       return new SVG12BridgeContext(getUserAgent(), getDocumentLoader());
/*      */     }
/*  366 */     return new BridgeContext(getUserAgent(), getDocumentLoader());
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void initializeDocument(Document document) {
/*  373 */     SVGOMDocument doc = (SVGOMDocument)document;
/*  374 */     CSSEngine eng = doc.getCSSEngine();
/*  375 */     if (eng == null) {
/*      */       
/*  377 */       SVGDOMImplementation impl = (SVGDOMImplementation)doc.getImplementation();
/*  378 */       eng = impl.createCSSEngine(doc, this);
/*  379 */       eng.setCSSEngineUserAgent(new CSSEngineUserAgentWrapper(this.userAgent));
/*  380 */       doc.setCSSEngine(eng);
/*  381 */       eng.setMedia(this.userAgent.getMedia());
/*  382 */       String uri = this.userAgent.getUserStyleSheetURI();
/*  383 */       if (uri != null) {
/*      */         try {
/*  385 */           ParsedURL url = new ParsedURL(uri);
/*  386 */           eng.setUserAgentStyleSheet(eng.parseStyleSheet(url, "all"));
/*      */         } catch (Exception e) {
/*      */           
/*  389 */           this.userAgent.displayError(e);
/*      */         } 
/*      */       }
/*  392 */       eng.setAlternateStyleSheet(this.userAgent.getAlternateStyleSheet());
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public CSSEngine getCSSEngineForElement(Element e) {
/*  400 */     SVGOMDocument doc = (SVGOMDocument)e.getOwnerDocument();
/*  401 */     return doc.getCSSEngine();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  414 */   public void setTextPainter(TextPainter textPainter) { this.textPainter = textPainter; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  421 */   public TextPainter getTextPainter() { return this.textPainter; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  428 */   public Document getDocument() { return this.document; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void setDocument(Document document) {
/*  437 */     if (this.document != document) {
/*  438 */       this.fontFamilyMap = null;
/*      */     }
/*  440 */     this.document = document;
/*  441 */     this.isSVG12 = ((SVGOMDocument)document).isSVG12();
/*  442 */     registerSVGBridges();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Map getFontFamilyMap() {
/*  449 */     if (this.fontFamilyMap == null) {
/*  450 */       this.fontFamilyMap = new HashMap();
/*      */     }
/*  452 */     return this.fontFamilyMap;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  461 */   protected void setFontFamilyMap(Map fontFamilyMap) { this.fontFamilyMap = fontFamilyMap; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void setElementData(Node n, Object data) {
/*  470 */     if (this.elementDataMap == null) {
/*  471 */       this.elementDataMap = new WeakHashMap();
/*      */     }
/*  473 */     this.elementDataMap.put(n, new SoftReference(data));
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Object getElementData(Node n) {
/*  480 */     if (this.elementDataMap == null)
/*  481 */       return null; 
/*  482 */     Object o = this.elementDataMap.get(n);
/*  483 */     if (o == null) return null; 
/*  484 */     SoftReference sr = (SoftReference)o;
/*  485 */     o = sr.get();
/*  486 */     if (o == null) {
/*  487 */       this.elementDataMap.remove(n);
/*      */     }
/*  489 */     return o;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  496 */   public UserAgent getUserAgent() { return this.userAgent; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  504 */   protected void setUserAgent(UserAgent userAgent) { this.userAgent = userAgent; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  511 */   public GVTBuilder getGVTBuilder() { return this.gvtBuilder; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  518 */   protected void setGVTBuilder(GVTBuilder gvtBuilder) { this.gvtBuilder = gvtBuilder; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  525 */   public InterpreterPool getInterpreterPool() { return this.interpreterPool; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  532 */   public FocusManager getFocusManager() { return this.focusManager; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  539 */   public CursorManager getCursorManager() { return this.cursorManager; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  548 */   protected void setInterpreterPool(InterpreterPool interpreterPool) { this.interpreterPool = interpreterPool; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Interpreter getInterpreter(String language) {
/*  557 */     if (this.document == null) {
/*  558 */       throw new RuntimeException("Unknown document");
/*      */     }
/*  560 */     Interpreter interpreter = (Interpreter)this.interpreterMap.get(language);
/*  561 */     if (interpreter == null) {
/*      */       try {
/*  563 */         interpreter = this.interpreterPool.createInterpreter(this.document, language);
/*  564 */         this.interpreterMap.put(language, interpreter);
/*      */       } catch (Exception e) {
/*  566 */         if (this.userAgent != null) {
/*  567 */           this.userAgent.displayError(e);
/*  568 */           return null;
/*      */         } 
/*      */       } 
/*      */     }
/*      */     
/*  573 */     if (interpreter == null && 
/*  574 */       this.userAgent != null) {
/*  575 */       this.userAgent.displayError(new Exception("Unknown language: " + language));
/*      */     }
/*      */ 
/*      */     
/*  579 */     return interpreter;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  586 */   public DocumentLoader getDocumentLoader() { return this.documentLoader; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  594 */   protected void setDocumentLoader(DocumentLoader newDocumentLoader) { this.documentLoader = newDocumentLoader; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  602 */   public Dimension2D getDocumentSize() { return this.documentSize; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  611 */   protected void setDocumentSize(Dimension2D d) { this.documentSize = d; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  618 */   public boolean isDynamic() { return (this.dynamicStatus == 2); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  625 */   public boolean isInteractive() { return (this.dynamicStatus != 0); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  637 */   public void setDynamicState(int status) { this.dynamicStatus = status; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void setDynamic(boolean dynamic) {
/*  645 */     if (dynamic) {
/*  646 */       setDynamicState(2);
/*      */     } else {
/*  648 */       setDynamicState(0);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void setInteractive(boolean interactive) {
/*  656 */     if (interactive) {
/*  657 */       setDynamicState(1);
/*      */     } else {
/*  659 */       setDynamicState(0);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  666 */   public UpdateManager getUpdateManager() { return this.updateManager; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  673 */   protected void setUpdateManager(UpdateManager um) { this.updateManager = um; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  680 */   protected void setUpdateManager(BridgeContext ctx, UpdateManager um) { ctx.setUpdateManager(um); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  687 */   protected void setXBLManager(BridgeContext ctx, XBLManager xm) { ctx.xblManager = xm; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  694 */   public boolean isSVG12() { return this.isSVG12; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public BridgeContext getPrimaryBridgeContext() {
/*  701 */     if (this.primaryContext != null) {
/*  702 */       return this.primaryContext;
/*      */     }
/*  704 */     return this;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public BridgeContext[] getChildContexts() {
/*  711 */     BridgeContext[] res = new BridgeContext[this.childContexts.size()];
/*  712 */     Iterator it = this.childContexts.iterator();
/*  713 */     for (int i = 0; i < res.length; i++) {
/*  714 */       WeakReference wr = (WeakReference)it.next();
/*  715 */       res[i] = (BridgeContext)wr.get();
/*      */     } 
/*  717 */     return res;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public SVGAnimationEngine getAnimationEngine() {
/*  725 */     if (this.animationEngine == null) {
/*  726 */       this.animationEngine = new SVGAnimationEngine(this.document, this);
/*  727 */       setAnimationLimitingMode();
/*      */     } 
/*  729 */     return this.animationEngine;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  738 */   public URIResolver createURIResolver(SVGDocument doc, DocumentLoader dl) { return new URIResolver(doc, dl); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Node getReferencedNode(Element e, String uri) {
/*      */     try {
/*  750 */       SVGDocument document = (SVGDocument)e.getOwnerDocument();
/*  751 */       URIResolver ur = createURIResolver(document, this.documentLoader);
/*  752 */       Node ref = ur.getNode(uri, e);
/*  753 */       if (ref == null) {
/*  754 */         throw new BridgeException(this, e, "uri.badTarget", new Object[] { uri });
/*      */       }
/*      */       
/*  757 */       SVGOMDocument refDoc = (SVGOMDocument)((ref.getNodeType() == 9) ? ref : ref.getOwnerDocument());
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  768 */       if (refDoc != document) {
/*  769 */         createSubBridgeContext(refDoc);
/*      */       }
/*  771 */       return ref;
/*      */     } catch (MalformedURLException ex) {
/*      */       
/*  774 */       throw new BridgeException(this, e, ex, "uri.malformed", new Object[] { uri });
/*      */     } catch (InterruptedIOException ex) {
/*      */       
/*  777 */       throw new InterruptedBridgeException();
/*      */     } catch (IOException ex) {
/*      */       
/*  780 */       throw new BridgeException(this, e, ex, "uri.io", new Object[] { uri });
/*      */     } catch (SecurityException ex) {
/*      */       
/*  783 */       throw new BridgeException(this, e, ex, "uri.unsecure", new Object[] { uri });
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Element getReferencedElement(Element e, String uri) {
/*  796 */     Node ref = getReferencedNode(e, uri);
/*  797 */     if (ref != null && ref.getNodeType() != 1) {
/*  798 */       throw new BridgeException(this, e, "uri.referenceDocument", new Object[] { uri });
/*      */     }
/*      */     
/*  801 */     return (Element)ref;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Viewport getViewport(Element e) {
/*  812 */     if (this.viewportStack != null) {
/*      */       
/*  814 */       if (this.viewportStack.size() == 0)
/*      */       {
/*  816 */         return (Viewport)this.viewportMap.get(this.userAgent);
/*      */       }
/*      */       
/*  819 */       return (Viewport)this.viewportStack.get(0);
/*      */     } 
/*      */ 
/*      */     
/*  823 */     e = SVGUtilities.getParentElement(e);
/*  824 */     while (e != null) {
/*  825 */       Viewport viewport = (Viewport)this.viewportMap.get(e);
/*  826 */       if (viewport != null) {
/*  827 */         return viewport;
/*      */       }
/*  829 */       e = SVGUtilities.getParentElement(e);
/*      */     } 
/*  831 */     return (Viewport)this.viewportMap.get(this.userAgent);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void openViewport(Element e, Viewport viewport) {
/*  842 */     this.viewportMap.put(e, viewport);
/*  843 */     if (this.viewportStack == null) {
/*  844 */       this.viewportStack = new LinkedList();
/*      */     }
/*  846 */     this.viewportStack.add(0, viewport);
/*      */   }
/*      */ 
/*      */   
/*  850 */   public void removeViewport(Element e) { this.viewportMap.remove(e); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void closeViewport(Element e) {
/*  859 */     this.viewportStack.remove(0);
/*  860 */     if (this.viewportStack.size() == 0) {
/*  861 */       this.viewportStack = null;
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void bind(Node node, GraphicsNode gn) {
/*  876 */     if (this.elementNodeMap == null) {
/*  877 */       this.elementNodeMap = new WeakHashMap();
/*  878 */       this.nodeElementMap = new WeakHashMap();
/*      */     } 
/*  880 */     this.elementNodeMap.put(node, new SoftReference(gn));
/*  881 */     this.nodeElementMap.put(gn, new SoftReference(node));
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void unbind(Node node) {
/*  890 */     if (this.elementNodeMap == null) {
/*      */       return;
/*      */     }
/*  893 */     GraphicsNode gn = null;
/*  894 */     SoftReference sr = (SoftReference)this.elementNodeMap.get(node);
/*  895 */     if (sr != null)
/*  896 */       gn = (GraphicsNode)sr.get(); 
/*  897 */     this.elementNodeMap.remove(node);
/*  898 */     if (gn != null) {
/*  899 */       this.nodeElementMap.remove(gn);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public GraphicsNode getGraphicsNode(Node node) {
/*  909 */     if (this.elementNodeMap != null) {
/*  910 */       SoftReference sr = (SoftReference)this.elementNodeMap.get(node);
/*  911 */       if (sr != null)
/*  912 */         return (GraphicsNode)sr.get(); 
/*      */     } 
/*  914 */     return null;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Element getElement(GraphicsNode gn) {
/*  924 */     if (this.nodeElementMap != null) {
/*  925 */       SoftReference sr = (SoftReference)this.nodeElementMap.get(gn);
/*  926 */       if (sr != null) {
/*  927 */         Node n = (Node)sr.get();
/*  928 */         if (n.getNodeType() == 1) {
/*  929 */           return (Element)n;
/*      */         }
/*      */       } 
/*      */     } 
/*  933 */     return null;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public boolean hasGraphicsNodeBridge(Element element) {
/*  945 */     if (this.namespaceURIMap == null || element == null) {
/*  946 */       return false;
/*      */     }
/*  948 */     String localName = element.getLocalName();
/*  949 */     String namespaceURI = element.getNamespaceURI();
/*  950 */     namespaceURI = (namespaceURI == null) ? "" : namespaceURI;
/*  951 */     HashMap localNameMap = (HashMap)this.namespaceURIMap.get(namespaceURI);
/*  952 */     if (localNameMap == null) {
/*  953 */       return false;
/*      */     }
/*  955 */     return localNameMap.get(localName) instanceof GraphicsNodeBridge;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  962 */   public DocumentBridge getDocumentBridge() { return new SVGDocumentBridge(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Bridge getBridge(Element element) {
/*  971 */     if (this.namespaceURIMap == null || element == null) {
/*  972 */       return null;
/*      */     }
/*  974 */     String localName = element.getLocalName();
/*  975 */     String namespaceURI = element.getNamespaceURI();
/*  976 */     namespaceURI = (namespaceURI == null) ? "" : namespaceURI;
/*  977 */     return getBridge(namespaceURI, localName);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Bridge getBridge(String namespaceURI, String localName) {
/*  988 */     Bridge bridge = null;
/*  989 */     if (this.namespaceURIMap != null) {
/*  990 */       HashMap localNameMap = (HashMap)this.namespaceURIMap.get(namespaceURI);
/*  991 */       if (localNameMap != null) {
/*  992 */         bridge = (Bridge)localNameMap.get(localName);
/*      */       }
/*      */     } 
/*  995 */     if (bridge == null && (this.reservedNamespaceSet == null || !this.reservedNamespaceSet.contains(namespaceURI)))
/*      */     {
/*      */       
/*  998 */       bridge = this.defaultBridge;
/*      */     }
/* 1000 */     if (isDynamic()) {
/* 1001 */       return (bridge == null) ? null : bridge.getInstance();
/*      */     }
/* 1003 */     return bridge;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void putBridge(String namespaceURI, String localName, Bridge bridge) {
/* 1016 */     if (!namespaceURI.equals(bridge.getNamespaceURI()) || !localName.equals(bridge.getLocalName()))
/*      */     {
/* 1018 */       throw new Error("Invalid Bridge: " + namespaceURI + "/" + bridge.getNamespaceURI() + " " + localName + "/" + bridge.getLocalName() + " " + bridge.getClass());
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1024 */     if (this.namespaceURIMap == null) {
/* 1025 */       this.namespaceURIMap = new HashMap();
/*      */     }
/* 1027 */     namespaceURI = (namespaceURI == null) ? "" : namespaceURI;
/* 1028 */     HashMap localNameMap = (HashMap)this.namespaceURIMap.get(namespaceURI);
/* 1029 */     if (localNameMap == null) {
/* 1030 */       localNameMap = new HashMap();
/* 1031 */       this.namespaceURIMap.put(namespaceURI, localNameMap);
/*      */     } 
/* 1033 */     localNameMap.put(localName, bridge);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1043 */   public void putBridge(Bridge bridge) { putBridge(bridge.getNamespaceURI(), bridge.getLocalName(), bridge); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void removeBridge(String namespaceURI, String localName) {
/* 1054 */     if (this.namespaceURIMap == null) {
/*      */       return;
/*      */     }
/* 1057 */     namespaceURI = (namespaceURI == null) ? "" : namespaceURI;
/* 1058 */     HashMap localNameMap = (HashMap)this.namespaceURIMap.get(namespaceURI);
/* 1059 */     if (localNameMap != null) {
/* 1060 */       localNameMap.remove(localName);
/* 1061 */       if (localNameMap.isEmpty()) {
/* 1062 */         this.namespaceURIMap.remove(namespaceURI);
/* 1063 */         if (this.namespaceURIMap.isEmpty()) {
/* 1064 */           this.namespaceURIMap = null;
/*      */         }
/*      */       } 
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1077 */   public void setDefaultBridge(Bridge bridge) { this.defaultBridge = bridge; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void putReservedNamespaceURI(String namespaceURI) {
/* 1084 */     if (namespaceURI == null) {
/* 1085 */       namespaceURI = "";
/*      */     }
/* 1087 */     if (this.reservedNamespaceSet == null) {
/* 1088 */       this.reservedNamespaceSet = new HashSet();
/*      */     }
/* 1090 */     this.reservedNamespaceSet.add(namespaceURI);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void removeReservedNamespaceURI(String namespaceURI) {
/* 1097 */     if (namespaceURI == null) {
/* 1098 */       namespaceURI = "";
/*      */     }
/* 1100 */     if (this.reservedNamespaceSet != null) {
/* 1101 */       this.reservedNamespaceSet.remove(namespaceURI);
/* 1102 */       if (this.reservedNamespaceSet.isEmpty()) {
/* 1103 */         this.reservedNamespaceSet = null;
/*      */       }
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1114 */   protected Set eventListenerSet = new HashSet();
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected EventListener domCharacterDataModifiedEventListener;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected EventListener domAttrModifiedEventListener;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected EventListener domNodeInsertedEventListener;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected EventListener domNodeRemovedEventListener;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected CSSEngineListener cssPropertiesChangedListener;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected AnimatedAttributeListener animatedAttributeListener;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected FocusManager focusManager;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1154 */   protected CursorManager cursorManager = new CursorManager(this);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void addUIEventListeners(Document doc) {
/* 1169 */     NodeEventTarget evtTarget = (NodeEventTarget)doc.getDocumentElement();
/*      */     
/* 1171 */     DOMMouseOverEventListener domMouseOverListener = new DOMMouseOverEventListener(this);
/*      */     
/* 1173 */     evtTarget.addEventListenerNS("http://www.w3.org/2001/xml-events", "mouseover", domMouseOverListener, true, null);
/*      */ 
/*      */ 
/*      */     
/* 1177 */     storeEventListenerNS(evtTarget, "http://www.w3.org/2001/xml-events", "mouseover", domMouseOverListener, true);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1183 */     DOMMouseOutEventListener domMouseOutListener = new DOMMouseOutEventListener(this);
/*      */     
/* 1185 */     evtTarget.addEventListenerNS("http://www.w3.org/2001/xml-events", "mouseout", domMouseOutListener, true, null);
/*      */ 
/*      */ 
/*      */     
/* 1189 */     storeEventListenerNS(evtTarget, "http://www.w3.org/2001/xml-events", "mouseout", domMouseOutListener, true);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void removeUIEventListeners(Document doc) {
/* 1198 */     EventTarget evtTarget = (EventTarget)doc.getDocumentElement();
/* 1199 */     synchronized (this.eventListenerSet) {
/* 1200 */       Iterator i = this.eventListenerSet.iterator();
/* 1201 */       while (i.hasNext()) {
/* 1202 */         EventListenerMememto elm = (EventListenerMememto)i.next();
/* 1203 */         NodeEventTarget et = elm.getTarget();
/* 1204 */         if (et == evtTarget) {
/* 1205 */           EventListener el = elm.getListener();
/* 1206 */           boolean uc = elm.getUseCapture();
/* 1207 */           String t = elm.getEventType();
/* 1208 */           boolean n = elm.getNamespaced();
/* 1209 */           if (et == null || el == null || t == null) {
/*      */             continue;
/*      */           }
/* 1212 */           if (n) {
/* 1213 */             String ns = elm.getNamespaceURI();
/* 1214 */             et.removeEventListenerNS(ns, t, el, uc); continue;
/*      */           } 
/* 1216 */           et.removeEventListener(t, el, uc);
/*      */         } 
/*      */       } 
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void addDOMListeners() {
/* 1229 */     SVGOMDocument doc = (SVGOMDocument)this.document;
/*      */     
/* 1231 */     this.domAttrModifiedEventListener = new DOMAttrModifiedEventListener(this);
/* 1232 */     doc.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMAttrModified", this.domAttrModifiedEventListener, true, null);
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1237 */     this.domNodeInsertedEventListener = new DOMNodeInsertedEventListener(this);
/* 1238 */     doc.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeInserted", this.domNodeInsertedEventListener, true, null);
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1243 */     this.domNodeRemovedEventListener = new DOMNodeRemovedEventListener(this);
/* 1244 */     doc.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", this.domNodeRemovedEventListener, true, null);
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1249 */     this.domCharacterDataModifiedEventListener = new DOMCharacterDataModifiedEventListener(this);
/*      */     
/* 1251 */     doc.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMCharacterDataModified", this.domCharacterDataModifiedEventListener, true, null);
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1256 */     this.animatedAttributeListener = new AnimatedAttrListener(this);
/* 1257 */     doc.addAnimatedAttributeListener(this.animatedAttributeListener);
/*      */     
/* 1259 */     this.focusManager = new FocusManager(this.document);
/*      */     
/* 1261 */     CSSEngine cssEngine = doc.getCSSEngine();
/* 1262 */     this.cssPropertiesChangedListener = new CSSPropertiesChangedListener(this);
/* 1263 */     cssEngine.addCSSEngineListener(this.cssPropertiesChangedListener);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void removeDOMListeners() {
/* 1270 */     SVGOMDocument doc = (SVGOMDocument)this.document;
/*      */     
/* 1272 */     doc.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMAttrModified", this.domAttrModifiedEventListener, true);
/*      */ 
/*      */     
/* 1275 */     doc.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeInserted", this.domNodeInsertedEventListener, true);
/*      */ 
/*      */     
/* 1278 */     doc.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", this.domNodeRemovedEventListener, true);
/*      */ 
/*      */     
/* 1281 */     doc.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMCharacterDataModified", this.domCharacterDataModifiedEventListener, true);
/*      */ 
/*      */ 
/*      */     
/* 1285 */     doc.removeAnimatedAttributeListener(this.animatedAttributeListener);
/*      */     
/* 1287 */     CSSEngine cssEngine = doc.getCSSEngine();
/* 1288 */     if (cssEngine != null) {
/* 1289 */       cssEngine.removeCSSEngineListener(this.cssPropertiesChangedListener);
/*      */       
/* 1291 */       cssEngine.dispose();
/* 1292 */       doc.setCSSEngine(null);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void storeEventListener(EventTarget t, String s, EventListener l, boolean b) {
/* 1304 */     synchronized (this.eventListenerSet) {
/* 1305 */       this.eventListenerSet.add(new EventListenerMememto(t, s, l, b, this));
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void storeEventListenerNS(EventTarget t, String n, String s, EventListener l, boolean b) {
/* 1318 */     synchronized (this.eventListenerSet) {
/* 1319 */       this.eventListenerSet.add(new EventListenerMememto(t, n, s, l, b, this));
/*      */     } 
/*      */   }
/*      */   
/*      */   public static class SoftReferenceMememto
/*      */     extends CleanerThread.SoftReferenceCleared {
/*      */     Object mememto;
/*      */     Set set;
/*      */     
/*      */     SoftReferenceMememto(Object ref, Object mememto, Set set) {
/* 1329 */       super(ref);
/*      */       
/* 1331 */       this.mememto = mememto;
/* 1332 */       this.set = set;
/*      */     }
/*      */     
/*      */     public void cleared() {
/* 1336 */       synchronized (this.set) {
/*      */         
/* 1338 */         this.set.remove(this.mememto);
/* 1339 */         this.mememto = null;
/* 1340 */         this.set = null;
/*      */       } 
/*      */     }
/*      */   }
/*      */ 
/*      */   
/*      */   protected static class EventListenerMememto
/*      */   {
/*      */     public SoftReference target;
/*      */     
/*      */     public SoftReference listener;
/*      */     
/*      */     public boolean useCapture;
/*      */     
/*      */     public String namespaceURI;
/*      */     
/*      */     public String eventType;
/*      */     
/*      */     public boolean namespaced;
/*      */ 
/*      */     
/*      */     public EventListenerMememto(EventTarget t, String s, EventListener l, boolean b, BridgeContext ctx) {
/* 1362 */       Set set = ctx.eventListenerSet;
/* 1363 */       this.target = new BridgeContext.SoftReferenceMememto(t, this, set);
/* 1364 */       this.listener = new BridgeContext.SoftReferenceMememto(l, this, set);
/* 1365 */       this.eventType = s;
/* 1366 */       this.useCapture = b;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public EventListenerMememto(EventTarget t, String n, String s, EventListener l, boolean b, BridgeContext ctx) {
/* 1375 */       this(t, s, l, b, ctx);
/* 1376 */       this.namespaceURI = n;
/* 1377 */       this.namespaced = true;
/*      */     }
/*      */ 
/*      */     
/* 1381 */     public EventListener getListener() { return (EventListener)this.listener.get(); }
/*      */ 
/*      */     
/* 1384 */     public NodeEventTarget getTarget() { return (NodeEventTarget)this.target.get(); }
/*      */ 
/*      */     
/* 1387 */     public boolean getUseCapture() { return this.useCapture; }
/*      */ 
/*      */     
/* 1390 */     public String getNamespaceURI() { return this.namespaceURI; }
/*      */ 
/*      */     
/* 1393 */     public String getEventType() { return this.eventType; }
/*      */ 
/*      */     
/* 1396 */     public boolean getNamespaced() { return this.namespaced; }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1404 */   public void addGVTListener(Document doc) { BridgeEventSupport.addGVTListener(this, doc); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1412 */   protected void clearChildContexts() { this.childContexts.clear(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void dispose() {
/* 1419 */     clearChildContexts();
/*      */     
/* 1421 */     synchronized (this.eventListenerSet) {
/*      */       
/* 1423 */       Iterator iter = this.eventListenerSet.iterator();
/* 1424 */       while (iter.hasNext()) {
/* 1425 */         EventListenerMememto m = (EventListenerMememto)iter.next();
/* 1426 */         NodeEventTarget et = m.getTarget();
/* 1427 */         EventListener el = m.getListener();
/* 1428 */         boolean uc = m.getUseCapture();
/* 1429 */         String t = m.getEventType();
/* 1430 */         boolean n = m.getNamespaced();
/* 1431 */         if (et == null || el == null || t == null) {
/*      */           continue;
/*      */         }
/* 1434 */         if (n) {
/* 1435 */           String ns = m.getNamespaceURI();
/* 1436 */           et.removeEventListenerNS(ns, t, el, uc); continue;
/*      */         } 
/* 1438 */         et.removeEventListener(t, el, uc);
/*      */       } 
/*      */     } 
/*      */ 
/*      */     
/* 1443 */     if (this.document != null) {
/* 1444 */       removeDOMListeners();
/*      */     }
/*      */     
/* 1447 */     if (this.animationEngine != null) {
/* 1448 */       this.animationEngine.dispose();
/* 1449 */       this.animationEngine = null;
/*      */     } 
/*      */     
/* 1452 */     Iterator iter = this.interpreterMap.values().iterator();
/* 1453 */     while (iter.hasNext()) {
/* 1454 */       Interpreter interpreter = (Interpreter)iter.next();
/* 1455 */       if (interpreter != null)
/* 1456 */         interpreter.dispose(); 
/*      */     } 
/* 1458 */     this.interpreterMap.clear();
/*      */     
/* 1460 */     if (this.focusManager != null) {
/* 1461 */       this.focusManager.dispose();
/*      */     }
/* 1463 */     if (this.elementDataMap != null) {
/* 1464 */       this.elementDataMap.clear();
/*      */     }
/* 1466 */     if (this.nodeElementMap != null) {
/* 1467 */       this.nodeElementMap.clear();
/*      */     }
/* 1469 */     if (this.elementNodeMap != null) {
/* 1470 */       this.elementNodeMap.clear();
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected static SVGContext getSVGContext(Node node) {
/* 1479 */     if (node instanceof SVGOMElement)
/* 1480 */       return ((SVGOMElement)node).getSVGContext(); 
/* 1481 */     if (node instanceof SVGOMDocument) {
/* 1482 */       return ((SVGOMDocument)node).getSVGContext();
/*      */     }
/* 1484 */     return null;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected static BridgeUpdateHandler getBridgeUpdateHandler(Node node) {
/* 1493 */     SVGContext ctx = getSVGContext(node);
/* 1494 */     return (ctx == null) ? null : (BridgeUpdateHandler)ctx;
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class DOMAttrModifiedEventListener
/*      */     implements EventListener
/*      */   {
/*      */     private final BridgeContext this$0;
/*      */ 
/*      */     
/* 1505 */     public DOMAttrModifiedEventListener(BridgeContext this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1512 */       Node node = (Node)evt.getTarget();
/* 1513 */       BridgeUpdateHandler h = BridgeContext.getBridgeUpdateHandler(node);
/* 1514 */       if (h != null) {
/*      */         try {
/* 1516 */           h.handleDOMAttrModifiedEvent((MutationEvent)evt);
/*      */         } catch (Exception e) {
/* 1518 */           this.this$0.userAgent.displayError(e);
/*      */         } 
/*      */       }
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class DOMMouseOutEventListener
/*      */     implements EventListener
/*      */   {
/*      */     private final BridgeContext this$0;
/*      */ 
/*      */     
/* 1532 */     public DOMMouseOutEventListener(BridgeContext this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1539 */       MouseEvent me = (MouseEvent)evt;
/* 1540 */       Element newTarget = (Element)me.getRelatedTarget();
/* 1541 */       Cursor cursor = CursorManager.DEFAULT_CURSOR;
/* 1542 */       if (newTarget != null) {
/* 1543 */         cursor = CSSUtilities.convertCursor(newTarget, this.this$0);
/*      */       }
/* 1545 */       if (cursor == null) {
/* 1546 */         cursor = CursorManager.DEFAULT_CURSOR;
/*      */       }
/* 1548 */       this.this$0.userAgent.setSVGCursor(cursor);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class DOMMouseOverEventListener
/*      */     implements EventListener
/*      */   {
/*      */     private final BridgeContext this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1565 */     public DOMMouseOverEventListener(BridgeContext this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1572 */       Element target = (Element)evt.getTarget();
/* 1573 */       Cursor cursor = CSSUtilities.convertCursor(target, this.this$0);
/*      */       
/* 1575 */       if (cursor != null) {
/* 1576 */         this.this$0.userAgent.setSVGCursor(cursor);
/*      */       }
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class DOMNodeInsertedEventListener
/*      */     implements EventListener
/*      */   {
/*      */     private final BridgeContext this$0;
/*      */ 
/*      */     
/* 1589 */     public DOMNodeInsertedEventListener(BridgeContext this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1596 */       MutationEvent me = (MutationEvent)evt;
/* 1597 */       BridgeUpdateHandler h = BridgeContext.getBridgeUpdateHandler(me.getRelatedNode());
/*      */       
/* 1599 */       if (h != null) {
/*      */         try {
/* 1601 */           h.handleDOMNodeInsertedEvent(me);
/* 1602 */         } catch (InterruptedBridgeException ibe) {
/*      */         
/*      */         } catch (Exception e) {
/* 1605 */           this.this$0.userAgent.displayError(e);
/*      */         } 
/*      */       }
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class DOMNodeRemovedEventListener
/*      */     implements EventListener
/*      */   {
/*      */     private final BridgeContext this$0;
/*      */ 
/*      */     
/* 1619 */     public DOMNodeRemovedEventListener(BridgeContext this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1626 */       Node node = (Node)evt.getTarget();
/* 1627 */       BridgeUpdateHandler h = BridgeContext.getBridgeUpdateHandler(node);
/* 1628 */       if (h != null) {
/*      */         try {
/* 1630 */           h.handleDOMNodeRemovedEvent((MutationEvent)evt);
/*      */         } catch (Exception e) {
/* 1632 */           this.this$0.userAgent.displayError(e);
/*      */         } 
/*      */       }
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class DOMCharacterDataModifiedEventListener
/*      */     implements EventListener
/*      */   {
/*      */     private final BridgeContext this$0;
/*      */ 
/*      */ 
/*      */     
/* 1647 */     public DOMCharacterDataModifiedEventListener(BridgeContext this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1654 */       Node node = (Node)evt.getTarget();
/* 1655 */       while (node != null && !(node instanceof SVGOMElement)) {
/* 1656 */         node = (Node)((AbstractNode)node).getParentNodeEventTarget();
/*      */       }
/* 1658 */       BridgeUpdateHandler h = BridgeContext.getBridgeUpdateHandler(node);
/* 1659 */       if (h != null) {
/*      */         try {
/* 1661 */           h.handleDOMCharacterDataModified((MutationEvent)evt);
/*      */         } catch (Exception e) {
/* 1663 */           this.this$0.userAgent.displayError(e);
/*      */         } 
/*      */       }
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class CSSPropertiesChangedListener
/*      */     implements CSSEngineListener
/*      */   {
/*      */     private final BridgeContext this$0;
/*      */ 
/*      */ 
/*      */     
/* 1678 */     public CSSPropertiesChangedListener(BridgeContext this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void propertiesChanged(CSSEngineEvent evt) {
/* 1686 */       Element elem = evt.getElement();
/* 1687 */       SVGContext ctx = BridgeContext.getSVGContext(elem);
/* 1688 */       if (ctx == null) {
/* 1689 */         GraphicsNode pgn = this.this$0.getGraphicsNode(elem.getParentNode());
/* 1690 */         if (pgn == null || !(pgn instanceof CompositeGraphicsNode)) {
/*      */           return;
/*      */         }
/*      */ 
/*      */         
/* 1695 */         CompositeGraphicsNode parent = (CompositeGraphicsNode)pgn;
/*      */ 
/*      */         
/* 1698 */         int[] properties = evt.getProperties();
/* 1699 */         for (int i = 0; i < properties.length; i++) {
/* 1700 */           if (properties[i] == 12) {
/* 1701 */             if (!CSSUtilities.convertDisplay(elem)) {
/*      */               break;
/*      */             }
/*      */ 
/*      */             
/* 1706 */             GVTBuilder builder = this.this$0.getGVTBuilder();
/* 1707 */             GraphicsNode childNode = builder.build(this.this$0, elem);
/*      */             
/* 1709 */             if (childNode == null) {
/*      */               break;
/*      */             }
/*      */             
/* 1713 */             int idx = -1;
/* 1714 */             for (Node ps = elem.getPreviousSibling(); ps != null; 
/* 1715 */               ps = ps.getPreviousSibling()) {
/* 1716 */               if (ps.getNodeType() != 1)
/*      */                 continue; 
/* 1718 */               Element pse = (Element)ps;
/* 1719 */               GraphicsNode gn = this.this$0.getGraphicsNode(pse);
/* 1720 */               if (gn == null)
/*      */                 continue; 
/* 1722 */               idx = parent.indexOf(gn);
/* 1723 */               if (idx == -1) {
/*      */                 continue;
/*      */               }
/*      */               
/*      */               break;
/*      */             } 
/* 1729 */             idx++;
/* 1730 */             parent.add(idx, childNode); break;
/*      */           } 
/*      */         } 
/*      */       } 
/* 1734 */       if (ctx != null && ctx instanceof BridgeUpdateHandler) {
/* 1735 */         ((BridgeUpdateHandler)ctx).handleCSSEngineEvent(evt);
/*      */       }
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class AnimatedAttrListener
/*      */     implements AnimatedAttributeListener
/*      */   {
/*      */     private final BridgeContext this$0;
/*      */ 
/*      */ 
/*      */     
/* 1749 */     public AnimatedAttrListener(BridgeContext this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void animatedAttributeChanged(Element e, AnimatedLiveAttributeValue alav) {
/* 1760 */       BridgeUpdateHandler h = BridgeContext.getBridgeUpdateHandler(e);
/* 1761 */       if (h != null) {
/*      */         try {
/* 1763 */           h.handleAnimatedAttributeChanged(alav);
/*      */         } catch (Exception ex) {
/* 1765 */           this.this$0.userAgent.displayError(ex);
/*      */         } 
/*      */       }
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void otherAnimationChanged(Element e, String type) {
/* 1777 */       BridgeUpdateHandler h = BridgeContext.getBridgeUpdateHandler(e);
/* 1778 */       if (h != null) {
/*      */         try {
/* 1780 */           h.handleOtherAnimationChanged(type);
/*      */         } catch (Exception ex) {
/* 1782 */           this.this$0.userAgent.displayError(ex);
/*      */         } 
/*      */       }
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1794 */   public Value getSystemColor(String ident) { return SystemColorSupport.getSystemColor(ident); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Value getDefaultFontFamily() {
/* 1804 */     SVGOMDocument doc = (SVGOMDocument)this.document;
/* 1805 */     SVGStylableElement root = (SVGStylableElement)doc.getRootElement();
/* 1806 */     String str = this.userAgent.getDefaultFontFamily();
/* 1807 */     return doc.getCSSEngine().parsePropertyValue(root, "font-family", str);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1815 */   public float getLighterFontWeight(float f) { return this.userAgent.getLighterFontWeight(f); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1822 */   public float getBolderFontWeight(float f) { return this.userAgent.getBolderFontWeight(f); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1829 */   public float getPixelUnitToMillimeter() { return this.userAgent.getPixelUnitToMillimeter(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1838 */   public float getPixelToMillimeter() { return getPixelUnitToMillimeter(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1846 */   public float getMediumFontSize() { return this.userAgent.getMediumFontSize(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1854 */   public float getBlockWidth(Element elt) { return getViewport(elt).getWidth(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1862 */   public float getBlockHeight(Element elt) { return getViewport(elt).getHeight(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1884 */   public void checkLoadExternalResource(ParsedURL resourceURL, ParsedURL docURL) throws SecurityException { this.userAgent.checkLoadExternalResource(resourceURL, docURL); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1893 */   public boolean isDynamicDocument(Document doc) { return BaseScriptingEnvironment.isDynamicDocument(this, doc); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public boolean isInteractiveDocument(Document doc) {
/* 1903 */     SVGSVGElement sVGSVGElement = ((SVGDocument)doc).getRootElement();
/* 1904 */     if (!"http://www.w3.org/2000/svg".equals(sVGSVGElement.getNamespaceURI())) {
/* 1905 */       return false;
/*      */     }
/* 1907 */     return checkInteractiveElement(sVGSVGElement);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1915 */   public boolean checkInteractiveElement(Element e) { return checkInteractiveElement((SVGDocument)e.getOwnerDocument(), e); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public boolean checkInteractiveElement(SVGDocument doc, Element e) {
/* 1925 */     String tag = e.getLocalName();
/*      */ 
/*      */     
/* 1928 */     if ("a".equals(tag)) {
/* 1929 */       return true;
/*      */     }
/*      */ 
/*      */ 
/*      */     
/* 1934 */     if ("title".equals(tag)) {
/* 1935 */       return (e.getParentNode() != doc.getRootElement());
/*      */     }
/* 1937 */     if ("desc".equals(tag)) {
/* 1938 */       return (e.getParentNode() != doc.getRootElement());
/*      */     }
/* 1940 */     if ("cursor".equals(tag)) {
/* 1941 */       return true;
/*      */     }
/*      */ 
/*      */     
/* 1945 */     if (e.getAttribute("cursor").length() > 0) {
/* 1946 */       return true;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1962 */     String svg_ns = "http://www.w3.org/2000/svg";
/* 1963 */     Node n = e.getFirstChild();
/* 1964 */     for (; n != null; 
/* 1965 */       n = n.getNextSibling()) {
/* 1966 */       if (n.getNodeType() == 1) {
/* 1967 */         Element child = (Element)n;
/* 1968 */         if ("http://www.w3.org/2000/svg".equals(child.getNamespaceURI()) && 
/* 1969 */           checkInteractiveElement(child))
/* 1970 */           return true; 
/*      */       } 
/*      */     } 
/* 1973 */     return false;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void setAnimationLimitingNone() {
/* 1980 */     this.animationLimitingMode = 0;
/* 1981 */     if (this.animationEngine != null) {
/* 1982 */       setAnimationLimitingMode();
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void setAnimationLimitingCPU(float pc) {
/* 1991 */     this.animationLimitingMode = 1;
/* 1992 */     this.animationLimitingAmount = pc;
/* 1993 */     if (this.animationEngine != null) {
/* 1994 */       setAnimationLimitingMode();
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void setAnimationLimitingFPS(float fps) {
/* 2003 */     this.animationLimitingMode = 2;
/* 2004 */     this.animationLimitingAmount = fps;
/* 2005 */     if (this.animationEngine != null) {
/* 2006 */       setAnimationLimitingMode();
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void setAnimationLimitingMode() {
/* 2014 */     switch (this.animationLimitingMode) {
/*      */       case 0:
/* 2016 */         this.animationEngine.setAnimationLimitingNone();
/*      */         break;
/*      */       case 1:
/* 2019 */         this.animationEngine.setAnimationLimitingCPU(this.animationLimitingAmount);
/*      */         break;
/*      */       
/*      */       case 2:
/* 2023 */         this.animationEngine.setAnimationLimitingFPS(this.animationLimitingAmount);
/*      */         break;
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 2031 */   protected List extensions = null;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void registerSVGBridges() {
/* 2037 */     UserAgent ua = getUserAgent();
/* 2038 */     List ext = getBridgeExtensions(this.document);
/* 2039 */     Iterator iter = ext.iterator();
/*      */     
/* 2041 */     while (iter.hasNext()) {
/* 2042 */       BridgeExtension be = (BridgeExtension)iter.next();
/* 2043 */       be.registerTags(this);
/* 2044 */       ua.registerExtension(be);
/*      */     } 
/*      */   }
/*      */   public List getBridgeExtensions(Document doc) {
/*      */     SVG12BridgeExtension sVG12BridgeExtension;
/* 2049 */     SVGSVGElement sVGSVGElement = ((SVGOMDocument)doc).getRootElement();
/* 2050 */     String ver = sVGSVGElement.getAttributeNS(null, "version");
/*      */ 
/*      */     
/* 2053 */     if (ver.length() == 0 || ver.equals("1.0") || ver.equals("1.1")) {
/* 2054 */       sVG12BridgeExtension = new SVGBridgeExtension();
/*      */     } else {
/* 2056 */       sVG12BridgeExtension = new SVG12BridgeExtension();
/*      */     } 
/* 2058 */     float priority = sVG12BridgeExtension.getPriority();
/* 2059 */     this.extensions = new LinkedList(getGlobalBridgeExtensions());
/*      */     
/* 2061 */     ListIterator li = this.extensions.listIterator();
/*      */     while (true) {
/* 2063 */       if (!li.hasNext()) {
/* 2064 */         li.add(sVG12BridgeExtension);
/*      */         break;
/*      */       } 
/* 2067 */       BridgeExtension lbe = (BridgeExtension)li.next();
/* 2068 */       if (lbe.getPriority() > priority) {
/* 2069 */         li.previous();
/* 2070 */         li.add(sVG12BridgeExtension);
/*      */         
/*      */         break;
/*      */       } 
/*      */     } 
/* 2075 */     return this.extensions;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 2081 */   protected static List globalExtensions = null;
/*      */   
/*      */   public static List getGlobalBridgeExtensions() {
/* 2084 */     if (globalExtensions != null) {
/* 2085 */       return globalExtensions;
/*      */     }
/* 2087 */     globalExtensions = new LinkedList();
/*      */     
/* 2089 */     iter = Service.providers(BridgeExtension.class);
/*      */     
/* 2091 */     while (iter.hasNext()) {
/* 2092 */       BridgeExtension be = (BridgeExtension)iter.next();
/* 2093 */       float priority = be.getPriority();
/* 2094 */       ListIterator li = globalExtensions.listIterator();
/*      */       while (true) {
/* 2096 */         if (!li.hasNext()) {
/* 2097 */           li.add(be);
/*      */           break;
/*      */         } 
/* 2100 */         BridgeExtension lbe = (BridgeExtension)li.next();
/* 2101 */         if (lbe.getPriority() > priority) {
/* 2102 */           li.previous();
/* 2103 */           li.add(be);
/*      */         } 
/*      */       } 
/*      */     } 
/*      */     
/* 2108 */     return globalExtensions;
/*      */   }
/*      */   
/*      */   protected BridgeContext() {}
/*      */   
/*      */   public static class CSSEngineUserAgentWrapper implements CSSEngineUserAgent {
/* 2114 */     CSSEngineUserAgentWrapper(UserAgent ua) { this.ua = ua; }
/*      */ 
/*      */     
/*      */     UserAgent ua;
/*      */ 
/*      */     
/* 2120 */     public void displayError(Exception ex) { this.ua.displayError(ex); }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2125 */     public void displayMessage(String message) { this.ua.displayMessage(message); }
/*      */   }
/*      */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\BridgeContext.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */