/*     */ package org.apache.batik.bridge;
/*     */ 
/*     */ import java.awt.Point;
/*     */ import java.awt.geom.NoninvertibleTransformException;
/*     */ import java.awt.geom.Point2D;
/*     */ import java.awt.geom.Rectangle2D;
/*     */ import java.lang.ref.SoftReference;
/*     */ import java.text.AttributedCharacterIterator;
/*     */ import java.util.List;
/*     */ import org.apache.batik.dom.events.DOMKeyEvent;
/*     */ import org.apache.batik.dom.events.DOMMouseEvent;
/*     */ import org.apache.batik.dom.events.NodeEventTarget;
/*     */ import org.apache.batik.dom.util.DOMUtilities;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.apache.batik.gvt.TextNode;
/*     */ import org.apache.batik.gvt.event.EventDispatcher;
/*     */ import org.apache.batik.gvt.event.GraphicsNodeKeyEvent;
/*     */ import org.apache.batik.gvt.event.GraphicsNodeKeyListener;
/*     */ import org.apache.batik.gvt.event.GraphicsNodeMouseEvent;
/*     */ import org.apache.batik.gvt.event.GraphicsNodeMouseListener;
/*     */ import org.apache.batik.gvt.renderer.StrokingTextPainter;
/*     */ import org.apache.batik.gvt.text.GVTAttributedCharacterIterator;
/*     */ import org.apache.batik.gvt.text.TextHit;
/*     */ import org.apache.batik.gvt.text.TextSpanLayout;
/*     */ import org.apache.batik.util.SVGConstants;
/*     */ import org.w3c.dom.Document;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.events.DocumentEvent;
/*     */ import org.w3c.dom.events.Event;
/*     */ import org.w3c.dom.events.EventListener;
/*     */ import org.w3c.dom.events.EventTarget;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class BridgeEventSupport
/*     */   implements SVGConstants
/*     */ {
/*  65 */   public static final AttributedCharacterIterator.Attribute TEXT_COMPOUND_ID = GVTAttributedCharacterIterator.TextAttribute.TEXT_COMPOUND_ID;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static void addGVTListener(BridgeContext ctx, Document doc) {
/*  75 */     UserAgent ua = ctx.getUserAgent();
/*  76 */     if (ua != null) {
/*  77 */       EventDispatcher dispatcher = ua.getEventDispatcher();
/*  78 */       if (dispatcher != null) {
/*  79 */         Listener listener = new Listener(ctx, ua);
/*  80 */         dispatcher.addGraphicsNodeMouseListener(listener);
/*  81 */         dispatcher.addGraphicsNodeKeyListener(listener);
/*     */ 
/*     */         
/*  84 */         EventListener l = new GVTUnloadListener(dispatcher, listener);
/*  85 */         NodeEventTarget target = (NodeEventTarget)doc;
/*  86 */         target.addEventListenerNS("http://www.w3.org/2001/xml-events", "SVGUnload", l, false, null);
/*     */ 
/*     */ 
/*     */         
/*  90 */         storeEventListenerNS(ctx, target, "http://www.w3.org/2001/xml-events", "SVGUnload", l, false);
/*     */       } 
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 107 */   protected static void storeEventListener(BridgeContext ctx, EventTarget e, String t, EventListener l, boolean c) { ctx.storeEventListener(e, t, l, c); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 119 */   protected static void storeEventListenerNS(BridgeContext ctx, EventTarget e, String n, String t, EventListener l, boolean c) { ctx.storeEventListenerNS(e, n, t, l, c); }
/*     */ 
/*     */   
/*     */   protected static class GVTUnloadListener
/*     */     implements EventListener
/*     */   {
/*     */     protected EventDispatcher dispatcher;
/*     */     protected BridgeEventSupport.Listener listener;
/*     */     
/*     */     public GVTUnloadListener(EventDispatcher dispatcher, BridgeEventSupport.Listener listener) {
/* 129 */       this.dispatcher = dispatcher;
/* 130 */       this.listener = listener;
/*     */     }
/*     */     
/*     */     public void handleEvent(Event evt) {
/* 134 */       this.dispatcher.removeGraphicsNodeMouseListener(this.listener);
/* 135 */       this.dispatcher.removeGraphicsNodeKeyListener(this.listener);
/* 136 */       NodeEventTarget et = (NodeEventTarget)evt.getTarget();
/* 137 */       et.removeEventListenerNS("http://www.w3.org/2001/xml-events", "SVGUnload", this, false);
/*     */     }
/*     */   }
/*     */ 
/*     */   
/*     */   protected static class Listener
/*     */     implements GraphicsNodeMouseListener, GraphicsNodeKeyListener
/*     */   {
/*     */     protected BridgeContext context;
/*     */     
/*     */     protected UserAgent ua;
/*     */     
/*     */     protected Element lastTargetElement;
/*     */     
/*     */     protected boolean isDown;
/*     */ 
/*     */     
/*     */     public Listener(BridgeContext ctx, UserAgent u) {
/* 155 */       this.context = ctx;
/* 156 */       this.ua = u;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void keyPressed(GraphicsNodeKeyEvent evt) {
/* 167 */       if (!this.isDown) {
/* 168 */         this.isDown = true;
/* 169 */         dispatchKeyEvent("keydown", evt);
/*     */       } 
/* 171 */       if (evt.getKeyChar() == Character.MAX_VALUE)
/*     */       {
/*     */         
/* 174 */         dispatchKeyEvent("keypress", evt);
/*     */       }
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void keyReleased(GraphicsNodeKeyEvent evt) {
/* 183 */       dispatchKeyEvent("keyup", evt);
/* 184 */       this.isDown = false;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 192 */     public void keyTyped(GraphicsNodeKeyEvent evt) { dispatchKeyEvent("keypress", evt); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     protected void dispatchKeyEvent(String eventType, GraphicsNodeKeyEvent evt) {
/* 200 */       FocusManager fmgr = this.context.getFocusManager();
/* 201 */       if (fmgr == null)
/*     */         return; 
/* 203 */       Element targetElement = (Element)fmgr.getCurrentEventTarget();
/* 204 */       if (targetElement == null) {
/* 205 */         targetElement = this.context.getDocument().getDocumentElement();
/*     */       }
/* 207 */       DocumentEvent d = (DocumentEvent)targetElement.getOwnerDocument();
/* 208 */       DOMKeyEvent keyEvt = (DOMKeyEvent)d.createEvent("KeyEvents");
/* 209 */       keyEvt.initKeyEvent(eventType, true, true, evt.isControlDown(), evt.isAltDown(), evt.isShiftDown(), evt.isMetaDown(), mapKeyCode(evt.getKeyCode()), evt.getKeyChar(), null);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/*     */       try {
/* 221 */         ((EventTarget)targetElement).dispatchEvent(keyEvt);
/*     */       } catch (RuntimeException e) {
/* 223 */         this.ua.displayError(e);
/*     */       } 
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     protected final int mapKeyCode(int keyCode) {
/* 234 */       switch (keyCode) {
/*     */         case 10:
/* 236 */           return 13;
/*     */         case 262:
/* 238 */           return 0;
/*     */         case 263:
/* 240 */           return 0;
/*     */       } 
/* 242 */       return keyCode;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 249 */     public void mouseClicked(GraphicsNodeMouseEvent evt) { dispatchMouseEvent("click", evt, true); }
/*     */ 
/*     */ 
/*     */     
/* 253 */     public void mousePressed(GraphicsNodeMouseEvent evt) { dispatchMouseEvent("mousedown", evt, true); }
/*     */ 
/*     */ 
/*     */     
/* 257 */     public void mouseReleased(GraphicsNodeMouseEvent evt) { dispatchMouseEvent("mouseup", evt, true); }
/*     */ 
/*     */     
/*     */     public void mouseEntered(GraphicsNodeMouseEvent evt) {
/* 261 */       Point clientXY = evt.getClientPoint();
/* 262 */       GraphicsNode node = evt.getGraphicsNode();
/* 263 */       Element targetElement = getEventTarget(node, new Point2D.Float(evt.getX(), evt.getY()));
/*     */       
/* 265 */       Element relatedElement = getRelatedElement(evt);
/* 266 */       dispatchMouseEvent("mouseover", targetElement, relatedElement, clientXY, evt, true);
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void mouseExited(GraphicsNodeMouseEvent evt) {
/* 275 */       Point clientXY = evt.getClientPoint();
/*     */       
/* 277 */       GraphicsNode node = evt.getRelatedNode();
/* 278 */       Element targetElement = getEventTarget(node, clientXY);
/* 279 */       if (this.lastTargetElement != null) {
/* 280 */         dispatchMouseEvent("mouseout", this.lastTargetElement, targetElement, clientXY, evt, true);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         
/* 286 */         this.lastTargetElement = null;
/*     */       } 
/*     */     }
/*     */ 
/*     */     
/* 291 */     public void mouseDragged(GraphicsNodeMouseEvent evt) { dispatchMouseEvent("mousemove", evt, false); }
/*     */ 
/*     */     
/*     */     public void mouseMoved(GraphicsNodeMouseEvent evt) {
/* 295 */       Point clientXY = evt.getClientPoint();
/* 296 */       GraphicsNode node = evt.getGraphicsNode();
/* 297 */       Element targetElement = getEventTarget(node, clientXY);
/* 298 */       Element holdLTE = this.lastTargetElement;
/* 299 */       if (holdLTE != targetElement) {
/* 300 */         if (holdLTE != null) {
/* 301 */           dispatchMouseEvent("mouseout", holdLTE, targetElement, clientXY, evt, true);
/*     */         }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         
/* 308 */         if (targetElement != null) {
/* 309 */           dispatchMouseEvent("mouseover", targetElement, holdLTE, clientXY, evt, true);
/*     */         }
/*     */       } 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 317 */       dispatchMouseEvent("mousemove", targetElement, null, clientXY, evt, false);
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     protected void dispatchMouseEvent(String eventType, GraphicsNodeMouseEvent evt, boolean cancelable) {
/* 336 */       Point clientXY = evt.getClientPoint();
/* 337 */       GraphicsNode node = evt.getGraphicsNode();
/* 338 */       Element targetElement = getEventTarget(node, new Point2D.Float(evt.getX(), evt.getY()));
/*     */       
/* 340 */       Element relatedElement = getRelatedElement(evt);
/* 341 */       dispatchMouseEvent(eventType, targetElement, relatedElement, clientXY, evt, cancelable);
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     protected void dispatchMouseEvent(String eventType, Element targetElement, Element relatedElement, Point clientXY, GraphicsNodeMouseEvent evt, boolean cancelable) {
/* 366 */       if (targetElement == null) {
/*     */         return;
/*     */       }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 382 */       Point screenXY = evt.getScreenPoint();
/*     */       
/* 384 */       DocumentEvent d = (DocumentEvent)targetElement.getOwnerDocument();
/* 385 */       DOMMouseEvent mouseEvt = (DOMMouseEvent)d.createEvent("MouseEvents");
/*     */       
/* 387 */       String modifiers = DOMUtilities.getModifiersList(evt.getLockState(), evt.getModifiers());
/*     */ 
/*     */       
/* 390 */       mouseEvt.initMouseEventNS("http://www.w3.org/2001/xml-events", eventType, true, cancelable, null, evt.getClickCount(), screenXY.x, screenXY.y, clientXY.x, clientXY.y, (short)(evt.getButton() - 1), (EventTarget)relatedElement, modifiers);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/*     */       try {
/* 405 */         ((EventTarget)targetElement).dispatchEvent(mouseEvt);
/*     */       } catch (RuntimeException e) {
/* 407 */         this.ua.displayError(e);
/*     */       } finally {
/* 409 */         this.lastTargetElement = targetElement;
/*     */       } 
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     protected Element getRelatedElement(GraphicsNodeMouseEvent evt) {
/* 419 */       GraphicsNode relatedNode = evt.getRelatedNode();
/* 420 */       Element relatedElement = null;
/* 421 */       if (relatedNode != null) {
/* 422 */         relatedElement = this.context.getElement(relatedNode);
/*     */       }
/* 424 */       return relatedElement;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     protected Element getEventTarget(GraphicsNode node, Point2D coords) {
/* 435 */       Element target = this.context.getElement(node);
/*     */ 
/*     */ 
/*     */       
/* 439 */       if (target != null && node instanceof TextNode) {
/* 440 */         TextNode textNode = (TextNode)node;
/* 441 */         List list = textNode.getTextRuns();
/* 442 */         Point2D pt = (Point2D)coords.clone();
/*     */         
/*     */         try {
/* 445 */           node.getGlobalTransform().createInverse().transform(pt, pt);
/* 446 */         } catch (NoninvertibleTransformException ex) {}
/*     */         
/* 448 */         if (list != null) {
/* 449 */           for (int i = 0; i < list.size(); i++) {
/* 450 */             StrokingTextPainter.TextRun run = (StrokingTextPainter.TextRun)list.get(i);
/*     */             
/* 452 */             AttributedCharacterIterator aci = run.getACI();
/* 453 */             TextSpanLayout layout = run.getLayout();
/* 454 */             float x = (float)pt.getX();
/* 455 */             float y = (float)pt.getY();
/* 456 */             TextHit textHit = layout.hitTestChar(x, y);
/* 457 */             Rectangle2D bounds = layout.getBounds2D();
/* 458 */             if (textHit != null && bounds != null && bounds.contains(x, y)) {
/*     */ 
/*     */               
/* 461 */               SoftReference sr = (SoftReference)aci.getAttribute(BridgeEventSupport.TEXT_COMPOUND_ID);
/*     */               
/* 463 */               Object delimiter = sr.get();
/* 464 */               if (delimiter instanceof Element) {
/* 465 */                 return (Element)delimiter;
/*     */               }
/*     */             } 
/*     */           } 
/*     */         }
/*     */       } 
/* 471 */       return target;
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\BridgeEventSupport.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */