/*     */ package org.apache.batik.bridge;
/*     */ 
/*     */ import java.awt.AlphaComposite;
/*     */ import java.awt.Color;
/*     */ import java.awt.Composite;
/*     */ import java.awt.Cursor;
/*     */ import java.awt.RenderingHints;
/*     */ import java.awt.geom.Rectangle2D;
/*     */ import org.apache.batik.css.engine.CSSEngine;
/*     */ import org.apache.batik.css.engine.CSSStylableElement;
/*     */ import org.apache.batik.css.engine.value.ListValue;
/*     */ import org.apache.batik.css.engine.value.Value;
/*     */ import org.apache.batik.css.engine.value.svg.ICCColor;
/*     */ import org.apache.batik.dom.svg.SVGOMDocument;
/*     */ import org.apache.batik.ext.awt.MultipleGradientPaint;
/*     */ import org.apache.batik.ext.awt.image.renderable.ClipRable;
/*     */ import org.apache.batik.ext.awt.image.renderable.Filter;
/*     */ import org.apache.batik.gvt.CompositeGraphicsNode;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.apache.batik.gvt.filter.Mask;
/*     */ import org.apache.batik.util.CSSConstants;
/*     */ import org.apache.batik.util.XMLConstants;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.css.CSSPrimitiveValue;
/*     */ import org.w3c.dom.css.CSSValue;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class CSSUtilities
/*     */   implements CSSConstants, ErrorConstants, XMLConstants
/*     */ {
/*  73 */   public static CSSEngine getCSSEngine(Element e) { return ((SVGOMDocument)e.getOwnerDocument()).getCSSEngine(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Value getComputedStyle(Element e, int property) {
/*  80 */     CSSEngine engine = getCSSEngine(e);
/*  81 */     if (engine == null) return null; 
/*  82 */     return engine.getComputedStyle((CSSStylableElement)e, null, property);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static int convertPointerEvents(Element e) {
/* 104 */     Value v = getComputedStyle(e, 40);
/* 105 */     String s = v.getStringValue();
/* 106 */     switch (s.charAt(0)) {
/*     */       case 'v':
/* 108 */         if (s.length() == 7) {
/* 109 */           return 3;
/*     */         }
/* 111 */         switch (s.charAt(7)) {
/*     */           case 'p':
/* 113 */             return 0;
/*     */           case 'f':
/* 115 */             return 1;
/*     */           case 's':
/* 117 */             return 2;
/*     */         } 
/*     */         
/* 120 */         throw new IllegalStateException("unexpected event, must be one of (p,f,s) is:" + s.charAt(7));
/*     */ 
/*     */       
/*     */       case 'p':
/* 124 */         return 4;
/*     */       case 'f':
/* 126 */         return 5;
/*     */       case 's':
/* 128 */         return 6;
/*     */       case 'a':
/* 130 */         return 7;
/*     */       case 'n':
/* 132 */         return 8;
/*     */     } 
/*     */     
/* 135 */     throw new IllegalStateException("unexpected event, must be one of (v,p,f,s,a,n) is:" + s.charAt(0));
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Rectangle2D convertEnableBackground(Element e) {
/*     */     float h, w, y, x;
/* 151 */     Value v = getComputedStyle(e, 14);
/* 152 */     if (v.getCssValueType() != 2) {
/* 153 */       return null;
/*     */     }
/* 155 */     ListValue lv = (ListValue)v;
/* 156 */     int length = lv.getLength();
/* 157 */     switch (length) {
/*     */       case 1:
/* 159 */         return CompositeGraphicsNode.VIEWPORT;
/*     */       case 5:
/* 161 */         x = lv.item(1).getFloatValue();
/* 162 */         y = lv.item(2).getFloatValue();
/* 163 */         w = lv.item(3).getFloatValue();
/* 164 */         h = lv.item(4).getFloatValue();
/* 165 */         return new Rectangle2D.Float(x, y, w, h);
/*     */     } 
/*     */     
/* 168 */     throw new IllegalStateException("Unexpected length:" + length);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static boolean convertColorInterpolationFilters(Element e) {
/* 184 */     Value v = getComputedStyle(e, 7);
/*     */     
/* 186 */     return ("linearrgb" == v.getStringValue());
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static MultipleGradientPaint.ColorSpaceEnum convertColorInterpolation(Element e) {
/* 201 */     Value v = getComputedStyle(e, 6);
/* 202 */     return ("linearrgb" == v.getStringValue()) ? MultipleGradientPaint.LINEAR_RGB : MultipleGradientPaint.SRGB;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static boolean isAutoCursor(Element e) {
/* 215 */     Value cursorValue = getComputedStyle(e, 10);
/*     */ 
/*     */ 
/*     */     
/* 219 */     boolean isAuto = false;
/* 220 */     if (cursorValue != null) {
/* 221 */       if (cursorValue.getCssValueType() == 1 && cursorValue.getPrimitiveType() == 21 && cursorValue.getStringValue().charAt(0) == 'a') {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         
/* 228 */         isAuto = true;
/* 229 */       } else if (cursorValue.getCssValueType() == 2 && cursorValue.getLength() == 1) {
/*     */ 
/*     */ 
/*     */         
/* 233 */         Value lValue = cursorValue.item(0);
/* 234 */         if (lValue != null && lValue.getCssValueType() == 1 && lValue.getPrimitiveType() == 21 && lValue.getStringValue().charAt(0) == 'a')
/*     */         {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */           
/* 241 */           isAuto = true;
/*     */         }
/*     */       } 
/*     */     }
/*     */     
/* 246 */     return isAuto;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 257 */   public static Cursor convertCursor(Element e, BridgeContext ctx) { return ctx.getCursorManager().convertCursor(e); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static RenderingHints convertShapeRendering(Element e, RenderingHints hints) {
/* 303 */     Value v = getComputedStyle(e, 42);
/* 304 */     String s = v.getStringValue();
/* 305 */     int len = s.length();
/* 306 */     if (len == 4 && s.charAt(0) == 'a')
/* 307 */       return hints; 
/* 308 */     if (len < 10) return hints;
/*     */     
/* 310 */     if (hints == null) {
/* 311 */       hints = new RenderingHints(null);
/*     */     }
/* 313 */     switch (s.charAt(0)) {
/*     */       case 'o':
/* 315 */         hints.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);
/*     */         
/* 317 */         hints.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
/*     */         break;
/*     */       
/*     */       case 'c':
/* 321 */         hints.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_DEFAULT);
/*     */         
/* 323 */         hints.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
/*     */         break;
/*     */       
/*     */       case 'g':
/* 327 */         hints.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
/*     */         
/* 329 */         hints.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
/*     */         
/* 331 */         hints.put(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_PURE);
/*     */         break;
/*     */     } 
/*     */     
/* 335 */     return hints;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static RenderingHints convertTextRendering(Element e, RenderingHints hints) {
/* 387 */     Value v = getComputedStyle(e, 55);
/* 388 */     String s = v.getStringValue();
/* 389 */     int len = s.length();
/* 390 */     if (len == 4 && s.charAt(0) == 'a')
/* 391 */       return hints; 
/* 392 */     if (len < 13) return hints;
/*     */     
/* 394 */     if (hints == null) {
/* 395 */       hints = new RenderingHints(null);
/*     */     }
/* 397 */     switch (s.charAt(8)) {
/*     */       case 's':
/* 399 */         hints.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);
/*     */         
/* 401 */         hints.put(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
/*     */         
/* 403 */         hints.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
/*     */         break;
/*     */ 
/*     */ 
/*     */       
/*     */       case 'l':
/* 409 */         hints.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
/*     */         
/* 411 */         hints.put(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
/*     */         
/* 413 */         hints.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
/*     */         break;
/*     */ 
/*     */ 
/*     */       
/*     */       case 'c':
/* 419 */         hints.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
/*     */         
/* 421 */         hints.put(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
/*     */         
/* 423 */         hints.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
/*     */         
/* 425 */         hints.put(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
/*     */         
/* 427 */         hints.put(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_PURE);
/*     */         break;
/*     */     } 
/*     */     
/* 431 */     return hints;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static RenderingHints convertImageRendering(Element e, RenderingHints hints) {
/* 465 */     Value v = getComputedStyle(e, 30);
/* 466 */     String s = v.getStringValue();
/* 467 */     int len = s.length();
/* 468 */     if (len == 4 && s.charAt(0) == 'a')
/* 469 */       return hints; 
/* 470 */     if (len < 13) return hints;
/*     */     
/* 472 */     if (hints == null) {
/* 473 */       hints = new RenderingHints(null);
/*     */     }
/* 475 */     switch (s.charAt(8)) {
/*     */       case 's':
/* 477 */         hints.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);
/*     */         
/* 479 */         hints.put(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
/*     */         break;
/*     */       
/*     */       case 'q':
/* 483 */         hints.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
/*     */         
/* 485 */         hints.put(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
/*     */         break;
/*     */     } 
/*     */     
/* 489 */     return hints;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static RenderingHints convertColorRendering(Element e, RenderingHints hints) {
/* 523 */     Value v = getComputedStyle(e, 9);
/* 524 */     String s = v.getStringValue();
/* 525 */     int len = s.length();
/* 526 */     if (len == 4 && s.charAt(0) == 'a')
/* 527 */       return hints; 
/* 528 */     if (len < 13) return hints;
/*     */     
/* 530 */     if (hints == null) {
/* 531 */       hints = new RenderingHints(null);
/*     */     }
/* 533 */     switch (s.charAt(8)) {
/*     */       case 's':
/* 535 */         hints.put(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_SPEED);
/*     */         
/* 537 */         hints.put(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_SPEED);
/*     */         break;
/*     */       
/*     */       case 'q':
/* 541 */         hints.put(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY);
/*     */         
/* 543 */         hints.put(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
/*     */         break;
/*     */     } 
/*     */     
/* 547 */     return hints;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static boolean convertDisplay(Element e) {
/* 561 */     if (!(e instanceof CSSStylableElement)) {
/* 562 */       return true;
/*     */     }
/* 564 */     Value v = getComputedStyle(e, 12);
/* 565 */     return (v.getStringValue().charAt(0) != 'n');
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static boolean convertVisibility(Element e) {
/* 579 */     Value v = getComputedStyle(e, 57);
/* 580 */     return (v.getStringValue().charAt(0) == 'v');
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 587 */   public static final Composite TRANSPARENT = AlphaComposite.getInstance(3, 0.0F);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Composite convertOpacity(Element e) {
/* 597 */     Value v = getComputedStyle(e, 38);
/* 598 */     float f = v.getFloatValue();
/* 599 */     if (f <= 0.0F)
/* 600 */       return TRANSPARENT; 
/* 601 */     if (f >= 1.0F) {
/* 602 */       return AlphaComposite.SrcOver;
/*     */     }
/* 604 */     return AlphaComposite.getInstance(3, f);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static boolean convertOverflow(Element e) {
/* 621 */     Value v = getComputedStyle(e, 39);
/* 622 */     String s = v.getStringValue();
/* 623 */     return (s.charAt(0) == 'h' || s.charAt(0) == 's');
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static float[] convertClip(Element e) {
/*     */     float[] off;
/* 634 */     Value v = getComputedStyle(e, 2);
/* 635 */     int primitiveType = v.getPrimitiveType();
/* 636 */     switch (primitiveType) {
/*     */       case 24:
/* 638 */         off = new float[4];
/* 639 */         off[0] = v.getTop().getFloatValue();
/* 640 */         off[1] = v.getRight().getFloatValue();
/* 641 */         off[2] = v.getBottom().getFloatValue();
/* 642 */         off[3] = v.getLeft().getFloatValue();
/* 643 */         return off;
/*     */       case 21:
/* 645 */         return null;
/*     */     } 
/*     */     
/* 648 */     throw new IllegalStateException("Unexpected primitiveType:" + primitiveType);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Filter convertFilter(Element filteredElement, GraphicsNode filteredNode, BridgeContext ctx) {
/*     */     Bridge bridge;
/*     */     Element filter;
/*     */     String uri;
/* 669 */     Value v = getComputedStyle(filteredElement, 18);
/* 670 */     int primitiveType = v.getPrimitiveType();
/* 671 */     switch (primitiveType) {
/*     */       case 21:
/* 673 */         return null;
/*     */       
/*     */       case 20:
/* 676 */         uri = v.getStringValue();
/* 677 */         filter = ctx.getReferencedElement(filteredElement, uri);
/* 678 */         bridge = ctx.getBridge(filter);
/* 679 */         if (bridge == null || !(bridge instanceof FilterBridge)) {
/* 680 */           throw new BridgeException(ctx, filteredElement, "css.uri.badTarget", new Object[] { uri });
/*     */         }
/*     */ 
/*     */         
/* 684 */         return ((FilterBridge)bridge).createFilter(ctx, filter, filteredElement, filteredNode);
/*     */     } 
/*     */ 
/*     */ 
/*     */     
/* 689 */     throw new IllegalStateException("Unexpected primitive type:" + primitiveType);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static ClipRable convertClipPath(Element clippedElement, GraphicsNode clippedNode, BridgeContext ctx) {
/*     */     Bridge bridge;
/*     */     Element cp;
/*     */     String uri;
/* 710 */     Value v = getComputedStyle(clippedElement, 3);
/*     */     
/* 712 */     int primitiveType = v.getPrimitiveType();
/* 713 */     switch (primitiveType) {
/*     */       case 21:
/* 715 */         return null;
/*     */       
/*     */       case 20:
/* 718 */         uri = v.getStringValue();
/* 719 */         cp = ctx.getReferencedElement(clippedElement, uri);
/* 720 */         bridge = ctx.getBridge(cp);
/* 721 */         if (bridge == null || !(bridge instanceof ClipBridge)) {
/* 722 */           throw new BridgeException(ctx, clippedElement, "css.uri.badTarget", new Object[] { uri });
/*     */         }
/*     */ 
/*     */         
/* 726 */         return ((ClipBridge)bridge).createClip(ctx, cp, clippedElement, clippedNode);
/*     */     } 
/*     */ 
/*     */ 
/*     */     
/* 731 */     throw new IllegalStateException("Unexpected primitive type:" + primitiveType);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static int convertClipRule(Element e) {
/* 742 */     Value v = getComputedStyle(e, 4);
/* 743 */     return (v.getStringValue().charAt(0) == 'n') ? 1 : 0;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Mask convertMask(Element maskedElement, GraphicsNode maskedNode, BridgeContext ctx) {
/*     */     Bridge bridge;
/*     */     Element m;
/*     */     String uri;
/* 764 */     Value v = getComputedStyle(maskedElement, 37);
/* 765 */     int primitiveType = v.getPrimitiveType();
/* 766 */     switch (primitiveType) {
/*     */       case 21:
/* 768 */         return null;
/*     */       
/*     */       case 20:
/* 771 */         uri = v.getStringValue();
/* 772 */         m = ctx.getReferencedElement(maskedElement, uri);
/* 773 */         bridge = ctx.getBridge(m);
/* 774 */         if (bridge == null || !(bridge instanceof MaskBridge)) {
/* 775 */           throw new BridgeException(ctx, maskedElement, "css.uri.badTarget", new Object[] { uri });
/*     */         }
/*     */ 
/*     */         
/* 779 */         return ((MaskBridge)bridge).createMask(ctx, m, maskedElement, maskedNode);
/*     */     } 
/*     */ 
/*     */ 
/*     */     
/* 784 */     throw new IllegalStateException("Unexpected primitive type:" + primitiveType);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static int convertFillRule(Element e) {
/* 795 */     Value v = getComputedStyle(e, 17);
/* 796 */     return (v.getStringValue().charAt(0) == 'n') ? 1 : 0;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Color convertLightingColor(Element e, BridgeContext ctx) {
/* 813 */     Value v = getComputedStyle(e, 33);
/* 814 */     if (v.getCssValueType() == 1) {
/* 815 */       return PaintServer.convertColor(v, 1.0F);
/*     */     }
/* 817 */     return PaintServer.convertRGBICCColor(e, v.item(0), (ICCColor)v.item(1), 1.0F, ctx);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Color convertFloodColor(Element e, BridgeContext ctx) {
/* 834 */     Value v = getComputedStyle(e, 19);
/* 835 */     Value o = getComputedStyle(e, 20);
/* 836 */     float f = PaintServer.convertOpacity(o);
/* 837 */     if (v.getCssValueType() == 1) {
/* 838 */       return PaintServer.convertColor(v, f);
/*     */     }
/* 840 */     return PaintServer.convertRGBICCColor(e, v.item(0), (ICCColor)v.item(1), f, ctx);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Color convertStopColor(Element e, float opacity, BridgeContext ctx) {
/* 860 */     Value v = getComputedStyle(e, 43);
/* 861 */     Value o = getComputedStyle(e, 44);
/* 862 */     opacity *= PaintServer.convertOpacity(o);
/* 863 */     if (v.getCssValueType() == 1) {
/* 864 */       return PaintServer.convertColor(v, opacity);
/*     */     }
/* 866 */     return PaintServer.convertRGBICCColor(e, v.item(0), (ICCColor)v.item(1), opacity, ctx);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static void computeStyleAndURIs(Element refElement, Element localRefElement, String uri) {
/* 888 */     int idx = uri.indexOf('#');
/* 889 */     if (idx != -1) {
/* 890 */       uri = uri.substring(0, idx);
/*     */     }
/*     */     
/* 893 */     if (uri.length() != 0) {
/* 894 */       localRefElement.setAttributeNS("http://www.w3.org/XML/1998/namespace", "base", uri);
/*     */     }
/*     */ 
/*     */     
/* 898 */     CSSEngine engine = getCSSEngine(localRefElement);
/* 899 */     CSSEngine refEngine = getCSSEngine(refElement);
/*     */     
/* 901 */     engine.importCascadedStyleMaps(refElement, refEngine, localRefElement);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 915 */   protected static int rule(CSSValue v) { return (((CSSPrimitiveValue)v).getStringValue().charAt(0) == 'n') ? 1 : 0; }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\CSSUtilities.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */