/*     */ package org.apache.batik.bridge;
/*     */ 
/*     */ import java.awt.Font;
/*     */ import java.util.Iterator;
/*     */ import java.util.LinkedList;
/*     */ import java.util.List;
/*     */ import org.apache.batik.dom.AbstractNode;
/*     */ import org.apache.batik.gvt.font.AWTFontFamily;
/*     */ import org.apache.batik.gvt.font.FontFamilyResolver;
/*     */ import org.apache.batik.gvt.font.GVTFontFace;
/*     */ import org.apache.batik.gvt.font.GVTFontFamily;
/*     */ import org.apache.batik.util.ParsedURL;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.Node;
/*     */ import org.w3c.dom.svg.SVGDocument;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class FontFace
/*     */   extends GVTFontFace
/*     */   implements ErrorConstants
/*     */ {
/*     */   List srcs;
/*     */   
/*     */   public FontFace(List srcs, String familyName, float unitsPerEm, String fontWeight, String fontStyle, String fontVariant, String fontStretch, float slope, String panose1, float ascent, float descent, float strikethroughPosition, float strikethroughThickness, float underlinePosition, float underlineThickness, float overlinePosition, float overlineThickness) {
/*  64 */     super(familyName, unitsPerEm, fontWeight, fontStyle, fontVariant, fontStretch, slope, panose1, ascent, descent, strikethroughPosition, strikethroughThickness, underlinePosition, underlineThickness, overlinePosition, overlineThickness);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*  70 */     this.srcs = srcs;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  77 */   protected FontFace(String familyName) { super(familyName); }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  82 */   public static CSSFontFace createFontFace(String familyName, FontFace src) { return new CSSFontFace(new LinkedList(src.srcs), familyName, src.unitsPerEm, src.fontWeight, src.fontStyle, src.fontVariant, src.fontStretch, src.slope, src.panose1, src.ascent, src.descent, src.strikethroughPosition, src.strikethroughThickness, src.underlinePosition, src.underlineThickness, src.overlinePosition, src.overlineThickness); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public GVTFontFamily getFontFamily(BridgeContext ctx) {
/*  96 */     String name = FontFamilyResolver.lookup(this.familyName);
/*  97 */     if (name != null) {
/*  98 */       GVTFontFace ff = createFontFace(name, this);
/*  99 */       return new AWTFontFamily(ff);
/*     */     } 
/*     */     
/* 102 */     Iterator iter = this.srcs.iterator();
/* 103 */     while (iter.hasNext()) {
/* 104 */       Object o = iter.next();
/* 105 */       if (o instanceof String) {
/* 106 */         String str = (String)o;
/* 107 */         name = FontFamilyResolver.lookup(str);
/* 108 */         if (name != null) {
/* 109 */           GVTFontFace ff = createFontFace(str, this);
/* 110 */           return new AWTFontFamily(ff);
/*     */         }  continue;
/* 112 */       }  if (o instanceof ParsedURL) {
/*     */         try {
/* 114 */           GVTFontFamily ff = getFontFamily(ctx, (ParsedURL)o);
/* 115 */           if (ff != null) {
/* 116 */             return ff;
/*     */           }
/*     */         } catch (SecurityException ex) {
/* 119 */           ctx.getUserAgent().displayError(ex);
/*     */         }
/*     */         catch (BridgeException ex) {
/*     */           
/* 123 */           if ("uri.unsecure".equals(ex.getCode()))
/* 124 */             ctx.getUserAgent().displayError(ex); 
/* 125 */         } catch (Exception ex) {}
/*     */       }
/*     */     } 
/*     */ 
/*     */ 
/*     */     
/* 131 */     return new AWTFontFamily(this);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected GVTFontFamily getFontFamily(BridgeContext ctx, ParsedURL purl) {
/* 139 */     String purlStr = purl.toString();
/*     */     
/* 141 */     Element e = getBaseElement(ctx);
/* 142 */     SVGDocument svgDoc = (SVGDocument)e.getOwnerDocument();
/* 143 */     String docURL = svgDoc.getURL();
/* 144 */     ParsedURL pDocURL = null;
/* 145 */     if (docURL != null) {
/* 146 */       pDocURL = new ParsedURL(docURL);
/*     */     }
/*     */     
/* 149 */     String baseURI = AbstractNode.getBaseURI(e);
/* 150 */     purl = new ParsedURL(baseURI, purlStr);
/* 151 */     UserAgent userAgent = ctx.getUserAgent();
/*     */     
/*     */     try {
/* 154 */       userAgent.checkLoadExternalResource(purl, pDocURL);
/*     */ 
/*     */     
/*     */     }
/*     */     catch (SecurityException ex) {
/*     */ 
/*     */       
/* 161 */       userAgent.displayError(ex);
/* 162 */       return null;
/*     */     } 
/*     */     
/* 165 */     if (purl.getRef() != null) {
/*     */       
/* 167 */       Element ref = ctx.getReferencedElement(e, purlStr);
/* 168 */       if (!ref.getNamespaceURI().equals("http://www.w3.org/2000/svg") || !ref.getLocalName().equals("font"))
/*     */       {
/* 170 */         return null;
/*     */       }
/*     */       
/* 173 */       SVGDocument doc = (SVGDocument)e.getOwnerDocument();
/* 174 */       SVGDocument rdoc = (SVGDocument)ref.getOwnerDocument();
/*     */       
/* 176 */       Element fontElt = ref;
/* 177 */       if (doc != rdoc) {
/* 178 */         fontElt = (Element)doc.importNode(ref, true);
/* 179 */         String base = AbstractNode.getBaseURI(ref);
/* 180 */         Element g = doc.createElementNS("http://www.w3.org/2000/svg", "g");
/* 181 */         g.appendChild(fontElt);
/* 182 */         g.setAttributeNS("http://www.w3.org/XML/1998/namespace", "xml:base", base);
/*     */         
/* 184 */         CSSUtilities.computeStyleAndURIs(ref, fontElt, purlStr);
/*     */       } 
/*     */ 
/*     */       
/* 188 */       Element fontFaceElt = null;
/* 189 */       Node n = fontElt.getFirstChild();
/* 190 */       for (; n != null; 
/* 191 */         n = n.getNextSibling()) {
/* 192 */         if (n.getNodeType() == 1 && n.getNamespaceURI().equals("http://www.w3.org/2000/svg") && n.getLocalName().equals("font-face")) {
/*     */ 
/*     */           
/* 195 */           fontFaceElt = (Element)n;
/*     */ 
/*     */           
/*     */           break;
/*     */         } 
/*     */       } 
/*     */       
/* 202 */       SVGFontFaceElementBridge fontFaceBridge = (SVGFontFaceElementBridge)ctx.getBridge("http://www.w3.org/2000/svg", "font-face");
/*     */       
/* 204 */       GVTFontFace gff = fontFaceBridge.createFontFace(ctx, fontFaceElt);
/*     */ 
/*     */       
/* 207 */       return new SVGFontFamily(gff, fontElt, ctx);
/*     */     } 
/*     */ 
/*     */     
/*     */     try {
/* 212 */       Font font = Font.createFont(0, purl.openStream());
/*     */       
/* 214 */       return new AWTFontFamily(this, font);
/* 215 */     } catch (Exception ex) {
/*     */       
/* 217 */       return null;
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected Element getBaseElement(BridgeContext ctx) {
/* 225 */     SVGDocument d = (SVGDocument)ctx.getDocument();
/* 226 */     return d.getRootElement();
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\FontFace.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */