/*     */ package org.apache.batik.bridge;
/*     */ 
/*     */ import java.awt.BasicStroke;
/*     */ import java.awt.Color;
/*     */ import java.awt.Paint;
/*     */ import java.awt.Shape;
/*     */ import java.awt.Stroke;
/*     */ import org.apache.batik.css.engine.value.Value;
/*     */ import org.apache.batik.css.engine.value.svg.ICCColor;
/*     */ import org.apache.batik.ext.awt.color.ICCColorSpaceExt;
/*     */ import org.apache.batik.gvt.CompositeShapePainter;
/*     */ import org.apache.batik.gvt.FillShapePainter;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.apache.batik.gvt.Marker;
/*     */ import org.apache.batik.gvt.MarkerShapePainter;
/*     */ import org.apache.batik.gvt.ShapeNode;
/*     */ import org.apache.batik.gvt.ShapePainter;
/*     */ import org.apache.batik.gvt.StrokeShapePainter;
/*     */ import org.apache.batik.util.CSSConstants;
/*     */ import org.apache.batik.util.SVGConstants;
/*     */ import org.w3c.dom.Element;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class PaintServer
/*     */   implements SVGConstants, CSSConstants, ErrorConstants
/*     */ {
/*     */   public static ShapePainter convertMarkers(Element e, ShapeNode node, BridgeContext ctx) {
/*  79 */     Value v = CSSUtilities.getComputedStyle(e, 36);
/*  80 */     Marker startMarker = convertMarker(e, v, ctx);
/*  81 */     v = CSSUtilities.getComputedStyle(e, 35);
/*  82 */     Marker midMarker = convertMarker(e, v, ctx);
/*  83 */     v = CSSUtilities.getComputedStyle(e, 34);
/*  84 */     Marker endMarker = convertMarker(e, v, ctx);
/*     */     
/*  86 */     if (startMarker != null || midMarker != null || endMarker != null) {
/*  87 */       MarkerShapePainter p = new MarkerShapePainter(node.getShape());
/*  88 */       p.setStartMarker(startMarker);
/*  89 */       p.setMiddleMarker(midMarker);
/*  90 */       p.setEndMarker(endMarker);
/*  91 */       return p;
/*     */     } 
/*  93 */     return null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Marker convertMarker(Element e, Value v, BridgeContext ctx) {
/* 113 */     if (v.getPrimitiveType() == 21) {
/* 114 */       return null;
/*     */     }
/* 116 */     String uri = v.getStringValue();
/* 117 */     Element markerElement = ctx.getReferencedElement(e, uri);
/* 118 */     Bridge bridge = ctx.getBridge(markerElement);
/* 119 */     if (bridge == null || !(bridge instanceof MarkerBridge)) {
/* 120 */       throw new BridgeException(ctx, e, "css.uri.badTarget", new Object[] { uri });
/*     */     }
/*     */     
/* 123 */     return ((MarkerBridge)bridge).createMarker(ctx, markerElement, e);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static ShapePainter convertFillAndStroke(Element e, ShapeNode node, BridgeContext ctx) {
/* 143 */     Shape shape = node.getShape();
/* 144 */     if (shape == null) return null;
/*     */     
/* 146 */     Paint fillPaint = convertFillPaint(e, node, ctx);
/* 147 */     FillShapePainter fp = new FillShapePainter(shape);
/* 148 */     fp.setPaint(fillPaint);
/*     */     
/* 150 */     Stroke stroke = convertStroke(e);
/* 151 */     if (stroke == null) {
/* 152 */       return fp;
/*     */     }
/* 154 */     Paint strokePaint = convertStrokePaint(e, node, ctx);
/* 155 */     StrokeShapePainter sp = new StrokeShapePainter(shape);
/* 156 */     sp.setStroke(stroke);
/* 157 */     sp.setPaint(strokePaint);
/*     */     
/* 159 */     CompositeShapePainter cp = new CompositeShapePainter(shape);
/* 160 */     cp.addShapePainter(fp);
/* 161 */     cp.addShapePainter(sp);
/* 162 */     return cp;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static ShapePainter convertStrokePainter(Element e, ShapeNode node, BridgeContext ctx) {
/* 169 */     Shape shape = node.getShape();
/* 170 */     if (shape == null) return null;
/*     */     
/* 172 */     Stroke stroke = convertStroke(e);
/* 173 */     if (stroke == null) {
/* 174 */       return null;
/*     */     }
/* 176 */     Paint strokePaint = convertStrokePaint(e, node, ctx);
/* 177 */     StrokeShapePainter sp = new StrokeShapePainter(shape);
/* 178 */     sp.setStroke(stroke);
/* 179 */     sp.setPaint(strokePaint);
/* 180 */     return sp;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Paint convertStrokePaint(Element strokedElement, GraphicsNode strokedNode, BridgeContext ctx) {
/* 198 */     Value v = CSSUtilities.getComputedStyle(strokedElement, 51);
/*     */     
/* 200 */     float opacity = convertOpacity(v);
/* 201 */     v = CSSUtilities.getComputedStyle(strokedElement, 45);
/*     */ 
/*     */     
/* 204 */     return convertPaint(strokedElement, strokedNode, v, opacity, ctx);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Paint convertFillPaint(Element filledElement, GraphicsNode filledNode, BridgeContext ctx) {
/* 222 */     Value v = CSSUtilities.getComputedStyle(filledElement, 16);
/*     */     
/* 224 */     float opacity = convertOpacity(v);
/* 225 */     v = CSSUtilities.getComputedStyle(filledElement, 15);
/*     */ 
/*     */     
/* 228 */     return convertPaint(filledElement, filledNode, v, opacity, ctx);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Paint convertPaint(Element paintedElement, GraphicsNode paintedNode, Value paintDef, float opacity, BridgeContext ctx) {
/*     */     Paint result;
/* 250 */     if (paintDef.getCssValueType() == 1) {
/* 251 */       switch (paintDef.getPrimitiveType()) {
/*     */         case 21:
/* 253 */           return null;
/*     */         
/*     */         case 25:
/* 256 */           return convertColor(paintDef, opacity);
/*     */         
/*     */         case 20:
/* 259 */           return convertURIPaint(paintedElement, paintedNode, paintDef, opacity, ctx);
/*     */       } 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 266 */       throw new IllegalArgumentException("Paint argument is not an appropriate CSS value");
/*     */     } 
/*     */ 
/*     */     
/* 270 */     Value v = paintDef.item(0);
/* 271 */     switch (v.getPrimitiveType()) {
/*     */       case 25:
/* 273 */         return convertRGBICCColor(paintedElement, v, (ICCColor)paintDef.item(1), opacity, ctx);
/*     */ 
/*     */ 
/*     */       
/*     */       case 20:
/* 278 */         result = silentConvertURIPaint(paintedElement, paintedNode, v, opacity, ctx);
/*     */ 
/*     */         
/* 281 */         if (result != null) return result;
/*     */         
/* 283 */         v = paintDef.item(1);
/* 284 */         switch (v.getPrimitiveType()) {
/*     */           case 21:
/* 286 */             return null;
/*     */           
/*     */           case 25:
/* 289 */             if (paintDef.getLength() == 2) {
/* 290 */               return convertColor(v, opacity);
/*     */             }
/* 292 */             return convertRGBICCColor(paintedElement, v, (ICCColor)paintDef.item(2), opacity, ctx);
/*     */         } 
/*     */ 
/*     */ 
/*     */         
/* 297 */         throw new IllegalArgumentException("Paint argument is not an appropriate CSS value");
/*     */     } 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 303 */     throw new IllegalArgumentException("Paint argument is not an appropriate CSS value");
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Paint silentConvertURIPaint(Element paintedElement, GraphicsNode paintedNode, Value paintDef, float opacity, BridgeContext ctx) {
/* 326 */     Paint paint = null;
/*     */     try {
/* 328 */       paint = convertURIPaint(paintedElement, paintedNode, paintDef, opacity, ctx);
/*     */     }
/* 330 */     catch (BridgeException ex) {}
/*     */     
/* 332 */     return paint;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Paint convertURIPaint(Element paintedElement, GraphicsNode paintedNode, Value paintDef, float opacity, BridgeContext ctx) {
/* 350 */     String uri = paintDef.getStringValue();
/* 351 */     Element paintElement = ctx.getReferencedElement(paintedElement, uri);
/*     */     
/* 353 */     Bridge bridge = ctx.getBridge(paintElement);
/* 354 */     if (bridge == null || !(bridge instanceof PaintBridge)) {
/* 355 */       throw new BridgeException(ctx, paintedElement, "css.uri.badTarget", new Object[] { uri });
/*     */     }
/*     */ 
/*     */     
/* 359 */     return ((PaintBridge)bridge).createPaint(ctx, paintElement, paintedElement, paintedNode, opacity);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Color convertRGBICCColor(Element paintedElement, Value colorDef, ICCColor iccColor, float opacity, BridgeContext ctx) {
/* 382 */     Color color = null;
/* 383 */     if (iccColor != null) {
/* 384 */       color = convertICCColor(paintedElement, iccColor, opacity, ctx);
/*     */     }
/* 386 */     if (color == null) {
/* 387 */       color = convertColor(colorDef, opacity);
/*     */     }
/* 389 */     return color;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Color convertICCColor(Element e, ICCColor c, float opacity, BridgeContext ctx) {
/* 407 */     String iccProfileName = c.getColorProfile();
/* 408 */     if (iccProfileName == null) {
/* 409 */       return null;
/*     */     }
/*     */     
/* 412 */     SVGColorProfileElementBridge profileBridge = (SVGColorProfileElementBridge)ctx.getBridge("http://www.w3.org/2000/svg", "color-profile");
/*     */ 
/*     */     
/* 415 */     if (profileBridge == null) {
/* 416 */       return null;
/*     */     }
/*     */     
/* 419 */     ICCColorSpaceExt profileCS = profileBridge.createICCColorSpaceExt(ctx, e, iccProfileName);
/*     */     
/* 421 */     if (profileCS == null) {
/* 422 */       return null;
/*     */     }
/*     */ 
/*     */     
/* 426 */     int n = c.getNumberOfColors();
/* 427 */     float[] colorValue = new float[n];
/* 428 */     if (n == 0) {
/* 429 */       return null;
/*     */     }
/* 431 */     for (int i = 0; i < n; i++) {
/* 432 */       colorValue[i] = c.getColor(i);
/*     */     }
/*     */ 
/*     */     
/* 436 */     float[] rgb = profileCS.intendedToRGB(colorValue);
/* 437 */     return new Color(rgb[0], rgb[1], rgb[2], opacity);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Color convertColor(Value c, float opacity) {
/* 446 */     int r = resolveColorComponent(c.getRed());
/* 447 */     int g = resolveColorComponent(c.getGreen());
/* 448 */     int b = resolveColorComponent(c.getBlue());
/* 449 */     return new Color(r, g, b, Math.round(opacity * 255.0F));
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static Stroke convertStroke(Element e) {
/* 463 */     Value v = CSSUtilities.getComputedStyle(e, 52);
/*     */     
/* 465 */     float width = v.getFloatValue();
/* 466 */     if (width == 0.0F) {
/* 467 */       return null;
/*     */     }
/* 469 */     v = CSSUtilities.getComputedStyle(e, 48);
/*     */     
/* 471 */     int linecap = convertStrokeLinecap(v);
/* 472 */     v = CSSUtilities.getComputedStyle(e, 49);
/*     */     
/* 474 */     int linejoin = convertStrokeLinejoin(v);
/* 475 */     v = CSSUtilities.getComputedStyle(e, 50);
/*     */     
/* 477 */     float miterlimit = convertStrokeMiterlimit(v);
/* 478 */     v = CSSUtilities.getComputedStyle(e, 46);
/*     */     
/* 480 */     float[] dasharray = convertStrokeDasharray(v);
/*     */     
/* 482 */     float dashoffset = 0.0F;
/* 483 */     if (dasharray != null) {
/* 484 */       v = CSSUtilities.getComputedStyle(e, 47);
/*     */       
/* 486 */       dashoffset = v.getFloatValue();
/*     */ 
/*     */ 
/*     */       
/* 490 */       if (dashoffset < 0.0F) {
/* 491 */         float dashpatternlength = 0.0F;
/* 492 */         for (int i = 0; i < dasharray.length; i++) {
/* 493 */           dashpatternlength += dasharray[i];
/*     */         }
/*     */ 
/*     */         
/* 497 */         if (dasharray.length % 2 != 0) {
/* 498 */           dashpatternlength *= 2.0F;
/*     */         }
/* 500 */         if (dashpatternlength == 0.0F) {
/* 501 */           dashoffset = 0.0F;
/*     */         } else {
/* 503 */           while (dashoffset < 0.0F)
/* 504 */             dashoffset += dashpatternlength; 
/*     */         } 
/*     */       } 
/*     */     } 
/* 508 */     return new BasicStroke(width, linecap, linejoin, miterlimit, dasharray, dashoffset);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static float[] convertStrokeDasharray(Value v) {
/* 527 */     float[] dasharray = null;
/* 528 */     if (v.getCssValueType() == 2) {
/* 529 */       int length = v.getLength();
/* 530 */       dasharray = new float[length];
/* 531 */       float sum = 0.0F;
/* 532 */       for (int i = 0; i < dasharray.length; i++) {
/* 533 */         dasharray[i] = v.item(i).getFloatValue();
/* 534 */         sum += dasharray[i];
/*     */       } 
/* 536 */       if (sum == 0.0F)
/*     */       {
/*     */ 
/*     */         
/* 540 */         dasharray = null;
/*     */       }
/*     */     } 
/* 543 */     return dasharray;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static float convertStrokeMiterlimit(Value v) {
/* 551 */     float miterlimit = v.getFloatValue();
/* 552 */     return (miterlimit < 1.0F) ? 1.0F : miterlimit;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static int convertStrokeLinecap(Value v) {
/* 560 */     String s = v.getStringValue();
/* 561 */     switch (s.charAt(0)) {
/*     */       case 'b':
/* 563 */         return 0;
/*     */       case 'r':
/* 565 */         return 1;
/*     */       case 's':
/* 567 */         return 2;
/*     */     } 
/* 569 */     throw new IllegalArgumentException("Linecap argument is not an appropriate CSS value");
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static int convertStrokeLinejoin(Value v) {
/* 580 */     String s = v.getStringValue();
/* 581 */     switch (s.charAt(0)) {
/*     */       case 'm':
/* 583 */         return 0;
/*     */       case 'r':
/* 585 */         return 1;
/*     */       case 'b':
/* 587 */         return 2;
/*     */     } 
/* 589 */     throw new IllegalArgumentException("Linejoin argument is not an appropriate CSS value");
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static int resolveColorComponent(Value v) {
/*     */     float f;
/* 604 */     switch (v.getPrimitiveType()) {
/*     */       case 2:
/* 606 */         f = v.getFloatValue();
/* 607 */         f = (f > 100.0F) ? 100.0F : ((f < 0.0F) ? 0.0F : f);
/* 608 */         return Math.round(255.0F * f / 100.0F);
/*     */       case 1:
/* 610 */         f = v.getFloatValue();
/* 611 */         f = (f > 255.0F) ? 255.0F : ((f < 0.0F) ? 0.0F : f);
/* 612 */         return Math.round(f);
/*     */     } 
/* 614 */     throw new IllegalArgumentException("Color component argument is not an appropriate CSS value");
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static float convertOpacity(Value v) {
/* 625 */     float r = v.getFloatValue();
/* 626 */     return (r < 0.0F) ? 0.0F : ((r > 1.0F) ? 1.0F : r);
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\PaintServer.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */