/*     */ package org.apache.batik.bridge;
/*     */ 
/*     */ import java.util.ArrayList;
/*     */ import java.util.List;
/*     */ import org.apache.batik.anim.AbstractAnimation;
/*     */ import org.apache.batik.anim.MotionAnimation;
/*     */ import org.apache.batik.anim.values.AnimatableMotionPointValue;
/*     */ import org.apache.batik.anim.values.AnimatableValue;
/*     */ import org.apache.batik.dom.anim.AnimationTarget;
/*     */ import org.apache.batik.dom.svg.SVGAnimatedPathDataSupport;
/*     */ import org.apache.batik.dom.svg.SVGOMElement;
/*     */ import org.apache.batik.dom.svg.SVGOMPathElement;
/*     */ import org.apache.batik.dom.util.XLinkSupport;
/*     */ import org.apache.batik.ext.awt.geom.ExtendedGeneralPath;
/*     */ import org.apache.batik.parser.AWTPathProducer;
/*     */ import org.apache.batik.parser.AngleHandler;
/*     */ import org.apache.batik.parser.AngleParser;
/*     */ import org.apache.batik.parser.LengthArrayProducer;
/*     */ import org.apache.batik.parser.LengthPairListParser;
/*     */ import org.apache.batik.parser.ParseException;
/*     */ import org.apache.batik.parser.PathParser;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.Node;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SVGAnimateMotionElementBridge
/*     */   extends SVGAnimateElementBridge
/*     */ {
/*  59 */   public String getLocalName() { return "animateMotion"; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  66 */   public Bridge getInstance() { return new SVGAnimateMotionElementBridge(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected AbstractAnimation createAnimation(AnimationTarget target) {
/*  73 */     this.animationType = 2;
/*  74 */     this.attributeLocalName = "motion";
/*     */     
/*  76 */     AnimatableValue from = parseLengthPair("from");
/*  77 */     AnimatableValue to = parseLengthPair("to");
/*  78 */     AnimatableValue by = parseLengthPair("by");
/*     */     
/*  80 */     boolean rotateAuto = false, rotateAutoReverse = false;
/*  81 */     float rotateAngle = 0.0F;
/*  82 */     short rotateAngleUnit = 0;
/*  83 */     String rotateString = this.element.getAttributeNS(null, "rotate");
/*     */     
/*  85 */     if (rotateString.length() != 0) {
/*  86 */       if (rotateString.equals("auto")) {
/*  87 */         rotateAuto = true;
/*  88 */       } else if (rotateString.equals("auto-reverse")) {
/*  89 */         rotateAuto = true;
/*  90 */         rotateAutoReverse = true;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/*     */       }
/*     */       else {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         
/* 112 */         AngleParser ap = new AngleParser();
/* 113 */         SVGAnimateMotionElementBridge$1$Handler h = new SVGAnimateMotionElementBridge$1$Handler(this);
/* 114 */         ap.setAngleHandler(h);
/*     */         try {
/* 116 */           ap.parse(rotateString);
/*     */         } catch (ParseException pEx) {
/* 118 */           throw new BridgeException(this.ctx, this.element, pEx, "attribute.malformed", new Object[] { "rotate", rotateString });
/*     */         } 
/*     */ 
/*     */ 
/*     */         
/* 123 */         rotateAngle = h.theAngle;
/* 124 */         rotateAngleUnit = h.theUnit;
/*     */       } 
/*     */     }
/* 127 */     return new MotionAnimation(this.timedElement, this, parseCalcMode(), parseKeyTimes(), parseKeySplines(), parseAdditive(), parseAccumulate(), parseValues(), from, to, by, parsePath(), parseKeyPoints(), rotateAuto, rotateAutoReverse, rotateAngle, rotateAngleUnit);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected ExtendedGeneralPath parsePath() {
/* 151 */     Node n = this.element.getFirstChild();
/* 152 */     while (n != null) {
/* 153 */       if (n.getNodeType() == 1 && "http://www.w3.org/2000/svg".equals(n.getNamespaceURI()) && "mpath".equals(n.getLocalName())) {
/*     */ 
/*     */         
/* 156 */         String uri = XLinkSupport.getXLinkHref((Element)n);
/* 157 */         Element path = this.ctx.getReferencedElement(this.element, uri);
/* 158 */         if (!"http://www.w3.org/2000/svg".equals(path.getNamespaceURI()) || !"path".equals(path.getLocalName()))
/*     */         {
/* 160 */           throw new BridgeException(this.ctx, this.element, "uri.badTarget", new Object[] { uri });
/*     */         }
/*     */ 
/*     */         
/* 164 */         SVGOMPathElement pathElt = (SVGOMPathElement)path;
/* 165 */         AWTPathProducer app = new AWTPathProducer();
/* 166 */         SVGAnimatedPathDataSupport.handlePathSegList(pathElt.getPathSegList(), app);
/*     */         
/* 168 */         return (ExtendedGeneralPath)app.getShape();
/*     */       } 
/* 170 */       n = n.getNextSibling();
/*     */     } 
/* 172 */     String pathString = this.element.getAttributeNS(null, "path");
/* 173 */     if (pathString.length() == 0) {
/* 174 */       return null;
/*     */     }
/*     */     try {
/* 177 */       AWTPathProducer app = new AWTPathProducer();
/* 178 */       PathParser pp = new PathParser();
/* 179 */       pp.setPathHandler(app);
/* 180 */       pp.parse(pathString);
/* 181 */       return (ExtendedGeneralPath)app.getShape();
/*     */     } catch (ParseException pEx) {
/* 183 */       throw new BridgeException(this.ctx, this.element, pEx, "attribute.malformed", new Object[] { "path", pathString });
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected float[] parseKeyPoints() {
/* 193 */     String keyPointsString = this.element.getAttributeNS(null, "keyPoints");
/*     */     
/* 195 */     int len = keyPointsString.length();
/* 196 */     if (len == 0) {
/* 197 */       return null;
/*     */     }
/* 199 */     List keyPoints = new ArrayList(7);
/* 200 */     int i = 0, start = 0;
/*     */     
/* 202 */     label36: while (i < len) {
/* 203 */       while (keyPointsString.charAt(i) == ' ') {
/* 204 */         i++;
/* 205 */         if (i == len) {
/*     */           break label36;
/*     */         }
/*     */       } 
/* 209 */       start = i++;
/* 210 */       if (i != len) {
/* 211 */         char c = keyPointsString.charAt(i);
/*     */         
/* 213 */         i++;
/* 214 */         while (c != ' ' && c != ';' && c != ',' && i != len)
/*     */         {
/*     */           
/* 217 */           c = keyPointsString.charAt(i);
/*     */         }
/*     */       } 
/* 220 */       int end = i++;
/*     */       try {
/* 222 */         float keyPointCoord = Float.parseFloat(keyPointsString.substring(start, end));
/*     */         
/* 224 */         keyPoints.add(new Float(keyPointCoord));
/*     */       } catch (NumberFormatException nfEx) {
/* 226 */         throw new BridgeException(this.ctx, this.element, nfEx, "attribute.malformed", new Object[] { "keyPoints", keyPointsString });
/*     */       } 
/*     */     } 
/*     */ 
/*     */     
/* 231 */     len = keyPoints.size();
/* 232 */     float[] ret = new float[len];
/* 233 */     for (int j = 0; j < len; j++) {
/* 234 */       ret[j] = ((Float)keyPoints.get(j)).floatValue();
/*     */     }
/* 236 */     return ret;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 243 */   protected int getDefaultCalcMode() { return 2; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected AnimatableValue[] parseValues() {
/* 250 */     String valuesString = this.element.getAttributeNS(null, "values");
/*     */     
/* 252 */     int len = valuesString.length();
/* 253 */     if (len == 0) {
/* 254 */       return null;
/*     */     }
/* 256 */     return parseValues(valuesString);
/*     */   }
/*     */   
/*     */   protected AnimatableValue[] parseValues(String s) {
/*     */     try {
/* 261 */       LengthPairListParser lplp = new LengthPairListParser();
/* 262 */       LengthArrayProducer lap = new LengthArrayProducer();
/* 263 */       lplp.setLengthListHandler(lap);
/* 264 */       lplp.parse(s);
/* 265 */       short[] types = lap.getLengthTypeArray();
/* 266 */       float[] values = lap.getLengthValueArray();
/* 267 */       AnimatableValue[] ret = new AnimatableValue[types.length / 2];
/* 268 */       for (int i = 0; i < types.length; i += 2) {
/* 269 */         float x = this.animationTarget.svgToUserSpace(values[i], types[i], (short)1);
/*     */         
/* 271 */         float y = this.animationTarget.svgToUserSpace(values[i + 1], types[i + 1], (short)2);
/*     */         
/* 273 */         ret[i / 2] = new AnimatableMotionPointValue(this.animationTarget, x, y, 0.0F);
/*     */       } 
/* 275 */       return ret;
/*     */     } catch (ParseException pEx) {
/* 277 */       throw new BridgeException(this.ctx, this.element, pEx, "attribute.malformed", new Object[] { "values", s });
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected AnimatableValue parseLengthPair(String ln) {
/* 287 */     String s = this.element.getAttributeNS(null, ln);
/* 288 */     if (s.length() == 0) {
/* 289 */       return null;
/*     */     }
/* 291 */     return parseValues(s)[0];
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 301 */   public AnimatableValue getUnderlyingValue() { return new AnimatableMotionPointValue(this.animationTarget, 0.0F, 0.0F, 0.0F); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void initializeAnimation() {
/*     */     Element element;
/* 310 */     String uri = XLinkSupport.getXLinkHref(this.element);
/*     */     
/* 312 */     if (uri.length() == 0) {
/* 313 */       element = this.element.getParentNode();
/*     */     } else {
/* 315 */       element = this.ctx.getReferencedElement(this.element, uri);
/* 316 */       if (element.getOwnerDocument() != this.element.getOwnerDocument()) {
/* 317 */         throw new BridgeException(this.ctx, this.element, "uri.badTarget", new Object[] { uri });
/*     */       }
/*     */     } 
/*     */ 
/*     */     
/* 322 */     this.animationTarget = null;
/* 323 */     if (element instanceof SVGOMElement) {
/* 324 */       this.targetElement = (SVGOMElement)element;
/* 325 */       this.animationTarget = this.targetElement;
/*     */     } 
/* 327 */     if (this.animationTarget == null) {
/* 328 */       throw new BridgeException(this.ctx, this.element, "uri.badTarget", new Object[] { uri });
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 334 */     this.timedElement = createTimedElement();
/* 335 */     this.animation = createAnimation(this.animationTarget);
/* 336 */     this.eng.addAnimation(this.animationTarget, (short)2, this.attributeNamespaceURI, this.attributeLocalName, this.animation);
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\SVGAnimateMotionElementBridge.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */