/*     */ package org.apache.batik.bridge;
/*     */ 
/*     */ import java.awt.geom.AffineTransform;
/*     */ import java.awt.geom.Rectangle2D;
/*     */ import java.util.Calendar;
/*     */ import org.apache.batik.anim.AbstractAnimation;
/*     */ import org.apache.batik.anim.timing.TimedElement;
/*     */ import org.apache.batik.anim.values.AnimatableValue;
/*     */ import org.apache.batik.css.engine.CSSEngineEvent;
/*     */ import org.apache.batik.dom.anim.AnimatableElement;
/*     */ import org.apache.batik.dom.anim.AnimationTarget;
/*     */ import org.apache.batik.dom.anim.AnimationTargetListener;
/*     */ import org.apache.batik.dom.svg.AnimatedLiveAttributeValue;
/*     */ import org.apache.batik.dom.svg.SVGAnimationContext;
/*     */ import org.apache.batik.dom.svg.SVGOMElement;
/*     */ import org.apache.batik.dom.util.XLinkSupport;
/*     */ import org.w3c.dom.DOMException;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.events.EventTarget;
/*     */ import org.w3c.dom.events.MutationEvent;
/*     */ import org.w3c.dom.svg.SVGElement;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class SVGAnimationElementBridge
/*     */   extends AbstractSVGBridge
/*     */   implements GenericBridge, BridgeUpdateHandler, SVGAnimationContext, AnimatableElement
/*     */ {
/*     */   protected SVGOMElement element;
/*     */   protected BridgeContext ctx;
/*     */   protected SVGAnimationEngine eng;
/*     */   protected TimedElement timedElement;
/*     */   protected AbstractAnimation animation;
/*     */   protected String attributeNamespaceURI;
/*     */   protected String attributeLocalName;
/*     */   protected short animationType;
/*     */   protected SVGOMElement targetElement;
/*     */   protected AnimationTarget animationTarget;
/*     */   
/* 116 */   public TimedElement getTimedElement() { return this.timedElement; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public AnimatableValue getUnderlyingValue() {
/* 127 */     if (this.animationType == 0) {
/* 128 */       return this.animationTarget.getUnderlyingValue(this.attributeNamespaceURI, this.attributeLocalName);
/*     */     }
/*     */     
/* 131 */     return this.eng.getUnderlyingCSSValue(this.element, this.animationTarget, this.attributeLocalName);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleElement(BridgeContext ctx, Element e) {
/* 146 */     if (ctx.isDynamic() && BridgeContext.getSVGContext(e) == null) {
/* 147 */       SVGAnimationElementBridge b = (SVGAnimationElementBridge)getInstance();
/*     */       
/* 149 */       b.element = (SVGOMElement)e;
/* 150 */       b.ctx = ctx;
/* 151 */       b.eng = ctx.getAnimationEngine();
/* 152 */       b.element.setSVGContext(b);
/* 153 */       if (b.eng.hasStarted()) {
/* 154 */         b.initializeAnimation();
/* 155 */         b.initializeTimedElement();
/*     */       } else {
/* 157 */         b.eng.addInitialBridge(b);
/*     */       } 
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void initializeAnimation() {
/*     */     int i;
/*     */     Element element1;
/* 168 */     String uri = XLinkSupport.getXLinkHref(this.element);
/*     */     
/* 170 */     if (uri.length() == 0) {
/* 171 */       element1 = this.element.getParentNode();
/*     */     } else {
/* 173 */       element1 = this.ctx.getReferencedElement(this.element, uri);
/* 174 */       if (element1.getOwnerDocument() != this.element.getOwnerDocument()) {
/* 175 */         throw new BridgeException(this.ctx, this.element, "uri.badTarget", new Object[] { uri });
/*     */       }
/*     */     } 
/*     */ 
/*     */     
/* 180 */     this.animationTarget = null;
/* 181 */     if (element1 instanceof SVGOMElement) {
/* 182 */       this.targetElement = (SVGOMElement)element1;
/* 183 */       this.animationTarget = this.targetElement;
/*     */     } 
/* 185 */     if (this.animationTarget == null) {
/* 186 */       throw new BridgeException(this.ctx, this.element, "uri.badTarget", new Object[] { uri });
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 192 */     String an = this.element.getAttributeNS(null, "attributeName");
/* 193 */     int ci = an.indexOf(':');
/* 194 */     if (ci == -1) {
/* 195 */       if (this.element.hasProperty(an)) {
/* 196 */         this.animationType = 1;
/* 197 */         this.attributeLocalName = an;
/*     */       } else {
/* 199 */         this.animationType = 0;
/* 200 */         this.attributeLocalName = an;
/*     */       } 
/*     */     } else {
/* 203 */       this.animationType = 0;
/* 204 */       String prefix = an.substring(0, ci);
/* 205 */       this.attributeNamespaceURI = this.element.lookupNamespaceURI(prefix);
/* 206 */       this.attributeLocalName = an.substring(ci + 1);
/*     */     } 
/* 208 */     if ((this.animationType == 1 && !this.targetElement.isPropertyAnimatable(this.attributeLocalName)) || (this.animationType == 0 && !this.targetElement.isAttributeAnimatable(this.attributeNamespaceURI, this.attributeLocalName)))
/*     */     {
/*     */ 
/*     */ 
/*     */       
/* 213 */       throw new BridgeException(this.ctx, this.element, "attribute.not.animatable", new Object[] { this.targetElement.getNodeName(), an });
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 221 */     if (this.animationType == 1) {
/* 222 */       i = this.targetElement.getPropertyType(this.attributeLocalName);
/*     */     } else {
/* 224 */       i = this.targetElement.getAttributeType(this.attributeNamespaceURI, this.attributeLocalName);
/*     */     } 
/*     */     
/* 227 */     if (!canAnimateType(i)) {
/* 228 */       throw new BridgeException(this.ctx, this.element, "type.not.animatable", new Object[] { this.targetElement.getNodeName(), an, this.element.getNodeName() });
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 235 */     this.timedElement = createTimedElement();
/* 236 */     this.animation = createAnimation(this.animationTarget);
/* 237 */     this.eng.addAnimation(this.animationTarget, this.animationType, this.attributeNamespaceURI, this.attributeLocalName, this.animation);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected abstract boolean canAnimateType(int paramInt);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 253 */   protected boolean checkValueType(AnimatableValue v) { return true; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void initializeTimedElement() {
/* 261 */     initializeTimedElement(this.timedElement);
/* 262 */     this.timedElement.initialize();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 269 */   protected TimedElement createTimedElement() { return new SVGTimedElement(this); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected abstract AbstractAnimation createAnimation(AnimationTarget paramAnimationTarget);
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected AnimatableValue parseAnimatableValue(String an) {
/* 281 */     if (!this.element.hasAttributeNS(null, an)) {
/* 282 */       return null;
/*     */     }
/* 284 */     String s = this.element.getAttributeNS(null, an);
/* 285 */     AnimatableValue val = this.eng.parseAnimatableValue(this.element, this.animationTarget, this.attributeNamespaceURI, this.attributeLocalName, (this.animationType == 1), s);
/*     */ 
/*     */ 
/*     */     
/* 289 */     if (!checkValueType(val)) {
/* 290 */       throw new BridgeException(this.ctx, this.element, "attribute.malformed", new Object[] { an, s });
/*     */     }
/*     */ 
/*     */     
/* 294 */     return val;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 301 */   protected void initializeTimedElement(TimedElement timedElement) { timedElement.parseAttributes(this.element.getAttributeNS(null, "begin"), this.element.getAttributeNS(null, "dur"), this.element.getAttributeNS(null, "end"), this.element.getAttributeNS(null, "min"), this.element.getAttributeNS(null, "max"), this.element.getAttributeNS(null, "repeatCount"), this.element.getAttributeNS(null, "repeatDur"), this.element.getAttributeNS(null, "fill"), this.element.getAttributeNS(null, "restart")); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleDOMAttrModifiedEvent(MutationEvent evt) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleDOMNodeInsertedEvent(MutationEvent evt) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleDOMNodeRemovedEvent(MutationEvent evt) {
/* 331 */     this.element.setSVGContext(null);
/* 332 */     dispose();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleDOMCharacterDataModified(MutationEvent evt) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleCSSEngineEvent(CSSEngineEvent evt) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleAnimatedAttributeChanged(AnimatedLiveAttributeValue alav) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleOtherAnimationChanged(String type) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void dispose() {
/* 365 */     if (this.element.getSVGContext() == null) {
/*     */       
/* 367 */       this.eng.removeAnimation(this.animation);
/* 368 */       this.timedElement.deinitialize();
/* 369 */       this.timedElement = null;
/* 370 */       this.element = null;
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 380 */   public float getPixelUnitToMillimeter() { return this.ctx.getUserAgent().getPixelUnitToMillimeter(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 389 */   public float getPixelToMM() { return getPixelUnitToMillimeter(); }
/*     */ 
/*     */ 
/*     */   
/* 393 */   public Rectangle2D getBBox() { return null; }
/*     */   
/* 395 */   public AffineTransform getScreenTransform() { return this.ctx.getUserAgent().getTransform(); }
/*     */ 
/*     */   
/* 398 */   public void setScreenTransform(AffineTransform at) { this.ctx.getUserAgent().setTransform(at); }
/*     */   
/* 400 */   public AffineTransform getCTM() { return null; }
/* 401 */   public AffineTransform getGlobalTransform() { return null; }
/*     */   
/* 403 */   public float getViewportWidth() { return this.ctx.getBlockWidth(this.element); }
/*     */ 
/*     */   
/* 406 */   public float getViewportHeight() { return this.ctx.getBlockHeight(this.element); }
/*     */   
/* 408 */   public float getFontSize() { return 0.0F; }
/*     */   
/* 410 */   public float svgToUserSpace(float v, int type, int pcInterp) { return 0.0F; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void addTargetListener(String pn, AnimationTargetListener l) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void removeTargetListener(String pn, AnimationTargetListener l) {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 432 */   public SVGElement getTargetElement() { return this.targetElement; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 440 */   public float getStartTime() { return this.timedElement.getCurrentBeginTime(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 448 */   public float getCurrentTime() { return this.timedElement.getLastSampleTime(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 458 */   public float getSimpleDuration() { return this.timedElement.getSimpleDur(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 467 */   public float getHyperlinkBeginTime() { return this.timedElement.getHyperlinkBeginTime(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public boolean beginElement() throws DOMException {
/* 477 */     this.timedElement.beginElement();
/* 478 */     return this.timedElement.canBegin();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public boolean beginElementAt(float offset) throws DOMException {
/* 486 */     this.timedElement.beginElement(offset);
/*     */ 
/*     */     
/* 489 */     return true;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public boolean endElement() throws DOMException {
/* 497 */     this.timedElement.endElement();
/* 498 */     return this.timedElement.canEnd();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public boolean endElementAt(float offset) throws DOMException {
/* 506 */     this.timedElement.endElement(offset);
/*     */ 
/*     */     
/* 509 */     return true;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 516 */   protected boolean isConstantAnimation() throws DOMException { return false; }
/*     */   
/*     */   protected class SVGTimedElement
/*     */     extends TimedElement {
/*     */     private final SVGAnimationElementBridge this$0;
/*     */     
/* 522 */     protected SVGTimedElement(SVGAnimationElementBridge this$0) { this.this$0 = this$0; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 528 */     public Element getElement() { return this.this$0.element; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 539 */     protected void fireTimeEvent(String eventType, Calendar time, int detail) { AnimationSupport.fireTimeEvent(this.this$0.element, eventType, time, detail); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 549 */     protected void toActive(float begin) { this.this$0.eng.toActive(this.this$0.animation, begin); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 561 */     protected void toInactive(boolean stillActive, boolean isFrozen) { this.this$0.eng.toInactive(this.this$0.animation, isFrozen); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 568 */     protected void removeFill() { this.this$0.eng.removeFill(this.this$0.animation); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 581 */     protected void sampledAt(float simpleTime, float simpleDur, int repeatIteration) { this.this$0.eng.sampledAt(this.this$0.animation, simpleTime, simpleDur, repeatIteration); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 591 */     protected void sampledLastValue(int repeatIteration) { this.this$0.eng.sampledLastValue(this.this$0.animation, repeatIteration); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 598 */     protected TimedElement getTimedElementById(String id) { return AnimationSupport.getTimedElementById(id, this.this$0.element); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 605 */     protected EventTarget getEventTargetById(String id) { return AnimationSupport.getEventTargetById(id, this.this$0.element); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 613 */     protected EventTarget getRootEventTarget() { return (EventTarget)this.this$0.element.getOwnerDocument(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 621 */     protected EventTarget getAnimationEventTarget() { return this.this$0.targetElement; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public boolean isBefore(TimedElement other) {
/* 629 */       Element e = ((SVGTimedElement)other).getElement();
/* 630 */       int pos = this.this$0.element.compareDocumentPosition(e);
/* 631 */       return ((pos & 0x2) != 0);
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public String toString() {
/* 638 */       if (this.this$0.element != null) {
/* 639 */         String id = this.this$0.element.getAttributeNS(null, "id");
/* 640 */         if (id.length() != 0) {
/* 641 */           return id;
/*     */         }
/*     */       } 
/* 644 */       return super.toString();
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 652 */     protected boolean isConstantAnimation() throws DOMException { return this.this$0.isConstantAnimation(); }
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\SVGAnimationElementBridge.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */