/*      */ package org.apache.batik.bridge;
/*      */ 
/*      */ import java.awt.Color;
/*      */ import java.awt.Paint;
/*      */ import java.lang.ref.WeakReference;
/*      */ import java.util.Arrays;
/*      */ import java.util.Calendar;
/*      */ import java.util.Date;
/*      */ import java.util.HashSet;
/*      */ import java.util.LinkedList;
/*      */ import java.util.Set;
/*      */ import org.apache.batik.anim.AnimationEngine;
/*      */ import org.apache.batik.anim.AnimationException;
/*      */ import org.apache.batik.anim.timing.TimedDocumentRoot;
/*      */ import org.apache.batik.anim.timing.TimedElement;
/*      */ import org.apache.batik.anim.values.AnimatableAngleOrIdentValue;
/*      */ import org.apache.batik.anim.values.AnimatableAngleValue;
/*      */ import org.apache.batik.anim.values.AnimatableBooleanValue;
/*      */ import org.apache.batik.anim.values.AnimatableColorValue;
/*      */ import org.apache.batik.anim.values.AnimatableIntegerValue;
/*      */ import org.apache.batik.anim.values.AnimatableLengthListValue;
/*      */ import org.apache.batik.anim.values.AnimatableLengthOrIdentValue;
/*      */ import org.apache.batik.anim.values.AnimatableLengthValue;
/*      */ import org.apache.batik.anim.values.AnimatableNumberListValue;
/*      */ import org.apache.batik.anim.values.AnimatableNumberOrIdentValue;
/*      */ import org.apache.batik.anim.values.AnimatableNumberOrPercentageValue;
/*      */ import org.apache.batik.anim.values.AnimatableNumberValue;
/*      */ import org.apache.batik.anim.values.AnimatablePaintValue;
/*      */ import org.apache.batik.anim.values.AnimatablePathDataValue;
/*      */ import org.apache.batik.anim.values.AnimatablePointListValue;
/*      */ import org.apache.batik.anim.values.AnimatablePreserveAspectRatioValue;
/*      */ import org.apache.batik.anim.values.AnimatableRectValue;
/*      */ import org.apache.batik.anim.values.AnimatableStringValue;
/*      */ import org.apache.batik.anim.values.AnimatableValue;
/*      */ import org.apache.batik.css.engine.CSSEngine;
/*      */ import org.apache.batik.css.engine.CSSStylableElement;
/*      */ import org.apache.batik.css.engine.StyleMap;
/*      */ import org.apache.batik.css.engine.value.FloatValue;
/*      */ import org.apache.batik.css.engine.value.Value;
/*      */ import org.apache.batik.css.engine.value.ValueManager;
/*      */ import org.apache.batik.dom.anim.AnimationTarget;
/*      */ import org.apache.batik.dom.svg.SVGOMDocument;
/*      */ import org.apache.batik.dom.svg.SVGOMElement;
/*      */ import org.apache.batik.dom.svg.SVGStylableElement;
/*      */ import org.apache.batik.parser.DefaultPreserveAspectRatioHandler;
/*      */ import org.apache.batik.parser.FloatArrayProducer;
/*      */ import org.apache.batik.parser.LengthArrayProducer;
/*      */ import org.apache.batik.parser.LengthHandler;
/*      */ import org.apache.batik.parser.LengthListParser;
/*      */ import org.apache.batik.parser.LengthParser;
/*      */ import org.apache.batik.parser.NumberListParser;
/*      */ import org.apache.batik.parser.ParseException;
/*      */ import org.apache.batik.parser.PathArrayProducer;
/*      */ import org.apache.batik.parser.PathParser;
/*      */ import org.apache.batik.parser.PointsParser;
/*      */ import org.apache.batik.parser.PreserveAspectRatioParser;
/*      */ import org.apache.batik.util.RunnableQueue;
/*      */ import org.w3c.dom.Document;
/*      */ import org.w3c.dom.Element;
/*      */ import org.w3c.dom.css.CSSStyleDeclaration;
/*      */ import org.w3c.dom.events.EventTarget;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class SVGAnimationEngine
/*      */   extends AnimationEngine
/*      */ {
/*      */   protected BridgeContext ctx;
/*      */   protected CSSEngine cssEngine;
/*      */   protected boolean started;
/*      */   protected AnimationTickRunnable animationTickRunnable;
/*  127 */   protected UncomputedAnimatableStringValueFactory uncomputedAnimatableStringValueFactory = new UncomputedAnimatableStringValueFactory(this);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  134 */   protected AnimatableLengthOrIdentFactory animatableLengthOrIdentFactory = new AnimatableLengthOrIdentFactory(this);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  140 */   protected AnimatableNumberOrIdentFactory animatableNumberOrIdentFactory = new AnimatableNumberOrIdentFactory(this, false);
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Factory[] factories = { 
/*  146 */       null, new AnimatableIntegerValueFactory(this), new AnimatableNumberValueFactory(this), new AnimatableLengthValueFactory(this), null, new AnimatableAngleValueFactory(this), new AnimatableColorValueFactory(this), new AnimatablePaintValueFactory(this), null, null, this.uncomputedAnimatableStringValueFactory, null, null, new AnimatableNumberListValueFactory(this), new AnimatableLengthListValueFactory(this), this.uncomputedAnimatableStringValueFactory, this.uncomputedAnimatableStringValueFactory, this.animatableLengthOrIdentFactory, this.uncomputedAnimatableStringValueFactory, this.uncomputedAnimatableStringValueFactory, this.uncomputedAnimatableStringValueFactory, this.uncomputedAnimatableStringValueFactory, new AnimatablePathDataFactory(this), this.uncomputedAnimatableStringValueFactory, null, this.animatableNumberOrIdentFactory, this.uncomputedAnimatableStringValueFactory, null, new AnimatableNumberOrIdentFactory(this, true), new AnimatableAngleOrIdentFactory(this), null, new AnimatablePointListValueFactory(this), new AnimatablePreserveAspectRatioValueFactory(this), null, this.uncomputedAnimatableStringValueFactory, null, null, null, null, this.animatableLengthOrIdentFactory, this.animatableLengthOrIdentFactory, this.animatableLengthOrIdentFactory, this.animatableLengthOrIdentFactory, this.animatableLengthOrIdentFactory, this.animatableNumberOrIdentFactory, null, null, new AnimatableNumberOrPercentageValueFactory(this), null, new AnimatableBooleanValueFactory(this), new AnimatableRectValueFactory(this) };
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected boolean isSVG12;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  208 */   protected LinkedList initialBridges = new LinkedList();
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected StyleMap dummyStyleMap;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected AnimationThread animationThread;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected int animationLimitingMode;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected float animationLimitingAmount;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  233 */   protected static final Set animationEventNames11 = new HashSet();
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  238 */   protected static final Set animationEventNames12 = new HashSet();
/*      */   
/*      */   static  {
/*  241 */     eventNamesCommon = new String[] { "click", "mousedown", "mouseup", "mouseover", "mousemove", "mouseout", "beginEvent", "endEvent" };
/*      */ 
/*      */ 
/*      */     
/*  245 */     String[] eventNamesSVG11 = { "DOMSubtreeModified", "DOMNodeInserted", "DOMNodeRemoved", "DOMNodeRemovedFromDocument", "DOMNodeInsertedIntoDocument", "DOMAttrModified", "DOMCharacterDataModified", "SVGLoad", "SVGUnload", "SVGAbort", "SVGError", "SVGResize", "SVGScroll", "repeatEvent" };
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  252 */     String[] eventNamesSVG12 = { "load", "resize", "scroll", "zoom" };
/*      */ 
/*      */     
/*  255 */     for (int i = 0; i < eventNamesCommon.length; i++) {
/*  256 */       animationEventNames11.add(eventNamesCommon[i]);
/*  257 */       animationEventNames12.add(eventNamesCommon[i]);
/*      */     } 
/*  259 */     for (int i = 0; i < eventNamesSVG11.length; i++) {
/*  260 */       animationEventNames11.add(eventNamesSVG11[i]);
/*      */     }
/*  262 */     for (int i = 0; i < eventNamesSVG12.length; i++) {
/*  263 */       animationEventNames12.add(eventNamesSVG12[i]);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public SVGAnimationEngine(Document doc, BridgeContext ctx) {
/*  271 */     super(doc);
/*  272 */     this.ctx = ctx;
/*  273 */     SVGOMDocument d = (SVGOMDocument)doc;
/*  274 */     this.cssEngine = d.getCSSEngine();
/*  275 */     this.dummyStyleMap = new StyleMap(this.cssEngine.getNumberOfProperties());
/*  276 */     this.isSVG12 = d.isSVG12();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void dispose() {
/*  283 */     synchronized (this) {
/*  284 */       pause();
/*  285 */       super.dispose();
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void addInitialBridge(SVGAnimationElementBridge b) {
/*  294 */     if (this.initialBridges != null) {
/*  295 */       this.initialBridges.add(b);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  303 */   public boolean hasStarted() { return this.started; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public AnimatableValue parseAnimatableValue(Element animElt, AnimationTarget target, String ns, String ln, boolean isCSS, String s) {
/*      */     int i;
/*  314 */     SVGOMElement elt = (SVGOMElement)target.getElement();
/*      */     
/*  316 */     if (isCSS) {
/*  317 */       i = elt.getPropertyType(ln);
/*      */     } else {
/*  319 */       i = elt.getAttributeType(ns, ln);
/*      */     } 
/*  321 */     Factory factory = this.factories[i];
/*  322 */     if (factory == null) {
/*  323 */       String an = (ns == null) ? ln : ('{' + ns + '}' + ln);
/*  324 */       throw new BridgeException(this.ctx, animElt, "attribute.not.animatable", new Object[] { target.getElement().getNodeName(), an });
/*      */     } 
/*      */ 
/*      */     
/*  328 */     return this.factories[i].createValue(target, ns, ln, isCSS, s);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public AnimatableValue getUnderlyingCSSValue(Element animElt, AnimationTarget target, String pn) {
/*  337 */     ValueManager[] vms = this.cssEngine.getValueManagers();
/*  338 */     int idx = this.cssEngine.getPropertyIndex(pn);
/*  339 */     if (idx != -1) {
/*  340 */       int type = vms[idx].getPropertyType();
/*  341 */       Factory factory = this.factories[type];
/*  342 */       if (factory == null) {
/*  343 */         throw new BridgeException(this.ctx, animElt, "attribute.not.animatable", new Object[] { target.getElement().getNodeName(), pn });
/*      */       }
/*      */ 
/*      */       
/*  347 */       SVGStylableElement e = (SVGStylableElement)target.getElement();
/*  348 */       CSSStyleDeclaration over = e.getOverrideStyle();
/*  349 */       String oldValue = over.getPropertyValue(pn);
/*  350 */       if (oldValue != null) {
/*  351 */         over.removeProperty(pn);
/*      */       }
/*  353 */       Value v = this.cssEngine.getComputedStyle(e, null, idx);
/*  354 */       if (oldValue != null && !oldValue.equals("")) {
/*  355 */         over.setProperty(pn, oldValue, null);
/*      */       }
/*  357 */       return this.factories[type].createValue(target, pn, v);
/*      */     } 
/*      */     
/*  360 */     return null;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void pause() {
/*  367 */     super.pause();
/*  368 */     UpdateManager um = this.ctx.getUpdateManager();
/*  369 */     if (um != null) {
/*  370 */       um.getUpdateRunnableQueue().setIdleRunnable(null);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void unpause() {
/*  378 */     super.unpause();
/*  379 */     UpdateManager um = this.ctx.getUpdateManager();
/*  380 */     if (um != null) {
/*  381 */       um.getUpdateRunnableQueue().setIdleRunnable(this.animationTickRunnable);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public float getCurrentTime() {
/*  389 */     boolean p = (this.pauseTime != 0L);
/*  390 */     unpause();
/*  391 */     float t = this.timedDocumentRoot.getCurrentTime();
/*  392 */     if (p) {
/*  393 */       pause();
/*      */     }
/*  395 */     return t;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public float setCurrentTime(float t) {
/*  402 */     float ret = super.setCurrentTime(t);
/*  403 */     if (this.animationTickRunnable != null) {
/*  404 */       this.animationTickRunnable.resume();
/*      */     }
/*  406 */     return ret;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  413 */   protected TimedDocumentRoot createDocumentRoot() { return new AnimationRoot(this); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void start(long documentStartTime) {
/*  420 */     if (this.started) {
/*      */       return;
/*      */     }
/*  423 */     this.started = true;
/*      */     try {
/*      */       try {
/*  426 */         Calendar cal = Calendar.getInstance();
/*  427 */         cal.setTime(new Date(documentStartTime));
/*  428 */         this.timedDocumentRoot.resetDocument(cal);
/*  429 */         Object[] bridges = this.initialBridges.toArray();
/*  430 */         this.initialBridges = null;
/*  431 */         for (int i = 0; i < bridges.length; i++) {
/*  432 */           SVGAnimationElementBridge bridge = (SVGAnimationElementBridge)bridges[i];
/*      */           
/*  434 */           bridge.initializeAnimation();
/*      */         } 
/*  436 */         for (int i = 0; i < bridges.length; i++) {
/*  437 */           SVGAnimationElementBridge bridge = (SVGAnimationElementBridge)bridges[i];
/*      */           
/*  439 */           bridge.initializeTimedElement();
/*      */         } 
/*      */ 
/*      */ 
/*      */         
/*  444 */         UpdateManager um = this.ctx.getUpdateManager();
/*  445 */         if (um != null) {
/*  446 */           RunnableQueue q = um.getUpdateRunnableQueue();
/*  447 */           this.animationTickRunnable = new AnimationTickRunnable(q, this);
/*  448 */           q.setIdleRunnable(this.animationTickRunnable);
/*      */         } 
/*      */       } catch (AnimationException ex) {
/*  451 */         throw new BridgeException(this.ctx, ex.getElement().getElement(), ex.getMessage());
/*      */       } 
/*      */     } catch (Exception ex) {
/*      */       
/*  455 */       if (this.ctx.getUserAgent() == null) {
/*  456 */         ex.printStackTrace();
/*      */       } else {
/*  458 */         this.ctx.getUserAgent().displayError(ex);
/*      */       } 
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  467 */   public void setAnimationLimitingNone() { this.animationLimitingMode = 0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void setAnimationLimitingCPU(float pc) {
/*  475 */     this.animationLimitingMode = 1;
/*  476 */     this.animationLimitingAmount = pc;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void setAnimationLimitingFPS(float fps) {
/*  484 */     this.animationLimitingMode = 2;
/*  485 */     this.animationLimitingAmount = fps;
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class AnimationRoot
/*      */     extends TimedDocumentRoot
/*      */   {
/*      */     private final SVGAnimationEngine this$0;
/*      */ 
/*      */     
/*      */     public AnimationRoot(SVGAnimationEngine this$0) {
/*  497 */       super(!this$0.isSVG12, this$0.isSVG12);
/*      */       this.this$0 = this$0;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected String getEventNamespaceURI(String eventName) {
/*  505 */       if (!this.this$0.isSVG12) {
/*  506 */         return null;
/*      */       }
/*  508 */       if (eventName.equals("focusin") || eventName.equals("focusout") || eventName.equals("activate") || SVGAnimationEngine.animationEventNames12.contains(eventName))
/*      */       {
/*      */ 
/*      */         
/*  512 */         return "http://www.w3.org/2001/xml-events";
/*      */       }
/*  514 */       return null;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected String getEventType(String eventName) {
/*  522 */       if (eventName.equals("focusin"))
/*  523 */         return "DOMFocusIn"; 
/*  524 */       if (eventName.equals("focusout"))
/*  525 */         return "DOMFocusOut"; 
/*  526 */       if (eventName.equals("activate")) {
/*  527 */         return "DOMActivate";
/*      */       }
/*  529 */       if (this.this$0.isSVG12) {
/*  530 */         if (SVGAnimationEngine.animationEventNames12.contains(eventName)) {
/*  531 */           return eventName;
/*      */         }
/*      */       }
/*  534 */       else if (SVGAnimationEngine.animationEventNames11.contains(eventName)) {
/*  535 */         return eventName;
/*      */       } 
/*      */       
/*  538 */       return null;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  546 */     protected String getRepeatEventName() { return "repeatEvent"; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  557 */     protected void fireTimeEvent(String eventType, Calendar time, int detail) { AnimationSupport.fireTimeEvent((EventTarget)this.this$0.document, eventType, time, detail); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected void toActive(float begin) {}
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected void toInactive(boolean stillActive, boolean isFrozen) {}
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected void removeFill() {}
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected void sampledAt(float simpleTime, float simpleDur, int repeatIteration) {}
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected void sampledLastValue(int repeatIteration) {}
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  611 */     protected TimedElement getTimedElementById(String id) { return AnimationSupport.getTimedElementById(id, this.this$0.document); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  618 */     protected EventTarget getEventTargetById(String id) { return AnimationSupport.getEventTargetById(id, this.this$0.document); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  626 */     protected EventTarget getAnimationEventTarget() { return null; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  634 */     protected EventTarget getRootEventTarget() { return (EventTarget)this.this$0.document; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  642 */     public Element getElement() { return null; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  650 */     public boolean isBefore(TimedElement other) { return false; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected void currentIntervalWillUpdate() {
/*  658 */       if (this.this$0.animationTickRunnable != null) {
/*  659 */         this.this$0.animationTickRunnable.resume();
/*      */       }
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected static class DebugAnimationTickRunnable
/*      */     extends AnimationTickRunnable
/*      */   {
/*  669 */     float t = 0.0F;
/*      */     
/*      */     public DebugAnimationTickRunnable(RunnableQueue q, SVGAnimationEngine eng) {
/*  672 */       super(q, eng);
/*  673 */       this.waitTime = Float.MAX_VALUE;
/*  674 */       (new SVGAnimationEngine$1(this)).start();
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void resume() {
/*  696 */       this.waitTime = 0L;
/*  697 */       Object lock = this.q.getIteratorLock();
/*  698 */       synchronized (lock) {
/*  699 */         lock.notify();
/*      */       } 
/*      */     }
/*      */     
/*      */     public long getWaitTime() {
/*  704 */       long wt = this.waitTime;
/*  705 */       this.waitTime = Float.MAX_VALUE;
/*  706 */       return wt;
/*      */     }
/*      */     
/*      */     public void run() {
/*  710 */       SVGAnimationEngine eng = getAnimationEngine();
/*  711 */       synchronized (eng) {
/*      */         try {
/*      */           try {
/*  714 */             eng.tick(this.t, false);
/*      */           } catch (AnimationException ex) {
/*  716 */             throw new BridgeException(eng.ctx, ex.getElement().getElement(), ex.getMessage());
/*      */           }
/*      */         
/*      */         } catch (Exception ex) {
/*      */           
/*  721 */           if (eng.ctx.getUserAgent() == null) {
/*  722 */             ex.printStackTrace();
/*      */           } else {
/*  724 */             eng.ctx.getUserAgent().displayError(ex);
/*      */           } 
/*      */         } 
/*      */       } 
/*      */     }
/*      */   }
/*      */   
/*      */   protected static class AnimationTickRunnable
/*      */     implements RunnableQueue.IdleRunnable
/*      */   {
/*      */     protected Calendar time;
/*      */     protected long waitTime;
/*      */     protected RunnableQueue q;
/*      */     private static final int NUM_TIMES = 8;
/*      */     protected long[] times;
/*      */     
/*      */     public AnimationTickRunnable(RunnableQueue q, SVGAnimationEngine eng) {
/*  741 */       this.time = Calendar.getInstance();
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  774 */       this.times = new long[8];
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  810 */       this.q = q;
/*  811 */       this.engRef = new WeakReference(eng);
/*      */       
/*  813 */       Arrays.fill(this.times, 100L);
/*  814 */       this.sumTime = 800L;
/*      */     }
/*      */     protected long sumTime; protected int timeIndex;
/*      */     protected WeakReference engRef;
/*      */     protected static final int MAX_EXCEPTION_COUNT = 10;
/*      */     protected int exceptionCount;
/*      */     
/*      */     public void resume() {
/*  822 */       this.waitTime = 0L;
/*  823 */       Object lock = this.q.getIteratorLock();
/*  824 */       synchronized (lock) {
/*  825 */         lock.notify();
/*      */       } 
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  838 */     public long getWaitTime() { return this.waitTime; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void run() {
/*  845 */       SVGAnimationEngine eng = getAnimationEngine();
/*  846 */       synchronized (eng) {
/*  847 */         int animationLimitingMode = eng.animationLimitingMode;
/*  848 */         float animationLimitingAmount = eng.animationLimitingAmount;
/*      */         try {
/*      */           try {
/*  851 */             long before = System.currentTimeMillis();
/*  852 */             this.time.setTime(new Date(before));
/*  853 */             float t = eng.timedDocumentRoot.convertWallclockTime(this.time);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */             
/*  859 */             float t2 = eng.tick(t, false);
/*  860 */             long after = System.currentTimeMillis();
/*  861 */             long dur = after - before;
/*  862 */             if (dur == 0L) {
/*  863 */               dur = 1L;
/*      */             }
/*  865 */             this.sumTime -= this.times[this.timeIndex];
/*  866 */             this.sumTime += dur;
/*  867 */             this.times[this.timeIndex] = dur;
/*  868 */             this.timeIndex = (this.timeIndex + 1) % 8;
/*      */             
/*  870 */             if (t2 == Float.POSITIVE_INFINITY) {
/*  871 */               this.waitTime = Float.MAX_VALUE;
/*      */             } else {
/*  873 */               this.waitTime = before + (long)(t2 * 1000.0F) - 1000L;
/*  874 */               if (this.waitTime < after) {
/*  875 */                 this.waitTime = after;
/*      */               }
/*  877 */               if (animationLimitingMode != 0) {
/*  878 */                 float f, ave = (float)this.sumTime / 8.0F;
/*      */                 
/*  880 */                 if (animationLimitingMode == 1) {
/*      */                   
/*  882 */                   f = ave / animationLimitingAmount - ave;
/*      */                 } else {
/*      */                   
/*  885 */                   f = 1000.0F / animationLimitingAmount - ave;
/*      */                 } 
/*  887 */                 long newWaitTime = after + (long)f;
/*  888 */                 if (newWaitTime > this.waitTime) {
/*  889 */                   this.waitTime = newWaitTime;
/*      */                 }
/*      */               } 
/*      */             } 
/*      */           } catch (AnimationException ex) {
/*      */             
/*  895 */             throw new BridgeException(eng.ctx, ex.getElement().getElement(), ex.getMessage());
/*      */           } 
/*      */ 
/*      */           
/*  899 */           this.exceptionCount = 0;
/*      */         } catch (Exception ex) {
/*  901 */           if (++this.exceptionCount < 10) {
/*  902 */             if (eng.ctx.getUserAgent() == null) {
/*  903 */               ex.printStackTrace();
/*      */             } else {
/*  905 */               eng.ctx.getUserAgent().displayError(ex);
/*      */             } 
/*      */           }
/*      */         } 
/*      */         
/*  910 */         if (animationLimitingMode == 0) {
/*      */           
/*      */           try {
/*  913 */             Thread.sleep(1L);
/*  914 */           } catch (InterruptedException ie) {}
/*      */         }
/*      */       } 
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  924 */     protected SVGAnimationEngine getAnimationEngine() { return (SVGAnimationEngine)this.engRef.get(); } }
/*      */   
/*      */   protected class AnimationThread extends Thread {
/*      */     protected Calendar time;
/*      */     protected RunnableQueue runnableQueue;
/*      */     
/*      */     protected AnimationThread(SVGAnimationEngine this$0) {
/*  931 */       this.this$0 = this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  936 */       this.time = Calendar.getInstance();
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  941 */       this.runnableQueue = this.this$0.ctx.getUpdateManager().getUpdateRunnableQueue();
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  947 */       this.ticker = new Ticker(this);
/*      */     }
/*      */     
/*      */     protected Ticker ticker;
/*      */     private final SVGAnimationEngine this$0;
/*      */     
/*      */     public void run() {
/*      */       while (true) {
/*  955 */         this.time.setTime(new Date());
/*  956 */         this.ticker.t = this.this$0.timedDocumentRoot.convertWallclockTime(this.time);
/*      */         try {
/*  958 */           this.runnableQueue.invokeAndWait(this.ticker);
/*      */         }
/*  960 */         catch (InterruptedException e) {
/*      */           break;
/*      */         } 
/*      */       } 
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected class Ticker
/*      */       implements Runnable
/*      */     {
/*      */       protected float t;
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*      */       private final SVGAnimationEngine.AnimationThread this$1;
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  983 */       protected Ticker(SVGAnimationEngine.AnimationThread this$1) { this.this$1 = this$1; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  994 */       public void run() { this.this$1.this$0.tick(this.t, false); }
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected static interface Factory
/*      */   {
/*      */     AnimatableValue createValue(AnimationTarget param1AnimationTarget, String param1String1, String param1String2, boolean param1Boolean, String param1String3);
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     AnimatableValue createValue(AnimationTarget param1AnimationTarget, String param1String, Value param1Value);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected abstract class CSSValueFactory
/*      */     implements Factory
/*      */   {
/*      */     private final SVGAnimationEngine this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1022 */     protected CSSValueFactory(SVGAnimationEngine this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1027 */     public AnimatableValue createValue(AnimationTarget target, String ns, String ln, boolean isCSS, String s) { return createValue(target, ln, createCSSValue(target, ln, s)); }
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableValue createValue(AnimationTarget target, String pn, Value v) {
/* 1032 */       CSSStylableElement elt = (CSSStylableElement)target.getElement();
/* 1033 */       v = computeValue(elt, pn, v);
/* 1034 */       return createAnimatableValue(target, pn, v);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected abstract AnimatableValue createAnimatableValue(AnimationTarget param1AnimationTarget, String param1String, Value param1Value);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected Value createCSSValue(AnimationTarget t, String pn, String s) {
/* 1048 */       CSSStylableElement elt = (CSSStylableElement)t.getElement();
/* 1049 */       Value v = this.this$0.cssEngine.parsePropertyValue(elt, pn, s);
/* 1050 */       return computeValue(elt, pn, v);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected Value computeValue(CSSStylableElement elt, String pn, Value v) {
/* 1059 */       ValueManager[] vms = this.this$0.cssEngine.getValueManagers();
/* 1060 */       int idx = this.this$0.cssEngine.getPropertyIndex(pn);
/* 1061 */       if (idx != -1) {
/* 1062 */         if (v.getCssValueType() == 0) {
/* 1063 */           elt = CSSEngine.getParentCSSStylableElement(elt);
/* 1064 */           if (elt != null) {
/* 1065 */             return this.this$0.cssEngine.getComputedStyle(elt, null, idx);
/*      */           }
/* 1067 */           return vms[idx].getDefaultValue();
/*      */         } 
/* 1069 */         v = vms[idx].computeValue(elt, null, this.this$0.cssEngine, idx, this.this$0.dummyStyleMap, v);
/*      */       } 
/*      */       
/* 1072 */       return v;
/*      */     }
/*      */   }
/*      */   
/*      */   protected class AnimatableBooleanValueFactory implements Factory {
/*      */     private final SVGAnimationEngine this$0;
/*      */     
/* 1079 */     protected AnimatableBooleanValueFactory(SVGAnimationEngine this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1086 */     public AnimatableValue createValue(AnimationTarget target, String ns, String ln, boolean isCSS, String s) { return new AnimatableBooleanValue(target, "true".equals(s)); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1094 */     public AnimatableValue createValue(AnimationTarget target, String pn, Value v) { return new AnimatableBooleanValue(target, "true".equals(v.getCssText())); }
/*      */   }
/*      */   
/*      */   protected class AnimatableIntegerValueFactory
/*      */     implements Factory
/*      */   {
/*      */     private final SVGAnimationEngine this$0;
/*      */     
/* 1102 */     protected AnimatableIntegerValueFactory(SVGAnimationEngine this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1109 */     public AnimatableValue createValue(AnimationTarget target, String ns, String ln, boolean isCSS, String s) { return new AnimatableIntegerValue(target, Integer.parseInt(s)); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1117 */     public AnimatableValue createValue(AnimationTarget target, String pn, Value v) { return new AnimatableIntegerValue(target, Math.round(v.getFloatValue())); }
/*      */   }
/*      */   
/*      */   protected class AnimatableNumberValueFactory
/*      */     implements Factory
/*      */   {
/*      */     private final SVGAnimationEngine this$0;
/*      */     
/* 1125 */     protected AnimatableNumberValueFactory(SVGAnimationEngine this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1132 */     public AnimatableValue createValue(AnimationTarget target, String ns, String ln, boolean isCSS, String s) { return new AnimatableNumberValue(target, Float.parseFloat(s)); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1140 */     public AnimatableValue createValue(AnimationTarget target, String pn, Value v) { return new AnimatableNumberValue(target, v.getFloatValue()); }
/*      */   }
/*      */   
/*      */   protected class AnimatableNumberOrPercentageValueFactory
/*      */     implements Factory {
/*      */     private final SVGAnimationEngine this$0;
/*      */     
/* 1147 */     protected AnimatableNumberOrPercentageValueFactory(SVGAnimationEngine this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableValue createValue(AnimationTarget target, String ns, String ln, boolean isCSS, String s) {
/*      */       boolean bool;
/*      */       float f;
/* 1157 */       if (s.charAt(s.length() - 1) == '%') {
/* 1158 */         f = Float.parseFloat(s.substring(0, s.length() - 1));
/* 1159 */         bool = true;
/*      */       } else {
/* 1161 */         f = Float.parseFloat(s);
/* 1162 */         bool = false;
/*      */       } 
/* 1164 */       return new AnimatableNumberOrPercentageValue(target, f, bool);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableValue createValue(AnimationTarget target, String pn, Value v) {
/* 1172 */       switch (v.getPrimitiveType()) {
/*      */         case 2:
/* 1174 */           return new AnimatableNumberOrPercentageValue(target, v.getFloatValue(), true);
/*      */         
/*      */         case 1:
/* 1177 */           return new AnimatableNumberOrPercentageValue(target, v.getFloatValue());
/*      */       } 
/*      */ 
/*      */       
/* 1181 */       return null;
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class AnimatablePreserveAspectRatioValueFactory
/*      */     implements Factory
/*      */   {
/*      */     protected short align;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected short meetOrSlice;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected PreserveAspectRatioParser parser;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected DefaultPreserveAspectRatioHandler handler;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     private final SVGAnimationEngine this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatablePreserveAspectRatioValueFactory(SVGAnimationEngine this$0) {
/* 1309 */       this.this$0 = this$0; this.parser = new PreserveAspectRatioParser(); this.handler = new SVGAnimationEngine$2(this);
/* 1310 */       this.parser.setPreserveAspectRatioHandler(this.handler);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableValue createValue(AnimationTarget target, String ns, String ln, boolean isCSS, String s) {
/*      */       try {
/* 1319 */         this.parser.parse(s);
/* 1320 */         return new AnimatablePreserveAspectRatioValue(target, this.align, this.meetOrSlice);
/*      */       }
/*      */       catch (ParseException e) {
/*      */         
/* 1324 */         return null;
/*      */       } 
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1334 */     public AnimatableValue createValue(AnimationTarget target, String pn, Value v) { return null; }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class AnimatableLengthValueFactory
/*      */     implements Factory
/*      */   {
/*      */     protected short type;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected float value;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected LengthParser parser;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected LengthHandler handler;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     private final SVGAnimationEngine this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableLengthValueFactory(SVGAnimationEngine this$0) {
/* 1402 */       this.this$0 = this$0; this.parser = new LengthParser(); this.handler = new SVGAnimationEngine$3(this);
/* 1403 */       this.parser.setLengthHandler(this.handler);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableValue createValue(AnimationTarget target, String ns, String ln, boolean isCSS, String s) {
/* 1411 */       short pcInterp = target.getPercentageInterpretation(ns, ln, isCSS);
/*      */       try {
/* 1413 */         this.parser.parse(s);
/* 1414 */         return new AnimatableLengthValue(target, this.type, this.value, pcInterp);
/*      */       }
/*      */       catch (ParseException e) {
/*      */         
/* 1418 */         return null;
/*      */       } 
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1427 */     public AnimatableValue createValue(AnimationTarget target, String pn, Value v) { return new AnimatableIntegerValue(target, Math.round(v.getFloatValue())); }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class AnimatableLengthListValueFactory
/*      */     implements Factory
/*      */   {
/*      */     protected LengthListParser parser;
/*      */ 
/*      */ 
/*      */     
/*      */     protected LengthArrayProducer producer;
/*      */ 
/*      */ 
/*      */     
/*      */     private final SVGAnimationEngine this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableLengthListValueFactory(SVGAnimationEngine this$0) {
/* 1450 */       this.this$0 = this$0; this.parser = new LengthListParser(); this.producer = new LengthArrayProducer();
/* 1451 */       this.parser.setLengthListHandler(this.producer);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableValue createValue(AnimationTarget target, String ns, String ln, boolean isCSS, String s) {
/*      */       try {
/* 1460 */         short pcInterp = target.getPercentageInterpretation(ns, ln, isCSS);
/*      */         
/* 1462 */         this.parser.parse(s);
/* 1463 */         return new AnimatableLengthListValue(target, this.producer.getLengthTypeArray(), this.producer.getLengthValueArray(), pcInterp);
/*      */       
/*      */       }
/*      */       catch (ParseException e) {
/*      */ 
/*      */         
/* 1469 */         return null;
/*      */       } 
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1479 */     public AnimatableValue createValue(AnimationTarget target, String pn, Value v) { return null; }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class AnimatableNumberListValueFactory
/*      */     implements Factory
/*      */   {
/*      */     protected NumberListParser parser;
/*      */ 
/*      */ 
/*      */     
/*      */     protected FloatArrayProducer producer;
/*      */ 
/*      */ 
/*      */     
/*      */     private final SVGAnimationEngine this$0;
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableNumberListValueFactory(SVGAnimationEngine this$0) {
/* 1501 */       this.this$0 = this$0; this.parser = new NumberListParser(); this.producer = new FloatArrayProducer();
/* 1502 */       this.parser.setNumberListHandler(this.producer);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableValue createValue(AnimationTarget target, String ns, String ln, boolean isCSS, String s) {
/*      */       try {
/* 1511 */         this.parser.parse(s);
/* 1512 */         return new AnimatableNumberListValue(target, this.producer.getFloatArray());
/*      */       }
/*      */       catch (ParseException e) {
/*      */         
/* 1516 */         return null;
/*      */       } 
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1526 */     public AnimatableValue createValue(AnimationTarget target, String pn, Value v) { return null; }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class AnimatableRectValueFactory
/*      */     implements Factory
/*      */   {
/*      */     protected NumberListParser parser;
/*      */ 
/*      */ 
/*      */     
/*      */     protected FloatArrayProducer producer;
/*      */ 
/*      */ 
/*      */     
/*      */     private final SVGAnimationEngine this$0;
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableRectValueFactory(SVGAnimationEngine this$0) {
/* 1548 */       this.this$0 = this$0; this.parser = new NumberListParser(); this.producer = new FloatArrayProducer();
/* 1549 */       this.parser.setNumberListHandler(this.producer);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableValue createValue(AnimationTarget target, String ns, String ln, boolean isCSS, String s) {
/*      */       try {
/* 1558 */         this.parser.parse(s);
/* 1559 */         float[] r = this.producer.getFloatArray();
/* 1560 */         if (r.length != 4)
/*      */         {
/* 1562 */           return null;
/*      */         }
/* 1564 */         return new AnimatableRectValue(target, r[0], r[1], r[2], r[3]);
/*      */       } catch (ParseException e) {
/*      */         
/* 1567 */         return null;
/*      */       } 
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1577 */     public AnimatableValue createValue(AnimationTarget target, String pn, Value v) { return null; }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class AnimatablePointListValueFactory
/*      */     implements Factory
/*      */   {
/*      */     protected PointsParser parser;
/*      */ 
/*      */ 
/*      */     
/*      */     protected FloatArrayProducer producer;
/*      */ 
/*      */ 
/*      */     
/*      */     private final SVGAnimationEngine this$0;
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatablePointListValueFactory(SVGAnimationEngine this$0) {
/* 1599 */       this.this$0 = this$0; this.parser = new PointsParser(); this.producer = new FloatArrayProducer();
/* 1600 */       this.parser.setPointsHandler(this.producer);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableValue createValue(AnimationTarget target, String ns, String ln, boolean isCSS, String s) {
/*      */       try {
/* 1609 */         this.parser.parse(s);
/* 1610 */         return new AnimatablePointListValue(target, this.producer.getFloatArray());
/*      */       }
/*      */       catch (ParseException e) {
/*      */         
/* 1614 */         return null;
/*      */       } 
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1624 */     public AnimatableValue createValue(AnimationTarget target, String pn, Value v) { return null; }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class AnimatablePathDataFactory
/*      */     implements Factory
/*      */   {
/*      */     protected PathParser parser;
/*      */ 
/*      */ 
/*      */     
/*      */     protected PathArrayProducer producer;
/*      */ 
/*      */ 
/*      */     
/*      */     private final SVGAnimationEngine this$0;
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatablePathDataFactory(SVGAnimationEngine this$0) {
/* 1646 */       this.this$0 = this$0; this.parser = new PathParser(); this.producer = new PathArrayProducer();
/* 1647 */       this.parser.setPathHandler(this.producer);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AnimatableValue createValue(AnimationTarget target, String ns, String ln, boolean isCSS, String s) {
/*      */       try {
/* 1656 */         this.parser.parse(s);
/* 1657 */         return new AnimatablePathDataValue(target, this.producer.getPathCommands(), this.producer.getPathParameters());
/*      */       
/*      */       }
/*      */       catch (ParseException e) {
/*      */         
/* 1662 */         return null;
/*      */       } 
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1672 */     public AnimatableValue createValue(AnimationTarget target, String pn, Value v) { return null; }
/*      */   }
/*      */   
/*      */   protected class UncomputedAnimatableStringValueFactory
/*      */     implements Factory {
/*      */     private final SVGAnimationEngine this$0;
/*      */     
/* 1679 */     protected UncomputedAnimatableStringValueFactory(SVGAnimationEngine this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */     
/* 1683 */     public AnimatableValue createValue(AnimationTarget target, String ns, String ln, boolean isCSS, String s) { return new AnimatableStringValue(target, s); }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1688 */     public AnimatableValue createValue(AnimationTarget target, String pn, Value v) { return new AnimatableStringValue(target, v.getCssText()); }
/*      */   }
/*      */   
/*      */   protected class AnimatableLengthOrIdentFactory
/*      */     extends CSSValueFactory {
/*      */     private final SVGAnimationEngine this$0;
/*      */     
/* 1695 */     protected AnimatableLengthOrIdentFactory(SVGAnimationEngine this$0) { super(this$0); this.this$0 = this$0; }
/*      */ 
/*      */     
/*      */     protected AnimatableValue createAnimatableValue(AnimationTarget target, String pn, Value v) {
/* 1699 */       if (v instanceof org.apache.batik.css.engine.value.StringValue) {
/* 1700 */         return new AnimatableLengthOrIdentValue(target, v.getStringValue());
/*      */       }
/*      */       
/* 1703 */       short pcInterp = target.getPercentageInterpretation(null, pn, true);
/* 1704 */       FloatValue fv = (FloatValue)v;
/* 1705 */       return new AnimatableLengthOrIdentValue(target, fv.getPrimitiveType(), fv.getFloatValue(), pcInterp);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class AnimatableNumberOrIdentFactory
/*      */     extends CSSValueFactory
/*      */   {
/*      */     protected boolean numericIdents;
/*      */ 
/*      */     
/*      */     private final SVGAnimationEngine this$0;
/*      */ 
/*      */     
/*      */     public AnimatableNumberOrIdentFactory(SVGAnimationEngine this$0, boolean numericIdents) {
/* 1721 */       super(this$0); this.this$0 = this$0;
/* 1722 */       this.numericIdents = numericIdents;
/*      */     }
/*      */ 
/*      */     
/*      */     protected AnimatableValue createAnimatableValue(AnimationTarget target, String pn, Value v) {
/* 1727 */       if (v instanceof org.apache.batik.css.engine.value.StringValue) {
/* 1728 */         return new AnimatableNumberOrIdentValue(target, v.getStringValue());
/*      */       }
/*      */       
/* 1731 */       FloatValue fv = (FloatValue)v;
/* 1732 */       return new AnimatableNumberOrIdentValue(target, fv.getFloatValue(), this.numericIdents);
/*      */     }
/*      */   }
/*      */   
/*      */   protected class AnimatableAngleValueFactory
/*      */     extends CSSValueFactory {
/*      */     private final SVGAnimationEngine this$0;
/*      */     
/* 1740 */     protected AnimatableAngleValueFactory(SVGAnimationEngine this$0) { super(this$0); this.this$0 = this$0; }
/*      */     
/*      */     protected AnimatableValue createAnimatableValue(AnimationTarget target, String pn, Value v) {
/*      */       short unit;
/* 1744 */       FloatValue fv = (FloatValue)v;
/*      */       
/* 1746 */       switch (fv.getPrimitiveType()) {
/*      */         case 1:
/*      */         case 11:
/* 1749 */           unit = 2;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/* 1761 */           return new AnimatableAngleValue(target, fv.getFloatValue(), unit);case 12: unit = 3; return new AnimatableAngleValue(target, fv.getFloatValue(), unit);case 13: unit = 4; return new AnimatableAngleValue(target, fv.getFloatValue(), unit);
/*      */       } 
/*      */       return null;
/*      */     } }
/*      */   
/*      */   protected class AnimatableAngleOrIdentFactory extends CSSValueFactory { private final SVGAnimationEngine this$0;
/*      */     
/* 1768 */     protected AnimatableAngleOrIdentFactory(SVGAnimationEngine this$0) { super(this$0); this.this$0 = this$0; }
/*      */     
/*      */     protected AnimatableValue createAnimatableValue(AnimationTarget target, String pn, Value v) {
/*      */       short unit;
/* 1772 */       if (v instanceof org.apache.batik.css.engine.value.StringValue) {
/* 1773 */         return new AnimatableAngleOrIdentValue(target, v.getStringValue());
/*      */       }
/*      */       
/* 1776 */       FloatValue fv = (FloatValue)v;
/*      */       
/* 1778 */       switch (fv.getPrimitiveType()) {
/*      */         case 1:
/*      */         case 11:
/* 1781 */           unit = 2;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/* 1793 */           return new AnimatableAngleOrIdentValue(target, fv.getFloatValue(), unit);case 12: unit = 3; return new AnimatableAngleOrIdentValue(target, fv.getFloatValue(), unit);case 13: unit = 4; return new AnimatableAngleOrIdentValue(target, fv.getFloatValue(), unit);
/*      */       } 
/*      */       return null;
/*      */     } }
/*      */   
/*      */   protected class AnimatableColorValueFactory extends CSSValueFactory {
/*      */     private final SVGAnimationEngine this$0;
/*      */     
/* 1801 */     protected AnimatableColorValueFactory(SVGAnimationEngine this$0) { super(this$0); this.this$0 = this$0; }
/*      */ 
/*      */     
/*      */     protected AnimatableValue createAnimatableValue(AnimationTarget target, String pn, Value v) {
/* 1805 */       Paint p = PaintServer.convertPaint(target.getElement(), null, v, 1.0F, this.this$0.ctx);
/*      */       
/* 1807 */       if (p instanceof Color) {
/* 1808 */         Color c = (Color)p;
/* 1809 */         return new AnimatableColorValue(target, c.getRed() / 255.0F, c.getGreen() / 255.0F, c.getBlue() / 255.0F);
/*      */       } 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1815 */       return null;
/*      */     }
/*      */   }
/*      */   
/*      */   protected class AnimatablePaintValueFactory extends CSSValueFactory {
/*      */     private final SVGAnimationEngine this$0;
/*      */     
/* 1822 */     protected AnimatablePaintValueFactory(SVGAnimationEngine this$0) { super(this$0); this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1830 */     protected AnimatablePaintValue createColorPaintValue(AnimationTarget t, Color c) { return AnimatablePaintValue.createColorPaintValue(t, c.getRed() / 255.0F, c.getGreen() / 255.0F, c.getBlue() / 255.0F); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected AnimatableValue createAnimatableValue(AnimationTarget target, String pn, Value v) {
/* 1837 */       if (v.getCssValueType() == 1) {
/* 1838 */         Paint p; switch (v.getPrimitiveType()) {
/*      */           case 21:
/* 1840 */             return AnimatablePaintValue.createNonePaintValue(target);
/*      */           case 25:
/* 1842 */             p = PaintServer.convertPaint(target.getElement(), null, v, 1.0F, this.this$0.ctx);
/*      */             
/* 1844 */             return createColorPaintValue(target, (Color)p);
/*      */           
/*      */           case 20:
/* 1847 */             return AnimatablePaintValue.createURIPaintValue(target, v.getStringValue());
/*      */         } 
/*      */       } else {
/*      */         Paint p, p;
/* 1851 */         Value v2, v1 = v.item(0);
/* 1852 */         switch (v1.getPrimitiveType()) {
/*      */           case 25:
/* 1854 */             p = PaintServer.convertPaint(target.getElement(), null, v, 1.0F, this.this$0.ctx);
/*      */             
/* 1856 */             return createColorPaintValue(target, (Color)p);
/*      */           
/*      */           case 20:
/* 1859 */             v2 = v.item(1);
/* 1860 */             switch (v2.getPrimitiveType()) {
/*      */               case 21:
/* 1862 */                 return AnimatablePaintValue.createURINonePaintValue(target, v1.getStringValue());
/*      */               
/*      */               case 25:
/* 1865 */                 p = PaintServer.convertPaint(target.getElement(), null, v.item(1), 1.0F, this.this$0.ctx);
/*      */                 
/* 1867 */                 return createColorPaintValue(target, (Color)p);
/*      */             } 
/*      */             
/*      */             break;
/*      */         } 
/*      */       
/*      */       } 
/* 1874 */       return null;
/*      */     }
/*      */   }
/*      */   
/*      */   protected class AnimatableStringValueFactory extends CSSValueFactory {
/*      */     private final SVGAnimationEngine this$0;
/*      */     
/* 1881 */     protected AnimatableStringValueFactory(SVGAnimationEngine this$0) { super(this$0); this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */     
/* 1885 */     protected AnimatableValue createAnimatableValue(AnimationTarget target, String pn, Value v) { return new AnimatableStringValue(target, v.getCssText()); }
/*      */   }
/*      */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\SVGAnimationEngine.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */