/*     */ package org.apache.batik.bridge;
/*     */ 
/*     */ import java.util.ArrayList;
/*     */ import java.util.Iterator;
/*     */ import java.util.LinkedList;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.StringTokenizer;
/*     */ import org.apache.batik.css.engine.CSSEngine;
/*     */ import org.apache.batik.css.engine.FontFaceRule;
/*     */ import org.apache.batik.dom.svg.SVGOMDocument;
/*     */ import org.apache.batik.gvt.font.GVTFontFace;
/*     */ import org.apache.batik.gvt.font.GVTFontFamily;
/*     */ import org.apache.batik.gvt.font.UnresolvedFontFamily;
/*     */ import org.apache.batik.util.SVGConstants;
/*     */ import org.w3c.dom.Document;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.NodeList;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class SVGFontUtilities
/*     */   implements SVGConstants
/*     */ {
/*     */   public static List getFontFaces(Document doc, BridgeContext ctx) {
/*  53 */     Map fontFamilyMap = ctx.getFontFamilyMap();
/*  54 */     List ret = (List)fontFamilyMap.get(doc);
/*  55 */     if (ret != null) {
/*  56 */       return ret;
/*     */     }
/*  58 */     ret = new LinkedList();
/*     */     
/*  60 */     NodeList fontFaceElements = doc.getElementsByTagNameNS("http://www.w3.org/2000/svg", "font-face");
/*     */ 
/*     */ 
/*     */     
/*  64 */     SVGFontFaceElementBridge fontFaceBridge = (SVGFontFaceElementBridge)ctx.getBridge("http://www.w3.org/2000/svg", "font-face");
/*     */ 
/*     */     
/*  67 */     for (int i = 0; i < fontFaceElements.getLength(); i++) {
/*  68 */       Element fontFaceElement = (Element)fontFaceElements.item(i);
/*  69 */       ret.add(fontFaceBridge.createFontFace(ctx, fontFaceElement));
/*     */     } 
/*     */ 
/*     */     
/*  73 */     CSSEngine engine = ((SVGOMDocument)doc).getCSSEngine();
/*  74 */     List sms = engine.getFontFaces();
/*  75 */     Iterator iter = sms.iterator();
/*  76 */     while (iter.hasNext()) {
/*  77 */       FontFaceRule ffr = (FontFaceRule)iter.next();
/*  78 */       ret.add(CSSFontFace.createCSSFontFace(engine, ffr));
/*     */     } 
/*  80 */     return ret;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static GVTFontFamily getFontFamily(Element textElement, BridgeContext ctx, String fontFamilyName, String fontWeight, String fontStyle) {
/* 110 */     String fontKeyName = fontFamilyName.toLowerCase() + " " + fontWeight + " " + fontStyle;
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 115 */     Map fontFamilyMap = ctx.getFontFamilyMap();
/* 116 */     GVTFontFamily fontFamily = (GVTFontFamily)fontFamilyMap.get(fontKeyName);
/*     */     
/* 118 */     if (fontFamily != null) {
/* 119 */       return fontFamily;
/*     */     }
/*     */ 
/*     */     
/* 123 */     Document doc = textElement.getOwnerDocument();
/*     */     
/* 125 */     List fontFaces = (List)fontFamilyMap.get(doc);
/*     */     
/* 127 */     if (fontFaces == null) {
/* 128 */       fontFaces = getFontFaces(doc, ctx);
/* 129 */       fontFamilyMap.put(doc, fontFaces);
/*     */     } 
/*     */ 
/*     */     
/* 133 */     Iterator iter = fontFaces.iterator();
/* 134 */     List svgFontFamilies = new LinkedList();
/* 135 */     while (iter.hasNext()) {
/* 136 */       FontFace fontFace = (FontFace)iter.next();
/*     */       
/* 138 */       if (!fontFace.hasFamilyName(fontFamilyName)) {
/*     */         continue;
/*     */       }
/*     */       
/* 142 */       String fontFaceStyle = fontFace.getFontStyle();
/* 143 */       if (fontFaceStyle.equals("all") || fontFaceStyle.indexOf(fontStyle) != -1) {
/*     */         
/* 145 */         GVTFontFamily ffam = fontFace.getFontFamily(ctx);
/* 146 */         if (ffam != null) {
/* 147 */           svgFontFamilies.add(ffam);
/*     */         }
/*     */       } 
/*     */     } 
/* 151 */     if (svgFontFamilies.size() == 1) {
/*     */       
/* 153 */       fontFamilyMap.put(fontKeyName, svgFontFamilies.get(0));
/* 154 */       return (GVTFontFamily)svgFontFamilies.get(0);
/*     */     } 
/* 156 */     if (svgFontFamilies.size() > 1) {
/*     */       
/* 158 */       String fontWeightNumber = getFontWeightNumberString(fontWeight);
/*     */ 
/*     */       
/* 161 */       List fontFamilyWeights = new ArrayList(svgFontFamilies.size());
/* 162 */       Iterator ffiter = svgFontFamilies.iterator();
/* 163 */       while (ffiter.hasNext()) {
/*     */         
/* 165 */         GVTFontFace fontFace = ((GVTFontFamily)ffiter.next()).getFontFace();
/* 166 */         String fontFaceWeight = fontFace.getFontWeight();
/* 167 */         fontFaceWeight = getFontWeightNumberString(fontFaceWeight);
/* 168 */         fontFamilyWeights.add(fontFaceWeight);
/*     */       } 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 175 */       List newFontFamilyWeights = new ArrayList(fontFamilyWeights);
/* 176 */       for (int i = 100; i <= 900; i += 100) {
/* 177 */         String weightString = String.valueOf(i);
/* 178 */         boolean matched = false;
/* 179 */         int minDifference = 1000;
/* 180 */         int minDifferenceIndex = 0;
/* 181 */         for (int j = 0; j < fontFamilyWeights.size(); j++) {
/* 182 */           String fontFamilyWeight = (String)fontFamilyWeights.get(j);
/* 183 */           if (fontFamilyWeight.indexOf(weightString) > -1) {
/* 184 */             matched = true;
/*     */             break;
/*     */           } 
/* 187 */           StringTokenizer st = new StringTokenizer(fontFamilyWeight, " ,");
/*     */           
/* 189 */           while (st.hasMoreTokens()) {
/* 190 */             int weightNum = Integer.parseInt(st.nextToken());
/* 191 */             int difference = Math.abs(weightNum - i);
/* 192 */             if (difference < minDifference) {
/* 193 */               minDifference = difference;
/* 194 */               minDifferenceIndex = j;
/*     */             } 
/*     */           } 
/*     */         } 
/* 198 */         if (!matched) {
/* 199 */           String newFontFamilyWeight = newFontFamilyWeights.get(minDifferenceIndex) + ", " + weightString;
/*     */ 
/*     */           
/* 202 */           newFontFamilyWeights.set(minDifferenceIndex, newFontFamilyWeight);
/*     */         } 
/*     */       } 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 209 */       for (int i = 0; i < svgFontFamilies.size(); i++) {
/* 210 */         String fontFaceWeight = (String)newFontFamilyWeights.get(i);
/* 211 */         if (fontFaceWeight.indexOf(fontWeightNumber) > -1) {
/* 212 */           fontFamilyMap.put(fontKeyName, svgFontFamilies.get(i));
/* 213 */           return (GVTFontFamily)svgFontFamilies.get(i);
/*     */         } 
/*     */       } 
/*     */       
/* 217 */       fontFamilyMap.put(fontKeyName, svgFontFamilies.get(0));
/* 218 */       return (GVTFontFamily)svgFontFamilies.get(0);
/*     */     } 
/*     */ 
/*     */     
/* 222 */     UnresolvedFontFamily unresolvedFontFamily = new UnresolvedFontFamily(fontFamilyName);
/*     */     
/* 224 */     fontFamilyMap.put(fontKeyName, unresolvedFontFamily);
/* 225 */     return unresolvedFontFamily;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected static String getFontWeightNumberString(String fontWeight) {
/* 239 */     if (fontWeight.equals("normal"))
/* 240 */       return "400"; 
/* 241 */     if (fontWeight.equals("bold"))
/* 242 */       return "700"; 
/* 243 */     if (fontWeight.equals("all")) {
/* 244 */       return "100, 200, 300, 400, 500, 600, 700, 800, 900";
/*     */     }
/* 246 */     return fontWeight;
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\SVGFontUtilities.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */