/*      */ package org.apache.batik.bridge;
/*      */ 
/*      */ import java.awt.RenderingHints;
/*      */ import java.awt.Shape;
/*      */ import java.awt.color.ICC_Profile;
/*      */ import java.awt.geom.AffineTransform;
/*      */ import java.awt.geom.NoninvertibleTransformException;
/*      */ import java.awt.geom.Rectangle2D;
/*      */ import java.io.BufferedInputStream;
/*      */ import java.io.IOException;
/*      */ import java.io.InputStream;
/*      */ import java.io.InterruptedIOException;
/*      */ import java.util.ArrayList;
/*      */ import java.util.List;
/*      */ import org.apache.batik.css.engine.CSSEngine;
/*      */ import org.apache.batik.dom.AbstractNode;
/*      */ import org.apache.batik.dom.events.DOMMouseEvent;
/*      */ import org.apache.batik.dom.events.NodeEventTarget;
/*      */ import org.apache.batik.dom.svg.AbstractSVGAnimatedLength;
/*      */ import org.apache.batik.dom.svg.AnimatedLiveAttributeValue;
/*      */ import org.apache.batik.dom.svg.LiveAttributeException;
/*      */ import org.apache.batik.dom.svg.SVGOMAnimatedPreserveAspectRatio;
/*      */ import org.apache.batik.dom.svg.SVGOMDocument;
/*      */ import org.apache.batik.dom.svg.SVGOMElement;
/*      */ import org.apache.batik.ext.awt.color.ICCColorSpaceExt;
/*      */ import org.apache.batik.ext.awt.image.renderable.ClipRable8Bit;
/*      */ import org.apache.batik.ext.awt.image.renderable.Filter;
/*      */ import org.apache.batik.ext.awt.image.spi.BrokenLinkProvider;
/*      */ import org.apache.batik.ext.awt.image.spi.ImageTagRegistry;
/*      */ import org.apache.batik.gvt.CanvasGraphicsNode;
/*      */ import org.apache.batik.gvt.CompositeGraphicsNode;
/*      */ import org.apache.batik.gvt.GraphicsNode;
/*      */ import org.apache.batik.gvt.ImageNode;
/*      */ import org.apache.batik.gvt.RasterImageNode;
/*      */ import org.apache.batik.gvt.ShapeNode;
/*      */ import org.apache.batik.util.HaltingThread;
/*      */ import org.apache.batik.util.MimeTypeConstants;
/*      */ import org.apache.batik.util.ParsedURL;
/*      */ import org.w3c.dom.Document;
/*      */ import org.w3c.dom.Element;
/*      */ import org.w3c.dom.events.DocumentEvent;
/*      */ import org.w3c.dom.events.Event;
/*      */ import org.w3c.dom.events.EventListener;
/*      */ import org.w3c.dom.events.EventTarget;
/*      */ import org.w3c.dom.svg.SVGDocument;
/*      */ import org.w3c.dom.svg.SVGImageElement;
/*      */ import org.w3c.dom.svg.SVGSVGElement;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class SVGImageElementBridge
/*      */   extends AbstractGraphicsNodeBridge
/*      */ {
/*      */   protected SVGDocument imgDocument;
/*   80 */   protected EventListener listener = null;
/*   81 */   protected BridgeContext subCtx = null;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected boolean hitCheckChildren = false;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*   92 */   public String getLocalName() { return "image"; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*   99 */   public Bridge getInstance() { return new SVGImageElementBridge(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public GraphicsNode createGraphicsNode(BridgeContext ctx, Element e) {
/*  111 */     ImageNode imageNode = (ImageNode)super.createGraphicsNode(ctx, e);
/*  112 */     if (imageNode == null) {
/*  113 */       return null;
/*      */     }
/*      */     
/*  116 */     associateSVGContext(ctx, e, imageNode);
/*      */     
/*  118 */     this.hitCheckChildren = false;
/*  119 */     GraphicsNode node = buildImageGraphicsNode(ctx, e);
/*      */     
/*  121 */     if (node == null) {
/*  122 */       SVGImageElement ie = (SVGImageElement)e;
/*  123 */       String uriStr = ie.getHref().getAnimVal();
/*  124 */       throw new BridgeException(ctx, e, "uri.image.invalid", new Object[] { uriStr });
/*      */     } 
/*      */ 
/*      */     
/*  128 */     imageNode.setImage(node);
/*  129 */     imageNode.setHitCheckChildren(this.hitCheckChildren);
/*      */ 
/*      */     
/*  132 */     RenderingHints hints = null;
/*  133 */     hints = CSSUtilities.convertImageRendering(e, hints);
/*  134 */     hints = CSSUtilities.convertColorRendering(e, hints);
/*  135 */     if (hints != null) {
/*  136 */       imageNode.setRenderingHints(hints);
/*      */     }
/*  138 */     return imageNode;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected GraphicsNode buildImageGraphicsNode(BridgeContext ctx, Element e) {
/*      */     ParsedURL parsedURL;
/*  155 */     SVGImageElement ie = (SVGImageElement)e;
/*      */ 
/*      */     
/*  158 */     String uriStr = ie.getHref().getAnimVal();
/*  159 */     if (uriStr.length() == 0) {
/*  160 */       throw new BridgeException(ctx, e, "attribute.missing", new Object[] { "xlink:href" });
/*      */     }
/*      */     
/*  163 */     if (uriStr.indexOf('#') != -1) {
/*  164 */       throw new BridgeException(ctx, e, "attribute.malformed", new Object[] { "xlink:href", uriStr });
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*  169 */     String baseURI = AbstractNode.getBaseURI(e);
/*      */     
/*  171 */     if (baseURI == null) {
/*  172 */       parsedURL = new ParsedURL(uriStr);
/*      */     } else {
/*  174 */       parsedURL = new ParsedURL(baseURI, uriStr);
/*      */     } 
/*      */     
/*  177 */     return createImageGraphicsNode(ctx, e, parsedURL);
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected GraphicsNode createImageGraphicsNode(BridgeContext ctx, Element e, ParsedURL purl) {
/*  183 */     Rectangle2D bounds = getImageBounds(ctx, e);
/*  184 */     if (bounds.getWidth() == 0.0D || bounds.getHeight() == 0.0D) {
/*  185 */       ShapeNode sn = new ShapeNode();
/*  186 */       sn.setShape(bounds);
/*  187 */       return sn;
/*      */     } 
/*      */     
/*  190 */     SVGDocument svgDoc = (SVGDocument)e.getOwnerDocument();
/*  191 */     String docURL = svgDoc.getURL();
/*  192 */     ParsedURL pDocURL = null;
/*  193 */     if (docURL != null) {
/*  194 */       pDocURL = new ParsedURL(docURL);
/*      */     }
/*  196 */     UserAgent userAgent = ctx.getUserAgent();
/*      */     
/*      */     try {
/*  199 */       userAgent.checkLoadExternalResource(purl, pDocURL);
/*      */     } catch (SecurityException secEx) {
/*  201 */       throw new BridgeException(ctx, e, secEx, "uri.unsecure", new Object[] { purl });
/*      */     } 
/*      */ 
/*      */     
/*  205 */     DocumentLoader loader = ctx.getDocumentLoader();
/*  206 */     ImageTagRegistry reg = ImageTagRegistry.getRegistry();
/*  207 */     ICCColorSpaceExt colorspace = extractColorSpace(e, ctx);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     try {
/*  215 */       Document doc = loader.checkCache(purl.toString());
/*  216 */       if (doc != null) {
/*  217 */         this.imgDocument = (SVGDocument)doc;
/*  218 */         return createSVGImageNode(ctx, e, this.imgDocument);
/*      */       } 
/*      */     } catch (BridgeException ex) {
/*  221 */       throw ex;
/*  222 */     } catch (Exception ex) {}
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  227 */     Filter img = reg.checkCache(purl, colorspace);
/*  228 */     if (img != null) {
/*  229 */       return createRasterImageNode(ctx, e, img, purl);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  240 */     ProtectedStream reference = null;
/*      */     try {
/*  242 */       reference = openStream(e, purl);
/*      */     } catch (SecurityException secEx) {
/*  244 */       throw new BridgeException(ctx, e, secEx, "uri.unsecure", new Object[] { purl });
/*      */     } catch (IOException ioe) {
/*      */       
/*  247 */       return createBrokenImageNode(ctx, e, purl.toString(), ioe.getLocalizedMessage());
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  258 */     img = reg.readURL(reference, purl, colorspace, false, false);
/*      */     
/*  260 */     if (img != null)
/*      */     {
/*  262 */       return createRasterImageNode(ctx, e, img, purl);
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*      */     try {
/*  268 */       reference.retry();
/*      */     } catch (IOException img) {
/*  270 */       IOException ioe; reference.release();
/*  271 */       reference = null;
/*      */       
/*      */       try {
/*  274 */         reference = openStream(e, purl);
/*      */       } catch (IOException ioe2) {
/*      */         
/*  277 */         return createBrokenImageNode(ctx, e, purl.toString(), ioe2.getLocalizedMessage());
/*      */       } 
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     try {
/*  286 */       Document doc = loader.loadDocument(purl.toString(), reference);
/*  287 */       this.imgDocument = (SVGDocument)doc;
/*  288 */       return createSVGImageNode(ctx, e, this.imgDocument);
/*      */     } catch (BridgeException img) {
/*  290 */       BridgeException ex; throw ex;
/*      */     } catch (SecurityException secEx) {
/*  292 */       throw new BridgeException(ctx, e, secEx, "uri.unsecure", new Object[] { purl });
/*      */     } catch (InterruptedIOException iioe) {
/*      */       
/*  295 */       if (HaltingThread.hasBeenHalted()) {
/*  296 */         throw new InterruptedBridgeException();
/*      */       }
/*      */     } catch (InterruptedBridgeException ibe) {
/*  299 */       throw ibe;
/*  300 */     } catch (Exception ex) {}
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     try {
/*  306 */       reference.retry();
/*      */     } catch (IOException img) {
/*  308 */       IOException ioe; reference.release();
/*  309 */       reference = null;
/*      */       
/*      */       try {
/*  312 */         reference = openStream(e, purl);
/*      */       } catch (IOException ioe2) {
/*  314 */         return createBrokenImageNode(ctx, e, purl.toString(), ioe2.getLocalizedMessage());
/*      */       } 
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     try {
/*  323 */       Filter img = reg.readURL(reference, purl, colorspace, true, true);
/*      */       
/*  325 */       if (img != null)
/*      */       {
/*  327 */         return createRasterImageNode(ctx, e, img, purl);
/*      */       }
/*      */     } finally {
/*  330 */       reference.release();
/*      */     } 
/*  332 */     return null;
/*      */   }
/*      */   
/*      */   public static class ProtectedStream extends BufferedInputStream { static final int BUFFER_SIZE = 8192;
/*      */     
/*      */     ProtectedStream(InputStream is) {
/*  338 */       super(is, 8192);
/*  339 */       super.mark(8192);
/*      */     }
/*      */     ProtectedStream(InputStream is, int size) {
/*  342 */       super(is, size);
/*  343 */       super.mark(size);
/*      */     }
/*      */ 
/*      */     
/*  347 */     public boolean markSupported() { return false; }
/*      */ 
/*      */     
/*      */     public void mark(int sz) {}
/*      */     
/*  352 */     public void reset() { throw new IOException("Reset unsupported"); }
/*      */ 
/*      */ 
/*      */     
/*  356 */     public void retry() { super.reset(); }
/*      */ 
/*      */ 
/*      */     
/*      */     public void close() {}
/*      */ 
/*      */     
/*      */     public void release() {
/*      */       try {
/*  365 */         super.close();
/*  366 */       } catch (IOException ioe) {}
/*      */     } }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected ProtectedStream openStream(Element e, ParsedURL purl) throws IOException {
/*  374 */     List mimeTypes = new ArrayList(ImageTagRegistry.getRegistry().getRegisteredMimeTypes());
/*      */     
/*  376 */     mimeTypes.add(MimeTypeConstants.MIME_TYPES_SVG);
/*  377 */     InputStream reference = purl.openStream(mimeTypes.iterator());
/*  378 */     return new ProtectedStream(reference);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  385 */   protected GraphicsNode instantiateGraphicsNode() { return new ImageNode(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  392 */   public boolean isComposite() { return false; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void initializeDynamicSupport(BridgeContext ctx, Element e, GraphicsNode node) {
/*  406 */     if (!ctx.isInteractive()) {
/*      */       return;
/*      */     }
/*      */ 
/*      */     
/*  411 */     ctx.bind(e, node);
/*      */     
/*  413 */     if (ctx.isDynamic()) {
/*      */       
/*  415 */       this.e = e;
/*  416 */       this.node = node;
/*  417 */       this.ctx = ctx;
/*  418 */       ((SVGOMElement)e).setSVGContext(this);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void handleAnimatedAttributeChanged(AnimatedLiveAttributeValue alav) {
/*      */     try {
/*  430 */       String ns = alav.getNamespaceURI();
/*  431 */       String ln = alav.getLocalName();
/*  432 */       if (ns == null) {
/*  433 */         if (ln.equals("x") || ln.equals("y")) {
/*      */           
/*  435 */           updateImageBounds(); return;
/*      */         } 
/*  437 */         if (ln.equals("width") || ln.equals("height")) {
/*      */           AbstractSVGAnimatedLength abstractSVGAnimatedLength;
/*  439 */           SVGImageElement ie = (SVGImageElement)this.e;
/*  440 */           ImageNode imageNode = (ImageNode)this.node;
/*      */           
/*  442 */           if (ln.charAt(0) == 'w') {
/*  443 */             abstractSVGAnimatedLength = (AbstractSVGAnimatedLength)ie.getWidth();
/*      */           } else {
/*  445 */             abstractSVGAnimatedLength = (AbstractSVGAnimatedLength)ie.getHeight();
/*      */           } 
/*  447 */           float val = abstractSVGAnimatedLength.getCheckedValue();
/*  448 */           if (val == 0.0F || imageNode.getImage() instanceof ShapeNode) {
/*  449 */             rebuildImageNode();
/*      */           } else {
/*  451 */             updateImageBounds();
/*      */           }  return;
/*      */         } 
/*  454 */         if (ln.equals("preserveAspectRatio")) {
/*  455 */           updateImageBounds();
/*      */           return;
/*      */         } 
/*  458 */       } else if (ns.equals("http://www.w3.org/1999/xlink") && ln.equals("href")) {
/*      */         
/*  460 */         rebuildImageNode();
/*      */         return;
/*      */       } 
/*      */     } catch (LiveAttributeException ex) {
/*  464 */       throw new BridgeException(this.ctx, ex);
/*      */     } 
/*  466 */     super.handleAnimatedAttributeChanged(alav);
/*      */   }
/*      */ 
/*      */   
/*      */   protected void updateImageBounds() {
/*  471 */     Rectangle2D bounds = getImageBounds(this.ctx, this.e);
/*  472 */     GraphicsNode imageNode = ((ImageNode)this.node).getImage();
/*  473 */     float[] vb = null;
/*  474 */     if (imageNode instanceof RasterImageNode) {
/*      */       
/*  476 */       Rectangle2D imgBounds = ((RasterImageNode)imageNode).getImageBounds();
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  481 */       vb = new float[4];
/*  482 */       vb[0] = 0.0F;
/*  483 */       vb[1] = 0.0F;
/*  484 */       vb[2] = (float)imgBounds.getWidth();
/*  485 */       vb[3] = (float)imgBounds.getHeight();
/*      */     }
/*  487 */     else if (this.imgDocument != null) {
/*  488 */       SVGSVGElement sVGSVGElement = this.imgDocument.getRootElement();
/*  489 */       String viewBox = sVGSVGElement.getAttributeNS(null, "viewBox");
/*      */       
/*  491 */       vb = ViewBox.parseViewBoxAttribute(this.e, viewBox, this.ctx);
/*      */     } 
/*      */     
/*  494 */     if (imageNode != null)
/*      */     {
/*      */ 
/*      */       
/*  498 */       initializeViewport(this.ctx, this.e, imageNode, vb, bounds);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected void rebuildImageNode() {
/*  505 */     if (this.imgDocument != null && this.listener != null) {
/*  506 */       NodeEventTarget tgt = (NodeEventTarget)this.imgDocument.getRootElement();
/*      */       
/*  508 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "click", this.listener, false);
/*      */ 
/*      */       
/*  511 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "keydown", this.listener, false);
/*      */ 
/*      */       
/*  514 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "keypress", this.listener, false);
/*      */ 
/*      */       
/*  517 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "keyup", this.listener, false);
/*      */ 
/*      */       
/*  520 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mousedown", this.listener, false);
/*      */ 
/*      */       
/*  523 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mousemove", this.listener, false);
/*      */ 
/*      */       
/*  526 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mouseout", this.listener, false);
/*      */ 
/*      */       
/*  529 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mouseover", this.listener, false);
/*      */ 
/*      */       
/*  532 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mouseup", this.listener, false);
/*      */ 
/*      */       
/*  535 */       this.listener = null;
/*      */     } 
/*      */     
/*  538 */     if (this.imgDocument != null) {
/*  539 */       SVGSVGElement svgElement = this.imgDocument.getRootElement();
/*  540 */       disposeTree(svgElement);
/*      */     } 
/*      */     
/*  543 */     this.imgDocument = null;
/*  544 */     this.subCtx = null;
/*      */ 
/*      */     
/*  547 */     GraphicsNode inode = buildImageGraphicsNode(this.ctx, this.e);
/*      */     
/*  549 */     ImageNode imgNode = (ImageNode)this.node;
/*  550 */     imgNode.setImage(inode);
/*      */     
/*  552 */     if (inode == null) {
/*  553 */       SVGImageElement ie = (SVGImageElement)this.e;
/*  554 */       String uriStr = ie.getHref().getAnimVal();
/*  555 */       throw new BridgeException(this.ctx, this.e, "uri.image.invalid", new Object[] { uriStr });
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void handleCSSPropertyChanged(int property) {
/*      */     RenderingHints hints;
/*  564 */     switch (property) {
/*      */       case 6:
/*      */       case 30:
/*  567 */         hints = CSSUtilities.convertImageRendering(this.e, null);
/*  568 */         hints = CSSUtilities.convertColorRendering(this.e, hints);
/*  569 */         if (hints != null) {
/*  570 */           this.node.setRenderingHints(hints);
/*      */         }
/*      */         return;
/*      */     } 
/*  574 */     super.handleCSSPropertyChanged(property);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected GraphicsNode createRasterImageNode(BridgeContext ctx, Element e, Filter img, ParsedURL purl) {
/*  592 */     Rectangle2D bounds = getImageBounds(ctx, e);
/*  593 */     if (bounds.getWidth() == 0.0D || bounds.getHeight() == 0.0D) {
/*  594 */       ShapeNode sn = new ShapeNode();
/*  595 */       sn.setShape(bounds);
/*  596 */       return sn;
/*      */     } 
/*      */     
/*  599 */     if (BrokenLinkProvider.hasBrokenLinkProperty(img)) {
/*  600 */       Object o = img.getProperty("org.apache.batik.BrokenLinkImage");
/*  601 */       String msg = "unknown";
/*  602 */       if (o instanceof String)
/*  603 */         msg = (String)o; 
/*  604 */       SVGDocument doc = ctx.getUserAgent().getBrokenLinkDocument(e, purl.toString(), msg);
/*      */       
/*  606 */       return createSVGImageNode(ctx, e, doc);
/*      */     } 
/*      */     
/*  609 */     RasterImageNode node = new RasterImageNode();
/*  610 */     node.setImage(img);
/*  611 */     Rectangle2D imgBounds = img.getBounds2D();
/*      */ 
/*      */ 
/*      */     
/*  615 */     float[] vb = new float[4];
/*  616 */     vb[0] = 0.0F;
/*  617 */     vb[1] = 0.0F;
/*  618 */     vb[2] = (float)imgBounds.getWidth();
/*  619 */     vb[3] = (float)imgBounds.getHeight();
/*      */ 
/*      */ 
/*      */     
/*  623 */     initializeViewport(ctx, e, node, vb, bounds);
/*      */     
/*  625 */     return node;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected GraphicsNode createSVGImageNode(BridgeContext ctx, Element e, SVGDocument imgDocument) {
/*  638 */     CSSEngine eng = ((SVGOMDocument)imgDocument).getCSSEngine();
/*  639 */     this.subCtx = ctx.createSubBridgeContext((SVGOMDocument)imgDocument);
/*      */     
/*  641 */     CompositeGraphicsNode result = new CompositeGraphicsNode();
/*      */ 
/*      */     
/*  644 */     Rectangle2D bounds = getImageBounds(ctx, e);
/*      */     
/*  646 */     if (bounds.getWidth() == 0.0D || bounds.getHeight() == 0.0D) {
/*  647 */       ShapeNode sn = new ShapeNode();
/*  648 */       sn.setShape(bounds);
/*  649 */       result.getChildren().add(sn);
/*  650 */       return result;
/*      */     } 
/*      */     
/*  653 */     Rectangle2D r = CSSUtilities.convertEnableBackground(e);
/*  654 */     if (r != null) {
/*  655 */       result.setBackgroundEnable(r);
/*      */     }
/*      */     
/*  658 */     SVGSVGElement svgElement = imgDocument.getRootElement();
/*      */     
/*  660 */     CanvasGraphicsNode node = (CanvasGraphicsNode)this.subCtx.getGVTBuilder().build(this.subCtx, svgElement);
/*      */ 
/*      */     
/*  663 */     if (eng == null && ctx.isInteractive())
/*      */     {
/*  665 */       this.subCtx.addUIEventListeners(imgDocument);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  671 */     node.setClip(null);
/*      */ 
/*      */ 
/*      */     
/*  675 */     node.setViewingTransform(new AffineTransform());
/*  676 */     result.getChildren().add(node);
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  681 */     String viewBox = svgElement.getAttributeNS(null, "viewBox");
/*      */     
/*  683 */     float[] vb = ViewBox.parseViewBoxAttribute(e, viewBox, ctx);
/*      */     
/*  685 */     initializeViewport(ctx, e, result, vb, bounds);
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  690 */     if (ctx.isInteractive()) {
/*  691 */       this.listener = new ForwardEventListener(svgElement, e);
/*  692 */       NodeEventTarget tgt = (NodeEventTarget)svgElement;
/*      */       
/*  694 */       tgt.addEventListenerNS("http://www.w3.org/2001/xml-events", "click", this.listener, false, null);
/*      */ 
/*      */       
/*  697 */       this.subCtx.storeEventListenerNS(tgt, "http://www.w3.org/2001/xml-events", "click", this.listener, false);
/*      */ 
/*      */ 
/*      */       
/*  701 */       tgt.addEventListenerNS("http://www.w3.org/2001/xml-events", "keydown", this.listener, false, null);
/*      */ 
/*      */       
/*  704 */       this.subCtx.storeEventListenerNS(tgt, "http://www.w3.org/2001/xml-events", "keydown", this.listener, false);
/*      */ 
/*      */ 
/*      */       
/*  708 */       tgt.addEventListenerNS("http://www.w3.org/2001/xml-events", "keypress", this.listener, false, null);
/*      */ 
/*      */       
/*  711 */       this.subCtx.storeEventListenerNS(tgt, "http://www.w3.org/2001/xml-events", "keypress", this.listener, false);
/*      */ 
/*      */ 
/*      */       
/*  715 */       tgt.addEventListenerNS("http://www.w3.org/2001/xml-events", "keyup", this.listener, false, null);
/*      */ 
/*      */       
/*  718 */       this.subCtx.storeEventListenerNS(tgt, "http://www.w3.org/2001/xml-events", "keyup", this.listener, false);
/*      */ 
/*      */ 
/*      */       
/*  722 */       tgt.addEventListenerNS("http://www.w3.org/2001/xml-events", "mousedown", this.listener, false, null);
/*      */ 
/*      */       
/*  725 */       this.subCtx.storeEventListenerNS(tgt, "http://www.w3.org/2001/xml-events", "mousedown", this.listener, false);
/*      */ 
/*      */ 
/*      */       
/*  729 */       tgt.addEventListenerNS("http://www.w3.org/2001/xml-events", "mousemove", this.listener, false, null);
/*      */ 
/*      */       
/*  732 */       this.subCtx.storeEventListenerNS(tgt, "http://www.w3.org/2001/xml-events", "mousemove", this.listener, false);
/*      */ 
/*      */ 
/*      */       
/*  736 */       tgt.addEventListenerNS("http://www.w3.org/2001/xml-events", "mouseout", this.listener, false, null);
/*      */ 
/*      */       
/*  739 */       this.subCtx.storeEventListenerNS(tgt, "http://www.w3.org/2001/xml-events", "mouseout", this.listener, false);
/*      */ 
/*      */ 
/*      */       
/*  743 */       tgt.addEventListenerNS("http://www.w3.org/2001/xml-events", "mouseover", this.listener, false, null);
/*      */ 
/*      */       
/*  746 */       this.subCtx.storeEventListenerNS(tgt, "http://www.w3.org/2001/xml-events", "mouseover", this.listener, false);
/*      */ 
/*      */ 
/*      */       
/*  750 */       tgt.addEventListenerNS("http://www.w3.org/2001/xml-events", "mouseup", this.listener, false, null);
/*      */ 
/*      */       
/*  753 */       this.subCtx.storeEventListenerNS(tgt, "http://www.w3.org/2001/xml-events", "mouseup", this.listener, false);
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/*  758 */     return result;
/*      */   }
/*      */   
/*      */   public void dispose() {
/*  762 */     if (this.imgDocument != null && this.listener != null) {
/*  763 */       NodeEventTarget tgt = (NodeEventTarget)this.imgDocument.getRootElement();
/*      */       
/*  765 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "click", this.listener, false);
/*      */ 
/*      */       
/*  768 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "keydown", this.listener, false);
/*      */ 
/*      */       
/*  771 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "keypress", this.listener, false);
/*      */ 
/*      */       
/*  774 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "keyup", this.listener, false);
/*      */ 
/*      */       
/*  777 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mousedown", this.listener, false);
/*      */ 
/*      */       
/*  780 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mousemove", this.listener, false);
/*      */ 
/*      */       
/*  783 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mouseout", this.listener, false);
/*      */ 
/*      */       
/*  786 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mouseover", this.listener, false);
/*      */ 
/*      */       
/*  789 */       tgt.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mouseup", this.listener, false);
/*      */ 
/*      */       
/*  792 */       this.listener = null;
/*      */     } 
/*      */     
/*  795 */     if (this.imgDocument != null) {
/*  796 */       SVGSVGElement svgElement = this.imgDocument.getRootElement();
/*  797 */       disposeTree(svgElement);
/*  798 */       this.imgDocument = null;
/*  799 */       this.subCtx = null;
/*      */     } 
/*  801 */     super.dispose();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected static class ForwardEventListener
/*      */     implements EventListener
/*      */   {
/*      */     protected Element svgElement;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected Element imgElement;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public ForwardEventListener(Element svgElement, Element imgElement) {
/*  824 */       this.svgElement = svgElement;
/*  825 */       this.imgElement = imgElement;
/*      */     }
/*      */     
/*      */     public void handleEvent(Event e) {
/*  829 */       DOMMouseEvent evt = (DOMMouseEvent)e;
/*  830 */       DOMMouseEvent newMouseEvent = (DOMMouseEvent)((DocumentEvent)this.imgElement.getOwnerDocument()).createEvent("MouseEvents");
/*      */ 
/*      */ 
/*      */       
/*  834 */       newMouseEvent.initMouseEventNS("http://www.w3.org/2001/xml-events", evt.getType(), evt.getBubbles(), evt.getCancelable(), evt.getView(), evt.getDetail(), evt.getScreenX(), evt.getScreenY(), evt.getClientX(), evt.getClientY(), evt.getButton(), (EventTarget)this.imgElement, evt.getModifiersString());
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  848 */       ((EventTarget)this.imgElement).dispatchEvent(newMouseEvent);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected static void initializeViewport(BridgeContext ctx, Element e, GraphicsNode node, float[] vb, Rectangle2D bounds) {
/*  870 */     float x = (float)bounds.getX();
/*  871 */     float y = (float)bounds.getY();
/*  872 */     float w = (float)bounds.getWidth();
/*  873 */     float h = (float)bounds.getHeight();
/*      */     
/*      */     try {
/*  876 */       SVGImageElement ie = (SVGImageElement)e;
/*  877 */       SVGOMAnimatedPreserveAspectRatio _par = (SVGOMAnimatedPreserveAspectRatio)ie.getPreserveAspectRatio();
/*      */       
/*  879 */       _par.check();
/*      */       
/*  881 */       AffineTransform at = ViewBox.getPreserveAspectRatioTransform(e, vb, w, h, _par, ctx);
/*      */       
/*  883 */       at.preConcatenate(AffineTransform.getTranslateInstance(x, y));
/*  884 */       node.setTransform(at);
/*      */ 
/*      */       
/*  887 */       Shape clip = null;
/*  888 */       if (CSSUtilities.convertOverflow(e)) {
/*  889 */         float[] offsets = CSSUtilities.convertClip(e);
/*  890 */         if (offsets == null) {
/*  891 */           clip = new Rectangle2D.Float(x, y, w, h);
/*      */         
/*      */         }
/*      */         else {
/*      */ 
/*      */           
/*  897 */           clip = new Rectangle2D.Float(x + offsets[3], y + offsets[0], w - offsets[1] - offsets[3], h - offsets[2] - offsets[0]);
/*      */         } 
/*      */       } 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  904 */       if (clip != null) {
/*      */         try {
/*  906 */           at = at.createInverse();
/*  907 */           Filter filter = node.getGraphicsNodeRable(true);
/*  908 */           clip = at.createTransformedShape(clip);
/*  909 */           node.setClip(new ClipRable8Bit(filter, clip));
/*  910 */         } catch (NoninvertibleTransformException ex) {}
/*      */       }
/*      */     } catch (LiveAttributeException ex) {
/*  913 */       throw new BridgeException(ctx, ex);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected static ICCColorSpaceExt extractColorSpace(Element element, BridgeContext ctx) {
/*  927 */     String colorProfileProperty = CSSUtilities.getComputedStyle(element, 8).getStringValue();
/*      */ 
/*      */ 
/*      */     
/*  931 */     ICCColorSpaceExt colorSpace = null;
/*  932 */     if ("srgb".equalsIgnoreCase(colorProfileProperty)) {
/*      */       
/*  934 */       colorSpace = new ICCColorSpaceExt(ICC_Profile.getInstance(1000), 4);
/*      */ 
/*      */     
/*      */     }
/*  938 */     else if (!"auto".equalsIgnoreCase(colorProfileProperty) && !"".equalsIgnoreCase(colorProfileProperty)) {
/*      */ 
/*      */ 
/*      */       
/*  942 */       SVGColorProfileElementBridge profileBridge = (SVGColorProfileElementBridge)ctx.getBridge("http://www.w3.org/2000/svg", "color-profile");
/*      */ 
/*      */       
/*  945 */       if (profileBridge != null) {
/*  946 */         colorSpace = profileBridge.createICCColorSpaceExt(ctx, element, colorProfileProperty);
/*      */       }
/*      */     } 
/*      */ 
/*      */     
/*  951 */     return colorSpace;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected static Rectangle2D getImageBounds(BridgeContext ctx, Element element) {
/*      */     try {
/*  963 */       SVGImageElement ie = (SVGImageElement)element;
/*      */ 
/*      */       
/*  966 */       AbstractSVGAnimatedLength _x = (AbstractSVGAnimatedLength)ie.getX();
/*      */       
/*  968 */       float x = _x.getCheckedValue();
/*      */ 
/*      */       
/*  971 */       AbstractSVGAnimatedLength _y = (AbstractSVGAnimatedLength)ie.getY();
/*      */       
/*  973 */       float y = _y.getCheckedValue();
/*      */ 
/*      */       
/*  976 */       AbstractSVGAnimatedLength _width = (AbstractSVGAnimatedLength)ie.getWidth();
/*      */       
/*  978 */       float w = _width.getCheckedValue();
/*      */ 
/*      */       
/*  981 */       AbstractSVGAnimatedLength _height = (AbstractSVGAnimatedLength)ie.getHeight();
/*      */       
/*  983 */       float h = _height.getCheckedValue();
/*      */       
/*  985 */       return new Rectangle2D.Float(x, y, w, h);
/*      */     } catch (LiveAttributeException ex) {
/*  987 */       throw new BridgeException(ctx, ex);
/*      */     } 
/*      */   }
/*      */ 
/*      */   
/*      */   GraphicsNode createBrokenImageNode(BridgeContext ctx, Element e, String uri, String message) {
/*  993 */     SVGDocument doc = ctx.getUserAgent().getBrokenLinkDocument(e, uri, Messages.formatMessage("uri.image.error", new Object[] { message }));
/*      */ 
/*      */     
/*  996 */     return createSVGImageNode(ctx, e, doc);
/*      */   }
/*      */ 
/*      */   
/* 1000 */   static SVGBrokenLinkProvider brokenLinkProvider = new SVGBrokenLinkProvider();
/*      */   
/*      */   static  {
/* 1003 */     ImageTagRegistry.setBrokenLinkProvider(brokenLinkProvider);
/*      */   }
/*      */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\SVGImageElementBridge.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */