/*     */ package org.apache.batik.bridge;
/*     */ 
/*     */ import java.awt.Dimension;
/*     */ import java.awt.RenderingHints;
/*     */ import java.awt.Shape;
/*     */ import java.awt.geom.AffineTransform;
/*     */ import java.awt.geom.NoninvertibleTransformException;
/*     */ import java.awt.geom.Rectangle2D;
/*     */ import java.util.ArrayList;
/*     */ import java.util.HashSet;
/*     */ import java.util.List;
/*     */ import java.util.Set;
/*     */ import org.apache.batik.dom.svg.AbstractSVGAnimatedLength;
/*     */ import org.apache.batik.dom.svg.AnimatedLiveAttributeValue;
/*     */ import org.apache.batik.dom.svg.LiveAttributeException;
/*     */ import org.apache.batik.dom.svg.SVGContext;
/*     */ import org.apache.batik.dom.svg.SVGOMAnimatedRect;
/*     */ import org.apache.batik.dom.svg.SVGOMElement;
/*     */ import org.apache.batik.dom.svg.SVGOMSVGElement;
/*     */ import org.apache.batik.dom.svg.SVGSVGContext;
/*     */ import org.apache.batik.ext.awt.image.renderable.ClipRable8Bit;
/*     */ import org.apache.batik.ext.awt.image.renderable.Filter;
/*     */ import org.apache.batik.gvt.CanvasGraphicsNode;
/*     */ import org.apache.batik.gvt.CompositeGraphicsNode;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.apache.batik.gvt.ShapeNode;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.Node;
/*     */ import org.w3c.dom.svg.SVGAnimatedPreserveAspectRatio;
/*     */ import org.w3c.dom.svg.SVGDocument;
/*     */ import org.w3c.dom.svg.SVGRect;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SVGSVGElementBridge
/*     */   extends SVGGElementBridge
/*     */   implements SVGSVGContext
/*     */ {
/*  73 */   public String getLocalName() { return "svg"; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  80 */   public Bridge getInstance() { return new SVGSVGElementBridge(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  87 */   protected GraphicsNode instantiateGraphicsNode() { return new CanvasGraphicsNode(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public GraphicsNode createGraphicsNode(BridgeContext ctx, Element e) {
/*  99 */     if (!SVGUtilities.matchUserAgent(e, ctx.getUserAgent())) {
/* 100 */       return null;
/*     */     }
/*     */ 
/*     */     
/* 104 */     CanvasGraphicsNode cgn = (CanvasGraphicsNode)instantiateGraphicsNode();
/*     */     
/* 106 */     associateSVGContext(ctx, e, cgn);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     try {
/* 113 */       SVGDocument doc = (SVGDocument)e.getOwnerDocument();
/* 114 */       SVGOMSVGElement se = (SVGOMSVGElement)e;
/* 115 */       boolean isOutermost = (doc.getRootElement() == e);
/* 116 */       float x = 0.0F;
/* 117 */       float y = 0.0F;
/*     */       
/* 119 */       if (!isOutermost) {
/*     */         
/* 121 */         AbstractSVGAnimatedLength _x = (AbstractSVGAnimatedLength)se.getX();
/*     */         
/* 123 */         x = _x.getCheckedValue();
/*     */ 
/*     */         
/* 126 */         AbstractSVGAnimatedLength _y = (AbstractSVGAnimatedLength)se.getY();
/*     */         
/* 128 */         y = _y.getCheckedValue();
/*     */       } 
/*     */ 
/*     */       
/* 132 */       AbstractSVGAnimatedLength _width = (AbstractSVGAnimatedLength)se.getWidth();
/*     */       
/* 134 */       float w = _width.getCheckedValue();
/*     */ 
/*     */       
/* 137 */       AbstractSVGAnimatedLength _height = (AbstractSVGAnimatedLength)se.getHeight();
/*     */       
/* 139 */       float h = _height.getCheckedValue();
/*     */ 
/*     */       
/* 142 */       cgn.setVisible(CSSUtilities.convertVisibility(e));
/*     */ 
/*     */       
/* 145 */       SVGOMAnimatedRect vb = (SVGOMAnimatedRect)se.getViewBox();
/* 146 */       SVGAnimatedPreserveAspectRatio par = se.getPreserveAspectRatio();
/* 147 */       AffineTransform viewingTransform = ViewBox.getPreserveAspectRatioTransform(e, vb, par, w, h, ctx);
/*     */ 
/*     */       
/* 150 */       float actualWidth = w;
/* 151 */       float actualHeight = h;
/*     */       try {
/* 153 */         AffineTransform vtInv = viewingTransform.createInverse();
/* 154 */         actualWidth = (float)(w * vtInv.getScaleX());
/* 155 */         actualHeight = (float)(h * vtInv.getScaleY());
/* 156 */       } catch (NoninvertibleTransformException ex) {}
/*     */       
/* 158 */       AffineTransform positionTransform = AffineTransform.getTranslateInstance(x, y);
/*     */ 
/*     */ 
/*     */       
/* 162 */       if (!isOutermost) {
/*     */         
/* 164 */         cgn.setPositionTransform(positionTransform);
/* 165 */       } else if (doc == ctx.getDocument()) {
/*     */         
/* 167 */         ctx.setDocumentSize(new Dimension((int)(w + 0.5F), (int)(h + 0.5F)));
/*     */       } 
/*     */ 
/*     */ 
/*     */       
/* 172 */       cgn.setViewingTransform(viewingTransform);
/*     */ 
/*     */       
/* 175 */       Shape clip = null;
/* 176 */       if (CSSUtilities.convertOverflow(e)) {
/* 177 */         float[] offsets = CSSUtilities.convertClip(e);
/* 178 */         if (offsets == null) {
/* 179 */           clip = new Rectangle2D.Float(x, y, w, h);
/*     */         
/*     */         }
/*     */         else {
/*     */ 
/*     */           
/* 185 */           clip = new Rectangle2D.Float(x + offsets[3], y + offsets[0], w - offsets[1] - offsets[3], h - offsets[2] - offsets[0]);
/*     */         } 
/*     */       } 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 192 */       if (clip != null) {
/*     */         try {
/* 194 */           AffineTransform at = new AffineTransform(positionTransform);
/* 195 */           at.concatenate(viewingTransform);
/* 196 */           at = at.createInverse();
/* 197 */           clip = at.createTransformedShape(clip);
/* 198 */           Filter filter = cgn.getGraphicsNodeRable(true);
/* 199 */           cgn.setClip(new ClipRable8Bit(filter, clip));
/* 200 */         } catch (NoninvertibleTransformException ex) {}
/*     */       }
/* 202 */       RenderingHints hints = null;
/* 203 */       hints = CSSUtilities.convertColorRendering(e, hints);
/* 204 */       if (hints != null) {
/* 205 */         cgn.setRenderingHints(hints);
/*     */       }
/*     */       
/* 208 */       Rectangle2D r = CSSUtilities.convertEnableBackground(e);
/* 209 */       if (r != null) {
/* 210 */         cgn.setBackgroundEnable(r);
/*     */       }
/*     */       
/* 213 */       if (vb.isSpecified()) {
/* 214 */         SVGRect vbr = vb.getAnimVal();
/* 215 */         actualWidth = vbr.getWidth();
/* 216 */         actualHeight = vbr.getHeight();
/*     */       } 
/* 218 */       ctx.openViewport(e, new SVGSVGElementViewport(actualWidth, actualHeight));
/*     */ 
/*     */       
/* 221 */       return cgn;
/*     */     } catch (LiveAttributeException ex) {
/* 223 */       throw new BridgeException(ctx, ex);
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void buildGraphicsNode(BridgeContext ctx, Element e, GraphicsNode node) {
/* 240 */     node.setComposite(CSSUtilities.convertOpacity(e));
/*     */     
/* 242 */     node.setFilter(CSSUtilities.convertFilter(e, node, ctx));
/*     */     
/* 244 */     node.setMask(CSSUtilities.convertMask(e, node, ctx));
/*     */     
/* 246 */     node.setPointerEventType(CSSUtilities.convertPointerEvents(e));
/*     */     
/* 248 */     initializeDynamicSupport(ctx, e, node);
/*     */     
/* 250 */     ctx.closeViewport(e);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void dispose() {
/* 259 */     this.ctx.removeViewport(this.e);
/* 260 */     super.dispose();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void handleAnimatedAttributeChanged(AnimatedLiveAttributeValue alav) {
/*     */     try {
/* 269 */       boolean rebuild = false;
/* 270 */       if (alav.getNamespaceURI() == null) {
/* 271 */         String ln = alav.getLocalName();
/* 272 */         if (ln.equals("width") || ln.equals("height")) {
/*     */           
/* 274 */           rebuild = true;
/* 275 */         } else if (ln.equals("x") || ln.equals("y")) {
/*     */           
/* 277 */           SVGDocument doc = (SVGDocument)this.e.getOwnerDocument();
/* 278 */           SVGOMSVGElement se = (SVGOMSVGElement)this.e;
/*     */           
/* 280 */           boolean isOutermost = (doc.getRootElement() == this.e);
/* 281 */           if (!isOutermost) {
/*     */             
/* 283 */             AbstractSVGAnimatedLength _x = (AbstractSVGAnimatedLength)se.getX();
/*     */             
/* 285 */             float x = _x.getCheckedValue();
/*     */ 
/*     */             
/* 288 */             AbstractSVGAnimatedLength _y = (AbstractSVGAnimatedLength)se.getY();
/*     */             
/* 290 */             float y = _y.getCheckedValue();
/*     */             
/* 292 */             AffineTransform positionTransform = AffineTransform.getTranslateInstance(x, y);
/*     */ 
/*     */             
/* 295 */             CanvasGraphicsNode cgn = (CanvasGraphicsNode)this.node;
/*     */             
/* 297 */             cgn.setPositionTransform(positionTransform);
/*     */             return;
/*     */           } 
/* 300 */         } else if (ln.equals("viewBox") || ln.equals("preserveAspectRatio")) {
/*     */           
/* 302 */           SVGDocument doc = (SVGDocument)this.e.getOwnerDocument();
/* 303 */           SVGOMSVGElement se = (SVGOMSVGElement)this.e;
/* 304 */           boolean isOutermost = (doc.getRootElement() == this.e);
/*     */ 
/*     */           
/* 307 */           float x = 0.0F;
/* 308 */           float y = 0.0F;
/* 309 */           if (!isOutermost) {
/*     */             
/* 311 */             AbstractSVGAnimatedLength _x = (AbstractSVGAnimatedLength)se.getX();
/*     */             
/* 313 */             x = _x.getCheckedValue();
/*     */ 
/*     */             
/* 316 */             AbstractSVGAnimatedLength _y = (AbstractSVGAnimatedLength)se.getY();
/*     */             
/* 318 */             y = _y.getCheckedValue();
/*     */           } 
/*     */ 
/*     */           
/* 322 */           AbstractSVGAnimatedLength _width = (AbstractSVGAnimatedLength)se.getWidth();
/*     */           
/* 324 */           float w = _width.getCheckedValue();
/*     */ 
/*     */           
/* 327 */           AbstractSVGAnimatedLength _height = (AbstractSVGAnimatedLength)se.getHeight();
/*     */           
/* 329 */           float h = _height.getCheckedValue();
/*     */ 
/*     */           
/* 332 */           CanvasGraphicsNode cgn = (CanvasGraphicsNode)this.node;
/*     */ 
/*     */           
/* 335 */           SVGOMAnimatedRect vb = (SVGOMAnimatedRect)se.getViewBox();
/* 336 */           SVGAnimatedPreserveAspectRatio par = se.getPreserveAspectRatio();
/* 337 */           AffineTransform newVT = ViewBox.getPreserveAspectRatioTransform(this.e, vb, par, w, h, this.ctx);
/*     */ 
/*     */           
/* 340 */           AffineTransform oldVT = cgn.getViewingTransform();
/* 341 */           if (newVT.getScaleX() != oldVT.getScaleX() || newVT.getScaleY() != oldVT.getScaleY() || newVT.getShearX() != oldVT.getShearX() || newVT.getShearY() != oldVT.getShearY()) {
/*     */ 
/*     */ 
/*     */             
/* 345 */             rebuild = true;
/*     */           } else {
/*     */             
/* 348 */             cgn.setViewingTransform(newVT);
/*     */ 
/*     */             
/* 351 */             Shape clip = null;
/* 352 */             if (CSSUtilities.convertOverflow(this.e)) {
/* 353 */               float[] offsets = CSSUtilities.convertClip(this.e);
/* 354 */               if (offsets == null) {
/* 355 */                 clip = new Rectangle2D.Float(x, y, w, h);
/*     */               
/*     */               }
/*     */               else {
/*     */ 
/*     */                 
/* 361 */                 clip = new Rectangle2D.Float(x + offsets[3], y + offsets[0], w - offsets[1] - offsets[3], h - offsets[2] - offsets[0]);
/*     */               } 
/*     */             } 
/*     */ 
/*     */ 
/*     */ 
/*     */             
/* 368 */             if (clip != null) {
/*     */               
/*     */               try {
/* 371 */                 AffineTransform at = cgn.getPositionTransform();
/* 372 */                 if (at == null) { at = new AffineTransform(); }
/* 373 */                 else { at = new AffineTransform(at); }
/* 374 */                  at.concatenate(newVT);
/* 375 */                 at = at.createInverse();
/* 376 */                 clip = at.createTransformedShape(clip);
/* 377 */                 Filter filter = cgn.getGraphicsNodeRable(true);
/* 378 */                 cgn.setClip(new ClipRable8Bit(filter, clip));
/* 379 */               } catch (NoninvertibleTransformException ex) {}
/*     */             }
/*     */           } 
/*     */         } 
/*     */         
/* 384 */         if (rebuild) {
/* 385 */           CompositeGraphicsNode gn = this.node.getParent();
/* 386 */           gn.remove(this.node);
/* 387 */           disposeTree(this.e, false);
/*     */           
/* 389 */           handleElementAdded(gn, this.e.getParentNode(), this.e);
/*     */           return;
/*     */         } 
/*     */       } 
/*     */     } catch (LiveAttributeException ex) {
/* 394 */       throw new BridgeException(this.ctx, ex);
/*     */     } 
/* 396 */     super.handleAnimatedAttributeChanged(alav);
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   public static class SVGSVGElementViewport
/*     */     implements Viewport
/*     */   {
/*     */     private float width;
/*     */ 
/*     */     
/*     */     private float height;
/*     */ 
/*     */ 
/*     */     
/*     */     public SVGSVGElementViewport(float w, float h) {
/* 412 */       this.width = w;
/* 413 */       this.height = h;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 420 */     public float getWidth() { return this.width; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 427 */     public float getHeight() { return this.height; }
/*     */   }
/*     */ 
/*     */   
/*     */   public List getIntersectionList(SVGRect svgRect, Element end) {
/* 432 */     List ret = new ArrayList();
/* 433 */     Rectangle2D rect = new Rectangle2D.Float(svgRect.getX(), svgRect.getY(), svgRect.getWidth(), svgRect.getHeight());
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 438 */     GraphicsNode svgGN = this.ctx.getGraphicsNode(this.e);
/* 439 */     if (svgGN == null) return ret;
/*     */     
/* 441 */     Rectangle2D svgBounds = svgGN.getSensitiveBounds();
/* 442 */     if (svgBounds == null) {
/* 443 */       return ret;
/*     */     }
/*     */ 
/*     */     
/* 447 */     if (!rect.intersects(svgBounds)) {
/* 448 */       return ret;
/*     */     }
/* 450 */     Element base = this.e;
/* 451 */     AffineTransform ati = svgGN.getGlobalTransform();
/*     */     try {
/* 453 */       ati = ati.createInverse();
/* 454 */     } catch (NoninvertibleTransformException e) {}
/*     */ 
/*     */ 
/*     */     
/* 458 */     Node next = base.getFirstChild();
/* 459 */     while (next != null && 
/* 460 */       !(next instanceof Element))
/*     */     {
/* 462 */       next = next.getNextSibling();
/*     */     }
/* 464 */     if (next == null) return ret; 
/* 465 */     Element curr = (Element)next;
/*     */     
/* 467 */     Set ancestors = null;
/* 468 */     if (end != null) {
/* 469 */       ancestors = getAncestors(end, base);
/* 470 */       if (ancestors == null)
/* 471 */         end = null; 
/*     */     } 
/* 473 */     while (curr != null) {
/* 474 */       String nsURI = curr.getNamespaceURI();
/* 475 */       String tag = curr.getLocalName();
/*     */       
/* 477 */       boolean isGroup = ("http://www.w3.org/2000/svg".equals(nsURI) && ("g".equals(tag) || "svg".equals(tag) || "a".equals(tag)));
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 482 */       GraphicsNode gn = this.ctx.getGraphicsNode(curr);
/* 483 */       if (gn == null) {
/*     */ 
/*     */         
/* 486 */         if (ancestors != null && ancestors.contains(curr))
/*     */           break; 
/* 488 */         curr = getNext(curr, base, end);
/*     */         
/*     */         continue;
/*     */       } 
/*     */       
/* 493 */       AffineTransform at = gn.getGlobalTransform();
/* 494 */       Rectangle2D gnBounds = gn.getSensitiveBounds();
/* 495 */       at.preConcatenate(ati);
/* 496 */       if (gnBounds != null) {
/* 497 */         gnBounds = at.createTransformedShape(gnBounds).getBounds2D();
/*     */       }
/* 499 */       if (gnBounds == null || !rect.intersects(gnBounds)) {
/*     */ 
/*     */ 
/*     */         
/* 503 */         if (ancestors != null && ancestors.contains(curr))
/*     */           break; 
/* 505 */         curr = getNext(curr, base, end);
/*     */ 
/*     */         
/*     */         continue;
/*     */       } 
/*     */ 
/*     */       
/* 512 */       if (isGroup) {
/*     */         
/* 514 */         next = curr.getFirstChild();
/* 515 */         while (next != null && 
/* 516 */           !(next instanceof Element))
/*     */         {
/* 518 */           next = next.getNextSibling();
/*     */         }
/* 520 */         if (next != null) {
/* 521 */           curr = (Element)next;
/*     */           continue;
/*     */         } 
/*     */       } else {
/* 525 */         if (curr == end) {
/*     */           break;
/*     */         }
/* 528 */         if ("http://www.w3.org/2000/svg".equals(nsURI) && "use".equals(tag))
/*     */         {
/*     */ 
/*     */           
/* 532 */           if (rect.contains(gnBounds))
/* 533 */             ret.add(curr);  } 
/* 534 */         if (gn instanceof ShapeNode) {
/* 535 */           ShapeNode sn = (ShapeNode)gn;
/* 536 */           Shape sensitive = sn.getSensitiveArea();
/* 537 */           if (sensitive != null) {
/* 538 */             sensitive = at.createTransformedShape(sensitive);
/* 539 */             if (sensitive.intersects(rect))
/* 540 */               ret.add(curr); 
/*     */           } 
/* 542 */         } else if (gn instanceof org.apache.batik.gvt.TextNode) {
/* 543 */           SVGOMElement svgElem = (SVGOMElement)curr;
/*     */           
/* 545 */           SVGTextElementBridge txtBridge = (SVGTextElementBridge)svgElem.getSVGContext();
/* 546 */           Set elems = txtBridge.getTextIntersectionSet(at, rect);
/*     */ 
/*     */ 
/*     */           
/* 550 */           if (ancestors != null && ancestors.contains(curr)) {
/* 551 */             filterChildren(curr, end, elems, ret);
/*     */           } else {
/* 553 */             ret.addAll(elems);
/*     */           } 
/*     */         } else {
/* 556 */           ret.add(curr);
/*     */         } 
/*     */       } 
/*     */       
/* 560 */       curr = getNext(curr, base, end);
/*     */     } 
/*     */     
/* 563 */     return ret;
/*     */   }
/*     */   
/*     */   public List getEnclosureList(SVGRect svgRect, Element end) {
/* 567 */     List ret = new ArrayList();
/* 568 */     Rectangle2D rect = new Rectangle2D.Float(svgRect.getX(), svgRect.getY(), svgRect.getWidth(), svgRect.getHeight());
/*     */ 
/*     */ 
/*     */     
/* 572 */     GraphicsNode svgGN = this.ctx.getGraphicsNode(this.e);
/* 573 */     if (svgGN == null) return ret;
/*     */     
/* 575 */     Rectangle2D svgBounds = svgGN.getSensitiveBounds();
/* 576 */     if (svgBounds == null) {
/* 577 */       return ret;
/*     */     }
/*     */ 
/*     */     
/* 581 */     if (!rect.intersects(svgBounds)) {
/* 582 */       return ret;
/*     */     }
/* 584 */     Element base = this.e;
/* 585 */     AffineTransform ati = svgGN.getGlobalTransform();
/*     */     try {
/* 587 */       ati = ati.createInverse();
/* 588 */     } catch (NoninvertibleTransformException e) {}
/*     */ 
/*     */ 
/*     */     
/* 592 */     Node next = base.getFirstChild();
/* 593 */     while (next != null && 
/* 594 */       !(next instanceof Element))
/*     */     {
/* 596 */       next = next.getNextSibling();
/*     */     }
/*     */     
/* 599 */     if (next == null) return ret; 
/* 600 */     Element curr = (Element)next;
/*     */     
/* 602 */     Set ancestors = null;
/* 603 */     if (end != null) {
/* 604 */       ancestors = getAncestors(end, base);
/* 605 */       if (ancestors == null) {
/* 606 */         end = null;
/*     */       }
/*     */     } 
/* 609 */     while (curr != null) {
/* 610 */       String nsURI = curr.getNamespaceURI();
/* 611 */       String tag = curr.getLocalName();
/*     */       
/* 613 */       boolean isGroup = ("http://www.w3.org/2000/svg".equals(nsURI) && ("g".equals(tag) || "svg".equals(tag) || "a".equals(tag)));
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 618 */       GraphicsNode gn = this.ctx.getGraphicsNode(curr);
/* 619 */       if (gn == null) {
/*     */ 
/*     */         
/* 622 */         if (ancestors != null && ancestors.contains(curr))
/*     */           break; 
/* 624 */         curr = getNext(curr, base, end);
/*     */         
/*     */         continue;
/*     */       } 
/*     */       
/* 629 */       AffineTransform at = gn.getGlobalTransform();
/* 630 */       Rectangle2D gnBounds = gn.getSensitiveBounds();
/* 631 */       at.preConcatenate(ati);
/* 632 */       if (gnBounds != null) {
/* 633 */         gnBounds = at.createTransformedShape(gnBounds).getBounds2D();
/*     */       }
/* 635 */       if (gnBounds == null || !rect.intersects(gnBounds)) {
/*     */ 
/*     */ 
/*     */         
/* 639 */         if (ancestors != null && ancestors.contains(curr))
/*     */           break; 
/* 641 */         curr = getNext(curr, base, end);
/*     */ 
/*     */         
/*     */         continue;
/*     */       } 
/*     */       
/* 647 */       if (isGroup) {
/*     */         
/* 649 */         next = curr.getFirstChild();
/* 650 */         while (next != null && 
/* 651 */           !(next instanceof Element))
/*     */         {
/* 653 */           next = next.getNextSibling();
/*     */         }
/* 655 */         if (next != null) {
/* 656 */           curr = (Element)next;
/*     */           continue;
/*     */         } 
/*     */       } else {
/* 660 */         if (curr == end)
/* 661 */           break;  if ("http://www.w3.org/2000/svg".equals(nsURI) && "use".equals(tag)) {
/*     */ 
/*     */ 
/*     */           
/* 665 */           if (rect.contains(gnBounds))
/* 666 */             ret.add(curr); 
/* 667 */         } else if (gn instanceof org.apache.batik.gvt.TextNode) {
/*     */ 
/*     */           
/* 670 */           SVGOMElement svgElem = (SVGOMElement)curr;
/*     */           
/* 672 */           SVGTextElementBridge txtBridge = (SVGTextElementBridge)svgElem.getSVGContext();
/* 673 */           Set elems = txtBridge.getTextEnclosureSet(at, rect);
/*     */ 
/*     */ 
/*     */           
/* 677 */           if (ancestors != null && ancestors.contains(curr))
/* 678 */           { filterChildren(curr, end, elems, ret); }
/*     */           else
/* 680 */           { ret.addAll(elems); } 
/* 681 */         } else if (rect.contains(gnBounds)) {
/*     */           
/* 683 */           ret.add(curr);
/*     */         } 
/*     */       } 
/*     */       
/* 687 */       curr = getNext(curr, base, end);
/*     */     } 
/* 689 */     return ret;
/*     */   }
/*     */ 
/*     */   
/*     */   public boolean checkIntersection(Element element, SVGRect svgRect) {
/* 694 */     GraphicsNode svgGN = this.ctx.getGraphicsNode(this.e);
/* 695 */     if (svgGN == null) return false;
/*     */     
/* 697 */     Rectangle2D rect = new Rectangle2D.Float(svgRect.getX(), svgRect.getY(), svgRect.getWidth(), svgRect.getHeight());
/*     */ 
/*     */     
/* 700 */     AffineTransform ati = svgGN.getGlobalTransform();
/*     */     
/*     */     try {
/* 703 */       ati = ati.createInverse();
/* 704 */     } catch (NoninvertibleTransformException e) {}
/*     */     
/* 706 */     SVGContext svgctx = null;
/* 707 */     if (element instanceof SVGOMElement) {
/* 708 */       svgctx = ((SVGOMElement)element).getSVGContext();
/* 709 */       if (svgctx instanceof SVGTextElementBridge || svgctx instanceof SVGTextElementBridge.AbstractTextChildSVGContext)
/*     */       {
/*     */         
/* 712 */         return SVGTextElementBridge.getTextIntersection(this.ctx, element, ati, rect, true);
/*     */       }
/*     */     } 
/*     */ 
/*     */     
/* 717 */     Rectangle2D gnBounds = null;
/* 718 */     GraphicsNode gn = this.ctx.getGraphicsNode(element);
/* 719 */     if (gn != null) {
/* 720 */       gnBounds = gn.getSensitiveBounds();
/*     */     }
/* 722 */     if (gnBounds == null) return false;
/*     */ 
/*     */     
/* 725 */     AffineTransform at = gn.getGlobalTransform();
/* 726 */     at.preConcatenate(ati);
/*     */     
/* 728 */     gnBounds = at.createTransformedShape(gnBounds).getBounds2D();
/* 729 */     if (!rect.intersects(gnBounds)) {
/* 730 */       return false;
/*     */     }
/*     */     
/* 733 */     if (!(gn instanceof ShapeNode)) {
/* 734 */       return true;
/*     */     }
/* 736 */     ShapeNode sn = (ShapeNode)gn;
/* 737 */     Shape sensitive = sn.getSensitiveArea();
/* 738 */     if (sensitive == null) return false;
/*     */     
/* 740 */     sensitive = at.createTransformedShape(sensitive);
/* 741 */     if (sensitive.intersects(rect)) {
/* 742 */       return true;
/*     */     }
/* 744 */     return false;
/*     */   }
/*     */   
/*     */   public boolean checkEnclosure(Element element, SVGRect svgRect) {
/* 748 */     GraphicsNode gn = this.ctx.getGraphicsNode(element);
/* 749 */     Rectangle2D gnBounds = null;
/* 750 */     SVGContext svgctx = null;
/* 751 */     if (element instanceof SVGOMElement) {
/* 752 */       svgctx = ((SVGOMElement)element).getSVGContext();
/* 753 */       if (svgctx instanceof SVGTextElementBridge || svgctx instanceof SVGTextElementBridge.AbstractTextChildSVGContext)
/*     */       
/*     */       { 
/* 756 */         gnBounds = SVGTextElementBridge.getTextBounds(this.ctx, element, true);
/*     */         
/* 758 */         Element p = (Element)element.getParentNode();
/*     */         
/* 760 */         while (p != null && gn == null) {
/* 761 */           gn = this.ctx.getGraphicsNode(p);
/* 762 */           p = (Element)p.getParentNode();
/*     */         }  }
/* 764 */       else if (gn != null)
/* 765 */       { gnBounds = gn.getSensitiveBounds(); } 
/* 766 */     } else if (gn != null) {
/* 767 */       gnBounds = gn.getSensitiveBounds();
/*     */     } 
/* 769 */     if (gnBounds == null) return false;
/*     */     
/* 771 */     GraphicsNode svgGN = this.ctx.getGraphicsNode(this.e);
/* 772 */     if (svgGN == null) return false;
/*     */     
/* 774 */     Rectangle2D rect = new Rectangle2D.Float(svgRect.getX(), svgRect.getY(), svgRect.getWidth(), svgRect.getHeight());
/*     */ 
/*     */     
/* 777 */     AffineTransform ati = svgGN.getGlobalTransform();
/*     */     try {
/* 779 */       ati = ati.createInverse();
/* 780 */     } catch (NoninvertibleTransformException e) {}
/*     */     
/* 782 */     AffineTransform at = gn.getGlobalTransform();
/* 783 */     at.preConcatenate(ati);
/*     */     
/* 785 */     gnBounds = at.createTransformedShape(gnBounds).getBounds2D();
/*     */     
/* 787 */     return rect.contains(gnBounds);
/*     */   }
/*     */ 
/*     */   
/*     */   public boolean filterChildren(Element curr, Element end, Set elems, List ret) {
/* 792 */     Node child = curr.getFirstChild();
/* 793 */     while (child != null) {
/* 794 */       if (child instanceof Element && filterChildren((Element)child, end, elems, ret))
/*     */       {
/* 796 */         return true; } 
/* 797 */       child = child.getNextSibling();
/*     */     } 
/*     */     
/* 800 */     if (curr == end) return true;
/*     */     
/* 802 */     if (elems.contains(curr)) {
/* 803 */       ret.add(curr);
/*     */     }
/* 805 */     return false;
/*     */   }
/*     */   
/*     */   protected Set getAncestors(Element end, Element base) {
/* 809 */     Set ret = new HashSet();
/* 810 */     Element p = end;
/*     */     do {
/* 812 */       ret.add(p);
/* 813 */       p = (Element)p.getParentNode();
/* 814 */     } while (p != null && p != base);
/*     */     
/* 816 */     if (p == null) {
/* 817 */       return null;
/*     */     }
/* 819 */     return ret;
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   protected Element getNext(Element curr, Element base, Element end) {
/* 825 */     Node next = curr.getNextSibling();
/* 826 */     while (next != null && 
/* 827 */       !(next instanceof Element))
/*     */     {
/* 829 */       next = next.getNextSibling();
/*     */     }
/* 831 */     while (next == null) {
/*     */       
/* 833 */       curr = (Element)curr.getParentNode();
/* 834 */       if (curr == end || curr == base) {
/* 835 */         next = null;
/*     */         break;
/*     */       } 
/* 838 */       next = curr.getNextSibling();
/* 839 */       while (next != null && 
/* 840 */         !(next instanceof Element))
/*     */       {
/* 842 */         next = next.getNextSibling();
/*     */       }
/*     */     } 
/*     */     
/* 846 */     return (Element)next;
/*     */   }
/*     */ 
/*     */   
/* 850 */   public void deselectAll() { this.ctx.getUserAgent().deselectAll(); }
/*     */ 
/*     */   
/*     */   public int suspendRedraw(int max_wait_milliseconds) {
/* 854 */     UpdateManager um = this.ctx.getUpdateManager();
/* 855 */     if (um != null)
/* 856 */       return um.addRedrawSuspension(max_wait_milliseconds); 
/* 857 */     return -1;
/*     */   }
/*     */   public boolean unsuspendRedraw(int suspend_handle_id) {
/* 860 */     UpdateManager um = this.ctx.getUpdateManager();
/* 861 */     if (um != null)
/* 862 */       return um.releaseRedrawSuspension(suspend_handle_id); 
/* 863 */     return false;
/*     */   }
/*     */   public void unsuspendRedrawAll() {
/* 866 */     UpdateManager um = this.ctx.getUpdateManager();
/* 867 */     if (um != null)
/* 868 */       um.releaseAllRedrawSuspension(); 
/*     */   }
/*     */   
/*     */   public void forceRedraw() {
/* 872 */     UpdateManager um = this.ctx.getUpdateManager();
/* 873 */     if (um != null) {
/* 874 */       um.forceRepaint();
/*     */     }
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 881 */   public void pauseAnimations() { this.ctx.getAnimationEngine().pause(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 888 */   public void unpauseAnimations() { this.ctx.getAnimationEngine().unpause(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 895 */   public boolean animationsPaused() { return this.ctx.getAnimationEngine().isPaused(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 902 */   public float getCurrentTime() { return this.ctx.getAnimationEngine().getCurrentTime(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 909 */   public void setCurrentTime(float t) { this.ctx.getAnimationEngine().setCurrentTime(t); }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\SVGSVGElementBridge.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */