/*      */ package org.apache.batik.bridge;
/*      */ 
/*      */ import java.awt.AlphaComposite;
/*      */ import java.awt.Color;
/*      */ import java.awt.RenderingHints;
/*      */ import java.awt.Shape;
/*      */ import java.awt.font.TextAttribute;
/*      */ import java.awt.geom.AffineTransform;
/*      */ import java.awt.geom.GeneralPath;
/*      */ import java.awt.geom.Point2D;
/*      */ import java.awt.geom.Rectangle2D;
/*      */ import java.lang.ref.SoftReference;
/*      */ import java.text.AttributedCharacterIterator;
/*      */ import java.text.AttributedString;
/*      */ import java.util.ArrayList;
/*      */ import java.util.HashMap;
/*      */ import java.util.HashSet;
/*      */ import java.util.Iterator;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import java.util.Set;
/*      */ import java.util.WeakHashMap;
/*      */ import org.apache.batik.css.engine.CSSEngineEvent;
/*      */ import org.apache.batik.css.engine.CSSStylableElement;
/*      */ import org.apache.batik.css.engine.StyleMap;
/*      */ import org.apache.batik.css.engine.value.ListValue;
/*      */ import org.apache.batik.css.engine.value.Value;
/*      */ import org.apache.batik.dom.events.NodeEventTarget;
/*      */ import org.apache.batik.dom.svg.AbstractSVGAnimatedLength;
/*      */ import org.apache.batik.dom.svg.AnimatedLiveAttributeValue;
/*      */ import org.apache.batik.dom.svg.LiveAttributeException;
/*      */ import org.apache.batik.dom.svg.SVGContext;
/*      */ import org.apache.batik.dom.svg.SVGOMAnimatedEnumeration;
/*      */ import org.apache.batik.dom.svg.SVGOMAnimatedLengthList;
/*      */ import org.apache.batik.dom.svg.SVGOMAnimatedNumberList;
/*      */ import org.apache.batik.dom.svg.SVGOMElement;
/*      */ import org.apache.batik.dom.svg.SVGOMTextPositioningElement;
/*      */ import org.apache.batik.dom.svg.SVGTextContent;
/*      */ import org.apache.batik.dom.util.XLinkSupport;
/*      */ import org.apache.batik.dom.util.XMLSupport;
/*      */ import org.apache.batik.gvt.GraphicsNode;
/*      */ import org.apache.batik.gvt.TextNode;
/*      */ import org.apache.batik.gvt.font.FontFamilyResolver;
/*      */ import org.apache.batik.gvt.font.GVTFont;
/*      */ import org.apache.batik.gvt.font.GVTFontFamily;
/*      */ import org.apache.batik.gvt.font.GVTGlyphMetrics;
/*      */ import org.apache.batik.gvt.font.GVTGlyphVector;
/*      */ import org.apache.batik.gvt.font.UnresolvedFontFamily;
/*      */ import org.apache.batik.gvt.renderer.StrokingTextPainter;
/*      */ import org.apache.batik.gvt.text.GVTAttributedCharacterIterator;
/*      */ import org.apache.batik.gvt.text.Mark;
/*      */ import org.apache.batik.gvt.text.TextHit;
/*      */ import org.apache.batik.gvt.text.TextPaintInfo;
/*      */ import org.apache.batik.gvt.text.TextPath;
/*      */ import org.apache.batik.gvt.text.TextSpanLayout;
/*      */ import org.w3c.dom.Element;
/*      */ import org.w3c.dom.Node;
/*      */ import org.w3c.dom.events.Event;
/*      */ import org.w3c.dom.events.EventListener;
/*      */ import org.w3c.dom.events.MutationEvent;
/*      */ import org.w3c.dom.svg.SVGLengthList;
/*      */ import org.w3c.dom.svg.SVGNumberList;
/*      */ import org.w3c.dom.svg.SVGTextContentElement;
/*      */ import org.w3c.dom.svg.SVGTextPositioningElement;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class SVGTextElementBridge
/*      */   extends AbstractGraphicsNodeBridge
/*      */   implements SVGTextContent
/*      */ {
/*  101 */   protected static final Integer ZERO = new Integer(0);
/*      */ 
/*      */   
/*  104 */   public static final AttributedCharacterIterator.Attribute TEXT_COMPOUND_DELIMITER = GVTAttributedCharacterIterator.TextAttribute.TEXT_COMPOUND_DELIMITER;
/*      */ 
/*      */ 
/*      */   
/*  108 */   public static final AttributedCharacterIterator.Attribute TEXT_COMPOUND_ID = GVTAttributedCharacterIterator.TextAttribute.TEXT_COMPOUND_ID;
/*      */ 
/*      */   
/*  111 */   public static final AttributedCharacterIterator.Attribute PAINT_INFO = GVTAttributedCharacterIterator.TextAttribute.PAINT_INFO;
/*      */ 
/*      */ 
/*      */   
/*  115 */   public static final AttributedCharacterIterator.Attribute ALT_GLYPH_HANDLER = GVTAttributedCharacterIterator.TextAttribute.ALT_GLYPH_HANDLER;
/*      */ 
/*      */ 
/*      */   
/*  119 */   public static final AttributedCharacterIterator.Attribute TEXTPATH = GVTAttributedCharacterIterator.TextAttribute.TEXTPATH;
/*      */ 
/*      */ 
/*      */   
/*  123 */   public static final AttributedCharacterIterator.Attribute ANCHOR_TYPE = GVTAttributedCharacterIterator.TextAttribute.ANCHOR_TYPE;
/*      */ 
/*      */ 
/*      */   
/*  127 */   public static final AttributedCharacterIterator.Attribute GVT_FONT_FAMILIES = GVTAttributedCharacterIterator.TextAttribute.GVT_FONT_FAMILIES;
/*      */ 
/*      */ 
/*      */   
/*  131 */   public static final AttributedCharacterIterator.Attribute GVT_FONTS = GVTAttributedCharacterIterator.TextAttribute.GVT_FONTS;
/*      */ 
/*      */ 
/*      */   
/*  135 */   public static final AttributedCharacterIterator.Attribute BASELINE_SHIFT = GVTAttributedCharacterIterator.TextAttribute.BASELINE_SHIFT;
/*      */ 
/*      */ 
/*      */   
/*      */   protected AttributedString laidoutText;
/*      */ 
/*      */   
/*  142 */   protected WeakHashMap elemTPI = new WeakHashMap();
/*      */ 
/*      */   
/*      */   protected boolean usingComplexSVGFont = false;
/*      */ 
/*      */   
/*      */   protected DOMChildNodeRemovedEventListener childNodeRemovedEventListener;
/*      */ 
/*      */   
/*      */   protected DOMSubtreeModifiedEventListener subtreeModifiedEventListener;
/*      */ 
/*      */   
/*      */   private boolean hasNewACI;
/*      */   
/*      */   private Element cssProceedElement;
/*      */   
/*      */   protected int endLimit;
/*      */ 
/*      */   
/*  161 */   public String getLocalName() { return "text"; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  168 */   public Bridge getInstance() { return new SVGTextElementBridge(); }
/*      */ 
/*      */ 
/*      */   
/*  172 */   protected TextNode getTextNode() { return (TextNode)this.node; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public GraphicsNode createGraphicsNode(BridgeContext ctx, Element e) {
/*  182 */     TextNode node = (TextNode)super.createGraphicsNode(ctx, e);
/*  183 */     if (node == null) {
/*  184 */       return null;
/*      */     }
/*  186 */     associateSVGContext(ctx, e, node);
/*      */ 
/*      */ 
/*      */     
/*  190 */     Node child = getFirstChild(e);
/*  191 */     while (child != null) {
/*  192 */       if (child.getNodeType() == 1) {
/*  193 */         addContextToChild(ctx, (Element)child);
/*      */       }
/*  195 */       child = getNextSibling(child);
/*      */     } 
/*      */ 
/*      */     
/*  199 */     if (ctx.getTextPainter() != null) {
/*  200 */       node.setTextPainter(ctx.getTextPainter());
/*      */     }
/*      */     
/*  203 */     RenderingHints hints = null;
/*  204 */     hints = CSSUtilities.convertColorRendering(e, hints);
/*  205 */     hints = CSSUtilities.convertTextRendering(e, hints);
/*  206 */     if (hints != null) {
/*  207 */       node.setRenderingHints(hints);
/*      */     }
/*  209 */     node.setLocation(getLocation(ctx, e));
/*      */     
/*  211 */     return node;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  219 */   protected GraphicsNode instantiateGraphicsNode() { return new TextNode(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Point2D getLocation(BridgeContext ctx, Element e) {
/*      */     try {
/*  231 */       SVGOMTextPositioningElement te = (SVGOMTextPositioningElement)e;
/*      */ 
/*      */       
/*  234 */       SVGOMAnimatedLengthList _x = (SVGOMAnimatedLengthList)te.getX();
/*  235 */       _x.check();
/*  236 */       SVGLengthList xs = _x.getAnimVal();
/*  237 */       float x = 0.0F;
/*  238 */       if (xs.getNumberOfItems() > 0) {
/*  239 */         x = xs.getItem(0).getValue();
/*      */       }
/*      */ 
/*      */       
/*  243 */       SVGOMAnimatedLengthList _y = (SVGOMAnimatedLengthList)te.getY();
/*  244 */       _y.check();
/*  245 */       SVGLengthList ys = _y.getAnimVal();
/*  246 */       float y = 0.0F;
/*  247 */       if (ys.getNumberOfItems() > 0) {
/*  248 */         y = ys.getItem(0).getValue();
/*      */       }
/*      */       
/*  251 */       return new Point2D.Float(x, y);
/*      */     } catch (LiveAttributeException ex) {
/*  253 */       throw new BridgeException(ctx, ex);
/*      */     } 
/*      */   }
/*      */   
/*      */   protected boolean isTextElement(Element e) {
/*  258 */     if (!"http://www.w3.org/2000/svg".equals(e.getNamespaceURI()))
/*  259 */       return false; 
/*  260 */     String nodeName = e.getLocalName();
/*  261 */     return (nodeName.equals("text") || nodeName.equals("tspan") || nodeName.equals("altGlyph") || nodeName.equals("a") || nodeName.equals("textPath") || nodeName.equals("tref"));
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected boolean isTextChild(Element e) {
/*  270 */     if (!"http://www.w3.org/2000/svg".equals(e.getNamespaceURI()))
/*  271 */       return false; 
/*  272 */     String nodeName = e.getLocalName();
/*  273 */     return (nodeName.equals("tspan") || nodeName.equals("altGlyph") || nodeName.equals("a") || nodeName.equals("textPath") || nodeName.equals("tref"));
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void buildGraphicsNode(BridgeContext ctx, Element e, GraphicsNode node) {
/*  291 */     e.normalize();
/*  292 */     computeLaidoutText(ctx, e, node);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  298 */     node.setComposite(CSSUtilities.convertOpacity(e));
/*      */     
/*  300 */     node.setFilter(CSSUtilities.convertFilter(e, node, ctx));
/*      */     
/*  302 */     node.setMask(CSSUtilities.convertMask(e, node, ctx));
/*      */     
/*  304 */     node.setClip(CSSUtilities.convertClipPath(e, node, ctx));
/*      */     
/*  306 */     node.setPointerEventType(CSSUtilities.convertPointerEvents(e));
/*      */     
/*  308 */     initializeDynamicSupport(ctx, e, node);
/*  309 */     if (!ctx.isDynamic()) {
/*  310 */       this.elemTPI.clear();
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  318 */   public boolean isComposite() { return false; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  328 */   protected Node getFirstChild(Node n) { return n.getFirstChild(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  336 */   protected Node getNextSibling(Node n) { return n.getNextSibling(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  344 */   protected Node getParentNode(Node n) { return n.getParentNode(); }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class DOMChildNodeRemovedEventListener
/*      */     implements EventListener
/*      */   {
/*      */     private final SVGTextElementBridge this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  357 */     protected DOMChildNodeRemovedEventListener(SVGTextElementBridge this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  363 */     public void handleEvent(Event evt) { this.this$0.handleDOMChildNodeRemovedEvent((MutationEvent)evt); }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class DOMSubtreeModifiedEventListener
/*      */     implements EventListener
/*      */   {
/*      */     private final SVGTextElementBridge this$0;
/*      */ 
/*      */ 
/*      */     
/*  375 */     protected DOMSubtreeModifiedEventListener(SVGTextElementBridge this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  381 */     public void handleEvent(Event evt) { this.this$0.handleDOMSubtreeModifiedEvent((MutationEvent)evt); }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void initializeDynamicSupport(BridgeContext ctx, Element e, GraphicsNode node) {
/*  395 */     super.initializeDynamicSupport(ctx, e, node);
/*      */     
/*  397 */     if (ctx.isDynamic())
/*      */     {
/*  399 */       addTextEventListeners(ctx, (NodeEventTarget)e);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addTextEventListeners(BridgeContext ctx, NodeEventTarget e) {
/*  407 */     if (this.childNodeRemovedEventListener == null) {
/*  408 */       this.childNodeRemovedEventListener = new DOMChildNodeRemovedEventListener(this);
/*      */     }
/*      */     
/*  411 */     if (this.subtreeModifiedEventListener == null) {
/*  412 */       this.subtreeModifiedEventListener = new DOMSubtreeModifiedEventListener(this);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  418 */     e.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", this.childNodeRemovedEventListener, true, null);
/*      */ 
/*      */     
/*  421 */     ctx.storeEventListenerNS(e, "http://www.w3.org/2001/xml-events", "DOMNodeRemoved", this.childNodeRemovedEventListener, true);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  427 */     e.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMSubtreeModified", this.subtreeModifiedEventListener, false, null);
/*      */ 
/*      */     
/*  430 */     ctx.storeEventListenerNS(e, "http://www.w3.org/2001/xml-events", "DOMSubtreeModified", this.subtreeModifiedEventListener, false);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void removeTextEventListeners(BridgeContext ctx, NodeEventTarget e) {
/*  440 */     e.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", this.childNodeRemovedEventListener, true);
/*      */ 
/*      */     
/*  443 */     e.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMSubtreeModified", this.subtreeModifiedEventListener, false);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void dispose() {
/*  453 */     removeTextEventListeners(this.ctx, (NodeEventTarget)this.e);
/*  454 */     super.dispose();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addContextToChild(BridgeContext ctx, Element e) {
/*  470 */     if ("http://www.w3.org/2000/svg".equals(e.getNamespaceURI())) {
/*  471 */       if (e.getLocalName().equals("tspan")) {
/*  472 */         ((SVGOMElement)e).setSVGContext(new TspanBridge(this, ctx, this, e));
/*      */       }
/*  474 */       else if (e.getLocalName().equals("textPath")) {
/*  475 */         ((SVGOMElement)e).setSVGContext(new TextPathBridge(this, ctx, this, e));
/*      */       }
/*  477 */       else if (e.getLocalName().equals("tref")) {
/*  478 */         ((SVGOMElement)e).setSVGContext(new TRefBridge(this, ctx, this, e));
/*      */       } 
/*      */     }
/*      */ 
/*      */     
/*  483 */     Node child = getFirstChild(e);
/*  484 */     while (child != null) {
/*  485 */       if (child.getNodeType() == 1) {
/*  486 */         addContextToChild(ctx, (Element)child);
/*      */       }
/*  488 */       child = getNextSibling(child);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void removeContextFromChild(BridgeContext ctx, Element e) {
/*  503 */     if ("http://www.w3.org/2000/svg".equals(e.getNamespaceURI())) {
/*  504 */       if (e.getLocalName().equals("tspan")) {
/*  505 */         ((AbstractTextChildBridgeUpdateHandler)((SVGOMElement)e).getSVGContext()).dispose();
/*      */       }
/*  507 */       else if (e.getLocalName().equals("textPath")) {
/*  508 */         ((AbstractTextChildBridgeUpdateHandler)((SVGOMElement)e).getSVGContext()).dispose();
/*      */       }
/*  510 */       else if (e.getLocalName().equals("tref")) {
/*  511 */         ((AbstractTextChildBridgeUpdateHandler)((SVGOMElement)e).getSVGContext()).dispose();
/*      */       } 
/*      */     }
/*      */ 
/*      */     
/*  516 */     Node child = getFirstChild(e);
/*  517 */     while (child != null) {
/*  518 */       if (child.getNodeType() == 1) {
/*  519 */         removeContextFromChild(ctx, (Element)child);
/*      */       }
/*  521 */       child = getNextSibling(child);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   public void handleDOMNodeInsertedEvent(MutationEvent evt) {
/*      */     Element childElement;
/*  529 */     Node childNode = (Node)evt.getTarget();
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  534 */     switch (childNode.getNodeType()) {
/*      */       case 3:
/*      */       case 4:
/*  537 */         this.laidoutText = null;
/*      */         break;
/*      */       case 1:
/*  540 */         childElement = (Element)childNode;
/*  541 */         if (isTextChild(childElement)) {
/*  542 */           addContextToChild(this.ctx, childElement);
/*  543 */           this.laidoutText = null;
/*      */         } 
/*      */         break;
/*      */     } 
/*      */     
/*  548 */     if (this.laidoutText == null) {
/*  549 */       computeLaidoutText(this.ctx, this.e, getTextNode());
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   public void handleDOMChildNodeRemovedEvent(MutationEvent evt) {
/*      */     Element childElt;
/*  557 */     Node childNode = (Node)evt.getTarget();
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  562 */     switch (childNode.getNodeType()) {
/*      */       
/*      */       case 3:
/*      */       case 4:
/*  566 */         if (isParentDisplayed(childNode)) {
/*  567 */           this.laidoutText = null;
/*      */         }
/*      */         break;
/*      */       case 1:
/*  571 */         childElt = (Element)childNode;
/*  572 */         if (isTextChild(childElt)) {
/*  573 */           this.laidoutText = null;
/*  574 */           removeContextFromChild(this.ctx, childElt);
/*      */         } 
/*      */         break;
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void handleDOMSubtreeModifiedEvent(MutationEvent evt) {
/*  590 */     if (this.laidoutText == null) {
/*  591 */       computeLaidoutText(this.ctx, this.e, getTextNode());
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void handleDOMCharacterDataModified(MutationEvent evt) {
/*  600 */     Node childNode = (Node)evt.getTarget();
/*      */     
/*  602 */     if (isParentDisplayed(childNode)) {
/*  603 */       this.laidoutText = null;
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected boolean isParentDisplayed(Node childNode) {
/*  618 */     Node parentNode = getParentNode(childNode);
/*  619 */     return isTextElement((Element)parentNode);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void computeLaidoutText(BridgeContext ctx, Element e, GraphicsNode node) {
/*  633 */     TextNode tn = (TextNode)node;
/*  634 */     this.elemTPI.clear();
/*      */     
/*  636 */     AttributedString as = buildAttributedString(ctx, e);
/*  637 */     if (as == null) {
/*  638 */       tn.setAttributedCharacterIterator(null);
/*      */       
/*      */       return;
/*      */     } 
/*  642 */     addGlyphPositionAttributes(as, e, ctx);
/*  643 */     if (ctx.isDynamic()) {
/*  644 */       this.laidoutText = new AttributedString(as.getIterator());
/*      */     }
/*      */ 
/*      */     
/*  648 */     tn.setAttributedCharacterIterator(as.getIterator());
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  653 */     TextPaintInfo pi = new TextPaintInfo();
/*  654 */     setBaseTextPaintInfo(pi, e, node, ctx);
/*      */     
/*  656 */     setDecorationTextPaintInfo(pi, e);
/*      */     
/*  658 */     addPaintAttributes(as, e, tn, pi, ctx);
/*      */     
/*  660 */     if (this.usingComplexSVGFont)
/*      */     {
/*  662 */       tn.setAttributedCharacterIterator(as.getIterator());
/*      */     }
/*      */     
/*  665 */     if (ctx.isDynamic()) {
/*  666 */       checkBBoxChange();
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void handleAnimatedAttributeChanged(AnimatedLiveAttributeValue alav) {
/*  687 */     if (alav.getNamespaceURI() == null) {
/*  688 */       String ln = alav.getLocalName();
/*  689 */       if (ln.equals("x") || ln.equals("y") || ln.equals("dx") || ln.equals("dy") || ln.equals("rotate") || ln.equals("textLength") || ln.equals("lengthAdjust")) {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         
/*  696 */         char c = ln.charAt(0);
/*  697 */         if (c == 'x' || c == 'y') {
/*  698 */           getTextNode().setLocation(getLocation(this.ctx, this.e));
/*      */         }
/*  700 */         computeLaidoutText(this.ctx, this.e, getTextNode());
/*      */         return;
/*      */       } 
/*      */     } 
/*  704 */     super.handleAnimatedAttributeChanged(alav);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void handleCSSEngineEvent(CSSEngineEvent evt) {
/*  713 */     this.hasNewACI = false;
/*  714 */     int[] properties = evt.getProperties();
/*      */     
/*  716 */     for (int i = 0; i < properties.length; i++) {
/*  717 */       switch (properties[i]) {
/*      */         case 1:
/*      */         case 11:
/*      */         case 12:
/*      */         case 21:
/*      */         case 22:
/*      */         case 24:
/*      */         case 25:
/*      */         case 27:
/*      */         case 28:
/*      */         case 29:
/*      */         case 31:
/*      */         case 32:
/*      */         case 53:
/*      */         case 56:
/*      */         case 58:
/*      */         case 59:
/*  734 */           if (!this.hasNewACI) {
/*  735 */             this.hasNewACI = true;
/*  736 */             computeLaidoutText(this.ctx, this.e, getTextNode());
/*      */           } 
/*      */           break;
/*      */       } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     } 
/*  745 */     this.cssProceedElement = evt.getElement();
/*      */     
/*  747 */     super.handleCSSEngineEvent(evt);
/*  748 */     this.cssProceedElement = null;
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected void handleCSSPropertyChanged(int property) {
/*      */     RenderingHints hints, hints;
/*  755 */     switch (property) {
/*      */       case 15:
/*      */       case 16:
/*      */       case 45:
/*      */       case 46:
/*      */       case 47:
/*      */       case 48:
/*      */       case 49:
/*      */       case 50:
/*      */       case 51:
/*      */       case 52:
/*      */       case 54:
/*  767 */         rebuildACI();
/*      */         return;
/*      */       
/*      */       case 57:
/*  771 */         rebuildACI();
/*  772 */         super.handleCSSPropertyChanged(property);
/*      */         return;
/*      */       case 55:
/*  775 */         hints = this.node.getRenderingHints();
/*  776 */         hints = CSSUtilities.convertTextRendering(this.e, hints);
/*  777 */         if (hints != null) {
/*  778 */           this.node.setRenderingHints(hints);
/*      */         }
/*      */         return;
/*      */       
/*      */       case 9:
/*  783 */         hints = this.node.getRenderingHints();
/*  784 */         hints = CSSUtilities.convertColorRendering(this.e, hints);
/*  785 */         if (hints != null) {
/*  786 */           this.node.setRenderingHints(hints);
/*      */         }
/*      */         return;
/*      */     } 
/*      */     
/*  791 */     super.handleCSSPropertyChanged(property);
/*      */   }
/*      */   
/*      */   protected void rebuildACI() {
/*      */     TextPaintInfo textPaintInfo2, textPaintInfo1;
/*  796 */     if (this.hasNewACI) {
/*      */       return;
/*      */     }
/*  799 */     TextNode textNode = getTextNode();
/*  800 */     if (textNode.getAttributedCharacterIterator() == null) {
/*      */       return;
/*      */     }
/*      */     
/*  804 */     if (this.cssProceedElement == this.e) {
/*  805 */       textPaintInfo1 = new TextPaintInfo();
/*  806 */       setBaseTextPaintInfo(textPaintInfo1, this.e, this.node, this.ctx);
/*  807 */       setDecorationTextPaintInfo(textPaintInfo1, this.e);
/*  808 */       textPaintInfo2 = (TextPaintInfo)this.elemTPI.get(this.e);
/*      */ 
/*      */     
/*      */     }
/*      */     else {
/*      */ 
/*      */       
/*  815 */       TextPaintInfo parentPI = getParentTextPaintInfo(this.cssProceedElement);
/*  816 */       textPaintInfo1 = getTextPaintInfo(this.cssProceedElement, textNode, parentPI, this.ctx);
/*  817 */       textPaintInfo2 = (TextPaintInfo)this.elemTPI.get(this.cssProceedElement);
/*      */     } 
/*  819 */     if (textPaintInfo2 == null)
/*      */       return; 
/*  821 */     textNode.swapTextPaintInfo(textPaintInfo1, textPaintInfo2);
/*  822 */     if (this.usingComplexSVGFont)
/*      */     {
/*  824 */       textNode.setAttributedCharacterIterator(textNode.getAttributedCharacterIterator());
/*      */     }
/*      */   }
/*      */   
/*      */   int getElementStartIndex(Element element) {
/*  829 */     TextPaintInfo tpi = (TextPaintInfo)this.elemTPI.get(element);
/*  830 */     if (tpi == null) return -1; 
/*  831 */     return tpi.startChar;
/*      */   }
/*      */   
/*      */   int getElementEndIndex(Element element) {
/*  835 */     TextPaintInfo tpi = (TextPaintInfo)this.elemTPI.get(element);
/*  836 */     if (tpi == null) return -1; 
/*  837 */     return tpi.endChar;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected AttributedString buildAttributedString(BridgeContext ctx, Element element) {
/*  855 */     AttributedStringBuffer asb = new AttributedStringBuffer();
/*  856 */     fillAttributedStringBuffer(ctx, element, true, null, null, null, asb);
/*  857 */     return asb.toAttributedString();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void fillAttributedStringBuffer(BridgeContext ctx, Element element, boolean top, TextPath textPath, Integer bidiLevel, Map initialAttributes, AttributedStringBuffer asb) {
/*  881 */     if (!SVGUtilities.matchUserAgent(element, ctx.getUserAgent()) || !CSSUtilities.convertDisplay(element)) {
/*      */       return;
/*      */     }
/*      */ 
/*      */     
/*  886 */     String s = XMLSupport.getXMLSpace(element);
/*  887 */     boolean preserve = s.equals("preserve");
/*      */     
/*  889 */     Element nodeElement = element;
/*  890 */     int elementStartChar = asb.length();
/*      */     
/*  892 */     if (top) {
/*  893 */       this.endLimit = 0;
/*      */     }
/*  895 */     if (preserve) {
/*  896 */       this.endLimit = asb.length();
/*      */     }
/*      */     
/*  899 */     Map map = (initialAttributes == null) ? new HashMap() : new HashMap(initialAttributes);
/*      */ 
/*      */     
/*  902 */     initialAttributes = getAttributeMap(ctx, element, textPath, bidiLevel, map);
/*      */     
/*  904 */     Object o = map.get(TextAttribute.BIDI_EMBEDDING);
/*  905 */     Integer subBidiLevel = bidiLevel;
/*  906 */     if (o != null) {
/*  907 */       subBidiLevel = (Integer)o;
/*      */     }
/*      */     
/*  910 */     Node n = getFirstChild(element);
/*  911 */     for (; n != null; 
/*  912 */       n = getNextSibling(n)) {
/*      */       String ln; boolean bool;
/*  914 */       if (preserve) {
/*  915 */         bool = false;
/*      */       }
/*  917 */       else if (asb.length() == 0) {
/*  918 */         bool = true;
/*      */       } else {
/*  920 */         bool = (asb.getLastChar() == 32);
/*      */       } 
/*      */ 
/*      */       
/*  924 */       switch (n.getNodeType()) {
/*      */         case 1:
/*  926 */           if (!"http://www.w3.org/2000/svg".equals(n.getNamespaceURI())) {
/*      */             break;
/*      */           }
/*  929 */           nodeElement = (Element)n;
/*      */           
/*  931 */           ln = n.getLocalName();
/*      */           
/*  933 */           if (ln.equals("tspan") || ln.equals("altGlyph")) {
/*      */             
/*  935 */             int before = asb.count;
/*  936 */             fillAttributedStringBuffer(ctx, nodeElement, false, textPath, subBidiLevel, initialAttributes, asb);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */             
/*  943 */             if (asb.count != before)
/*  944 */               initialAttributes = null;  break;
/*      */           } 
/*  946 */           if (ln.equals("textPath")) {
/*  947 */             SVGTextPathElementBridge textPathBridge = (SVGTextPathElementBridge)ctx.getBridge(nodeElement);
/*      */             
/*  949 */             TextPath newTextPath = textPathBridge.createTextPath(ctx, nodeElement);
/*      */             
/*  951 */             if (newTextPath != null) {
/*  952 */               int before = asb.count;
/*  953 */               fillAttributedStringBuffer(ctx, nodeElement, false, newTextPath, subBidiLevel, initialAttributes, asb);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */               
/*  960 */               if (asb.count != before)
/*  961 */                 initialAttributes = null; 
/*      */             }  break;
/*      */           } 
/*  964 */           if (ln.equals("tref")) {
/*  965 */             String uriStr = XLinkSupport.getXLinkHref((Element)n);
/*  966 */             Element ref = ctx.getReferencedElement((Element)n, uriStr);
/*  967 */             s = TextUtilities.getElementContent(ref);
/*  968 */             s = normalizeString(s, preserve, bool);
/*  969 */             if (s.length() != 0) {
/*  970 */               int trefStart = asb.length();
/*  971 */               Map m = (initialAttributes == null) ? new HashMap() : new HashMap(initialAttributes);
/*      */ 
/*      */               
/*  974 */               getAttributeMap(ctx, nodeElement, textPath, bidiLevel, m);
/*      */               
/*  976 */               asb.append(s, m);
/*  977 */               int trefEnd = asb.length() - 1;
/*      */               
/*  979 */               TextPaintInfo tpi = (TextPaintInfo)this.elemTPI.get(nodeElement);
/*  980 */               tpi.startChar = trefStart;
/*  981 */               tpi.endChar = trefEnd;
/*  982 */               initialAttributes = null;
/*      */             }  break;
/*  984 */           }  if (ln.equals("a")) {
/*  985 */             NodeEventTarget target = (NodeEventTarget)nodeElement;
/*  986 */             UserAgent ua = ctx.getUserAgent();
/*      */             
/*  988 */             SVGAElementBridge.CursorHolder ch = new SVGAElementBridge.CursorHolder(CursorManager.DEFAULT_CURSOR);
/*      */ 
/*      */             
/*  991 */             EventListener l = new SVGAElementBridge.AnchorListener(ua, ch);
/*  992 */             target.addEventListenerNS("http://www.w3.org/2001/xml-events", "click", l, false, null);
/*      */ 
/*      */             
/*  995 */             ctx.storeEventListenerNS(target, "http://www.w3.org/2001/xml-events", "click", l, false);
/*      */ 
/*      */ 
/*      */             
/*  999 */             int before = asb.count;
/* 1000 */             fillAttributedStringBuffer(ctx, nodeElement, false, textPath, subBidiLevel, initialAttributes, asb);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */             
/* 1007 */             if (asb.count != before) {
/* 1008 */               initialAttributes = null;
/*      */             }
/*      */           } 
/*      */           break;
/*      */         case 3:
/*      */         case 4:
/* 1014 */           s = n.getNodeValue();
/* 1015 */           s = normalizeString(s, preserve, bool);
/* 1016 */           if (s.length() != 0) {
/* 1017 */             asb.append(s, map);
/* 1018 */             if (preserve) {
/* 1019 */               this.endLimit = asb.length();
/*      */             }
/* 1021 */             initialAttributes = null;
/*      */           } 
/*      */           break;
/*      */       } 
/*      */     } 
/* 1026 */     if (top) {
/* 1027 */       boolean strippedSome = false;
/* 1028 */       while (this.endLimit < asb.length() && asb.getLastChar() == 32) {
/* 1029 */         asb.stripLast();
/* 1030 */         strippedSome = true;
/*      */       } 
/* 1032 */       if (strippedSome) {
/* 1033 */         Iterator iter = this.elemTPI.values().iterator();
/* 1034 */         while (iter.hasNext()) {
/* 1035 */           TextPaintInfo tpi = (TextPaintInfo)iter.next();
/* 1036 */           if (tpi.endChar >= asb.length()) {
/* 1037 */             tpi.endChar = asb.length() - 1;
/* 1038 */             if (tpi.startChar > tpi.endChar)
/* 1039 */               tpi.startChar = tpi.endChar; 
/*      */           } 
/*      */         } 
/*      */       } 
/*      */     } 
/* 1044 */     int elementEndChar = asb.length() - 1;
/* 1045 */     TextPaintInfo tpi = (TextPaintInfo)this.elemTPI.get(element);
/* 1046 */     tpi.startChar = elementStartChar;
/* 1047 */     tpi.endChar = elementEndChar;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected String normalizeString(String s, boolean preserve, boolean stripfirst) {
/* 1056 */     StringBuffer sb = new StringBuffer(s.length());
/* 1057 */     if (preserve) {
/* 1058 */       for (int i = 0; i < s.length(); i++) {
/* 1059 */         char c = s.charAt(i);
/* 1060 */         switch (c) {
/*      */           case '\t':
/*      */           case '\n':
/*      */           case '\r':
/* 1064 */             sb.append(' ');
/*      */             break;
/*      */           default:
/* 1067 */             sb.append(c); break;
/*      */         } 
/*      */       } 
/* 1070 */       return sb.toString();
/*      */     } 
/*      */     
/* 1073 */     int idx = 0;
/* 1074 */     if (stripfirst) {
/* 1075 */       while (idx < s.length()) {
/* 1076 */         switch (s.charAt(idx)) { default:
/*      */             break;
/*      */           case '\t':
/*      */           case '\n':
/*      */           case '\r':
/*      */           case ' ':
/*      */             break; }
/* 1083 */          idx++;
/*      */       } 
/*      */     }
/*      */ 
/*      */     
/* 1088 */     boolean space = false;
/* 1089 */     for (int i = idx; i < s.length(); i++) {
/* 1090 */       char c = s.charAt(i);
/* 1091 */       switch (c) {
/*      */         case '\n':
/*      */         case '\r':
/*      */           break;
/*      */         case '\t':
/*      */         case ' ':
/* 1097 */           if (!space) {
/* 1098 */             sb.append(' ');
/* 1099 */             space = true;
/*      */           } 
/*      */           break;
/*      */         default:
/* 1103 */           sb.append(c);
/* 1104 */           space = false;
/*      */           break;
/*      */       } 
/*      */     } 
/* 1108 */     return sb.toString();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected static class AttributedStringBuffer
/*      */   {
/* 1140 */     protected List strings = new ArrayList();
/* 1141 */     protected List attributes = new ArrayList();
/* 1142 */     protected int count = 0;
/* 1143 */     protected int length = 0;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1150 */     public boolean isEmpty() { return (this.count == 0); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1157 */     public int length() { return this.length; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void append(String s, Map m) {
/* 1164 */       if (s.length() == 0)
/* 1165 */         return;  this.strings.add(s);
/* 1166 */       this.attributes.add(m);
/* 1167 */       this.count++;
/* 1168 */       this.length += s.length();
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public int getLastChar() {
/* 1175 */       if (this.count == 0) {
/* 1176 */         return -1;
/*      */       }
/* 1178 */       String s = (String)this.strings.get(this.count - 1);
/* 1179 */       return s.charAt(s.length() - 1);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void stripFirst() {
/* 1186 */       String s = (String)this.strings.get(0);
/* 1187 */       if (s.charAt(s.length() - 1) != ' ') {
/*      */         return;
/*      */       }
/* 1190 */       this.length--;
/*      */       
/* 1192 */       if (s.length() == 1) {
/* 1193 */         this.attributes.remove(0);
/* 1194 */         this.strings.remove(0);
/* 1195 */         this.count--;
/*      */         
/*      */         return;
/*      */       } 
/* 1199 */       this.strings.set(0, s.substring(1));
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void stripLast() {
/* 1206 */       String s = (String)this.strings.get(this.count - 1);
/* 1207 */       if (s.charAt(s.length() - 1) != ' ') {
/*      */         return;
/*      */       }
/* 1210 */       this.length--;
/*      */       
/* 1212 */       if (s.length() == 1) {
/* 1213 */         this.attributes.remove(--this.count);
/* 1214 */         this.strings.remove(this.count);
/*      */         
/*      */         return;
/*      */       } 
/* 1218 */       this.strings.set(this.count - 1, s.substring(0, s.length() - 1));
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AttributedString toAttributedString() {
/* 1226 */       switch (this.count) {
/*      */         case 0:
/* 1228 */           return null;
/*      */         case 1:
/* 1230 */           return new AttributedString((String)this.strings.get(0), (Map)this.attributes.get(0));
/*      */       } 
/*      */ 
/*      */       
/* 1234 */       StringBuffer sb = new StringBuffer(this.strings.size() * 5);
/* 1235 */       Iterator it = this.strings.iterator();
/* 1236 */       while (it.hasNext()) {
/* 1237 */         sb.append((String)it.next());
/*      */       }
/*      */       
/* 1240 */       AttributedString result = new AttributedString(sb.toString());
/*      */ 
/*      */ 
/*      */       
/* 1244 */       Iterator sit = this.strings.iterator();
/* 1245 */       Iterator ait = this.attributes.iterator();
/* 1246 */       int idx = 0;
/* 1247 */       while (sit.hasNext()) {
/* 1248 */         String s = (String)sit.next();
/* 1249 */         int nidx = idx + s.length();
/* 1250 */         Map m = (Map)ait.next();
/* 1251 */         Iterator kit = m.keySet().iterator();
/* 1252 */         Iterator vit = m.values().iterator();
/* 1253 */         while (kit.hasNext()) {
/* 1254 */           AttributedCharacterIterator.Attribute attr = (AttributedCharacterIterator.Attribute)kit.next();
/* 1255 */           Object val = vit.next();
/* 1256 */           result.addAttribute(attr, val, idx, nidx);
/*      */         } 
/* 1258 */         idx = nidx;
/*      */       } 
/*      */       
/* 1261 */       return result;
/*      */     }
/*      */     
/*      */     public String toString() {
/* 1265 */       switch (this.count) {
/*      */         case 0:
/* 1267 */           return "";
/*      */         case 1:
/* 1269 */           return (String)this.strings.get(0);
/*      */       } 
/*      */       
/* 1272 */       StringBuffer sb = new StringBuffer(this.strings.size() * 5);
/* 1273 */       Iterator it = this.strings.iterator();
/* 1274 */       while (it.hasNext()) {
/* 1275 */         sb.append((String)it.next());
/*      */       }
/* 1277 */       return sb.toString();
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected boolean nodeAncestorOf(Node node1, Node node2) {
/* 1285 */     if (node2 == null || node1 == null) {
/* 1286 */       return false;
/*      */     }
/* 1288 */     Node parent = getParentNode(node2);
/* 1289 */     while (parent != null && parent != node1) {
/* 1290 */       parent = getParentNode(parent);
/*      */     }
/* 1292 */     return (parent == node1);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addGlyphPositionAttributes(AttributedString as, Element element, BridgeContext ctx) {
/* 1304 */     if (!SVGUtilities.matchUserAgent(element, ctx.getUserAgent()) || !CSSUtilities.convertDisplay(element)) {
/*      */       return;
/*      */     }
/*      */     
/* 1308 */     if (element.getLocalName().equals("textPath")) {
/*      */       
/* 1310 */       addChildGlyphPositionAttributes(as, element, ctx);
/*      */       
/*      */       return;
/*      */     } 
/*      */     
/* 1315 */     int firstChar = getElementStartIndex(element);
/*      */     
/* 1317 */     if (firstChar == -1)
/*      */       return; 
/* 1319 */     int lastChar = getElementEndIndex(element);
/*      */ 
/*      */ 
/*      */     
/* 1323 */     if (!(element instanceof SVGTextPositioningElement)) {
/* 1324 */       addChildGlyphPositionAttributes(as, element, ctx);
/*      */       
/*      */       return;
/*      */     } 
/*      */     
/* 1329 */     SVGTextPositioningElement te = (SVGTextPositioningElement)element;
/*      */     
/*      */     try {
/* 1332 */       SVGOMAnimatedLengthList _x = (SVGOMAnimatedLengthList)te.getX();
/*      */       
/* 1334 */       _x.check();
/* 1335 */       SVGOMAnimatedLengthList _y = (SVGOMAnimatedLengthList)te.getY();
/*      */       
/* 1337 */       _y.check();
/* 1338 */       SVGOMAnimatedLengthList _dx = (SVGOMAnimatedLengthList)te.getDx();
/*      */       
/* 1340 */       _dx.check();
/* 1341 */       SVGOMAnimatedLengthList _dy = (SVGOMAnimatedLengthList)te.getDy();
/*      */       
/* 1343 */       _dy.check();
/* 1344 */       SVGOMAnimatedNumberList _rotate = (SVGOMAnimatedNumberList)te.getRotate();
/*      */       
/* 1346 */       _rotate.check();
/*      */       
/* 1348 */       SVGLengthList xs = _x.getAnimVal();
/* 1349 */       SVGLengthList ys = _y.getAnimVal();
/* 1350 */       SVGLengthList dxs = _dx.getAnimVal();
/* 1351 */       SVGLengthList dys = _dy.getAnimVal();
/* 1352 */       SVGNumberList rs = _rotate.getAnimVal();
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1357 */       int len = xs.getNumberOfItems();
/* 1358 */       for (int i = 0; i < len && firstChar + i <= lastChar; i++) {
/* 1359 */         as.addAttribute(GVTAttributedCharacterIterator.TextAttribute.X, new Float(xs.getItem(i).getValue()), firstChar + i, firstChar + i + 1);
/*      */       }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1366 */       len = ys.getNumberOfItems();
/* 1367 */       for (int i = 0; i < len && firstChar + i <= lastChar; i++) {
/* 1368 */         as.addAttribute(GVTAttributedCharacterIterator.TextAttribute.Y, new Float(ys.getItem(i).getValue()), firstChar + i, firstChar + i + 1);
/*      */       }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1375 */       len = dxs.getNumberOfItems();
/* 1376 */       for (int i = 0; i < len && firstChar + i <= lastChar; i++) {
/* 1377 */         as.addAttribute(GVTAttributedCharacterIterator.TextAttribute.DX, new Float(dxs.getItem(i).getValue()), firstChar + i, firstChar + i + 1);
/*      */       }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1384 */       len = dys.getNumberOfItems();
/* 1385 */       for (int i = 0; i < len && firstChar + i <= lastChar; i++) {
/* 1386 */         as.addAttribute(GVTAttributedCharacterIterator.TextAttribute.DY, new Float(dys.getItem(i).getValue()), firstChar + i, firstChar + i + 1);
/*      */       }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1393 */       len = rs.getNumberOfItems();
/* 1394 */       if (len == 1) {
/*      */         
/* 1396 */         Float rad = new Float(Math.toRadians(rs.getItem(0).getValue()));
/* 1397 */         as.addAttribute(GVTAttributedCharacterIterator.TextAttribute.ROTATION, rad, firstChar, lastChar + 1);
/*      */ 
/*      */       
/*      */       }
/* 1401 */       else if (len > 1) {
/*      */         
/* 1403 */         for (int i = 0; i < len && firstChar + i <= lastChar; i++) {
/* 1404 */           Float rad = new Float(Math.toRadians(rs.getItem(i).getValue()));
/* 1405 */           as.addAttribute(GVTAttributedCharacterIterator.TextAttribute.ROTATION, rad, firstChar + i, firstChar + i + 1);
/*      */         } 
/*      */       } 
/*      */ 
/*      */ 
/*      */       
/* 1411 */       addChildGlyphPositionAttributes(as, element, ctx);
/*      */     } catch (LiveAttributeException ex) {
/* 1413 */       throw new BridgeException(ctx, ex);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addChildGlyphPositionAttributes(AttributedString as, Element element, BridgeContext ctx) {
/* 1421 */     Node child = getFirstChild(element);
/* 1422 */     for (; child != null; 
/* 1423 */       child = getNextSibling(child)) {
/* 1424 */       if (child.getNodeType() == 1) {
/*      */         
/* 1426 */         Element childElement = (Element)child;
/* 1427 */         if (isTextChild(childElement)) {
/* 1428 */           addGlyphPositionAttributes(as, childElement, ctx);
/*      */         }
/*      */       } 
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addPaintAttributes(AttributedString as, Element element, TextNode node, TextPaintInfo pi, BridgeContext ctx) {
/* 1442 */     if (!SVGUtilities.matchUserAgent(element, ctx.getUserAgent()) || !CSSUtilities.convertDisplay(element)) {
/*      */       return;
/*      */     }
/*      */     
/* 1446 */     Object o = this.elemTPI.get(element);
/* 1447 */     if (o != null) {
/* 1448 */       node.swapTextPaintInfo(pi, (TextPaintInfo)o);
/*      */     }
/* 1450 */     addChildPaintAttributes(as, element, node, pi, ctx);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addChildPaintAttributes(AttributedString as, Element element, TextNode node, TextPaintInfo parentPI, BridgeContext ctx) {
/* 1459 */     Node child = getFirstChild(element);
/* 1460 */     for (; child != null; 
/* 1461 */       child = getNextSibling(child)) {
/* 1462 */       if (child.getNodeType() == 1) {
/*      */ 
/*      */         
/* 1465 */         Element childElement = (Element)child;
/* 1466 */         if (isTextChild(childElement)) {
/* 1467 */           TextPaintInfo pi = getTextPaintInfo(childElement, node, parentPI, ctx);
/*      */           
/* 1469 */           addPaintAttributes(as, childElement, node, pi, ctx);
/*      */         } 
/*      */       } 
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected List getFontList(BridgeContext ctx, Element element, Map result) {
/* 1483 */     result.put(TEXT_COMPOUND_ID, new SoftReference(element));
/*      */     
/* 1485 */     Float fsFloat = TextUtilities.convertFontSize(element);
/* 1486 */     float fontSize = fsFloat.floatValue();
/*      */     
/* 1488 */     result.put(TextAttribute.SIZE, fsFloat);
/*      */ 
/*      */     
/* 1491 */     result.put(TextAttribute.WIDTH, TextUtilities.convertFontStretch(element));
/*      */ 
/*      */ 
/*      */     
/* 1495 */     result.put(TextAttribute.POSTURE, TextUtilities.convertFontStyle(element));
/*      */ 
/*      */ 
/*      */     
/* 1499 */     result.put(TextAttribute.WEIGHT, TextUtilities.convertFontWeight(element));
/*      */ 
/*      */ 
/*      */     
/* 1503 */     Value v = CSSUtilities.getComputedStyle(element, 27);
/*      */     
/* 1505 */     String fontWeightString = v.getCssText();
/*      */ 
/*      */     
/* 1508 */     String fontStyleString = CSSUtilities.getComputedStyle(element, 25).getStringValue();
/*      */ 
/*      */ 
/*      */     
/* 1512 */     result.put(TEXT_COMPOUND_DELIMITER, element);
/*      */ 
/*      */     
/* 1515 */     Value val = CSSUtilities.getComputedStyle(element, 21);
/*      */     
/* 1517 */     List fontFamilyList = new ArrayList();
/* 1518 */     List fontList = new ArrayList();
/* 1519 */     int len = val.getLength();
/* 1520 */     for (int i = 0; i < len; i++) {
/* 1521 */       Value it = val.item(i);
/* 1522 */       String fontFamilyName = it.getStringValue();
/*      */       
/* 1524 */       GVTFontFamily fontFamily = SVGFontUtilities.getFontFamily(element, ctx, fontFamilyName, fontWeightString, fontStyleString);
/*      */ 
/*      */       
/* 1527 */       if (fontFamily == null)
/* 1528 */         continue;  if (fontFamily instanceof UnresolvedFontFamily) {
/* 1529 */         fontFamily = FontFamilyResolver.resolve((UnresolvedFontFamily)fontFamily);
/*      */         
/* 1531 */         if (fontFamily == null)
/*      */           continue; 
/* 1533 */       }  fontFamilyList.add(fontFamily);
/* 1534 */       if (fontFamily instanceof SVGFontFamily) {
/* 1535 */         SVGFontFamily svgFF = (SVGFontFamily)fontFamily;
/* 1536 */         if (svgFF.isComplex()) {
/* 1537 */           this.usingComplexSVGFont = true;
/*      */         }
/*      */       } 
/* 1540 */       GVTFont ft = fontFamily.deriveFont(fontSize, result);
/* 1541 */       fontList.add(ft);
/*      */       
/*      */       continue;
/*      */     } 
/*      */     
/* 1546 */     result.put(GVT_FONT_FAMILIES, fontFamilyList);
/*      */     
/* 1548 */     if (!ctx.isDynamic())
/*      */     {
/*      */ 
/*      */       
/* 1552 */       result.remove(TEXT_COMPOUND_DELIMITER);
/*      */     }
/* 1554 */     return fontList;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Map getAttributeMap(BridgeContext ctx, Element element, TextPath textPath, Integer bidiLevel, Map result) {
/* 1576 */     SVGTextContentElement tce = null;
/* 1577 */     if (element instanceof SVGTextContentElement)
/*      */     {
/*      */       
/* 1580 */       tce = (SVGTextContentElement)element;
/*      */     }
/*      */     
/* 1583 */     Map inheritMap = null;
/*      */ 
/*      */     
/* 1586 */     if ("http://www.w3.org/2000/svg".equals(element.getNamespaceURI()) && element.getLocalName().equals("altGlyph"))
/*      */     {
/* 1588 */       result.put(ALT_GLYPH_HANDLER, new SVGAltGlyphHandler(ctx, element));
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1594 */     TextPaintInfo pi = new TextPaintInfo();
/*      */     
/* 1596 */     pi.visible = true;
/* 1597 */     pi.fillPaint = Color.black;
/* 1598 */     result.put(PAINT_INFO, pi);
/* 1599 */     this.elemTPI.put(element, pi);
/*      */     
/* 1601 */     if (textPath != null) {
/* 1602 */       result.put(TEXTPATH, textPath);
/*      */     }
/*      */ 
/*      */     
/* 1606 */     TextNode.Anchor a = TextUtilities.convertTextAnchor(element);
/* 1607 */     result.put(ANCHOR_TYPE, a);
/*      */ 
/*      */     
/* 1610 */     List fontList = getFontList(ctx, element, result);
/* 1611 */     result.put(GVT_FONTS, fontList);
/*      */ 
/*      */ 
/*      */     
/* 1615 */     Object bs = TextUtilities.convertBaselineShift(element);
/* 1616 */     if (bs != null) {
/* 1617 */       result.put(BASELINE_SHIFT, bs);
/*      */     }
/*      */ 
/*      */     
/* 1621 */     Value val = CSSUtilities.getComputedStyle(element, 56);
/*      */     
/* 1623 */     String s = val.getStringValue();
/* 1624 */     if (s.charAt(0) == 'n') {
/* 1625 */       if (bidiLevel != null) {
/* 1626 */         result.put(TextAttribute.BIDI_EMBEDDING, bidiLevel);
/*      */ 
/*      */ 
/*      */       
/*      */       }
/*      */ 
/*      */ 
/*      */     
/*      */     }
/*      */     else {
/*      */ 
/*      */ 
/*      */       
/* 1639 */       val = CSSUtilities.getComputedStyle(element, 11);
/*      */       
/* 1641 */       String rs = val.getStringValue();
/* 1642 */       int cbidi = 0;
/* 1643 */       if (bidiLevel != null) cbidi = bidiLevel.intValue();
/*      */ 
/*      */ 
/*      */       
/* 1647 */       if (cbidi < 0) cbidi = -cbidi;
/*      */       
/* 1649 */       switch (rs.charAt(0)) {
/*      */         case 'l':
/* 1651 */           result.put(TextAttribute.RUN_DIRECTION, TextAttribute.RUN_DIRECTION_LTR);
/*      */           
/* 1653 */           if ((cbidi & true) == 1) { cbidi++; break; }
/* 1654 */            cbidi += 2;
/*      */           break;
/*      */         case 'r':
/* 1657 */           result.put(TextAttribute.RUN_DIRECTION, TextAttribute.RUN_DIRECTION_RTL);
/*      */           
/* 1659 */           if ((cbidi & true) == 1) { cbidi += 2; break; }
/* 1660 */            cbidi++;
/*      */           break;
/*      */       } 
/*      */       
/* 1664 */       switch (s.charAt(0)) {
/*      */         case 'b':
/* 1666 */           cbidi = -cbidi;
/*      */           break;
/*      */       } 
/*      */       
/* 1670 */       result.put(TextAttribute.BIDI_EMBEDDING, new Integer(cbidi));
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/* 1675 */     val = CSSUtilities.getComputedStyle(element, 59);
/*      */     
/* 1677 */     s = val.getStringValue();
/* 1678 */     switch (s.charAt(0)) {
/*      */       case 'l':
/* 1680 */         result.put(GVTAttributedCharacterIterator.TextAttribute.WRITING_MODE, GVTAttributedCharacterIterator.TextAttribute.WRITING_MODE_LTR);
/*      */         break;
/*      */ 
/*      */ 
/*      */       
/*      */       case 'r':
/* 1686 */         result.put(GVTAttributedCharacterIterator.TextAttribute.WRITING_MODE, GVTAttributedCharacterIterator.TextAttribute.WRITING_MODE_RTL);
/*      */         break;
/*      */ 
/*      */ 
/*      */       
/*      */       case 't':
/* 1692 */         result.put(GVTAttributedCharacterIterator.TextAttribute.WRITING_MODE, GVTAttributedCharacterIterator.TextAttribute.WRITING_MODE_TTB);
/*      */         break;
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1701 */     val = CSSUtilities.getComputedStyle(element, 29);
/*      */     
/* 1703 */     int primitiveType = val.getPrimitiveType();
/* 1704 */     switch (primitiveType) {
/*      */       case 21:
/* 1706 */         result.put(GVTAttributedCharacterIterator.TextAttribute.VERTICAL_ORIENTATION, GVTAttributedCharacterIterator.TextAttribute.ORIENTATION_AUTO);
/*      */         break;
/*      */ 
/*      */ 
/*      */       
/*      */       case 11:
/* 1712 */         result.put(GVTAttributedCharacterIterator.TextAttribute.VERTICAL_ORIENTATION, GVTAttributedCharacterIterator.TextAttribute.ORIENTATION_ANGLE);
/*      */ 
/*      */ 
/*      */         
/* 1716 */         result.put(GVTAttributedCharacterIterator.TextAttribute.VERTICAL_ORIENTATION_ANGLE, new Float(val.getFloatValue()));
/*      */         break;
/*      */ 
/*      */       
/*      */       case 12:
/* 1721 */         result.put(GVTAttributedCharacterIterator.TextAttribute.VERTICAL_ORIENTATION, GVTAttributedCharacterIterator.TextAttribute.ORIENTATION_ANGLE);
/*      */ 
/*      */ 
/*      */         
/* 1725 */         result.put(GVTAttributedCharacterIterator.TextAttribute.VERTICAL_ORIENTATION_ANGLE, new Float(Math.toDegrees(val.getFloatValue())));
/*      */         break;
/*      */ 
/*      */       
/*      */       case 13:
/* 1730 */         result.put(GVTAttributedCharacterIterator.TextAttribute.VERTICAL_ORIENTATION, GVTAttributedCharacterIterator.TextAttribute.ORIENTATION_ANGLE);
/*      */ 
/*      */ 
/*      */         
/* 1734 */         result.put(GVTAttributedCharacterIterator.TextAttribute.VERTICAL_ORIENTATION_ANGLE, new Float(val.getFloatValue() * 9.0F / 5.0F));
/*      */         break;
/*      */ 
/*      */ 
/*      */       
/*      */       default:
/* 1740 */         throw new IllegalStateException("unexpected primitiveType (V):" + primitiveType);
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/* 1745 */     val = CSSUtilities.getComputedStyle(element, 28);
/*      */     
/* 1747 */     primitiveType = val.getPrimitiveType();
/* 1748 */     switch (primitiveType) {
/*      */       case 11:
/* 1750 */         result.put(GVTAttributedCharacterIterator.TextAttribute.HORIZONTAL_ORIENTATION_ANGLE, new Float(val.getFloatValue()));
/*      */         break;
/*      */ 
/*      */       
/*      */       case 12:
/* 1755 */         result.put(GVTAttributedCharacterIterator.TextAttribute.HORIZONTAL_ORIENTATION_ANGLE, new Float(Math.toDegrees(val.getFloatValue())));
/*      */         break;
/*      */ 
/*      */       
/*      */       case 13:
/* 1760 */         result.put(GVTAttributedCharacterIterator.TextAttribute.HORIZONTAL_ORIENTATION_ANGLE, new Float(val.getFloatValue() * 9.0F / 5.0F));
/*      */         break;
/*      */ 
/*      */ 
/*      */       
/*      */       default:
/* 1766 */         throw new IllegalStateException("unexpected primitiveType (H):" + primitiveType);
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1772 */     Float sp = TextUtilities.convertLetterSpacing(element);
/* 1773 */     if (sp != null) {
/* 1774 */       result.put(GVTAttributedCharacterIterator.TextAttribute.LETTER_SPACING, sp);
/*      */ 
/*      */       
/* 1777 */       result.put(GVTAttributedCharacterIterator.TextAttribute.CUSTOM_SPACING, Boolean.TRUE);
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1783 */     sp = TextUtilities.convertWordSpacing(element);
/* 1784 */     if (sp != null) {
/* 1785 */       result.put(GVTAttributedCharacterIterator.TextAttribute.WORD_SPACING, sp);
/*      */ 
/*      */       
/* 1788 */       result.put(GVTAttributedCharacterIterator.TextAttribute.CUSTOM_SPACING, Boolean.TRUE);
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1794 */     sp = TextUtilities.convertKerning(element);
/* 1795 */     if (sp != null) {
/* 1796 */       result.put(GVTAttributedCharacterIterator.TextAttribute.KERNING, sp);
/*      */       
/* 1798 */       result.put(GVTAttributedCharacterIterator.TextAttribute.CUSTOM_SPACING, Boolean.TRUE);
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/* 1803 */     if (tce == null) {
/* 1804 */       return inheritMap;
/*      */     }
/*      */ 
/*      */     
/*      */     try {
/* 1809 */       AbstractSVGAnimatedLength textLength = (AbstractSVGAnimatedLength)tce.getTextLength();
/*      */       
/* 1811 */       if (textLength.isSpecified()) {
/* 1812 */         if (inheritMap == null) {
/* 1813 */           inheritMap = new HashMap();
/*      */         }
/*      */         
/* 1816 */         Object value = new Float(textLength.getCheckedValue());
/* 1817 */         result.put(GVTAttributedCharacterIterator.TextAttribute.BBOX_WIDTH, value);
/*      */ 
/*      */         
/* 1820 */         inheritMap.put(GVTAttributedCharacterIterator.TextAttribute.BBOX_WIDTH, value);
/*      */ 
/*      */ 
/*      */ 
/*      */         
/* 1825 */         SVGOMAnimatedEnumeration _lengthAdjust = (SVGOMAnimatedEnumeration)tce.getLengthAdjust();
/*      */         
/* 1827 */         if (_lengthAdjust.getCheckedVal() == 2)
/*      */         {
/* 1829 */           result.put(GVTAttributedCharacterIterator.TextAttribute.LENGTH_ADJUST, GVTAttributedCharacterIterator.TextAttribute.ADJUST_ALL);
/*      */ 
/*      */ 
/*      */           
/* 1833 */           inheritMap.put(GVTAttributedCharacterIterator.TextAttribute.LENGTH_ADJUST, GVTAttributedCharacterIterator.TextAttribute.ADJUST_ALL);
/*      */         
/*      */         }
/*      */         else
/*      */         {
/* 1838 */           result.put(GVTAttributedCharacterIterator.TextAttribute.LENGTH_ADJUST, GVTAttributedCharacterIterator.TextAttribute.ADJUST_SPACING);
/*      */ 
/*      */ 
/*      */           
/* 1842 */           inheritMap.put(GVTAttributedCharacterIterator.TextAttribute.LENGTH_ADJUST, GVTAttributedCharacterIterator.TextAttribute.ADJUST_SPACING);
/*      */ 
/*      */ 
/*      */           
/* 1846 */           result.put(GVTAttributedCharacterIterator.TextAttribute.CUSTOM_SPACING, Boolean.TRUE);
/*      */ 
/*      */           
/* 1849 */           inheritMap.put(GVTAttributedCharacterIterator.TextAttribute.CUSTOM_SPACING, Boolean.TRUE);
/*      */         }
/*      */       
/*      */       } 
/*      */     } catch (LiveAttributeException ex) {
/*      */       
/* 1855 */       throw new BridgeException(ctx, ex);
/*      */     } 
/*      */     
/* 1858 */     return inheritMap;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected TextPaintInfo getParentTextPaintInfo(Element child) {
/* 1871 */     Node parent = getParentNode(child);
/* 1872 */     while (parent != null) {
/* 1873 */       TextPaintInfo tpi = (TextPaintInfo)this.elemTPI.get(parent);
/* 1874 */       if (tpi != null) return tpi; 
/* 1875 */       parent = getParentNode(parent);
/*      */     } 
/* 1877 */     return null;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected TextPaintInfo getTextPaintInfo(Element element, GraphicsNode node, TextPaintInfo parentTPI, BridgeContext ctx) {
/* 1890 */     CSSUtilities.getComputedStyle(element, 54);
/*      */ 
/*      */     
/* 1893 */     TextPaintInfo pi = new TextPaintInfo(parentTPI);
/*      */ 
/*      */     
/* 1896 */     StyleMap sm = ((CSSStylableElement)element).getComputedStyleMap(null);
/* 1897 */     if (sm.isNullCascaded(54) && sm.isNullCascaded(15) && sm.isNullCascaded(45) && sm.isNullCascaded(52) && sm.isNullCascaded(38))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1903 */       return pi;
/*      */     }
/*      */     
/* 1906 */     setBaseTextPaintInfo(pi, element, node, ctx);
/*      */     
/* 1908 */     if (!sm.isNullCascaded(54)) {
/* 1909 */       setDecorationTextPaintInfo(pi, element);
/*      */     }
/* 1911 */     return pi;
/*      */   }
/*      */ 
/*      */   
/*      */   public void setBaseTextPaintInfo(TextPaintInfo pi, Element element, GraphicsNode node, BridgeContext ctx) {
/* 1916 */     if (!element.getLocalName().equals("text")) {
/* 1917 */       pi.composite = CSSUtilities.convertOpacity(element);
/*      */     } else {
/* 1919 */       pi.composite = AlphaComposite.SrcOver;
/*      */     } 
/* 1921 */     pi.visible = CSSUtilities.convertVisibility(element);
/* 1922 */     pi.fillPaint = PaintServer.convertFillPaint(element, node, ctx);
/* 1923 */     pi.strokePaint = PaintServer.convertStrokePaint(element, node, ctx);
/* 1924 */     pi.strokeStroke = PaintServer.convertStroke(element);
/*      */   } public void setDecorationTextPaintInfo(TextPaintInfo pi, Element element) {
/*      */     int i, len;
/*      */     ListValue lst;
/* 1928 */     Value val = CSSUtilities.getComputedStyle(element, 54);
/*      */ 
/*      */     
/* 1931 */     switch (val.getCssValueType()) {
/*      */       case 2:
/* 1933 */         lst = (ListValue)val;
/*      */         
/* 1935 */         len = lst.getLength();
/* 1936 */         for (i = 0; i < len; i++) {
/* 1937 */           Value v = lst.item(i);
/* 1938 */           String s = v.getStringValue();
/* 1939 */           switch (s.charAt(0)) {
/*      */             case 'u':
/* 1941 */               if (pi.fillPaint != null) {
/* 1942 */                 pi.underlinePaint = pi.fillPaint;
/*      */               }
/* 1944 */               if (pi.strokePaint != null) {
/* 1945 */                 pi.underlineStrokePaint = pi.strokePaint;
/*      */               }
/* 1947 */               if (pi.strokeStroke != null) {
/* 1948 */                 pi.underlineStroke = pi.strokeStroke;
/*      */               }
/*      */               break;
/*      */             case 'o':
/* 1952 */               if (pi.fillPaint != null) {
/* 1953 */                 pi.overlinePaint = pi.fillPaint;
/*      */               }
/* 1955 */               if (pi.strokePaint != null) {
/* 1956 */                 pi.overlineStrokePaint = pi.strokePaint;
/*      */               }
/* 1958 */               if (pi.strokeStroke != null) {
/* 1959 */                 pi.overlineStroke = pi.strokeStroke;
/*      */               }
/*      */               break;
/*      */             case 'l':
/* 1963 */               if (pi.fillPaint != null) {
/* 1964 */                 pi.strikethroughPaint = pi.fillPaint;
/*      */               }
/* 1966 */               if (pi.strokePaint != null) {
/* 1967 */                 pi.strikethroughStrokePaint = pi.strokePaint;
/*      */               }
/* 1969 */               if (pi.strokeStroke != null) {
/* 1970 */                 pi.strikethroughStroke = pi.strokeStroke;
/*      */               }
/*      */               break;
/*      */           } 
/*      */         
/*      */         } 
/*      */         return;
/*      */     } 
/* 1978 */     pi.underlinePaint = null;
/* 1979 */     pi.underlineStrokePaint = null;
/* 1980 */     pi.underlineStroke = null;
/*      */     
/* 1982 */     pi.overlinePaint = null;
/* 1983 */     pi.overlineStrokePaint = null;
/* 1984 */     pi.overlineStroke = null;
/*      */     
/* 1986 */     pi.strikethroughPaint = null;
/* 1987 */     pi.strikethroughStrokePaint = null;
/* 1988 */     pi.strikethroughStroke = null;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public abstract class AbstractTextChildSVGContext
/*      */     extends AnimatableSVGBridge
/*      */   {
/*      */     protected SVGTextElementBridge textBridge;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     private final SVGTextElementBridge this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public AbstractTextChildSVGContext(SVGTextElementBridge this$0, BridgeContext ctx, SVGTextElementBridge parent, Element e) {
/* 2010 */       this.this$0 = this$0;
/* 2011 */       this.ctx = ctx;
/* 2012 */       this.textBridge = parent;
/* 2013 */       this.e = e;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2021 */     public String getNamespaceURI() { return null; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2029 */     public String getLocalName() { return null; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2036 */     public Bridge getInstance() { return null; }
/*      */ 
/*      */     
/* 2039 */     public SVGTextElementBridge getTextBridge() { return this.textBridge; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2045 */     public float getPixelUnitToMillimeter() { return this.ctx.getUserAgent().getPixelUnitToMillimeter(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2054 */     public float getPixelToMM() { return getPixelUnitToMillimeter(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2066 */     public Rectangle2D getBBox() { return null; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2077 */     public AffineTransform getCTM() { return null; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2086 */     public AffineTransform getGlobalTransform() { return null; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2095 */     public AffineTransform getScreenTransform() { return null; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void setScreenTransform(AffineTransform at) {}
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2112 */     public float getViewportWidth() { return this.ctx.getBlockWidth(this.e); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2120 */     public float getViewportHeight() { return this.ctx.getBlockHeight(this.e); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2127 */     public float getFontSize() { return CSSUtilities.getComputedStyle(this.e, 22).getFloatValue(); }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected abstract class AbstractTextChildBridgeUpdateHandler
/*      */     extends AbstractTextChildSVGContext
/*      */     implements BridgeUpdateHandler
/*      */   {
/*      */     private final SVGTextElementBridge this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected AbstractTextChildBridgeUpdateHandler(SVGTextElementBridge this$0, BridgeContext ctx, SVGTextElementBridge parent, Element e) {
/* 2153 */       super(this$0, ctx, parent, e);
/*      */       this.this$0 = this$0;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleDOMAttrModifiedEvent(MutationEvent evt) {}
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2167 */     public void handleDOMNodeInsertedEvent(MutationEvent evt) { this.textBridge.handleDOMNodeInsertedEvent(evt); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleDOMNodeRemovedEvent(MutationEvent evt) {}
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2181 */     public void handleDOMCharacterDataModified(MutationEvent evt) { this.textBridge.handleDOMCharacterDataModified(evt); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 2188 */     public void handleCSSEngineEvent(CSSEngineEvent evt) { this.textBridge.handleCSSEngineEvent(evt); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleAnimatedAttributeChanged(AnimatedLiveAttributeValue alav) {}
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleOtherAnimationChanged(String type) {}
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void dispose() {
/* 2209 */       ((SVGOMElement)this.e).setSVGContext(null);
/* 2210 */       this.this$0.elemTPI.remove(this.e);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class AbstractTextChildTextContent
/*      */     extends AbstractTextChildBridgeUpdateHandler
/*      */     implements SVGTextContent
/*      */   {
/*      */     private final SVGTextElementBridge this$0;
/*      */ 
/*      */ 
/*      */     
/*      */     protected AbstractTextChildTextContent(SVGTextElementBridge this$0, BridgeContext ctx, SVGTextElementBridge parent, Element e) {
/* 2226 */       super(this$0, ctx, parent, e);
/*      */       this.this$0 = this$0;
/*      */     }
/*      */ 
/*      */ 
/*      */     
/* 2232 */     public int getNumberOfChars() { return this.textBridge.getNumberOfChars(this.e); }
/*      */ 
/*      */ 
/*      */     
/* 2236 */     public Rectangle2D getExtentOfChar(int charnum) { return this.textBridge.getExtentOfChar(this.e, charnum); }
/*      */ 
/*      */ 
/*      */     
/* 2240 */     public Point2D getStartPositionOfChar(int charnum) { return this.textBridge.getStartPositionOfChar(this.e, charnum); }
/*      */ 
/*      */ 
/*      */     
/* 2244 */     public Point2D getEndPositionOfChar(int charnum) { return this.textBridge.getEndPositionOfChar(this.e, charnum); }
/*      */ 
/*      */ 
/*      */     
/* 2248 */     public void selectSubString(int charnum, int nchars) { this.textBridge.selectSubString(this.e, charnum, nchars); }
/*      */ 
/*      */ 
/*      */     
/* 2252 */     public float getRotationOfChar(int charnum) { return this.textBridge.getRotationOfChar(this.e, charnum); }
/*      */ 
/*      */ 
/*      */     
/* 2256 */     public float getComputedTextLength() { return this.textBridge.getComputedTextLength(this.e); }
/*      */ 
/*      */ 
/*      */     
/* 2260 */     public float getSubStringLength(int charnum, int nchars) { return this.textBridge.getSubStringLength(this.e, charnum, nchars); }
/*      */ 
/*      */ 
/*      */     
/* 2264 */     public int getCharNumAtPosition(float x, float y) { return this.textBridge.getCharNumAtPosition(this.e, x, y); }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class TRefBridge
/*      */     extends AbstractTextChildTextContent
/*      */   {
/*      */     private final SVGTextElementBridge this$0;
/*      */ 
/*      */ 
/*      */     
/*      */     protected TRefBridge(SVGTextElementBridge this$0, BridgeContext ctx, SVGTextElementBridge parent, Element e) {
/* 2277 */       super(this$0, ctx, parent, e);
/*      */       this.this$0 = this$0;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleAnimatedAttributeChanged(AnimatedLiveAttributeValue alav) {
/* 2286 */       if (alav.getNamespaceURI() == null) {
/* 2287 */         String ln = alav.getLocalName();
/* 2288 */         if (ln.equals("x") || ln.equals("y") || ln.equals("dx") || ln.equals("dy") || ln.equals("rotate") || ln.equals("textLength") || ln.equals("lengthAdjust")) {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/* 2296 */           this.textBridge.computeLaidoutText(this.ctx, this.textBridge.e, this.textBridge.getTextNode());
/*      */           
/*      */           return;
/*      */         } 
/*      */       } 
/* 2301 */       super.handleAnimatedAttributeChanged(alav);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class TextPathBridge
/*      */     extends AbstractTextChildTextContent
/*      */   {
/*      */     private final SVGTextElementBridge this$0;
/*      */ 
/*      */     
/*      */     protected TextPathBridge(SVGTextElementBridge this$0, BridgeContext ctx, SVGTextElementBridge parent, Element e) {
/* 2314 */       super(this$0, ctx, parent, e);
/*      */       this.this$0 = this$0;
/*      */     }
/*      */   }
/*      */ 
/*      */   
/*      */   protected class TspanBridge
/*      */     extends AbstractTextChildTextContent
/*      */   {
/*      */     private final SVGTextElementBridge this$0;
/*      */ 
/*      */     
/*      */     protected TspanBridge(SVGTextElementBridge this$0, BridgeContext ctx, SVGTextElementBridge parent, Element e) {
/* 2327 */       super(this$0, ctx, parent, e);
/*      */       this.this$0 = this$0;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleAnimatedAttributeChanged(AnimatedLiveAttributeValue alav) {
/* 2336 */       if (alav.getNamespaceURI() == null) {
/* 2337 */         String ln = alav.getLocalName();
/* 2338 */         if (ln.equals("x") || ln.equals("y") || ln.equals("dx") || ln.equals("dy") || ln.equals("rotate") || ln.equals("textLength") || ln.equals("lengthAdjust")) {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/* 2346 */           this.textBridge.computeLaidoutText(this.ctx, this.textBridge.e, this.textBridge.getTextNode());
/*      */           
/*      */           return;
/*      */         } 
/*      */       } 
/* 2351 */       super.handleAnimatedAttributeChanged(alav);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/* 2357 */   public int getNumberOfChars() { return getNumberOfChars(this.e); }
/*      */ 
/*      */ 
/*      */   
/* 2361 */   public Rectangle2D getExtentOfChar(int charnum) { return getExtentOfChar(this.e, charnum); }
/*      */ 
/*      */ 
/*      */   
/* 2365 */   public Point2D getStartPositionOfChar(int charnum) { return getStartPositionOfChar(this.e, charnum); }
/*      */ 
/*      */ 
/*      */   
/* 2369 */   public Point2D getEndPositionOfChar(int charnum) { return getEndPositionOfChar(this.e, charnum); }
/*      */ 
/*      */ 
/*      */   
/* 2373 */   public void selectSubString(int charnum, int nchars) { selectSubString(this.e, charnum, nchars); }
/*      */ 
/*      */ 
/*      */   
/* 2377 */   public float getRotationOfChar(int charnum) { return getRotationOfChar(this.e, charnum); }
/*      */ 
/*      */ 
/*      */   
/* 2381 */   public float getComputedTextLength() { return getComputedTextLength(this.e); }
/*      */ 
/*      */ 
/*      */   
/* 2385 */   public float getSubStringLength(int charnum, int nchars) { return getSubStringLength(this.e, charnum, nchars); }
/*      */ 
/*      */ 
/*      */   
/* 2389 */   public int getCharNumAtPosition(float x, float y) { return getCharNumAtPosition(this.e, x, y); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected int getNumberOfChars(Element element) {
/* 2399 */     AttributedCharacterIterator aci = getTextNode().getAttributedCharacterIterator();
/* 2400 */     if (aci == null) {
/* 2401 */       return 0;
/*      */     }
/*      */ 
/*      */     
/* 2405 */     int firstChar = getElementStartIndex(element);
/*      */     
/* 2407 */     if (firstChar == -1) {
/* 2408 */       return 0;
/*      */     }
/* 2410 */     int lastChar = getElementEndIndex(element);
/*      */     
/* 2412 */     return lastChar - firstChar + 1;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Rectangle2D getExtentOfChar(Element element, int charnum) {
/* 2421 */     TextNode textNode = getTextNode();
/*      */ 
/*      */     
/* 2424 */     AttributedCharacterIterator aci = textNode.getAttributedCharacterIterator();
/* 2425 */     if (aci == null) return null;
/*      */     
/* 2427 */     int firstChar = getElementStartIndex(element);
/*      */     
/* 2429 */     if (firstChar == -1) {
/* 2430 */       return null;
/*      */     }
/*      */     
/* 2433 */     List list = getTextRuns(textNode);
/*      */ 
/*      */ 
/*      */     
/* 2437 */     CharacterInformation info = getCharacterInformation(list, firstChar, charnum, aci);
/*      */     
/* 2439 */     if (info == null) {
/* 2440 */       return null;
/*      */     }
/*      */ 
/*      */     
/* 2444 */     GVTGlyphVector it = info.layout.getGlyphVector();
/*      */     
/* 2446 */     Shape b = null;
/*      */     
/* 2448 */     if (info.glyphIndexStart == info.glyphIndexEnd) {
/* 2449 */       if (it.isGlyphVisible(info.glyphIndexStart)) {
/* 2450 */         b = it.getGlyphCellBounds(info.glyphIndexStart);
/*      */       }
/*      */     } else {
/* 2453 */       GeneralPath path = null;
/* 2454 */       for (int k = info.glyphIndexStart; k <= info.glyphIndexEnd; k++) {
/* 2455 */         if (it.isGlyphVisible(k)) {
/* 2456 */           Rectangle2D gb = it.getGlyphCellBounds(k);
/* 2457 */           if (path == null) {
/* 2458 */             path = new GeneralPath(gb);
/*      */           } else {
/* 2460 */             path.append(gb, false);
/*      */           } 
/*      */         } 
/*      */       } 
/* 2464 */       b = path;
/*      */     } 
/*      */     
/* 2467 */     if (b == null) {
/* 2468 */       return null;
/*      */     }
/*      */ 
/*      */     
/* 2472 */     return b.getBounds2D();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Point2D getStartPositionOfChar(Element element, int charnum) {
/* 2481 */     TextNode textNode = getTextNode();
/*      */ 
/*      */     
/* 2484 */     AttributedCharacterIterator aci = textNode.getAttributedCharacterIterator();
/* 2485 */     if (aci == null) {
/* 2486 */       return null;
/*      */     }
/* 2488 */     int firstChar = getElementStartIndex(element);
/* 2489 */     if (firstChar == -1) {
/* 2490 */       return null;
/*      */     }
/*      */     
/* 2493 */     List list = getTextRuns(textNode);
/*      */ 
/*      */ 
/*      */     
/* 2497 */     CharacterInformation info = getCharacterInformation(list, firstChar, charnum, aci);
/*      */     
/* 2499 */     if (info == null) {
/* 2500 */       return null;
/*      */     }
/* 2502 */     return getStartPoint(info);
/*      */   }
/*      */ 
/*      */   
/*      */   protected Point2D getStartPoint(CharacterInformation info) {
/* 2507 */     GVTGlyphVector it = info.layout.getGlyphVector();
/* 2508 */     if (!it.isGlyphVisible(info.glyphIndexStart)) {
/* 2509 */       return null;
/*      */     }
/* 2511 */     Point2D b = it.getGlyphPosition(info.glyphIndexStart);
/*      */ 
/*      */     
/* 2514 */     AffineTransform glyphTransform = it.getGlyphTransform(info.glyphIndexStart);
/*      */ 
/*      */ 
/*      */     
/* 2518 */     Point2D.Float result = new Point2D.Float(0.0F, 0.0F);
/* 2519 */     if (glyphTransform != null)
/*      */     {
/* 2521 */       glyphTransform.transform(result, result);
/*      */     }
/* 2523 */     result.x = (float)(result.x + b.getX());
/* 2524 */     result.y = (float)(result.y + b.getY());
/* 2525 */     return result;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Point2D getEndPositionOfChar(Element element, int charnum) {
/* 2533 */     TextNode textNode = getTextNode();
/*      */ 
/*      */     
/* 2536 */     AttributedCharacterIterator aci = textNode.getAttributedCharacterIterator();
/* 2537 */     if (aci == null) {
/* 2538 */       return null;
/*      */     }
/* 2540 */     int firstChar = getElementStartIndex(element);
/* 2541 */     if (firstChar == -1) {
/* 2542 */       return null;
/*      */     }
/*      */     
/* 2545 */     List list = getTextRuns(textNode);
/*      */ 
/*      */ 
/*      */     
/* 2549 */     CharacterInformation info = getCharacterInformation(list, firstChar, charnum, aci);
/*      */     
/* 2551 */     if (info == null)
/* 2552 */       return null; 
/* 2553 */     return getEndPoint(info);
/*      */   }
/*      */ 
/*      */   
/*      */   protected Point2D getEndPoint(CharacterInformation info) {
/* 2558 */     GVTGlyphVector it = info.layout.getGlyphVector();
/* 2559 */     if (!it.isGlyphVisible(info.glyphIndexEnd)) {
/* 2560 */       return null;
/*      */     }
/* 2562 */     Point2D b = it.getGlyphPosition(info.glyphIndexEnd);
/*      */ 
/*      */     
/* 2565 */     AffineTransform glyphTransform = it.getGlyphTransform(info.glyphIndexEnd);
/*      */     
/* 2567 */     GVTGlyphMetrics metrics = it.getGlyphMetrics(info.glyphIndexEnd);
/*      */ 
/*      */     
/* 2570 */     Point2D.Float result = new Point2D.Float(metrics.getHorizontalAdvance(), 0.0F);
/*      */ 
/*      */     
/* 2573 */     if (glyphTransform != null) {
/* 2574 */       glyphTransform.transform(result, result);
/*      */     }
/* 2576 */     result.x = (float)(result.x + b.getX());
/* 2577 */     result.y = (float)(result.y + b.getY());
/* 2578 */     return result;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected float getRotationOfChar(Element element, int charnum) {
/* 2586 */     TextNode textNode = getTextNode();
/*      */ 
/*      */     
/* 2589 */     AttributedCharacterIterator aci = textNode.getAttributedCharacterIterator();
/* 2590 */     if (aci == null) {
/* 2591 */       return 0.0F;
/*      */     }
/*      */     
/* 2594 */     int firstChar = getElementStartIndex(element);
/* 2595 */     if (firstChar == -1) {
/* 2596 */       return 0.0F;
/*      */     }
/*      */     
/* 2599 */     List list = getTextRuns(textNode);
/*      */ 
/*      */ 
/*      */     
/* 2603 */     CharacterInformation info = getCharacterInformation(list, firstChar, charnum, aci);
/*      */     
/* 2605 */     double angle = 0.0D;
/* 2606 */     int nbGlyphs = 0;
/*      */     
/* 2608 */     if (info != null) {
/* 2609 */       GVTGlyphVector it = info.layout.getGlyphVector();
/*      */       
/* 2611 */       int k = info.glyphIndexStart;
/* 2612 */       for (; k <= info.glyphIndexEnd; 
/* 2613 */         k++) {
/* 2614 */         if (it.isGlyphVisible(k)) {
/*      */           
/* 2616 */           nbGlyphs++;
/*      */ 
/*      */           
/* 2619 */           AffineTransform glyphTransform = it.getGlyphTransform(k);
/* 2620 */           if (glyphTransform != null)
/*      */           
/* 2622 */           { double glyphAngle = 0.0D;
/* 2623 */             double cosTheta = glyphTransform.getScaleX();
/* 2624 */             double sinTheta = glyphTransform.getShearX();
/*      */ 
/*      */             
/* 2627 */             if (cosTheta == 0.0D)
/* 2628 */             { if (sinTheta > 0.0D) { glyphAngle = Math.PI; }
/* 2629 */               else { glyphAngle = -3.141592653589793D; }
/*      */                }
/* 2631 */             else { glyphAngle = Math.atan(sinTheta / cosTheta);
/* 2632 */               if (cosTheta < 0.0D) {
/* 2633 */                 glyphAngle += Math.PI;
/*      */               } }
/*      */ 
/*      */ 
/*      */             
/* 2638 */             glyphAngle = Math.toDegrees(-glyphAngle) % 360.0D;
/*      */ 
/*      */             
/* 2641 */             angle += glyphAngle - info.getComputedOrientationAngle(); } 
/*      */         } 
/*      */       } 
/* 2644 */     }  if (nbGlyphs == 0) return 0.0F; 
/* 2645 */     return (float)(angle / nbGlyphs);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 2653 */   protected float getComputedTextLength(Element e) { return getSubStringLength(e, 0, getNumberOfChars(e)); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected float getSubStringLength(Element element, int charnum, int nchars) {
/* 2663 */     if (nchars == 0) {
/* 2664 */       return 0.0F;
/*      */     }
/*      */     
/* 2667 */     float length = 0.0F;
/*      */     
/* 2669 */     TextNode textNode = getTextNode();
/*      */ 
/*      */     
/* 2672 */     AttributedCharacterIterator aci = textNode.getAttributedCharacterIterator();
/* 2673 */     if (aci == null) {
/* 2674 */       return -1.0F;
/*      */     }
/* 2676 */     int firstChar = getElementStartIndex(element);
/*      */     
/* 2678 */     if (firstChar == -1) {
/* 2679 */       return -1.0F;
/*      */     }
/* 2681 */     List list = getTextRuns(textNode);
/*      */ 
/*      */     
/* 2684 */     CharacterInformation currentInfo = getCharacterInformation(list, firstChar, charnum, aci);
/* 2685 */     CharacterInformation lastCharacterInRunInfo = null;
/* 2686 */     int chIndex = currentInfo.characterIndex + 1;
/* 2687 */     GVTGlyphVector vector = currentInfo.layout.getGlyphVector();
/* 2688 */     float[] advs = currentInfo.layout.getGlyphAdvances();
/* 2689 */     boolean[] glyphTrack = new boolean[advs.length];
/* 2690 */     for (int k = charnum + 1; k < charnum + nchars; k++) {
/* 2691 */       if (currentInfo.layout.isOnATextPath()) {
/* 2692 */         int gi = currentInfo.glyphIndexStart;
/* 2693 */         for (; gi <= currentInfo.glyphIndexEnd; gi++) {
/* 2694 */           if (vector.isGlyphVisible(gi) && !glyphTrack[gi])
/* 2695 */             length += advs[gi + 1] - advs[gi]; 
/* 2696 */           glyphTrack[gi] = true;
/*      */         } 
/*      */         
/* 2699 */         CharacterInformation newInfo = getCharacterInformation(list, firstChar, k, aci);
/* 2700 */         if (newInfo.layout != currentInfo.layout) {
/* 2701 */           vector = newInfo.layout.getGlyphVector();
/* 2702 */           advs = newInfo.layout.getGlyphAdvances();
/* 2703 */           glyphTrack = new boolean[advs.length];
/* 2704 */           chIndex = currentInfo.characterIndex + 1;
/*      */         } 
/* 2706 */         currentInfo = newInfo;
/*      */       
/*      */       }
/* 2709 */       else if (currentInfo.layout.hasCharacterIndex(chIndex)) {
/* 2710 */         chIndex++;
/*      */       }
/*      */       else {
/*      */         
/* 2714 */         lastCharacterInRunInfo = getCharacterInformation(list, firstChar, k - 1, aci);
/*      */ 
/*      */ 
/*      */ 
/*      */         
/* 2719 */         length += distanceFirstLastCharacterInRun(currentInfo, lastCharacterInRunInfo);
/*      */ 
/*      */         
/* 2722 */         currentInfo = getCharacterInformation(list, firstChar, k, aci);
/* 2723 */         chIndex = currentInfo.characterIndex + 1;
/* 2724 */         vector = currentInfo.layout.getGlyphVector();
/* 2725 */         advs = currentInfo.layout.getGlyphAdvances();
/* 2726 */         glyphTrack = new boolean[advs.length];
/* 2727 */         lastCharacterInRunInfo = null;
/*      */       } 
/*      */     } 
/*      */     
/* 2731 */     if (currentInfo.layout.isOnATextPath()) {
/* 2732 */       int gi = currentInfo.glyphIndexStart;
/* 2733 */       for (; gi <= currentInfo.glyphIndexEnd; gi++) {
/* 2734 */         if (vector.isGlyphVisible(gi) && !glyphTrack[gi])
/* 2735 */           length += advs[gi + 1] - advs[gi]; 
/* 2736 */         glyphTrack[gi] = true;
/*      */       } 
/*      */     } else {
/* 2739 */       if (lastCharacterInRunInfo == null) {
/* 2740 */         lastCharacterInRunInfo = getCharacterInformation(list, firstChar, charnum + nchars - 1, aci);
/*      */       }
/*      */ 
/*      */ 
/*      */       
/* 2745 */       length += distanceFirstLastCharacterInRun(currentInfo, lastCharacterInRunInfo);
/*      */     } 
/*      */ 
/*      */     
/* 2749 */     return length;
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected float distanceFirstLastCharacterInRun(CharacterInformation first, CharacterInformation last) {
/* 2755 */     float[] advs = first.layout.getGlyphAdvances();
/*      */     
/* 2757 */     int firstStart = first.glyphIndexStart;
/* 2758 */     int firstEnd = first.glyphIndexEnd;
/* 2759 */     int lastStart = last.glyphIndexStart;
/* 2760 */     int lastEnd = last.glyphIndexEnd;
/*      */     
/* 2762 */     int start = (firstStart < lastStart) ? firstStart : lastStart;
/* 2763 */     int end = (firstEnd < lastEnd) ? lastEnd : firstEnd;
/* 2764 */     return advs[end + 1] - advs[start];
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected float distanceBetweenRun(CharacterInformation last, CharacterInformation first) {
/*      */     float f;
/* 2773 */     CharacterInformation info = new CharacterInformation();
/*      */ 
/*      */ 
/*      */     
/* 2777 */     info.layout = last.layout;
/* 2778 */     info.glyphIndexEnd = last.layout.getGlyphCount() - 1;
/*      */     
/* 2780 */     Point2D startPoint = getEndPoint(info);
/*      */ 
/*      */     
/* 2783 */     info.layout = first.layout;
/* 2784 */     info.glyphIndexStart = 0;
/*      */     
/* 2786 */     Point2D endPoint = getStartPoint(info);
/*      */     
/* 2788 */     if (first.isVertical()) {
/* 2789 */       f = (float)(endPoint.getY() - startPoint.getY());
/*      */     } else {
/*      */       
/* 2792 */       f = (float)(endPoint.getX() - startPoint.getX());
/*      */     } 
/*      */     
/* 2795 */     return f;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void selectSubString(Element element, int charnum, int nchars) {
/*      */     Mark mark;
/* 2806 */     TextNode textNode = getTextNode();
/*      */ 
/*      */     
/* 2809 */     AttributedCharacterIterator aci = textNode.getAttributedCharacterIterator();
/* 2810 */     if (aci == null) {
/*      */       return;
/*      */     }
/* 2813 */     int firstChar = getElementStartIndex(element);
/*      */     
/* 2815 */     if (firstChar == -1) {
/*      */       return;
/*      */     }
/* 2818 */     List list = getTextRuns(textNode);
/*      */     
/* 2820 */     int lastChar = getElementEndIndex(element);
/*      */ 
/*      */     
/* 2823 */     CharacterInformation firstInfo = getCharacterInformation(list, firstChar, charnum, aci);
/* 2824 */     CharacterInformation lastInfo = getCharacterInformation(list, firstChar, charnum + nchars - 1, aci);
/*      */ 
/*      */     
/* 2827 */     Mark firstMark = textNode.getMarkerForChar(firstInfo.characterIndex, true);
/*      */     
/* 2829 */     if (lastInfo != null && lastInfo.characterIndex <= lastChar) {
/* 2830 */       mark = textNode.getMarkerForChar(lastInfo.characterIndex, false);
/*      */     } else {
/*      */       
/* 2833 */       mark = textNode.getMarkerForChar(lastChar, false);
/*      */     } 
/*      */     
/* 2836 */     this.ctx.getUserAgent().setTextSelection(firstMark, mark);
/*      */   }
/*      */   
/*      */   protected int getCharNumAtPosition(Element e, float x, float y) {
/* 2840 */     TextNode textNode = getTextNode();
/*      */ 
/*      */     
/* 2843 */     AttributedCharacterIterator aci = textNode.getAttributedCharacterIterator();
/* 2844 */     if (aci == null) {
/* 2845 */       return -1;
/*      */     }
/*      */     
/* 2848 */     List list = getTextRuns(textNode);
/*      */ 
/*      */ 
/*      */     
/* 2852 */     TextHit hit = null;
/*      */     
/* 2854 */     for (int i = list.size() - 1; i >= 0 && hit == null; i--) {
/*      */       
/* 2856 */       StrokingTextPainter.TextRun textRun = (StrokingTextPainter.TextRun)list.get(i);
/* 2857 */       hit = textRun.getLayout().hitTestChar(x, y);
/*      */     } 
/*      */     
/* 2860 */     if (hit == null) {
/* 2861 */       return -1;
/*      */     }
/*      */ 
/*      */     
/* 2865 */     int first = getElementStartIndex(e);
/* 2866 */     int last = getElementEndIndex(e);
/*      */     
/* 2868 */     int hitIndex = hit.getCharIndex();
/*      */     
/* 2870 */     if (hitIndex >= first && hitIndex <= last) {
/* 2871 */       return hitIndex - first;
/*      */     }
/* 2873 */     return -1;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected List getTextRuns(TextNode node) {
/* 2882 */     if (node.getTextRuns() == null)
/*      */     {
/*      */       
/* 2885 */       node.getPrimitiveBounds();
/*      */     }
/*      */     
/* 2888 */     return node.getTextRuns();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected CharacterInformation getCharacterInformation(List list, int startIndex, int charnum, AttributedCharacterIterator aci) {
/* 2911 */     CharacterInformation info = new CharacterInformation();
/* 2912 */     info.characterIndex = startIndex + charnum;
/*      */     
/* 2914 */     for (int i = 0; i < list.size(); ) {
/*      */       
/* 2916 */       StrokingTextPainter.TextRun run = (StrokingTextPainter.TextRun)list.get(i);
/*      */       
/* 2918 */       if (!run.getLayout().hasCharacterIndex(info.characterIndex)) {
/*      */         i++; continue;
/*      */       } 
/* 2921 */       info.layout = run.getLayout();
/*      */       
/* 2923 */       aci.setIndex(info.characterIndex);
/*      */ 
/*      */       
/* 2926 */       if (aci.getAttribute(ALT_GLYPH_HANDLER) != null) {
/* 2927 */         info.glyphIndexStart = 0;
/* 2928 */         info.glyphIndexEnd = info.layout.getGlyphCount() - 1;
/*      */       } else {
/* 2930 */         info.glyphIndexStart = info.layout.getGlyphIndex(info.characterIndex);
/*      */ 
/*      */ 
/*      */ 
/*      */         
/* 2935 */         if (info.glyphIndexStart == -1) {
/* 2936 */           info.glyphIndexStart = 0;
/* 2937 */           info.glyphIndexEnd = info.layout.getGlyphCount() - 1;
/*      */         } else {
/*      */           
/* 2940 */           info.glyphIndexEnd = info.glyphIndexStart;
/*      */         } 
/*      */       } 
/* 2943 */       return info;
/*      */     } 
/* 2945 */     return null;
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected static class CharacterInformation
/*      */   {
/*      */     TextSpanLayout layout;
/*      */ 
/*      */     
/*      */     int glyphIndexStart;
/*      */ 
/*      */     
/*      */     int glyphIndexEnd;
/*      */ 
/*      */     
/*      */     int characterIndex;
/*      */ 
/*      */ 
/*      */     
/* 2965 */     public boolean isVertical() { return this.layout.isVertical(); }
/*      */ 
/*      */ 
/*      */     
/* 2969 */     public double getComputedOrientationAngle() { return this.layout.getComputedOrientationAngle(this.characterIndex); }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Set getTextIntersectionSet(AffineTransform at, Rectangle2D rect) {
/* 2976 */     Set elems = new HashSet();
/*      */     
/* 2978 */     TextNode tn = getTextNode();
/*      */     
/* 2980 */     List list = tn.getTextRuns();
/* 2981 */     if (list == null) {
/* 2982 */       return elems;
/*      */     }
/* 2984 */     for (int i = 0; i < list.size(); i++) {
/*      */       
/* 2986 */       StrokingTextPainter.TextRun run = (StrokingTextPainter.TextRun)list.get(i);
/* 2987 */       TextSpanLayout layout = run.getLayout();
/* 2988 */       AttributedCharacterIterator aci = run.getACI();
/* 2989 */       aci.first();
/*      */       
/* 2991 */       SoftReference sr = (SoftReference)aci.getAttribute(TEXT_COMPOUND_ID);
/* 2992 */       Element elem = (Element)sr.get();
/*      */       
/* 2994 */       if (elem == null || 
/* 2995 */         elems.contains(elem) || 
/* 2996 */         !isTextSensitive(elem))
/*      */         continue; 
/* 2998 */       Rectangle2D glBounds = layout.getBounds2D();
/* 2999 */       if (glBounds != null) {
/* 3000 */         glBounds = at.createTransformedShape(glBounds).getBounds2D();
/*      */         
/* 3002 */         if (!rect.intersects(glBounds)) {
/*      */           continue;
/*      */         }
/*      */       } 
/*      */       
/* 3007 */       GVTGlyphVector gv = layout.getGlyphVector();
/* 3008 */       for (int g = 0; g < gv.getNumGlyphs(); g++) {
/* 3009 */         Shape gBounds = gv.getGlyphLogicalBounds(g);
/* 3010 */         if (gBounds != null) {
/* 3011 */           gBounds = at.createTransformedShape(gBounds).getBounds2D();
/*      */ 
/*      */           
/* 3014 */           if (gBounds.intersects(rect)) {
/* 3015 */             elems.add(elem); break;
/*      */           } 
/*      */         } 
/*      */       } 
/*      */       continue;
/*      */     } 
/* 3021 */     return elems;
/*      */   }
/*      */ 
/*      */   
/*      */   public Set getTextEnclosureSet(AffineTransform at, Rectangle2D rect) {
/* 3026 */     TextNode tn = getTextNode();
/*      */     
/* 3028 */     Set elems = new HashSet();
/* 3029 */     List list = tn.getTextRuns();
/* 3030 */     if (list == null) {
/* 3031 */       return elems;
/*      */     }
/* 3033 */     Set reject = new HashSet();
/* 3034 */     for (int i = 0; i < list.size(); i++) {
/*      */       
/* 3036 */       StrokingTextPainter.TextRun run = (StrokingTextPainter.TextRun)list.get(i);
/* 3037 */       TextSpanLayout layout = run.getLayout();
/* 3038 */       AttributedCharacterIterator aci = run.getACI();
/* 3039 */       aci.first();
/*      */       
/* 3041 */       SoftReference sr = (SoftReference)aci.getAttribute(TEXT_COMPOUND_ID);
/* 3042 */       Element elem = (Element)sr.get();
/*      */       
/* 3044 */       if (elem != null && 
/* 3045 */         !reject.contains(elem))
/* 3046 */         if (!isTextSensitive(elem)) {
/* 3047 */           reject.add(elem);
/*      */         }
/*      */         else {
/*      */           
/* 3051 */           Rectangle2D glBounds = layout.getBounds2D();
/* 3052 */           if (glBounds != null) {
/*      */ 
/*      */ 
/*      */             
/* 3056 */             glBounds = at.createTransformedShape(glBounds).getBounds2D();
/*      */             
/* 3058 */             if (rect.contains(glBounds)) {
/* 3059 */               elems.add(elem);
/*      */             } else {
/* 3061 */               reject.add(elem);
/* 3062 */               elems.remove(elem);
/*      */             } 
/*      */           } 
/*      */         }  
/* 3066 */     }  return elems;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static boolean getTextIntersection(BridgeContext ctx, Element elem, AffineTransform ati, Rectangle2D rect, boolean checkSensitivity) {
/* 3074 */     SVGContext svgCtx = null;
/* 3075 */     if (elem instanceof SVGOMElement)
/* 3076 */       svgCtx = ((SVGOMElement)elem).getSVGContext(); 
/* 3077 */     if (svgCtx == null) return false;
/*      */     
/* 3079 */     SVGTextElementBridge txtBridge = null;
/* 3080 */     if (svgCtx instanceof SVGTextElementBridge) {
/* 3081 */       txtBridge = (SVGTextElementBridge)svgCtx;
/* 3082 */     } else if (svgCtx instanceof AbstractTextChildSVGContext) {
/*      */       
/* 3084 */       AbstractTextChildSVGContext childCtx = (AbstractTextChildSVGContext)svgCtx;
/* 3085 */       txtBridge = childCtx.getTextBridge();
/*      */     } 
/* 3087 */     if (txtBridge == null) return false;
/*      */     
/* 3089 */     TextNode tn = txtBridge.getTextNode();
/* 3090 */     List list = tn.getTextRuns();
/* 3091 */     if (list == null) {
/* 3092 */       return false;
/*      */     }
/* 3094 */     Element txtElem = txtBridge.e;
/*      */     
/* 3096 */     AffineTransform at = tn.getGlobalTransform();
/* 3097 */     at.preConcatenate(ati);
/*      */ 
/*      */     
/* 3100 */     Rectangle2D tnRect = tn.getBounds();
/* 3101 */     tnRect = at.createTransformedShape(tnRect).getBounds2D();
/* 3102 */     if (!rect.intersects(tnRect)) return false;
/*      */     
/* 3104 */     for (int i = 0; i < list.size(); i++) {
/*      */       
/* 3106 */       StrokingTextPainter.TextRun run = (StrokingTextPainter.TextRun)list.get(i);
/* 3107 */       TextSpanLayout layout = run.getLayout();
/* 3108 */       AttributedCharacterIterator aci = run.getACI();
/* 3109 */       aci.first();
/*      */       
/* 3111 */       SoftReference sr = (SoftReference)aci.getAttribute(TEXT_COMPOUND_ID);
/* 3112 */       Element runElem = (Element)sr.get();
/* 3113 */       if (runElem != null)
/*      */       {
/*      */         
/* 3116 */         if (!checkSensitivity || isTextSensitive(runElem)) {
/*      */           
/* 3118 */           Element p = runElem;
/* 3119 */           while (p != null && p != txtElem && p != elem) {
/* 3120 */             p = (Element)txtBridge.getParentNode(p);
/*      */           }
/* 3122 */           if (p == elem) {
/*      */ 
/*      */             
/* 3125 */             Rectangle2D glBounds = layout.getBounds2D();
/* 3126 */             if (glBounds != null) {
/* 3127 */               glBounds = at.createTransformedShape(glBounds).getBounds2D();
/* 3128 */               if (rect.intersects(glBounds)) {
/*      */                 
/* 3130 */                 GVTGlyphVector gv = layout.getGlyphVector();
/* 3131 */                 for (int g = 0; g < gv.getNumGlyphs(); g++)
/* 3132 */                 { Shape gBounds = gv.getGlyphLogicalBounds(g);
/* 3133 */                   if (gBounds != null)
/* 3134 */                   { gBounds = at.createTransformedShape(gBounds).getBounds2D();
/*      */ 
/*      */                     
/* 3137 */                     if (gBounds.intersects(rect))
/* 3138 */                       return true;  }  } 
/*      */               } 
/*      */             } 
/*      */           } 
/*      */         }  } 
/* 3143 */     }  return false;
/*      */   }
/*      */ 
/*      */   
/*      */   public static Rectangle2D getTextBounds(BridgeContext ctx, Element elem, boolean checkSensitivity) {
/* 3148 */     SVGContext svgCtx = null;
/* 3149 */     if (elem instanceof SVGOMElement)
/* 3150 */       svgCtx = ((SVGOMElement)elem).getSVGContext(); 
/* 3151 */     if (svgCtx == null) return null;
/*      */     
/* 3153 */     SVGTextElementBridge txtBridge = null;
/* 3154 */     if (svgCtx instanceof SVGTextElementBridge) {
/* 3155 */       txtBridge = (SVGTextElementBridge)svgCtx;
/* 3156 */     } else if (svgCtx instanceof AbstractTextChildSVGContext) {
/*      */       
/* 3158 */       AbstractTextChildSVGContext childCtx = (AbstractTextChildSVGContext)svgCtx;
/* 3159 */       txtBridge = childCtx.getTextBridge();
/*      */     } 
/* 3161 */     if (txtBridge == null) return null;
/*      */     
/* 3163 */     TextNode tn = txtBridge.getTextNode();
/* 3164 */     List list = tn.getTextRuns();
/* 3165 */     if (list == null) {
/* 3166 */       return null;
/*      */     }
/* 3168 */     Element txtElem = txtBridge.e;
/* 3169 */     Rectangle2D ret = null;
/*      */     
/* 3171 */     for (int i = 0; i < list.size(); i++) {
/*      */       
/* 3173 */       StrokingTextPainter.TextRun run = (StrokingTextPainter.TextRun)list.get(i);
/* 3174 */       TextSpanLayout layout = run.getLayout();
/* 3175 */       AttributedCharacterIterator aci = run.getACI();
/* 3176 */       aci.first();
/*      */       
/* 3178 */       SoftReference sr = (SoftReference)aci.getAttribute(TEXT_COMPOUND_ID);
/* 3179 */       Element runElem = (Element)sr.get();
/* 3180 */       if (runElem != null)
/*      */       {
/*      */         
/* 3183 */         if (!checkSensitivity || isTextSensitive(runElem)) {
/*      */           
/* 3185 */           Element p = runElem;
/* 3186 */           while (p != null && p != txtElem && p != elem) {
/* 3187 */             p = (Element)txtBridge.getParentNode(p);
/*      */           }
/* 3189 */           if (p == elem) {
/*      */ 
/*      */             
/* 3192 */             Rectangle2D glBounds = layout.getBounds2D();
/* 3193 */             if (glBounds != null)
/* 3194 */               if (ret == null) { ret = (Rectangle2D)glBounds.clone(); }
/* 3195 */               else { ret.add(glBounds); }  
/*      */           } 
/*      */         }  } 
/* 3198 */     }  return ret;
/*      */   }
/*      */ 
/*      */   
/*      */   public static boolean isTextSensitive(Element e) {
/* 3203 */     int ptrEvts = CSSUtilities.convertPointerEvents(e);
/* 3204 */     switch (ptrEvts) {
/*      */       case 0:
/*      */       case 1:
/*      */       case 2:
/*      */       case 3:
/* 3209 */         return CSSUtilities.convertVisibility(e);
/*      */       case 4:
/*      */       case 5:
/*      */       case 6:
/*      */       case 7:
/* 3214 */         return true;
/*      */     } 
/*      */     
/* 3217 */     return false;
/*      */   }
/*      */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\SVGTextElementBridge.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */