/*      */ package org.apache.batik.bridge;
/*      */ 
/*      */ import java.awt.geom.AffineTransform;
/*      */ import java.awt.geom.Point2D;
/*      */ import java.awt.geom.Rectangle2D;
/*      */ import java.io.IOException;
/*      */ import java.util.Iterator;
/*      */ import java.util.LinkedList;
/*      */ import java.util.List;
/*      */ import java.util.StringTokenizer;
/*      */ import org.apache.batik.css.engine.CSSEngine;
/*      */ import org.apache.batik.dom.AbstractNode;
/*      */ import org.apache.batik.dom.util.XLinkSupport;
/*      */ import org.apache.batik.dom.util.XMLSupport;
/*      */ import org.apache.batik.gvt.GraphicsNode;
/*      */ import org.apache.batik.parser.AWTTransformProducer;
/*      */ import org.apache.batik.parser.ClockHandler;
/*      */ import org.apache.batik.parser.ClockParser;
/*      */ import org.apache.batik.parser.ParseException;
/*      */ import org.apache.batik.parser.UnitProcessor;
/*      */ import org.apache.batik.util.ParsedURL;
/*      */ import org.apache.batik.util.SVGConstants;
/*      */ import org.w3c.dom.Element;
/*      */ import org.w3c.dom.Node;
/*      */ import org.w3c.dom.svg.SVGDocument;
/*      */ import org.w3c.dom.svg.SVGElement;
/*      */ import org.w3c.dom.svg.SVGLangSpace;
/*      */ import org.w3c.dom.svg.SVGNumberList;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public abstract class SVGUtilities
/*      */   implements SVGConstants, ErrorConstants
/*      */ {
/*      */   public static final short USER_SPACE_ON_USE = 1;
/*      */   public static final short OBJECT_BOUNDING_BOX = 2;
/*      */   public static final short STROKE_WIDTH = 3;
/*      */   
/*      */   public static Element getParentElement(Element elt) {
/*   73 */     Node n = CSSEngine.getCSSParentNode(elt);
/*   74 */     while (n != null && n.getNodeType() != 1) {
/*   75 */       n = CSSEngine.getCSSParentNode(n);
/*      */     }
/*   77 */     return (Element)n;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static float[] convertSVGNumberList(SVGNumberList l) {
/*   85 */     int n = l.getNumberOfItems();
/*   86 */     if (n == 0) {
/*   87 */       return null;
/*      */     }
/*   89 */     float[] fl = new float[n];
/*   90 */     for (int i = 0; i < n; i++) {
/*   91 */       fl[i] = l.getItem(i).getValue();
/*      */     }
/*   93 */     return fl;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  101 */   public static float convertSVGNumber(String s) { return Float.parseFloat(s); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  109 */   public static int convertSVGInteger(String s) { return Integer.parseInt(s); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static float convertRatio(String v) {
/*  119 */     float d = 1.0F;
/*  120 */     if (v.endsWith("%")) {
/*  121 */       v = v.substring(0, v.length() - 1);
/*  122 */       d = 100.0F;
/*      */     } 
/*  124 */     float r = Float.parseFloat(v) / d;
/*  125 */     if (r < 0.0F) {
/*  126 */       r = 0.0F;
/*  127 */     } else if (r > 1.0F) {
/*  128 */       r = 1.0F;
/*      */     } 
/*  130 */     return r;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static String getDescription(SVGElement elt) {
/*  137 */     String result = "";
/*  138 */     boolean preserve = false;
/*  139 */     Node n = elt.getFirstChild();
/*  140 */     if (n != null && n.getNodeType() == 1) {
/*  141 */       String name = (n.getPrefix() == null) ? n.getNodeName() : n.getLocalName();
/*      */       
/*  143 */       if (name.equals("desc")) {
/*  144 */         preserve = ((SVGLangSpace)n).getXMLspace().equals("preserve");
/*      */         
/*  146 */         n = n.getFirstChild();
/*  147 */         for (; n != null; 
/*  148 */           n = n.getNextSibling()) {
/*  149 */           if (n.getNodeType() == 3) {
/*  150 */             result = result + n.getNodeValue();
/*      */           }
/*      */         } 
/*      */       } 
/*      */     } 
/*  155 */     return preserve ? XMLSupport.preserveXMLSpace(result) : XMLSupport.defaultXMLSpace(result);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static boolean matchUserAgent(Element elt, UserAgent ua) {
/*  167 */     if (elt.hasAttributeNS(null, "systemLanguage")) {
/*      */       
/*  169 */       String s, sl = elt.getAttributeNS(null, "systemLanguage");
/*      */       
/*  171 */       if (sl.length() == 0)
/*  172 */         return false; 
/*  173 */       StringTokenizer st = new StringTokenizer(sl, ", "); do {
/*  174 */         if (!st.hasMoreTokens())
/*      */         {
/*      */ 
/*      */ 
/*      */ 
/*      */           
/*  180 */           return false; }  s = st.nextToken();
/*      */       } while (!matchUserLanguage(s, ua.getLanguages()));
/*  182 */     }  if (elt.hasAttributeNS(null, "requiredFeatures")) {
/*      */       
/*  184 */       String rf = elt.getAttributeNS(null, "requiredFeatures");
/*      */       
/*  186 */       if (rf.length() == 0)
/*  187 */         return false; 
/*  188 */       StringTokenizer st = new StringTokenizer(rf, " ");
/*  189 */       while (st.hasMoreTokens()) {
/*  190 */         String s = st.nextToken();
/*  191 */         if (!ua.hasFeature(s)) {
/*  192 */           return false;
/*      */         }
/*      */       } 
/*      */     } 
/*  196 */     if (elt.hasAttributeNS(null, "requiredExtensions")) {
/*      */       
/*  198 */       String re = elt.getAttributeNS(null, "requiredExtensions");
/*      */       
/*  200 */       if (re.length() == 0)
/*  201 */         return false; 
/*  202 */       StringTokenizer st = new StringTokenizer(re, " ");
/*  203 */       while (st.hasMoreTokens()) {
/*  204 */         String s = st.nextToken();
/*  205 */         if (!ua.supportExtension(s)) {
/*  206 */           return false;
/*      */         }
/*      */       } 
/*      */     } 
/*  210 */     return true;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected static boolean matchUserLanguage(String s, String userLanguages) {
/*  222 */     StringTokenizer st = new StringTokenizer(userLanguages, ", ");
/*  223 */     while (st.hasMoreTokens()) {
/*  224 */       String t = st.nextToken();
/*  225 */       if (s.startsWith(t)) {
/*  226 */         if (s.length() > t.length()) {
/*  227 */           return (s.charAt(t.length()) == '-');
/*      */         }
/*  229 */         return true;
/*      */       } 
/*      */     } 
/*  232 */     return false;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static String getChainableAttributeNS(Element element, String namespaceURI, String attrName, BridgeContext ctx) {
/*  251 */     DocumentLoader loader = ctx.getDocumentLoader();
/*  252 */     Element e = element;
/*  253 */     List refs = new LinkedList();
/*      */     while (true) {
/*  255 */       String v = e.getAttributeNS(namespaceURI, attrName);
/*  256 */       if (v.length() > 0) {
/*  257 */         return v;
/*      */       }
/*  259 */       String uriStr = XLinkSupport.getXLinkHref(e);
/*  260 */       if (uriStr.length() == 0) {
/*  261 */         return "";
/*      */       }
/*  263 */       String baseURI = ((AbstractNode)e).getBaseURI();
/*  264 */       ParsedURL purl = new ParsedURL(baseURI, uriStr);
/*      */       
/*  266 */       Iterator iter = refs.iterator();
/*  267 */       while (iter.hasNext()) {
/*  268 */         if (purl.equals(iter.next())) {
/*  269 */           throw new BridgeException(ctx, e, "xlink.href.circularDependencies", new Object[] { uriStr });
/*      */         }
/*      */       } 
/*      */ 
/*      */       
/*      */       try {
/*  275 */         SVGDocument svgDoc = (SVGDocument)e.getOwnerDocument();
/*  276 */         URIResolver resolver = ctx.createURIResolver(svgDoc, loader);
/*  277 */         e = resolver.getElement(purl.toString(), e);
/*  278 */         refs.add(purl);
/*      */       } catch (IOException ioEx) {
/*  280 */         throw new BridgeException(ctx, e, ioEx, "uri.io", new Object[] { uriStr });
/*      */       } catch (SecurityException secEx) {
/*      */         
/*  283 */         throw new BridgeException(ctx, e, secEx, "uri.unsecure", new Object[] { uriStr });
/*      */       } 
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static Point2D convertPoint(String xStr, String xAttr, String yStr, String yAttr, short unitsType, UnitProcessor.Context uctx) {
/*      */     float y;
/*      */     float x;
/*  311 */     switch (unitsType) {
/*      */       case 2:
/*  313 */         x = UnitProcessor.svgHorizontalCoordinateToObjectBoundingBox(xStr, xAttr, uctx);
/*      */         
/*  315 */         y = UnitProcessor.svgVerticalCoordinateToObjectBoundingBox(yStr, yAttr, uctx);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         
/*  327 */         return new Point2D.Float(x, y);case 1: x = UnitProcessor.svgHorizontalCoordinateToUserSpace(xStr, xAttr, uctx); y = UnitProcessor.svgVerticalCoordinateToUserSpace(yStr, yAttr, uctx); return new Point2D.Float(x, y);
/*      */     } 
/*      */     throw new IllegalArgumentException("Invalid unit type");
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static float convertLength(String length, String attr, short unitsType, UnitProcessor.Context uctx) {
/*  343 */     switch (unitsType) {
/*      */       case 2:
/*  345 */         return UnitProcessor.svgOtherLengthToObjectBoundingBox(length, attr, uctx);
/*      */       
/*      */       case 1:
/*  348 */         return UnitProcessor.svgOtherLengthToUserSpace(length, attr, uctx);
/*      */     } 
/*  350 */     throw new IllegalArgumentException("Invalid unit type");
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static Rectangle2D convertMaskRegion(Element maskElement, Element maskedElement, GraphicsNode maskedNode, BridgeContext ctx) {
/*      */     short s;
/*  373 */     String xStr = maskElement.getAttributeNS(null, "x");
/*  374 */     if (xStr.length() == 0) {
/*  375 */       xStr = "-10%";
/*      */     }
/*      */     
/*  378 */     String yStr = maskElement.getAttributeNS(null, "y");
/*  379 */     if (yStr.length() == 0) {
/*  380 */       yStr = "-10%";
/*      */     }
/*      */     
/*  383 */     String wStr = maskElement.getAttributeNS(null, "width");
/*  384 */     if (wStr.length() == 0) {
/*  385 */       wStr = "120%";
/*      */     }
/*      */     
/*  388 */     String hStr = maskElement.getAttributeNS(null, "height");
/*  389 */     if (hStr.length() == 0) {
/*  390 */       hStr = "120%";
/*      */     }
/*      */ 
/*      */     
/*  394 */     String units = maskElement.getAttributeNS(null, "maskUnits");
/*      */     
/*  396 */     if (units.length() == 0) {
/*  397 */       s = 2;
/*      */     } else {
/*  399 */       s = parseCoordinateSystem(maskElement, "maskUnits", units, ctx);
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/*  404 */     UnitProcessor.Context uctx = UnitProcessor.createContext(ctx, maskedElement);
/*      */ 
/*      */     
/*  407 */     return convertRegion(xStr, yStr, wStr, hStr, s, maskedNode, uctx);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static Rectangle2D convertPatternRegion(Element patternElement, Element paintedElement, GraphicsNode paintedNode, BridgeContext ctx) {
/*      */     short s;
/*  435 */     String xStr = getChainableAttributeNS(patternElement, null, "x", ctx);
/*      */     
/*  437 */     if (xStr.length() == 0) {
/*  438 */       xStr = "0";
/*      */     }
/*      */     
/*  441 */     String yStr = getChainableAttributeNS(patternElement, null, "y", ctx);
/*      */     
/*  443 */     if (yStr.length() == 0) {
/*  444 */       yStr = "0";
/*      */     }
/*      */     
/*  447 */     String wStr = getChainableAttributeNS(patternElement, null, "width", ctx);
/*      */     
/*  449 */     if (wStr.length() == 0) {
/*  450 */       throw new BridgeException(ctx, patternElement, "attribute.missing", new Object[] { "width" });
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*  455 */     String hStr = getChainableAttributeNS(patternElement, null, "height", ctx);
/*      */     
/*  457 */     if (hStr.length() == 0) {
/*  458 */       throw new BridgeException(ctx, patternElement, "attribute.missing", new Object[] { "height" });
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  464 */     String units = getChainableAttributeNS(patternElement, null, "patternUnits", ctx);
/*      */     
/*  466 */     if (units.length() == 0) {
/*  467 */       s = 2;
/*      */     } else {
/*  469 */       s = parseCoordinateSystem(patternElement, "patternUnits", units, ctx);
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/*  474 */     UnitProcessor.Context uctx = UnitProcessor.createContext(ctx, paintedElement);
/*      */ 
/*      */     
/*  477 */     return convertRegion(xStr, yStr, wStr, hStr, s, paintedNode, uctx);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static float[] convertFilterRes(Element filterElement, BridgeContext ctx) {
/*  500 */     float[] filterRes = new float[2];
/*  501 */     String s = getChainableAttributeNS(filterElement, null, "filterRes", ctx);
/*      */     
/*  503 */     Float[] vals = convertSVGNumberOptionalNumber(filterElement, "filterRes", s, ctx);
/*      */ 
/*      */     
/*  506 */     if (filterRes[0] < 0.0F || filterRes[1] < 0.0F) {
/*  507 */       throw new BridgeException(ctx, filterElement, "attribute.malformed", new Object[] { "filterRes", s });
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*  512 */     if (vals[false] == null) {
/*  513 */       filterRes[0] = -1.0F;
/*      */     } else {
/*  515 */       filterRes[0] = vals[0].floatValue();
/*  516 */       if (filterRes[0] < 0.0F) {
/*  517 */         throw new BridgeException(ctx, filterElement, "attribute.malformed", new Object[] { "filterRes", s });
/*      */       }
/*      */     } 
/*      */ 
/*      */     
/*  522 */     if (vals[true] == null) {
/*  523 */       filterRes[1] = filterRes[0];
/*      */     } else {
/*  525 */       filterRes[1] = vals[1].floatValue();
/*  526 */       if (filterRes[1] < 0.0F) {
/*  527 */         throw new BridgeException(ctx, filterElement, "attribute.malformed", new Object[] { "filterRes", s });
/*      */       }
/*      */     } 
/*      */     
/*  531 */     return filterRes;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static Float[] convertSVGNumberOptionalNumber(Element elem, String attrName, String attrValue, BridgeContext ctx) {
/*  544 */     Float[] ret = new Float[2];
/*  545 */     if (attrValue.length() == 0) {
/*  546 */       return ret;
/*      */     }
/*      */     try {
/*  549 */       StringTokenizer tokens = new StringTokenizer(attrValue, " ");
/*  550 */       ret[0] = new Float(Float.parseFloat(tokens.nextToken()));
/*  551 */       if (tokens.hasMoreTokens()) {
/*  552 */         ret[1] = new Float(Float.parseFloat(tokens.nextToken()));
/*      */       }
/*      */       
/*  555 */       if (tokens.hasMoreTokens()) {
/*  556 */         throw new BridgeException(ctx, elem, "attribute.malformed", new Object[] { attrName, attrValue });
/*      */       }
/*      */     }
/*      */     catch (NumberFormatException nfEx) {
/*      */       
/*  561 */       throw new BridgeException(ctx, elem, nfEx, "attribute.malformed", new Object[] { attrName, attrValue, nfEx });
/*      */     } 
/*      */ 
/*      */     
/*  565 */     return ret;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static Rectangle2D convertFilterChainRegion(Element filterElement, Element filteredElement, GraphicsNode filteredNode, BridgeContext ctx) {
/*      */     short s;
/*  585 */     String xStr = getChainableAttributeNS(filterElement, null, "x", ctx);
/*      */     
/*  587 */     if (xStr.length() == 0) {
/*  588 */       xStr = "-10%";
/*      */     }
/*      */     
/*  591 */     String yStr = getChainableAttributeNS(filterElement, null, "y", ctx);
/*      */     
/*  593 */     if (yStr.length() == 0) {
/*  594 */       yStr = "-10%";
/*      */     }
/*      */     
/*  597 */     String wStr = getChainableAttributeNS(filterElement, null, "width", ctx);
/*      */     
/*  599 */     if (wStr.length() == 0) {
/*  600 */       wStr = "120%";
/*      */     }
/*      */     
/*  603 */     String hStr = getChainableAttributeNS(filterElement, null, "height", ctx);
/*      */     
/*  605 */     if (hStr.length() == 0) {
/*  606 */       hStr = "120%";
/*      */     }
/*      */ 
/*      */     
/*  610 */     String units = getChainableAttributeNS(filterElement, null, "filterUnits", ctx);
/*      */     
/*  612 */     if (units.length() == 0) {
/*  613 */       s = 2;
/*      */     } else {
/*  615 */       s = parseCoordinateSystem(filterElement, "filterUnits", units, ctx);
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  634 */     UnitProcessor.Context uctx = UnitProcessor.createContext(ctx, filteredElement);
/*      */ 
/*      */     
/*  637 */     Rectangle2D region = convertRegion(xStr, yStr, wStr, hStr, s, filteredNode, uctx);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  647 */     units = getChainableAttributeNS(filterElement, null, "filterMarginsUnits", ctx);
/*      */ 
/*      */     
/*  650 */     if (units.length() == 0) {
/*      */       
/*  652 */       s = 1;
/*      */     } else {
/*  654 */       s = parseCoordinateSystem(filterElement, "filterMarginsUnits", units, ctx);
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  660 */     String dxStr = filterElement.getAttributeNS(null, "mx");
/*      */     
/*  662 */     if (dxStr.length() == 0) {
/*  663 */       dxStr = "0";
/*      */     }
/*      */     
/*  666 */     String dyStr = filterElement.getAttributeNS(null, "my");
/*  667 */     if (dyStr.length() == 0) {
/*  668 */       dyStr = "0";
/*      */     }
/*      */     
/*  671 */     String dwStr = filterElement.getAttributeNS(null, "mw");
/*  672 */     if (dwStr.length() == 0) {
/*  673 */       dwStr = "0";
/*      */     }
/*      */     
/*  676 */     String dhStr = filterElement.getAttributeNS(null, "mh");
/*  677 */     if (dhStr.length() == 0) {
/*  678 */       dhStr = "0";
/*      */     }
/*      */     
/*  681 */     return extendRegion(dxStr, dyStr, dwStr, dhStr, s, filteredNode, region, uctx);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected static Rectangle2D extendRegion(String dxStr, String dyStr, String dwStr, String dhStr, short unitsType, GraphicsNode filteredNode, Rectangle2D region, UnitProcessor.Context uctx) {
/*      */     Rectangle2D bounds;
/*      */     float dh;
/*      */     float dw;
/*      */     float dy;
/*      */     float dx;
/*  714 */     switch (unitsType) {
/*      */       case 1:
/*  716 */         dx = UnitProcessor.svgHorizontalCoordinateToUserSpace(dxStr, "mx", uctx);
/*      */         
/*  718 */         dy = UnitProcessor.svgVerticalCoordinateToUserSpace(dyStr, "my", uctx);
/*      */         
/*  720 */         dw = UnitProcessor.svgHorizontalCoordinateToUserSpace(dwStr, "mw", uctx);
/*      */         
/*  722 */         dh = UnitProcessor.svgVerticalCoordinateToUserSpace(dhStr, "mh", uctx);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         
/*  751 */         region.setRect(region.getX() + dx, region.getY() + dy, region.getWidth() + dw, region.getHeight() + dh);
/*      */ 
/*      */ 
/*      */ 
/*      */         
/*  756 */         return region;case 2: bounds = filteredNode.getGeometryBounds(); if (bounds == null) { dx = dy = dw = dh = 0.0F; } else { dx = UnitProcessor.svgHorizontalCoordinateToObjectBoundingBox(dxStr, "mx", uctx); dx = (float)(dx * bounds.getWidth()); dy = UnitProcessor.svgVerticalCoordinateToObjectBoundingBox(dyStr, "my", uctx); dy = (float)(dy * bounds.getHeight()); dw = UnitProcessor.svgHorizontalCoordinateToObjectBoundingBox(dwStr, "mw", uctx); dw = (float)(dw * bounds.getWidth()); dh = UnitProcessor.svgVerticalCoordinateToObjectBoundingBox(dhStr, "mh", uctx); dh = (float)(dh * bounds.getHeight()); }  region.setRect(region.getX() + dx, region.getY() + dy, region.getWidth() + dw, region.getHeight() + dh); return region;
/*      */     } 
/*      */     throw new IllegalArgumentException("Invalid unit type");
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static Rectangle2D getBaseFilterPrimitiveRegion(Element filterPrimitiveElement, Element filteredElement, GraphicsNode filteredNode, Rectangle2D defaultRegion, BridgeContext ctx) {
/*  771 */     UnitProcessor.Context uctx = UnitProcessor.createContext(ctx, filteredElement);
/*      */ 
/*      */     
/*  774 */     double x = defaultRegion.getX();
/*  775 */     String s = filterPrimitiveElement.getAttributeNS(null, "x");
/*  776 */     if (s.length() != 0) {
/*  777 */       x = UnitProcessor.svgHorizontalCoordinateToUserSpace(s, "x", uctx);
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*  782 */     double y = defaultRegion.getY();
/*  783 */     s = filterPrimitiveElement.getAttributeNS(null, "y");
/*  784 */     if (s.length() != 0) {
/*  785 */       y = UnitProcessor.svgVerticalCoordinateToUserSpace(s, "y", uctx);
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*  790 */     double w = defaultRegion.getWidth();
/*  791 */     s = filterPrimitiveElement.getAttributeNS(null, "width");
/*  792 */     if (s.length() != 0) {
/*  793 */       w = UnitProcessor.svgHorizontalLengthToUserSpace(s, "width", uctx);
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*  798 */     double h = defaultRegion.getHeight();
/*  799 */     s = filterPrimitiveElement.getAttributeNS(null, "height");
/*  800 */     if (s.length() != 0) {
/*  801 */       h = UnitProcessor.svgVerticalLengthToUserSpace(s, "height", uctx);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  808 */     return new Rectangle2D.Double(x, y, w, h);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static Rectangle2D convertFilterPrimitiveRegion(Element filterPrimitiveElement, Element filterElement, Element filteredElement, GraphicsNode filteredNode, Rectangle2D defaultRegion, Rectangle2D filterRegion, BridgeContext ctx) {
/*      */     Rectangle2D bounds;
/*      */     short s;
/*  834 */     String units = "";
/*  835 */     if (filterElement != null) {
/*  836 */       units = getChainableAttributeNS(filterElement, null, "primitiveUnits", ctx);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  842 */     if (units.length() == 0) {
/*  843 */       s = 1;
/*      */     } else {
/*  845 */       s = parseCoordinateSystem(filterElement, "filterUnits", units, ctx);
/*      */     } 
/*      */ 
/*      */     
/*  849 */     String xStr = "", yStr = "", wStr = "", hStr = "";
/*      */     
/*  851 */     if (filterPrimitiveElement != null) {
/*      */       
/*  853 */       xStr = filterPrimitiveElement.getAttributeNS(null, "x");
/*      */ 
/*      */ 
/*      */       
/*  857 */       yStr = filterPrimitiveElement.getAttributeNS(null, "y");
/*      */ 
/*      */ 
/*      */       
/*  861 */       wStr = filterPrimitiveElement.getAttributeNS(null, "width");
/*      */ 
/*      */ 
/*      */       
/*  865 */       hStr = filterPrimitiveElement.getAttributeNS(null, "height");
/*      */     } 
/*      */ 
/*      */     
/*  869 */     double x = defaultRegion.getX();
/*  870 */     double y = defaultRegion.getY();
/*  871 */     double w = defaultRegion.getWidth();
/*  872 */     double h = defaultRegion.getHeight();
/*      */ 
/*      */     
/*  875 */     UnitProcessor.Context uctx = UnitProcessor.createContext(ctx, filteredElement);
/*      */ 
/*      */     
/*  878 */     switch (s) {
/*      */       case 2:
/*  880 */         bounds = filteredNode.getGeometryBounds();
/*  881 */         if (bounds != null) {
/*  882 */           if (xStr.length() != 0) {
/*  883 */             x = UnitProcessor.svgHorizontalCoordinateToObjectBoundingBox(xStr, "x", uctx);
/*      */             
/*  885 */             x = bounds.getX() + x * bounds.getWidth();
/*      */           } 
/*  887 */           if (yStr.length() != 0) {
/*  888 */             y = UnitProcessor.svgVerticalCoordinateToObjectBoundingBox(yStr, "y", uctx);
/*      */             
/*  890 */             y = bounds.getY() + y * bounds.getHeight();
/*      */           } 
/*  892 */           if (wStr.length() != 0) {
/*  893 */             w = UnitProcessor.svgHorizontalLengthToObjectBoundingBox(wStr, "width", uctx);
/*      */             
/*  895 */             w *= bounds.getWidth();
/*      */           } 
/*  897 */           if (hStr.length() != 0) {
/*  898 */             h = UnitProcessor.svgVerticalLengthToObjectBoundingBox(hStr, "height", uctx);
/*      */             
/*  900 */             h *= bounds.getHeight();
/*      */           } 
/*      */         } 
/*      */         break;
/*      */       case 1:
/*  905 */         if (xStr.length() != 0) {
/*  906 */           x = UnitProcessor.svgHorizontalCoordinateToUserSpace(xStr, "x", uctx);
/*      */         }
/*      */         
/*  909 */         if (yStr.length() != 0) {
/*  910 */           y = UnitProcessor.svgVerticalCoordinateToUserSpace(yStr, "y", uctx);
/*      */         }
/*      */         
/*  913 */         if (wStr.length() != 0) {
/*  914 */           w = UnitProcessor.svgHorizontalLengthToUserSpace(wStr, "width", uctx);
/*      */         }
/*      */         
/*  917 */         if (hStr.length() != 0) {
/*  918 */           h = UnitProcessor.svgVerticalLengthToUserSpace(hStr, "height", uctx);
/*      */         }
/*      */         break;
/*      */       
/*      */       default:
/*  923 */         throw new Error("invalid unitsType:" + s);
/*      */     } 
/*      */     
/*  926 */     Rectangle2D region = new Rectangle2D.Double(x, y, w, h);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  932 */     units = "";
/*  933 */     if (filterElement != null) {
/*  934 */       units = getChainableAttributeNS(filterElement, null, "filterPrimitiveMarginsUnits", ctx);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  940 */     if (units.length() == 0) {
/*  941 */       s = 1;
/*      */     } else {
/*  943 */       s = parseCoordinateSystem(filterElement, "filterPrimitiveMarginsUnits", units, ctx);
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  949 */     String dxStr = "", dyStr = "", dwStr = "", dhStr = "";
/*      */     
/*  951 */     if (filterPrimitiveElement != null) {
/*      */       
/*  953 */       dxStr = filterPrimitiveElement.getAttributeNS(null, "mx");
/*      */ 
/*      */ 
/*      */       
/*  957 */       dyStr = filterPrimitiveElement.getAttributeNS(null, "my");
/*      */ 
/*      */ 
/*      */       
/*  961 */       dwStr = filterPrimitiveElement.getAttributeNS(null, "mw");
/*      */ 
/*      */ 
/*      */       
/*  965 */       dhStr = filterPrimitiveElement.getAttributeNS(null, "mh");
/*      */     } 
/*      */     
/*  968 */     if (dxStr.length() == 0) {
/*  969 */       dxStr = "0";
/*      */     }
/*  971 */     if (dyStr.length() == 0) {
/*  972 */       dyStr = "0";
/*      */     }
/*  974 */     if (dwStr.length() == 0) {
/*  975 */       dwStr = "0";
/*      */     }
/*  977 */     if (dhStr.length() == 0) {
/*  978 */       dhStr = "0";
/*      */     }
/*      */     
/*  981 */     region = extendRegion(dxStr, dyStr, dwStr, dhStr, s, filteredNode, region, uctx);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  990 */     Rectangle2D.intersect(region, filterRegion, region);
/*      */     
/*  992 */     return region;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static Rectangle2D convertFilterPrimitiveRegion(Element filterPrimitiveElement, Element filteredElement, GraphicsNode filteredNode, Rectangle2D defaultRegion, Rectangle2D filterRegion, BridgeContext ctx) {
/* 1015 */     Node parentNode = filterPrimitiveElement.getParentNode();
/* 1016 */     Element filterElement = null;
/* 1017 */     if (parentNode != null && parentNode.getNodeType() == 1)
/*      */     {
/* 1019 */       filterElement = (Element)parentNode;
/*      */     }
/* 1021 */     return convertFilterPrimitiveRegion(filterPrimitiveElement, filterElement, filteredElement, filteredNode, defaultRegion, filterRegion, ctx);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static short parseCoordinateSystem(Element e, String attr, String coordinateSystem, BridgeContext ctx) {
/* 1057 */     if ("userSpaceOnUse".equals(coordinateSystem))
/* 1058 */       return 1; 
/* 1059 */     if ("objectBoundingBox".equals(coordinateSystem)) {
/* 1060 */       return 2;
/*      */     }
/* 1062 */     throw new BridgeException(ctx, e, "attribute.malformed", new Object[] { attr, coordinateSystem });
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static short parseMarkerCoordinateSystem(Element e, String attr, String coordinateSystem, BridgeContext ctx) {
/* 1081 */     if ("userSpaceOnUse".equals(coordinateSystem))
/* 1082 */       return 1; 
/* 1083 */     if ("strokeWidth".equals(coordinateSystem)) {
/* 1084 */       return 3;
/*      */     }
/* 1086 */     throw new BridgeException(ctx, e, "attribute.malformed", new Object[] { attr, coordinateSystem });
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected static Rectangle2D convertRegion(String xStr, String yStr, String wStr, String hStr, short unitsType, GraphicsNode targetNode, UnitProcessor.Context uctx) {
/*      */     Rectangle2D bounds;
/*      */     double h;
/*      */     double w;
/*      */     double y;
/*      */     double x;
/* 1112 */     switch (unitsType) {
/*      */       case 2:
/* 1114 */         x = UnitProcessor.svgHorizontalCoordinateToObjectBoundingBox(xStr, "x", uctx);
/*      */         
/* 1116 */         y = UnitProcessor.svgVerticalCoordinateToObjectBoundingBox(yStr, "y", uctx);
/*      */         
/* 1118 */         w = UnitProcessor.svgHorizontalLengthToObjectBoundingBox(wStr, "width", uctx);
/*      */         
/* 1120 */         h = UnitProcessor.svgVerticalLengthToObjectBoundingBox(hStr, "height", uctx);
/*      */ 
/*      */         
/* 1123 */         bounds = targetNode.getGeometryBounds();
/* 1124 */         if (bounds != null) {
/* 1125 */           x = bounds.getX() + x * bounds.getWidth();
/* 1126 */           y = bounds.getY() + y * bounds.getHeight();
/* 1127 */           w *= bounds.getWidth();
/* 1128 */           h *= bounds.getHeight();
/*      */         } else {
/* 1130 */           x = y = w = h = 0.0D;
/*      */         } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         
/* 1146 */         return new Rectangle2D.Double(x, y, w, h);case 1: x = UnitProcessor.svgHorizontalCoordinateToUserSpace(xStr, "x", uctx); y = UnitProcessor.svgVerticalCoordinateToUserSpace(yStr, "y", uctx); w = UnitProcessor.svgHorizontalLengthToUserSpace(wStr, "width", uctx); h = UnitProcessor.svgVerticalLengthToUserSpace(hStr, "height", uctx); return new Rectangle2D.Double(x, y, w, h);
/*      */     } 
/*      */     throw new Error("invalid unitsType:" + unitsType);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static AffineTransform convertTransform(Element e, String attr, String transform, BridgeContext ctx) {
/*      */     try {
/* 1166 */       return AWTTransformProducer.createAffineTransform(transform);
/*      */     } catch (ParseException pEx) {
/* 1168 */       throw new BridgeException(ctx, e, pEx, "attribute.malformed", new Object[] { attr, transform, pEx });
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static AffineTransform toObjectBBox(AffineTransform Tx, GraphicsNode node) {
/* 1184 */     AffineTransform Mx = new AffineTransform();
/* 1185 */     Rectangle2D bounds = node.getGeometryBounds();
/* 1186 */     if (bounds != null) {
/* 1187 */       Mx.translate(bounds.getX(), bounds.getY());
/* 1188 */       Mx.scale(bounds.getWidth(), bounds.getHeight());
/*      */     } 
/* 1190 */     Mx.concatenate(Tx);
/* 1191 */     return Mx;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static Rectangle2D toObjectBBox(Rectangle2D r, GraphicsNode node) {
/* 1205 */     Rectangle2D bounds = node.getGeometryBounds();
/* 1206 */     if (bounds != null) {
/* 1207 */       return new Rectangle2D.Double(bounds.getX() + r.getX() * bounds.getWidth(), bounds.getY() + r.getY() * bounds.getHeight(), r.getWidth() * bounds.getWidth(), r.getHeight() * bounds.getHeight());
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1213 */     return new Rectangle2D.Double();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static float convertSnapshotTime(Element e, BridgeContext ctx) {
/* 1226 */     if (!e.hasAttributeNS(null, "snapshotTime")) {
/* 1227 */       return 0.0F;
/*      */     }
/* 1229 */     String t = e.getAttributeNS(null, "snapshotTime");
/* 1230 */     if (t.equals("none")) {
/* 1231 */       return 0.0F;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1240 */     ClockParser p = new ClockParser(false);
/* 1241 */     SVGUtilities$1$Handler h = new SVGUtilities$1$Handler();
/* 1242 */     p.setClockHandler(h);
/*      */     try {
/* 1244 */       p.parse(t);
/*      */     } catch (ParseException pEx) {
/* 1246 */       throw new BridgeException(null, e, pEx, "attribute.malformed", new Object[] { "snapshotTime", t, pEx });
/*      */     } 
/*      */ 
/*      */     
/* 1250 */     return h.time;
/*      */   }
/*      */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\SVGUtilities.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */