/*      */ package org.apache.batik.bridge;
/*      */ 
/*      */ import java.io.IOException;
/*      */ import java.io.StringReader;
/*      */ import java.net.URL;
/*      */ import java.util.HashMap;
/*      */ import java.util.Map;
/*      */ import java.util.Timer;
/*      */ import java.util.TimerTask;
/*      */ import org.apache.batik.dom.GenericDOMImplementation;
/*      */ import org.apache.batik.dom.events.NodeEventTarget;
/*      */ import org.apache.batik.dom.svg.SAXSVGDocumentFactory;
/*      */ import org.apache.batik.dom.svg.SVGOMDocument;
/*      */ import org.apache.batik.dom.util.DOMUtilities;
/*      */ import org.apache.batik.dom.util.SAXDocumentFactory;
/*      */ import org.apache.batik.script.Interpreter;
/*      */ import org.apache.batik.script.InterpreterException;
/*      */ import org.apache.batik.script.ScriptEventWrapper;
/*      */ import org.apache.batik.script.Window;
/*      */ import org.apache.batik.util.RunnableQueue;
/*      */ import org.apache.batik.util.XMLResourceDescriptor;
/*      */ import org.w3c.dom.Document;
/*      */ import org.w3c.dom.Element;
/*      */ import org.w3c.dom.Node;
/*      */ import org.w3c.dom.events.Event;
/*      */ import org.w3c.dom.events.EventListener;
/*      */ import org.w3c.dom.events.MutationEvent;
/*      */ import org.w3c.dom.svg.SVGDocument;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class ScriptingEnvironment
/*      */   extends BaseScriptingEnvironment
/*      */ {
/*      */   public static final String[] SVG_EVENT_ATTRS = { 
/*   73 */       "onabort", "onerror", "onresize", "onscroll", "onunload", "onzoom", "onbegin", "onend", "onrepeat", "onfocusin", "onfocusout", "onactivate", "onclick", "onmousedown", "onmouseup", "onmouseover", "onmouseout", "onmousemove", "onkeypress", "onkeydown", "onkeyup" };
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static final String[] SVG_DOM_EVENT = { 
/*  101 */       "SVGAbort", "SVGError", "SVGResize", "SVGScroll", "SVGUnload", "SVGZoom", "beginEvent", "endEvent", "repeatEvent", "DOMFocusIn", "DOMFocusOut", "DOMActivate", "click", "mousedown", "mouseup", "mouseover", "mouseout", "mousemove", "keypress", "keydown", "keyup" };
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  130 */   protected Timer timer = new Timer(true);
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected UpdateManager updateManager;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected RunnableQueue updateRunnableQueue;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected EventListener domNodeInsertedListener;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected EventListener domNodeRemovedListener;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected EventListener domAttrModifiedListener;
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  160 */   protected EventListener svgAbortListener = new ScriptingEventListener(this, "onabort");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  166 */   protected EventListener svgErrorListener = new ScriptingEventListener(this, "onerror");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  172 */   protected EventListener svgResizeListener = new ScriptingEventListener(this, "onresize");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  178 */   protected EventListener svgScrollListener = new ScriptingEventListener(this, "onscroll");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  184 */   protected EventListener svgUnloadListener = new ScriptingEventListener(this, "onunload");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  190 */   protected EventListener svgZoomListener = new ScriptingEventListener(this, "onzoom");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  196 */   protected EventListener beginListener = new ScriptingEventListener(this, "onbegin");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  202 */   protected EventListener endListener = new ScriptingEventListener(this, "onend");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  208 */   protected EventListener repeatListener = new ScriptingEventListener(this, "onrepeat");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  214 */   protected EventListener focusinListener = new ScriptingEventListener(this, "onfocusin");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  220 */   protected EventListener focusoutListener = new ScriptingEventListener(this, "onfocusout");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  226 */   protected EventListener activateListener = new ScriptingEventListener(this, "onactivate");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  232 */   protected EventListener clickListener = new ScriptingEventListener(this, "onclick");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  238 */   protected EventListener mousedownListener = new ScriptingEventListener(this, "onmousedown");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  244 */   protected EventListener mouseupListener = new ScriptingEventListener(this, "onmouseup");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  250 */   protected EventListener mouseoverListener = new ScriptingEventListener(this, "onmouseover");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  256 */   protected EventListener mouseoutListener = new ScriptingEventListener(this, "onmouseout");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  262 */   protected EventListener mousemoveListener = new ScriptingEventListener(this, "onmousemove");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  268 */   protected EventListener keypressListener = new ScriptingEventListener(this, "onkeypress");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  274 */   protected EventListener keydownListener = new ScriptingEventListener(this, "onkeydown");
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  280 */   protected EventListener keyupListener = new ScriptingEventListener(this, "onkeyup");
/*      */ 
/*      */   
/*      */   protected EventListener[] listeners = { 
/*  284 */       this.svgAbortListener, this.svgErrorListener, this.svgResizeListener, this.svgScrollListener, this.svgUnloadListener, this.svgZoomListener, this.beginListener, this.endListener, this.repeatListener, this.focusinListener, this.focusoutListener, this.activateListener, this.clickListener, this.mousedownListener, this.mouseupListener, this.mouseoverListener, this.mouseoutListener, this.mousemoveListener, this.keypressListener, this.keydownListener, this.keyupListener };
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  312 */   Map attrToDOMEvent = new HashMap(SVG_EVENT_ATTRS.length);
/*  313 */   Map attrToListener = new HashMap(SVG_EVENT_ATTRS.length);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public ScriptingEnvironment(BridgeContext ctx) {
/*  326 */     super(ctx); for (int i = 0; i < SVG_EVENT_ATTRS.length; i++) { this.attrToDOMEvent.put(SVG_EVENT_ATTRS[i], SVG_DOM_EVENT[i]); this.attrToListener.put(SVG_EVENT_ATTRS[i], this.listeners[i]); }
/*  327 */      this.updateManager = ctx.getUpdateManager();
/*  328 */     this.updateRunnableQueue = this.updateManager.getUpdateRunnableQueue();
/*      */ 
/*      */     
/*  331 */     addScriptingListeners(this.document.getDocumentElement());
/*      */ 
/*      */     
/*  334 */     addDocumentListeners();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addDocumentListeners() {
/*  341 */     this.domNodeInsertedListener = new DOMNodeInsertedListener(this);
/*  342 */     this.domNodeRemovedListener = new DOMNodeRemovedListener(this);
/*  343 */     this.domAttrModifiedListener = new DOMAttrModifiedListener(this);
/*  344 */     NodeEventTarget et = (NodeEventTarget)this.document;
/*  345 */     et.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeInserted", this.domNodeInsertedListener, false, null);
/*      */ 
/*      */     
/*  348 */     et.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", this.domNodeRemovedListener, false, null);
/*      */ 
/*      */     
/*  351 */     et.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMAttrModified", this.domAttrModifiedListener, false, null);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void removeDocumentListeners() {
/*  360 */     NodeEventTarget et = (NodeEventTarget)this.document;
/*  361 */     et.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeInserted", this.domNodeInsertedListener, false);
/*      */ 
/*      */     
/*  364 */     et.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", this.domNodeRemovedListener, false);
/*      */ 
/*      */     
/*  367 */     et.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMAttrModified", this.domAttrModifiedListener, false);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  377 */   public Window createWindow(Interpreter interp, String lang) { return new Window(this, interp, lang); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void runEventHandler(String script, Event evt, String lang, String desc) {
/*  385 */     Interpreter interpreter = getInterpreter(lang);
/*  386 */     if (interpreter == null)
/*      */       return; 
/*      */     try {
/*      */       Event event1;
/*  390 */       checkCompatibleScriptURL(lang, this.docPURL);
/*      */ 
/*      */       
/*  393 */       if (evt instanceof ScriptEventWrapper) {
/*  394 */         event1 = ((ScriptEventWrapper)evt).getEventObject();
/*      */       } else {
/*  396 */         event1 = evt;
/*      */       } 
/*  398 */       interpreter.bindObject("event", event1);
/*  399 */       interpreter.bindObject("evt", event1);
/*  400 */       interpreter.evaluate(new StringReader(script), desc);
/*  401 */     } catch (IOException ioe) {
/*      */     
/*      */     } catch (InterpreterException ie) {
/*  404 */       handleInterpreterException(ie);
/*      */     } catch (SecurityException se) {
/*  406 */       handleSecurityException(se);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void interrupt() {
/*  414 */     this.timer.cancel();
/*      */     
/*  416 */     removeScriptingListeners(this.document.getDocumentElement());
/*      */ 
/*      */     
/*  419 */     removeDocumentListeners();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void addScriptingListeners(Node node) {
/*  427 */     if (node.getNodeType() == 1) {
/*  428 */       addScriptingListenersOn((Element)node);
/*      */     }
/*      */ 
/*      */     
/*  432 */     Node n = node.getFirstChild();
/*  433 */     for (; n != null; 
/*  434 */       n = n.getNextSibling()) {
/*  435 */       addScriptingListeners(n);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addScriptingListenersOn(Element elt) {
/*  444 */     NodeEventTarget target = (NodeEventTarget)elt;
/*  445 */     if ("http://www.w3.org/2000/svg".equals(elt.getNamespaceURI())) {
/*  446 */       if ("svg".equals(elt.getLocalName())) {
/*      */         
/*  448 */         if (elt.hasAttributeNS(null, "onabort")) {
/*  449 */           target.addEventListenerNS("http://www.w3.org/2001/xml-events", "SVGAbort", this.svgAbortListener, false, null);
/*      */         }
/*      */ 
/*      */         
/*  453 */         if (elt.hasAttributeNS(null, "onerror")) {
/*  454 */           target.addEventListenerNS("http://www.w3.org/2001/xml-events", "SVGError", this.svgErrorListener, false, null);
/*      */         }
/*      */ 
/*      */         
/*  458 */         if (elt.hasAttributeNS(null, "onresize")) {
/*  459 */           target.addEventListenerNS("http://www.w3.org/2001/xml-events", "SVGResize", this.svgResizeListener, false, null);
/*      */         }
/*      */ 
/*      */         
/*  463 */         if (elt.hasAttributeNS(null, "onscroll")) {
/*  464 */           target.addEventListenerNS("http://www.w3.org/2001/xml-events", "SVGScroll", this.svgScrollListener, false, null);
/*      */         }
/*      */ 
/*      */         
/*  468 */         if (elt.hasAttributeNS(null, "onunload")) {
/*  469 */           target.addEventListenerNS("http://www.w3.org/2001/xml-events", "SVGUnload", this.svgUnloadListener, false, null);
/*      */         }
/*      */ 
/*      */         
/*  473 */         if (elt.hasAttributeNS(null, "onzoom")) {
/*  474 */           target.addEventListenerNS("http://www.w3.org/2001/xml-events", "SVGZoom", this.svgZoomListener, false, null);
/*      */         }
/*      */       }
/*      */       else {
/*      */         
/*  479 */         String name = elt.getLocalName();
/*  480 */         if (name.equals("set") || name.startsWith("animate")) {
/*      */ 
/*      */           
/*  483 */           if (elt.hasAttributeNS(null, "onbegin")) {
/*  484 */             target.addEventListenerNS("http://www.w3.org/2001/xml-events", "beginEvent", this.beginListener, false, null);
/*      */           }
/*      */ 
/*      */           
/*  488 */           if (elt.hasAttributeNS(null, "onend")) {
/*  489 */             target.addEventListenerNS("http://www.w3.org/2001/xml-events", "endEvent", this.endListener, false, null);
/*      */           }
/*      */ 
/*      */           
/*  493 */           if (elt.hasAttributeNS(null, "onrepeat")) {
/*  494 */             target.addEventListenerNS("http://www.w3.org/2001/xml-events", "repeatEvent", this.repeatListener, false, null);
/*      */           }
/*      */ 
/*      */           
/*      */           return;
/*      */         } 
/*      */       } 
/*      */     }
/*      */ 
/*      */     
/*  504 */     if (elt.hasAttributeNS(null, "onfocusin")) {
/*  505 */       target.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMFocusIn", this.focusinListener, false, null);
/*      */     }
/*      */ 
/*      */     
/*  509 */     if (elt.hasAttributeNS(null, "onfocusout")) {
/*  510 */       target.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMFocusOut", this.focusoutListener, false, null);
/*      */     }
/*      */ 
/*      */     
/*  514 */     if (elt.hasAttributeNS(null, "onactivate")) {
/*  515 */       target.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMActivate", this.activateListener, false, null);
/*      */     }
/*      */ 
/*      */     
/*  519 */     if (elt.hasAttributeNS(null, "onclick")) {
/*  520 */       target.addEventListenerNS("http://www.w3.org/2001/xml-events", "click", this.clickListener, false, null);
/*      */     }
/*      */ 
/*      */     
/*  524 */     if (elt.hasAttributeNS(null, "onmousedown")) {
/*  525 */       target.addEventListenerNS("http://www.w3.org/2001/xml-events", "mousedown", this.mousedownListener, false, null);
/*      */     }
/*      */ 
/*      */     
/*  529 */     if (elt.hasAttributeNS(null, "onmouseup")) {
/*  530 */       target.addEventListenerNS("http://www.w3.org/2001/xml-events", "mouseup", this.mouseupListener, false, null);
/*      */     }
/*      */ 
/*      */     
/*  534 */     if (elt.hasAttributeNS(null, "onmouseover")) {
/*  535 */       target.addEventListenerNS("http://www.w3.org/2001/xml-events", "mouseover", this.mouseoverListener, false, null);
/*      */     }
/*      */ 
/*      */     
/*  539 */     if (elt.hasAttributeNS(null, "onmouseout")) {
/*  540 */       target.addEventListenerNS("http://www.w3.org/2001/xml-events", "mouseout", this.mouseoutListener, false, null);
/*      */     }
/*      */ 
/*      */     
/*  544 */     if (elt.hasAttributeNS(null, "onmousemove")) {
/*  545 */       target.addEventListenerNS("http://www.w3.org/2001/xml-events", "mousemove", this.mousemoveListener, false, null);
/*      */     }
/*      */ 
/*      */     
/*  549 */     if (elt.hasAttributeNS(null, "onkeypress")) {
/*  550 */       target.addEventListenerNS("http://www.w3.org/2001/xml-events", "keypress", this.keypressListener, false, null);
/*      */     }
/*      */ 
/*      */     
/*  554 */     if (elt.hasAttributeNS(null, "onkeydown")) {
/*  555 */       target.addEventListenerNS("http://www.w3.org/2001/xml-events", "keydown", this.keydownListener, false, null);
/*      */     }
/*      */ 
/*      */     
/*  559 */     if (elt.hasAttributeNS(null, "onkeyup")) {
/*  560 */       target.addEventListenerNS("http://www.w3.org/2001/xml-events", "keyup", this.keyupListener, false, null);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void removeScriptingListeners(Node node) {
/*  571 */     if (node.getNodeType() == 1)
/*      */     {
/*  573 */       removeScriptingListenersOn((Element)node);
/*      */     }
/*      */ 
/*      */     
/*  577 */     Node n = node.getFirstChild();
/*  578 */     for (; n != null; 
/*  579 */       n = n.getNextSibling()) {
/*  580 */       removeScriptingListeners(n);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void removeScriptingListenersOn(Element elt) {
/*  588 */     NodeEventTarget target = (NodeEventTarget)elt;
/*  589 */     if ("http://www.w3.org/2000/svg".equals(elt.getNamespaceURI())) {
/*  590 */       if ("svg".equals(elt.getLocalName())) {
/*      */         
/*  592 */         target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "SVGAbort", this.svgAbortListener, false);
/*      */ 
/*      */         
/*  595 */         target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "SVGError", this.svgErrorListener, false);
/*      */ 
/*      */         
/*  598 */         target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "SVGResize", this.svgResizeListener, false);
/*      */ 
/*      */         
/*  601 */         target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "SVGScroll", this.svgScrollListener, false);
/*      */ 
/*      */         
/*  604 */         target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "SVGUnload", this.svgUnloadListener, false);
/*      */ 
/*      */         
/*  607 */         target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "SVGZoom", this.svgZoomListener, false);
/*      */       }
/*      */       else {
/*      */         
/*  611 */         String name = elt.getLocalName();
/*  612 */         if (name.equals("set") || name.startsWith("animate")) {
/*      */ 
/*      */           
/*  615 */           target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "beginEvent", this.beginListener, false);
/*      */ 
/*      */           
/*  618 */           target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "endEvent", this.endListener, false);
/*      */ 
/*      */           
/*  621 */           target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "repeatEvent", this.repeatListener, false);
/*      */ 
/*      */           
/*      */           return;
/*      */         } 
/*      */       } 
/*      */     }
/*      */ 
/*      */     
/*  630 */     target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMFocusIn", this.focusinListener, false);
/*      */ 
/*      */     
/*  633 */     target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMFocusOut", this.focusoutListener, false);
/*      */ 
/*      */     
/*  636 */     target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMActivate", this.activateListener, false);
/*      */ 
/*      */     
/*  639 */     target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "click", this.clickListener, false);
/*      */ 
/*      */     
/*  642 */     target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mousedown", this.mousedownListener, false);
/*      */ 
/*      */     
/*  645 */     target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mouseup", this.mouseupListener, false);
/*      */ 
/*      */     
/*  648 */     target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mouseover", this.mouseoverListener, false);
/*      */ 
/*      */     
/*  651 */     target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mouseout", this.mouseoutListener, false);
/*      */ 
/*      */     
/*  654 */     target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "mousemove", this.mousemoveListener, false);
/*      */ 
/*      */     
/*  657 */     target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "keypress", this.keypressListener, false);
/*      */ 
/*      */     
/*  660 */     target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "keydown", this.keydownListener, false);
/*      */ 
/*      */     
/*  663 */     target.removeEventListenerNS("http://www.w3.org/2001/xml-events", "keyup", this.keyupListener, false);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void updateScriptingListeners(Element elt, String attr) {
/*  672 */     String domEvt = (String)this.attrToDOMEvent.get(attr);
/*  673 */     if (domEvt == null) {
/*      */       return;
/*      */     }
/*  676 */     EventListener listener = (EventListener)this.attrToListener.get(attr);
/*  677 */     NodeEventTarget target = (NodeEventTarget)elt;
/*  678 */     if (elt.hasAttributeNS(null, attr)) {
/*  679 */       target.addEventListenerNS("http://www.w3.org/2001/xml-events", domEvt, listener, false, null);
/*      */     }
/*      */     else {
/*      */       
/*  683 */       target.removeEventListenerNS("http://www.w3.org/2001/xml-events", domEvt, listener, false);
/*      */     } 
/*      */   }
/*      */ 
/*      */   
/*      */   protected class EvaluateRunnable
/*      */     implements Runnable
/*      */   {
/*      */     protected Interpreter interpreter;
/*      */     protected String script;
/*      */     private final ScriptingEnvironment this$0;
/*      */     
/*      */     public EvaluateRunnable(ScriptingEnvironment this$0, String s, Interpreter interp) {
/*  696 */       this.this$0 = this$0;
/*  697 */       this.interpreter = interp;
/*  698 */       this.script = s;
/*      */     }
/*      */     public void run() {
/*      */       try {
/*  702 */         this.interpreter.evaluate(this.script);
/*      */       } catch (InterpreterException ie) {
/*  704 */         this.this$0.handleInterpreterException(ie);
/*      */       } 
/*      */     }
/*      */   }
/*      */ 
/*      */   
/*      */   protected class EvaluateIntervalRunnable
/*      */     implements Runnable
/*      */   {
/*      */     public int count;
/*      */     
/*      */     public boolean error;
/*      */     
/*      */     protected Interpreter interpreter;
/*      */     protected String script;
/*      */     private final ScriptingEnvironment this$0;
/*      */     
/*      */     public EvaluateIntervalRunnable(ScriptingEnvironment this$0, String s, Interpreter interp) {
/*  722 */       this.this$0 = this$0;
/*  723 */       this.interpreter = interp;
/*  724 */       this.script = s;
/*      */     }
/*      */     public void run() {
/*  727 */       synchronized (this) {
/*  728 */         if (this.error)
/*      */           return; 
/*  730 */         this.count--;
/*      */       } 
/*      */       try {
/*  733 */         this.interpreter.evaluate(this.script);
/*      */       } catch (InterpreterException ie) {
/*  735 */         this.this$0.handleInterpreterException(ie);
/*  736 */         synchronized (this) {
/*  737 */           this.error = true;
/*      */         } 
/*      */       } catch (Exception e) {
/*  740 */         if (this.this$0.userAgent != null) {
/*  741 */           this.this$0.userAgent.displayError(e);
/*      */         } else {
/*  743 */           e.printStackTrace();
/*      */         } 
/*  745 */         synchronized (this) {
/*  746 */           this.error = true;
/*      */         } 
/*      */       } 
/*      */     }
/*      */   }
/*      */ 
/*      */   
/*      */   protected class EvaluateRunnableRunnable
/*      */     implements Runnable
/*      */   {
/*      */     public int count;
/*      */     
/*      */     public boolean error;
/*      */     
/*      */     protected Runnable runnable;
/*      */     private final ScriptingEnvironment this$0;
/*      */     
/*      */     public EvaluateRunnableRunnable(ScriptingEnvironment this$0, Runnable r) {
/*  764 */       this.this$0 = this$0;
/*  765 */       this.runnable = r;
/*      */     }
/*      */     public void run() {
/*  768 */       synchronized (this) {
/*  769 */         if (this.error)
/*      */           return; 
/*  771 */         this.count--;
/*      */       } 
/*      */       try {
/*  774 */         this.runnable.run();
/*      */       } catch (Exception e) {
/*  776 */         if (this.this$0.userAgent != null) {
/*  777 */           this.this$0.userAgent.displayError(e);
/*      */         } else {
/*  779 */           e.printStackTrace();
/*      */         } 
/*  781 */         synchronized (this) {
/*  782 */           this.error = true;
/*      */         } 
/*      */       } 
/*      */     }
/*      */   }
/*      */ 
/*      */   
/*      */   protected class Window
/*      */     implements Window
/*      */   {
/*      */     protected Interpreter interpreter;
/*      */     
/*      */     protected String language;
/*      */     
/*      */     static final String DEFLATE = "deflate";
/*      */     
/*      */     static final String GZIP = "gzip";
/*      */     
/*      */     static final String UTF_8 = "UTF-8";
/*      */     
/*      */     private final ScriptingEnvironment this$0;
/*      */ 
/*      */     
/*      */     public Window(ScriptingEnvironment this$0, Interpreter interp, String lang) {
/*  806 */       this.this$0 = this$0;
/*  807 */       this.interpreter = interp;
/*  808 */       this.language = lang;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public Object setInterval(String script, long interval) {
/*  816 */       TimerTask tt = new ScriptingEnvironment$1(this, script);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  839 */       this.this$0.timer.schedule(tt, interval, interval);
/*  840 */       return tt;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public Object setInterval(Runnable r, long interval) {
/*  848 */       TimerTask tt = new ScriptingEnvironment$2(this, r);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  865 */       this.this$0.timer.schedule(tt, interval, interval);
/*  866 */       return tt;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void clearInterval(Object interval) {
/*  874 */       if (interval == null)
/*  875 */         return;  ((TimerTask)interval).cancel();
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public Object setTimeout(String script, long timeout) {
/*  883 */       TimerTask tt = new ScriptingEnvironment$3(this, script);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  890 */       this.this$0.timer.schedule(tt, timeout);
/*  891 */       return tt;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public Object setTimeout(Runnable r, long timeout) {
/*  899 */       TimerTask tt = new ScriptingEnvironment$4(this, r);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  915 */       this.this$0.timer.schedule(tt, timeout);
/*  916 */       return tt;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void clearTimeout(Object timeout) {
/*  924 */       if (timeout == null)
/*  925 */         return;  ((TimerTask)timeout).cancel();
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public Node parseXML(String text, Document doc) {
/*      */       SAXDocumentFactory sAXDocumentFactory;
/*  934 */       SAXSVGDocumentFactory df = new SAXSVGDocumentFactory(XMLResourceDescriptor.getXMLParserClassName());
/*      */       
/*  936 */       URL urlObj = null;
/*  937 */       if (doc instanceof SVGOMDocument) {
/*  938 */         urlObj = ((SVGOMDocument)doc).getURLObject();
/*      */       }
/*  940 */       if (urlObj == null) {
/*  941 */         urlObj = ((SVGOMDocument)this.this$0.bridgeContext.getDocument()).getURLObject();
/*      */       }
/*      */       
/*  944 */       String uri = (urlObj == null) ? "" : urlObj.toString();
/*  945 */       Node res = DOMUtilities.parseXML(text, doc, uri, null, null, df);
/*  946 */       if (res != null) {
/*  947 */         return res;
/*      */       }
/*  949 */       if (doc instanceof SVGOMDocument) {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         
/*  957 */         sAXDocumentFactory = new HashMap();
/*  958 */         sAXDocumentFactory.put("xmlns", "http://www.w3.org/2000/xmlns/");
/*      */         
/*  960 */         sAXDocumentFactory.put("xmlns:xlink", "http://www.w3.org/1999/xlink");
/*      */ 
/*      */         
/*  963 */         res = DOMUtilities.parseXML(text, doc, uri, sAXDocumentFactory, "svg", df);
/*      */         
/*  965 */         if (res != null) {
/*  966 */           return res;
/*      */         }
/*      */       } 
/*      */ 
/*      */       
/*  971 */       if (doc != null) {
/*  972 */         sAXDocumentFactory = new SAXDocumentFactory(doc.getImplementation(), XMLResourceDescriptor.getXMLParserClassName());
/*      */       } else {
/*      */         
/*  975 */         sAXDocumentFactory = new SAXDocumentFactory(new GenericDOMImplementation(), XMLResourceDescriptor.getXMLParserClassName());
/*      */       } 
/*      */       
/*  978 */       return DOMUtilities.parseXML(text, doc, uri, null, null, sAXDocumentFactory);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  986 */     public void getURL(String uri, Window.URLResponseHandler h) { getURL(uri, h, null); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void getURL(String uri, Window.URLResponseHandler h, String enc) {
/*  999 */       Thread t = new ScriptingEnvironment$6(this, uri, enc, h);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1065 */       t.setPriority(1);
/* 1066 */       t.start();
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1072 */     public void postURL(String uri, String content, Window.URLResponseHandler h) { postURL(uri, content, h, "text/plain", null); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1078 */     public void postURL(String uri, String content, Window.URLResponseHandler h, String mimeType) { postURL(uri, content, h, mimeType, null); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void postURL(String uri, String content, Window.URLResponseHandler h, String mimeType, String fEnc) {
/* 1086 */       Thread t = new ScriptingEnvironment$9(this, uri, mimeType, fEnc, content, h);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1192 */       t.setPriority(1);
/* 1193 */       t.start();
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void alert(String message) {
/* 1200 */       if (this.this$0.userAgent != null) {
/* 1201 */         this.this$0.userAgent.showAlert(message);
/*      */       }
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public boolean confirm(String message) {
/* 1209 */       if (this.this$0.userAgent != null) {
/* 1210 */         return this.this$0.userAgent.showConfirm(message);
/*      */       }
/* 1212 */       return false;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public String prompt(String message) {
/* 1219 */       if (this.this$0.userAgent != null) {
/* 1220 */         return this.this$0.userAgent.showPrompt(message);
/*      */       }
/* 1222 */       return null;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public String prompt(String message, String defVal) {
/* 1229 */       if (this.this$0.userAgent != null) {
/* 1230 */         return this.this$0.userAgent.showPrompt(message, defVal);
/*      */       }
/* 1232 */       return null;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1239 */     public BridgeContext getBridgeContext() { return this.this$0.bridgeContext; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1246 */     public Interpreter getInterpreter() { return this.interpreter; }
/*      */   }
/*      */   
/*      */   protected class DOMNodeInsertedListener
/*      */     implements EventListener {
/*      */     private final ScriptingEnvironment this$0;
/*      */     
/* 1253 */     protected DOMNodeInsertedListener(ScriptingEnvironment this$0) { this.this$0 = this$0; }
/*      */     
/* 1255 */     public void handleEvent(Event evt) { this.this$0.addScriptingListeners((Node)evt.getTarget()); }
/*      */   }
/*      */   
/*      */   protected class DOMNodeRemovedListener
/*      */     implements EventListener {
/*      */     private final ScriptingEnvironment this$0;
/*      */     
/* 1262 */     protected DOMNodeRemovedListener(ScriptingEnvironment this$0) { this.this$0 = this$0; }
/*      */     
/* 1264 */     public void handleEvent(Event evt) { this.this$0.removeScriptingListeners((Node)evt.getTarget()); } }
/*      */   
/*      */   protected class DOMAttrModifiedListener implements EventListener { private final ScriptingEnvironment this$0;
/*      */     
/* 1268 */     protected DOMAttrModifiedListener(ScriptingEnvironment this$0) { this.this$0 = this$0; }
/*      */     public void handleEvent(Event evt) {
/* 1270 */       MutationEvent me = (MutationEvent)evt;
/* 1271 */       if (me.getAttrChange() != 1) {
/* 1272 */         this.this$0.updateScriptingListeners((Element)me.getTarget(), me.getAttrName());
/*      */       }
/*      */     } }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class ScriptingEventListener
/*      */     implements EventListener
/*      */   {
/*      */     protected String attribute;
/*      */ 
/*      */     
/*      */     private final ScriptingEnvironment this$0;
/*      */ 
/*      */ 
/*      */     
/*      */     public ScriptingEventListener(ScriptingEnvironment this$0, String attr) {
/* 1290 */       this.this$0 = this$0;
/* 1291 */       this.attribute = attr;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1298 */       Element elt = (Element)evt.getCurrentTarget();
/*      */       
/* 1300 */       String script = elt.getAttributeNS(null, this.attribute);
/* 1301 */       if (script.length() == 0) {
/*      */         return;
/*      */       }
/* 1304 */       DocumentLoader dl = this.this$0.bridgeContext.getDocumentLoader();
/* 1305 */       SVGDocument d = (SVGDocument)elt.getOwnerDocument();
/* 1306 */       int line = dl.getLineNumber(elt);
/* 1307 */       String desc = Messages.formatMessage("BaseScriptingEnvironment.constant.event.script.description", new Object[] { d.getURL(), this.attribute, new Integer(line) });
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1312 */       Element e = elt;
/* 1313 */       while (e != null && (!"http://www.w3.org/2000/svg".equals(e.getNamespaceURI()) || !"svg".equals(e.getLocalName())))
/*      */       {
/*      */ 
/*      */         
/* 1317 */         e = SVGUtilities.getParentElement(e);
/*      */       }
/* 1319 */       if (e == null) {
/*      */         return;
/*      */       }
/* 1322 */       String lang = e.getAttributeNS(null, "contentScriptType");
/*      */ 
/*      */       
/* 1325 */       this.this$0.runEventHandler(script, evt, lang, desc);
/*      */     }
/*      */   }
/*      */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\ScriptingEnvironment.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */