/*     */ package org.apache.batik.bridge;
/*     */ 
/*     */ import java.awt.Shape;
/*     */ import java.awt.geom.AffineTransform;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Collection;
/*     */ import java.util.Collections;
/*     */ import java.util.Iterator;
/*     */ import java.util.LinkedList;
/*     */ import java.util.List;
/*     */ import java.util.Timer;
/*     */ import java.util.TimerTask;
/*     */ import org.apache.batik.bridge.svg12.DefaultXBLManager;
/*     */ import org.apache.batik.bridge.svg12.SVG12BridgeContext;
/*     */ import org.apache.batik.bridge.svg12.SVG12ScriptingEnvironment;
/*     */ import org.apache.batik.dom.events.AbstractEvent;
/*     */ import org.apache.batik.dom.svg.SVGOMDocument;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.apache.batik.gvt.RootGraphicsNode;
/*     */ import org.apache.batik.gvt.UpdateTracker;
/*     */ import org.apache.batik.gvt.renderer.ImageRenderer;
/*     */ import org.apache.batik.util.EventDispatcher;
/*     */ import org.apache.batik.util.RunnableQueue;
/*     */ import org.w3c.dom.Document;
/*     */ import org.w3c.dom.events.DocumentEvent;
/*     */ import org.w3c.dom.events.EventTarget;
/*     */ 
/*     */ public class UpdateManager
/*     */ {
/*     */   static final int MIN_REPAINT_TIME;
/*     */   protected BridgeContext bridgeContext;
/*     */   protected Document document;
/*     */   protected RunnableQueue updateRunnableQueue;
/*     */   protected RunnableQueue.RunHandler runHandler;
/*     */   protected List listeners;
/*     */   protected ScriptingEnvironment scriptingEnvironment;
/*     */   protected RepaintManager repaintManager;
/*     */   protected UpdateTracker updateTracker;
/*     */   protected GraphicsNode graphicsNode;
/*     */   protected boolean started;
/*     */   protected BridgeContext[] secondaryBridgeContexts;
/*     */   protected ScriptingEnvironment[] secondaryScriptingEnvironments;
/*     */   protected int minRepaintTime;
/*     */   long outOfDateTime;
/*     */   List suspensionList;
/*     */   int nextSuspensionIndex;
/*     */   long allResumeTime;
/*     */   Timer repaintTriggerTimer;
/*     */   TimerTask repaintTimerTask;
/*     */   static EventDispatcher.Dispatcher startedDispatcher;
/*     */   static EventDispatcher.Dispatcher stoppedDispatcher;
/*     */   static EventDispatcher.Dispatcher suspendedDispatcher;
/*     */   static EventDispatcher.Dispatcher resumedDispatcher;
/*     */   static EventDispatcher.Dispatcher updateStartedDispatcher;
/*     */   static EventDispatcher.Dispatcher updateCompletedDispatcher;
/*     */   static EventDispatcher.Dispatcher updateFailedDispatcher;
/*     */   
/*     */   static  {
/*  59 */     value = 20;
/*     */     
/*  61 */     try { String s = System.getProperty("org.apache.batik.min_repaint_time", "20");
/*     */       
/*  63 */       value = Integer.parseInt(s); }
/*  64 */     catch (SecurityException se) {  }
/*  65 */     catch (NumberFormatException nfe) {  }
/*     */     finally
/*  67 */     { MIN_REPAINT_TIME = value; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 764 */     startedDispatcher = new EventDispatcher.Dispatcher()
/*     */       {
/*     */         public void dispatch(Object listener, Object event) {
/* 767 */           ((UpdateManagerListener)listener).managerStarted((UpdateManagerEvent)event);
/*     */         }
/*     */       };
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 776 */     stoppedDispatcher = new EventDispatcher.Dispatcher()
/*     */       {
/*     */         public void dispatch(Object listener, Object event) {
/* 779 */           ((UpdateManagerListener)listener).managerStopped((UpdateManagerEvent)event);
/*     */         }
/*     */       };
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 788 */     suspendedDispatcher = new EventDispatcher.Dispatcher()
/*     */       {
/*     */         public void dispatch(Object listener, Object event) {
/* 791 */           ((UpdateManagerListener)listener).managerSuspended((UpdateManagerEvent)event);
/*     */         }
/*     */       };
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 800 */     resumedDispatcher = new EventDispatcher.Dispatcher()
/*     */       {
/*     */         public void dispatch(Object listener, Object event) {
/* 803 */           ((UpdateManagerListener)listener).managerResumed((UpdateManagerEvent)event);
/*     */         }
/*     */       };
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 812 */     updateStartedDispatcher = new EventDispatcher.Dispatcher()
/*     */       {
/*     */         public void dispatch(Object listener, Object event) {
/* 815 */           ((UpdateManagerListener)listener).updateStarted((UpdateManagerEvent)event);
/*     */         }
/*     */       };
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 824 */     updateCompletedDispatcher = new EventDispatcher.Dispatcher()
/*     */       {
/*     */         public void dispatch(Object listener, Object event) {
/* 827 */           ((UpdateManagerListener)listener).updateCompleted((UpdateManagerEvent)event);
/*     */         }
/*     */       };
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 836 */     updateFailedDispatcher = new EventDispatcher.Dispatcher()
/*     */       {
/*     */         public void dispatch(Object listener, Object event) {
/* 839 */           ((UpdateManagerListener)listener).updateFailed((UpdateManagerEvent)event); }
/*     */       }; } public UpdateManager(BridgeContext ctx, GraphicsNode gn, Document doc) { this.listeners = Collections.synchronizedList(new LinkedList()); this.outOfDateTime = 0L; this.suspensionList = new ArrayList(); this.nextSuspensionIndex = 1; this.allResumeTime = -1L; this.repaintTriggerTimer = null; this.repaintTimerTask = null; this.bridgeContext = ctx; this.bridgeContext.setUpdateManager(this); this.document = doc; this.updateRunnableQueue = RunnableQueue.createRunnableQueue(); this.runHandler = createRunHandler(); this.updateRunnableQueue.setRunHandler(this.runHandler); this.graphicsNode = gn; this.scriptingEnvironment = initializeScriptingEnvironment(this.bridgeContext); this.secondaryBridgeContexts = (BridgeContext[])ctx.getChildContexts().clone(); this.secondaryScriptingEnvironments = new ScriptingEnvironment[this.secondaryBridgeContexts.length]; for (int i = 0; i < this.secondaryBridgeContexts.length; i++) { BridgeContext resCtx = this.secondaryBridgeContexts[i]; if (((SVGOMDocument)resCtx.getDocument()).isSVG12()) { resCtx.setUpdateManager(this); ScriptingEnvironment se = initializeScriptingEnvironment(resCtx); this.secondaryScriptingEnvironments[i] = se; }  }  this.minRepaintTime = MIN_REPAINT_TIME; } public int getMinRepaintTime() { return this.minRepaintTime; } public void setMinRepaintTime(int minRepaintTime) { this.minRepaintTime = minRepaintTime; } protected ScriptingEnvironment initializeScriptingEnvironment(BridgeContext ctx) { ScriptingEnvironment scriptingEnvironment1; SVGOMDocument d = (SVGOMDocument)ctx.getDocument(); if (d.isSVG12()) { scriptingEnvironment1 = new SVG12ScriptingEnvironment(ctx); ctx.xblManager = new DefaultXBLManager(d, ctx); d.setXBLManager(ctx.xblManager); } else { scriptingEnvironment1 = new ScriptingEnvironment(ctx); }  return scriptingEnvironment1; } public void dispatchSVGLoadEvent() throws InterruptedException { dispatchSVGLoadEvent(this.bridgeContext, this.scriptingEnvironment); for (int i = 0; i < this.secondaryScriptingEnvironments.length; i++) { BridgeContext ctx = this.secondaryBridgeContexts[i]; if (((SVGOMDocument)ctx.getDocument()).isSVG12()) { ScriptingEnvironment se = this.secondaryScriptingEnvironments[i]; dispatchSVGLoadEvent(ctx, se); }  }  this.secondaryBridgeContexts = null; this.secondaryScriptingEnvironments = null; } protected void dispatchSVGLoadEvent(BridgeContext ctx, ScriptingEnvironment se) { se.loadScripts(); se.dispatchSVGLoadEvent(); if (ctx.isSVG12() && ctx.xblManager != null) { SVG12BridgeContext ctx12 = (SVG12BridgeContext)ctx; ctx12.addBindingListener(); ctx12.xblManager.startProcessing(); }  } public void dispatchSVGZoomEvent() throws InterruptedException { this.scriptingEnvironment.dispatchSVGZoomEvent(); } public void dispatchSVGScrollEvent() throws InterruptedException { this.scriptingEnvironment.dispatchSVGScrollEvent(); } public void dispatchSVGResizeEvent() throws InterruptedException { this.scriptingEnvironment.dispatchSVGResizeEvent(); } public void manageUpdates(ImageRenderer r) { this.updateRunnableQueue.preemptLater(new Runnable(this, r) { private final ImageRenderer val$r; private final UpdateManager this$0; public void run() throws InterruptedException { synchronized (this.this$0) { this.this$0.running = true; this.this$0.updateTracker = new UpdateTracker(); RootGraphicsNode root = this.this$0.graphicsNode.getRoot(); if (root != null) root.addTreeGraphicsNodeChangeListener(this.this$0.updateTracker);  this.this$0.repaintManager = new RepaintManager(this.val$r); UpdateManagerEvent ev = new UpdateManagerEvent(this.this$0, null, null); this.this$0.fireEvent(UpdateManager.startedDispatcher, ev); this.this$0.started = true; }  } }
/*     */       ); resume(); } public BridgeContext getBridgeContext() { return this.bridgeContext; } public RunnableQueue getUpdateRunnableQueue() { return this.updateRunnableQueue; } public RepaintManager getRepaintManager() { return this.repaintManager; } public UpdateTracker getUpdateTracker() { return this.updateTracker; } public Document getDocument() { return this.document; } public ScriptingEnvironment getScriptingEnvironment() { return this.scriptingEnvironment; } public boolean isRunning() { return this.running; } public void suspend() throws InterruptedException { if (this.updateRunnableQueue.getQueueState() == RunnableQueue.RUNNING) this.updateRunnableQueue.suspendExecution(false);  this.suspendCalled = true; } public void resume() throws InterruptedException { if (this.updateRunnableQueue.getQueueState() != RunnableQueue.RUNNING) this.updateRunnableQueue.resumeExecution();  } public void interrupt() throws InterruptedException { Runnable r = new Runnable(this) { private final UpdateManager this$0; public void run() throws InterruptedException { synchronized (this.this$0) { if (this.this$0.started) { this.this$0.dispatchSVGUnLoadEvent(); } else { this.this$0.running = false; this.this$0.scriptingEnvironment.interrupt(); this.this$0.updateRunnableQueue.getThread().halt(); }  }  } }
/*     */       ; try { this.updateRunnableQueue.preemptLater(r); this.updateRunnableQueue.resumeExecution(); } catch (IllegalStateException ise) {} } public void dispatchSVGUnLoadEvent() throws InterruptedException { if (!this.started) throw new IllegalStateException("UpdateManager not started.");  this.updateRunnableQueue.preemptLater(new Runnable(this) {
/*     */           private final UpdateManager this$0; public void run() throws InterruptedException { synchronized (this.this$0) { String str; AbstractEvent evt = (AbstractEvent)((DocumentEvent)this.this$0.document).createEvent("SVGEvents"); if (this.this$0.bridgeContext.isSVG12()) { str = "unload"; } else { str = "SVGUnload"; }  evt.initEventNS("http://www.w3.org/2001/xml-events", str, false, false); ((EventTarget)this.this$0.document.getDocumentElement()).dispatchEvent(evt); this.this$0.running = false; this.this$0.scriptingEnvironment.interrupt(); this.this$0.updateRunnableQueue.getThread().halt(); this.this$0.bridgeContext.dispose(); UpdateManagerEvent ev = new UpdateManagerEvent(this.this$0, null, null); this.this$0.fireEvent(UpdateManager.stoppedDispatcher, ev); }  }
/*     */         }); resume(); } public void updateRendering(AffineTransform u2d, boolean dbr, Shape aoi, int width, int height) { this.repaintManager.setupRenderer(u2d, dbr, aoi, width, height); List l = new ArrayList(1); l.add(aoi); updateRendering(l, false); } public void updateRendering(AffineTransform u2d, boolean dbr, boolean cpt, Shape aoi, int width, int height) { this.repaintManager.setupRenderer(u2d, dbr, aoi, width, height); List l = new ArrayList(1); l.add(aoi); updateRendering(l, cpt); } protected void updateRendering(List areas, boolean clearPaintingTransform) { try { UpdateManagerEvent ev = new UpdateManagerEvent(this, this.repaintManager.getOffScreen(), null); fireEvent(updateStartedDispatcher, ev); Collection c = this.repaintManager.updateRendering(areas); List l = new ArrayList(c); ev = new UpdateManagerEvent(this, this.repaintManager.getOffScreen(), l, clearPaintingTransform); fireEvent(updateCompletedDispatcher, ev); } catch (ThreadDeath td) { UpdateManagerEvent ev = new UpdateManagerEvent(this, null, null); fireEvent(updateFailedDispatcher, ev); throw td; } catch (Throwable t) { UpdateManagerEvent ev = new UpdateManagerEvent(this, null, null); fireEvent(updateFailedDispatcher, ev); }  } protected void repaint() throws InterruptedException { if (!this.updateTracker.hasChanged()) { this.outOfDateTime = 0L; return; }  long ctime = System.currentTimeMillis(); if (ctime < this.allResumeTime) { createRepaintTimer(); return; }  if (this.allResumeTime > 0L) releaseAllRedrawSuspension();  if (ctime - this.outOfDateTime < this.minRepaintTime) synchronized (this.updateRunnableQueue.getIteratorLock()) { Iterator i = this.updateRunnableQueue.iterator(); while (i.hasNext()) { if (!(i.next() instanceof NoRepaintRunnable)) return;  }  }   List dirtyAreas = this.updateTracker.getDirtyAreas(); this.updateTracker.clear(); if (dirtyAreas != null) updateRendering(dirtyAreas, false);  this.outOfDateTime = 0L; } public void forceRepaint() throws InterruptedException { if (!this.updateTracker.hasChanged()) { this.outOfDateTime = 0L; return; }  List dirtyAreas = this.updateTracker.getDirtyAreas(); this.updateTracker.clear(); if (dirtyAreas != null) updateRendering(dirtyAreas, false);  this.outOfDateTime = 0L; } protected class SuspensionInfo {
/*     */     int index; long resumeMilli; private final UpdateManager this$0; public SuspensionInfo(UpdateManager this$0, int index, long resumeMilli) { this.this$0 = this$0; this.index = index; this.resumeMilli = resumeMilli; } public int getIndex() { return this.index; } public long getResumeMilli() { return this.resumeMilli; }
/*     */   } protected class RepaintTimerTask extends TimerTask {
/*     */     UpdateManager um; private final UpdateManager this$0; RepaintTimerTask(UpdateManager this$0, UpdateManager um) { this.this$0 = this$0; this.um = um; } public void run() throws InterruptedException { RunnableQueue rq = this.um.getUpdateRunnableQueue(); if (rq == null) return;  rq.invokeLater(new UpdateManager$4(this)); }
/* 848 */   } protected RunnableQueue.RunHandler createRunHandler() { return new UpdateManagerRunHander(this); }
/*     */   void createRepaintTimer() throws InterruptedException { if (this.repaintTimerTask != null) return;  if (this.allResumeTime < 0L) return;  if (this.repaintTriggerTimer == null) this.repaintTriggerTimer = new Timer(true);  long delay = this.allResumeTime - System.currentTimeMillis(); if (delay < 0L) delay = 0L;  this.repaintTimerTask = new RepaintTimerTask(this, this); this.repaintTriggerTimer.schedule(this.repaintTimerTask, delay); } void resetRepaintTimer() throws InterruptedException { if (this.repaintTimerTask == null) return;  if (this.allResumeTime < 0L) return;  if (this.repaintTriggerTimer == null) this.repaintTriggerTimer = new Timer(true);  long delay = this.allResumeTime - System.currentTimeMillis(); if (delay < 0L) delay = 0L;  this.repaintTimerTask = new RepaintTimerTask(this, this); this.repaintTriggerTimer.schedule(this.repaintTimerTask, delay); } int addRedrawSuspension(int max_wait_milliseconds) { long resumeTime = System.currentTimeMillis() + max_wait_milliseconds; SuspensionInfo si = new SuspensionInfo(this, this.nextSuspensionIndex++, resumeTime); if (resumeTime > this.allResumeTime) { this.allResumeTime = resumeTime; resetRepaintTimer(); }  this.suspensionList.add(si); return si.getIndex(); } void releaseAllRedrawSuspension() throws InterruptedException { this.suspensionList.clear(); this.allResumeTime = -1L; resetRepaintTimer(); } boolean releaseRedrawSuspension(int index) { if (index > this.nextSuspensionIndex) return false;  if (this.suspensionList.size() == 0) return true;  int lo = 0, hi = this.suspensionList.size() - 1; while (lo < hi) { int mid = lo + hi >> 1; SuspensionInfo si = (SuspensionInfo)this.suspensionList.get(mid); int idx = si.getIndex(); if (idx == index) { lo = hi = mid; continue; }  if (idx < index) { lo = mid + 1; continue; }  hi = mid - 1; }  SuspensionInfo si = (SuspensionInfo)this.suspensionList.get(lo); int idx = si.getIndex(); if (idx != index) return true;  this.suspensionList.remove(lo); if (this.suspensionList.size() == 0) { this.allResumeTime = -1L; resetRepaintTimer(); } else { long resumeTime = si.getResumeMilli(); if (resumeTime == this.allResumeTime) { this.allResumeTime = findNewAllResumeTime(); resetRepaintTimer(); }  }  return true; } long findNewAllResumeTime() { long ret = -1L; Iterator i = this.suspensionList.iterator(); while (i.hasNext()) { SuspensionInfo si = (SuspensionInfo)i.next(); long t = si.getResumeMilli(); if (t > ret) ret = t;  }  return ret; } public void addUpdateManagerListener(UpdateManagerListener l) { this.listeners.add(l); } public void removeUpdateManagerListener(UpdateManagerListener l) { this.listeners.remove(l); } protected void fireEvent(EventDispatcher.Dispatcher dispatcher, Object event) { EventDispatcher.fireEvent(dispatcher, this.listeners, event, false); } protected class UpdateManagerRunHander extends RunnableQueue.RunHandlerAdapter
/*     */   {
/* 851 */     protected UpdateManagerRunHander(UpdateManager this$0) { this.this$0 = this$0; }
/*     */     private final UpdateManager this$0;
/*     */     
/*     */     public void runnableStart(RunnableQueue rq, Runnable r) {
/* 855 */       if (this.this$0.running && !(r instanceof NoRepaintRunnable))
/*     */       {
/*     */         
/* 858 */         if (this.this$0.outOfDateTime == 0L) {
/* 859 */           this.this$0.outOfDateTime = System.currentTimeMillis();
/*     */         }
/*     */       }
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void runnableInvoked(RunnableQueue rq, Runnable r) {
/* 869 */       if (this.this$0.running && !(r instanceof NoRepaintRunnable)) {
/* 870 */         this.this$0.repaint();
/*     */       }
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void executionSuspended(RunnableQueue rq) {
/* 878 */       synchronized (this.this$0) {
/*     */         
/* 880 */         if (this.this$0.suspendCalled) {
/* 881 */           this.this$0.running = false;
/* 882 */           UpdateManagerEvent ev = new UpdateManagerEvent(this, null, null);
/*     */           
/* 884 */           this.this$0.fireEvent(UpdateManager.suspendedDispatcher, ev);
/*     */         } 
/*     */       } 
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void executionResumed(RunnableQueue rq) {
/* 893 */       synchronized (this.this$0) {
/*     */ 
/*     */         
/* 896 */         if (this.this$0.suspendCalled && !this.this$0.running) {
/* 897 */           this.this$0.running = true;
/* 898 */           this.this$0.suspendCalled = false;
/*     */           
/* 900 */           UpdateManagerEvent ev = new UpdateManagerEvent(this, null, null);
/*     */           
/* 902 */           this.this$0.fireEvent(UpdateManager.resumedDispatcher, ev);
/*     */         } 
/*     */       } 
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\UpdateManager.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */