/*     */ package org.apache.batik.bridge;
/*     */ 
/*     */ import java.awt.geom.AffineTransform;
/*     */ import java.util.StringTokenizer;
/*     */ import org.apache.batik.dom.svg.LiveAttributeException;
/*     */ import org.apache.batik.dom.svg.SVGOMAnimatedRect;
/*     */ import org.apache.batik.parser.AWTTransformProducer;
/*     */ import org.apache.batik.parser.FragmentIdentifierHandler;
/*     */ import org.apache.batik.parser.FragmentIdentifierParser;
/*     */ import org.apache.batik.parser.ParseException;
/*     */ import org.apache.batik.parser.PreserveAspectRatioParser;
/*     */ import org.apache.batik.util.SVGConstants;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.Node;
/*     */ import org.w3c.dom.svg.SVGAnimatedPreserveAspectRatio;
/*     */ import org.w3c.dom.svg.SVGAnimatedRect;
/*     */ import org.w3c.dom.svg.SVGPreserveAspectRatio;
/*     */ import org.w3c.dom.svg.SVGRect;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class ViewBox
/*     */   implements SVGConstants, ErrorConstants
/*     */ {
/*     */   public static AffineTransform getViewTransform(String ref, Element e, float w, float h, BridgeContext ctx) {
/*     */     boolean bool;
/*     */     short s;
/*     */     float[] arrayOfFloat;
/*  74 */     if (ref == null || ref.length() == 0) {
/*  75 */       return getPreserveAspectRatioTransform(e, w, h, ctx);
/*     */     }
/*     */     
/*  78 */     ViewHandler vh = new ViewHandler();
/*  79 */     FragmentIdentifierParser p = new FragmentIdentifierParser();
/*  80 */     p.setFragmentIdentifierHandler(vh);
/*  81 */     p.parse(ref);
/*     */     
/*  83 */     Element attrDefElement = e;
/*  84 */     if (vh.hasId) {
/*  85 */       arrayOfFloat = e.getOwnerDocument();
/*  86 */       attrDefElement = arrayOfFloat.getElementById(vh.id);
/*     */     } 
/*  88 */     if (attrDefElement == null) {
/*  89 */       throw new BridgeException(ctx, e, "uri.malformed", new Object[] { ref });
/*     */     }
/*     */ 
/*     */ 
/*     */     
/*  94 */     if (!attrDefElement.getNamespaceURI().equals("http://www.w3.org/2000/svg") || !attrDefElement.getLocalName().equals("view"))
/*     */     {
/*  96 */       attrDefElement = getClosestAncestorSVGElement(e);
/*     */     }
/*     */ 
/*     */ 
/*     */     
/* 101 */     if (vh.hasViewBox) {
/* 102 */       arrayOfFloat = vh.viewBox;
/*     */     } else {
/* 104 */       String viewBoxStr = attrDefElement.getAttributeNS(null, "viewBox");
/*     */       
/* 106 */       arrayOfFloat = parseViewBoxAttribute(attrDefElement, viewBoxStr, ctx);
/*     */     } 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 112 */     if (vh.hasPreserveAspectRatio) {
/* 113 */       s = vh.align;
/* 114 */       bool = vh.meet;
/*     */     } else {
/* 116 */       String aspectRatio = attrDefElement.getAttributeNS(null, "preserveAspectRatio");
/*     */       
/* 118 */       PreserveAspectRatioParser pp = new PreserveAspectRatioParser();
/* 119 */       ViewHandler ph = new ViewHandler();
/* 120 */       pp.setPreserveAspectRatioHandler(ph);
/*     */       try {
/* 122 */         pp.parse(aspectRatio);
/*     */       } catch (ParseException pEx) {
/* 124 */         throw new BridgeException(ctx, attrDefElement, pEx, "attribute.malformed", new Object[] { "preserveAspectRatio", aspectRatio, pEx });
/*     */       } 
/*     */ 
/*     */ 
/*     */       
/* 129 */       s = ph.align;
/* 130 */       bool = ph.meet;
/*     */     } 
/*     */ 
/*     */     
/* 134 */     AffineTransform transform = getPreserveAspectRatioTransform(arrayOfFloat, s, bool, w, h);
/*     */     
/* 136 */     if (vh.hasTransform) {
/* 137 */       transform.concatenate(vh.getAffineTransform());
/*     */     }
/* 139 */     return transform;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   private static Element getClosestAncestorSVGElement(Element e) {
/* 148 */     Node n = e;
/* 149 */     for (; n != null && n.getNodeType() == 1; 
/* 150 */       n = n.getParentNode()) {
/* 151 */       Element tmp = (Element)n;
/* 152 */       if (tmp.getNamespaceURI().equals("http://www.w3.org/2000/svg") && tmp.getLocalName().equals("svg"))
/*     */       {
/* 154 */         return tmp;
/*     */       }
/*     */     } 
/* 157 */     return null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 174 */   public static AffineTransform getPreserveAspectRatioTransform(Element e, float w, float h) { return getPreserveAspectRatioTransform(e, w, h, null); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static AffineTransform getPreserveAspectRatioTransform(Element e, float w, float h, BridgeContext ctx) {
/* 188 */     String viewBox = e.getAttributeNS(null, "viewBox");
/*     */ 
/*     */     
/* 191 */     String aspectRatio = e.getAttributeNS(null, "preserveAspectRatio");
/*     */ 
/*     */     
/* 194 */     return getPreserveAspectRatioTransform(e, viewBox, aspectRatio, w, h, ctx);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static AffineTransform getPreserveAspectRatioTransform(Element e, String viewBox, String aspectRatio, float w, float h, BridgeContext ctx) {
/* 217 */     if (viewBox.length() == 0) {
/* 218 */       return new AffineTransform();
/*     */     }
/* 220 */     float[] vb = parseViewBoxAttribute(e, viewBox, ctx);
/*     */ 
/*     */     
/* 223 */     PreserveAspectRatioParser p = new PreserveAspectRatioParser();
/* 224 */     ViewHandler ph = new ViewHandler();
/* 225 */     p.setPreserveAspectRatioHandler(ph);
/*     */     try {
/* 227 */       p.parse(aspectRatio);
/*     */     } catch (ParseException pEx) {
/* 229 */       throw new BridgeException(ctx, e, pEx, "attribute.malformed", new Object[] { "preserveAspectRatio", aspectRatio, pEx });
/*     */     } 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 235 */     return getPreserveAspectRatioTransform(vb, ph.align, ph.meet, w, h);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static AffineTransform getPreserveAspectRatioTransform(Element e, float[] vb, float w, float h, BridgeContext ctx) {
/* 255 */     String aspectRatio = e.getAttributeNS(null, "preserveAspectRatio");
/*     */ 
/*     */ 
/*     */     
/* 259 */     PreserveAspectRatioParser p = new PreserveAspectRatioParser();
/* 260 */     ViewHandler ph = new ViewHandler();
/* 261 */     p.setPreserveAspectRatioHandler(ph);
/*     */     try {
/* 263 */       p.parse(aspectRatio);
/*     */     } catch (ParseException pEx) {
/* 265 */       throw new BridgeException(ctx, e, pEx, "attribute.malformed", new Object[] { "preserveAspectRatio", aspectRatio, pEx });
/*     */     } 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 271 */     return getPreserveAspectRatioTransform(vb, ph.align, ph.meet, w, h);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static AffineTransform getPreserveAspectRatioTransform(Element e, float[] vb, float w, float h, SVGAnimatedPreserveAspectRatio aPAR, BridgeContext ctx) {
/*     */     try {
/* 291 */       SVGPreserveAspectRatio pAR = aPAR.getAnimVal();
/* 292 */       short align = pAR.getAlign();
/* 293 */       boolean meet = (pAR.getMeetOrSlice() == 1);
/*     */       
/* 295 */       return getPreserveAspectRatioTransform(vb, align, meet, w, h);
/*     */     } catch (LiveAttributeException ex) {
/* 297 */       throw new BridgeException(ctx, ex);
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static AffineTransform getPreserveAspectRatioTransform(Element e, SVGAnimatedRect aViewBox, SVGAnimatedPreserveAspectRatio aPAR, float w, float h, BridgeContext ctx) {
/* 317 */     if (!((SVGOMAnimatedRect)aViewBox).isSpecified())
/*     */     {
/* 319 */       return new AffineTransform();
/*     */     }
/* 321 */     SVGRect viewBox = aViewBox.getAnimVal();
/* 322 */     float[] vb = { viewBox.getX(), viewBox.getY(), viewBox.getWidth(), viewBox.getHeight() };
/*     */ 
/*     */     
/* 325 */     return getPreserveAspectRatioTransform(e, vb, w, h, aPAR, ctx);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static float[] parseViewBoxAttribute(Element e, String value, BridgeContext ctx) {
/* 338 */     if (value.length() == 0) {
/* 339 */       return null;
/*     */     }
/* 341 */     int i = 0;
/* 342 */     float[] vb = new float[4];
/* 343 */     StringTokenizer st = new StringTokenizer(value, " ,");
/*     */     try {
/* 345 */       while (i < 4 && st.hasMoreTokens()) {
/* 346 */         vb[i] = Float.parseFloat(st.nextToken());
/* 347 */         i++;
/*     */       } 
/*     */     } catch (NumberFormatException nfEx) {
/* 350 */       throw new BridgeException(ctx, e, nfEx, "attribute.malformed", new Object[] { "viewBox", value, nfEx });
/*     */     } 
/*     */ 
/*     */     
/* 354 */     if (i != 4) {
/* 355 */       throw new BridgeException(ctx, e, "attribute.malformed", new Object[] { "viewBox", value });
/*     */     }
/*     */ 
/*     */ 
/*     */     
/* 360 */     if (vb[2] < 0.0F || vb[3] < 0.0F) {
/* 361 */       throw new BridgeException(ctx, e, "attribute.malformed", new Object[] { "viewBox", value });
/*     */     }
/*     */ 
/*     */ 
/*     */     
/* 366 */     if (vb[2] == 0.0F || vb[3] == 0.0F) {
/* 367 */       return null;
/*     */     }
/* 369 */     return vb;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static AffineTransform getPreserveAspectRatioTransform(float[] vb, short align, boolean meet, float w, float h) {
/* 388 */     if (vb == null) {
/* 389 */       return new AffineTransform();
/*     */     }
/*     */     
/* 392 */     AffineTransform result = new AffineTransform();
/* 393 */     float vpar = vb[2] / vb[3];
/* 394 */     float svgar = w / h;
/*     */     
/* 396 */     if (align == 1)
/* 397 */     { result.scale((w / vb[2]), (h / vb[3]));
/* 398 */       result.translate(-vb[0], -vb[1]); }
/* 399 */     else if ((vpar < svgar && meet) || (vpar >= svgar && !meet))
/* 400 */     { float sf = h / vb[3];
/* 401 */       result.scale(sf, sf);
/* 402 */       switch (align)
/*     */       { case 2:
/*     */         case 5:
/*     */         case 8:
/* 406 */           result.translate(-vb[0], -vb[1]);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */           
/* 434 */           return result;case 3: case 6: case 9: result.translate((-vb[0] - (vb[2] - w * vb[3] / h) / 2.0F), -vb[1]); return result; }  result.translate((-vb[0] - vb[2] - w * vb[3] / h), -vb[1]); } else { float sf = w / vb[2]; result.scale(sf, sf); switch (align) { case 2: case 3: case 4: result.translate(-vb[0], -vb[1]); return result;case 5: case 6: case 7: result.translate(-vb[0], (-vb[1] - (vb[3] - h * vb[2] / w) / 2.0F)); return result; }  result.translate(-vb[0], (-vb[1] - vb[3] - h * vb[2] / w)); }  return result;
/*     */   }
/*     */ 
/*     */   
/*     */   protected static class ViewHandler
/*     */     extends AWTTransformProducer
/*     */     implements FragmentIdentifierHandler
/*     */   {
/*     */     public boolean hasTransform;
/*     */     
/*     */     public boolean hasId;
/*     */     
/*     */     public boolean hasViewBox;
/*     */     
/*     */     public boolean hasViewTargetParams;
/*     */     public boolean hasZoomAndPanParams;
/*     */     public String id;
/*     */     public float[] viewBox;
/*     */     public String viewTargetParams;
/*     */     public boolean isMagnify;
/*     */     public boolean hasPreserveAspectRatio;
/*     */     public short align;
/*     */     
/*     */     public void endTransformList() {
/* 458 */       super.endTransformList();
/* 459 */       this.hasTransform = true;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void startFragmentIdentifier() {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void idReference(String s) throws ParseException {
/* 490 */       this.id = s;
/* 491 */       this.hasId = true;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void viewBox(float x, float y, float width, float height) throws ParseException {
/* 506 */       this.hasViewBox = true;
/* 507 */       this.viewBox = new float[4];
/* 508 */       this.viewBox[0] = x;
/* 509 */       this.viewBox[1] = y;
/* 510 */       this.viewBox[2] = width;
/* 511 */       this.viewBox[3] = height;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void startViewTarget() {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void viewTarget(String name) throws ParseException {
/* 529 */       this.viewTargetParams = name;
/* 530 */       this.hasViewTargetParams = true;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void endViewTarget() {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void zoomAndPan(boolean magnify) {
/* 547 */       this.isMagnify = magnify;
/* 548 */       this.hasZoomAndPanParams = true;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void endFragmentIdentifier() {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public boolean meet = true;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void startPreserveAspectRatio() {}
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 580 */     public void none() { this.align = 1; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 589 */     public void xMaxYMax() { this.align = 10; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 598 */     public void xMaxYMid() { this.align = 7; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 607 */     public void xMaxYMin() { this.align = 4; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 616 */     public void xMidYMax() { this.align = 9; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 625 */     public void xMidYMid() { this.align = 6; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 634 */     public void xMidYMin() { this.align = 3; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 643 */     public void xMinYMax() { this.align = 8; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 652 */     public void xMinYMid() { this.align = 5; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 661 */     public void xMinYMin() { this.align = 2; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 670 */     public void meet() { this.meet = true; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 679 */     public void slice() { this.meet = false; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 688 */     public void endPreserveAspectRatio() { this.hasPreserveAspectRatio = true; }
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\ViewBox.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */