/*      */ package org.apache.batik.bridge.svg12;
/*      */ 
/*      */ import java.util.ArrayList;
/*      */ import java.util.HashMap;
/*      */ import java.util.Iterator;
/*      */ import java.util.LinkedList;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import java.util.Set;
/*      */ import java.util.TreeSet;
/*      */ import javax.swing.event.EventListenerList;
/*      */ import org.apache.batik.bridge.BridgeContext;
/*      */ import org.apache.batik.bridge.BridgeException;
/*      */ import org.apache.batik.dom.AbstractDocument;
/*      */ import org.apache.batik.dom.AbstractNode;
/*      */ import org.apache.batik.dom.events.EventSupport;
/*      */ import org.apache.batik.dom.events.NodeEventTarget;
/*      */ import org.apache.batik.dom.svg12.BindableElement;
/*      */ import org.apache.batik.dom.svg12.XBLEventSupport;
/*      */ import org.apache.batik.dom.svg12.XBLOMContentElement;
/*      */ import org.apache.batik.dom.svg12.XBLOMDefinitionElement;
/*      */ import org.apache.batik.dom.svg12.XBLOMShadowTreeElement;
/*      */ import org.apache.batik.dom.svg12.XBLOMTemplateElement;
/*      */ import org.apache.batik.dom.xbl.NodeXBL;
/*      */ import org.apache.batik.dom.xbl.ShadowTreeEvent;
/*      */ import org.apache.batik.dom.xbl.XBLManager;
/*      */ import org.apache.batik.dom.xbl.XBLManagerData;
/*      */ import org.apache.batik.dom.xbl.XBLShadowTreeElement;
/*      */ import org.apache.batik.util.DoublyIndexedTable;
/*      */ import org.apache.batik.util.XBLConstants;
/*      */ import org.w3c.dom.Attr;
/*      */ import org.w3c.dom.Document;
/*      */ import org.w3c.dom.Element;
/*      */ import org.w3c.dom.NamedNodeMap;
/*      */ import org.w3c.dom.Node;
/*      */ import org.w3c.dom.NodeList;
/*      */ import org.w3c.dom.events.DocumentEvent;
/*      */ import org.w3c.dom.events.Event;
/*      */ import org.w3c.dom.events.EventListener;
/*      */ import org.w3c.dom.events.EventTarget;
/*      */ import org.w3c.dom.events.MutationEvent;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class DefaultXBLManager
/*      */   implements XBLManager, XBLConstants
/*      */ {
/*      */   protected boolean isProcessing;
/*      */   protected Document document;
/*      */   protected BridgeContext ctx;
/*      */   protected DoublyIndexedTable definitionLists;
/*      */   protected DoublyIndexedTable definitions;
/*      */   protected Map contentManagers;
/*      */   protected Map imports;
/*      */   protected DocInsertedListener docInsertedListener;
/*      */   protected DocRemovedListener docRemovedListener;
/*      */   protected DocSubtreeListener docSubtreeListener;
/*      */   protected ImportAttrListener importAttrListener;
/*      */   protected RefAttrListener refAttrListener;
/*      */   protected EventListenerList bindingListenerList;
/*      */   protected EventListenerList contentSelectionChangedListenerList;
/*      */   
/*      */   public DefaultXBLManager(Document doc, BridgeContext ctx) {
/*   94 */     this.definitionLists = new DoublyIndexedTable();
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*   99 */     this.definitions = new DoublyIndexedTable();
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  104 */     this.contentManagers = new HashMap();
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  109 */     this.imports = new HashMap();
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  114 */     this.docInsertedListener = new DocInsertedListener(this);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  120 */     this.docRemovedListener = new DocRemovedListener(this);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  126 */     this.docSubtreeListener = new DocSubtreeListener(this);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  132 */     this.importAttrListener = new ImportAttrListener(this);
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  137 */     this.refAttrListener = new RefAttrListener(this);
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  142 */     this.bindingListenerList = new EventListenerList();
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  147 */     this.contentSelectionChangedListenerList = new EventListenerList();
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  154 */     this.document = doc;
/*  155 */     this.ctx = ctx;
/*  156 */     ImportRecord ir = new ImportRecord(this, null, null);
/*  157 */     this.imports.put(null, ir);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void startProcessing() {
/*  164 */     if (this.isProcessing) {
/*      */       return;
/*      */     }
/*      */ 
/*      */     
/*  169 */     NodeList nl = this.document.getElementsByTagNameNS("http://www.w3.org/2004/xbl", "definition");
/*      */     
/*  171 */     XBLOMDefinitionElement[] defs = new XBLOMDefinitionElement[nl.getLength()];
/*      */     
/*  173 */     for (int i = 0; i < defs.length; i++) {
/*  174 */       defs[i] = (XBLOMDefinitionElement)nl.item(i);
/*      */     }
/*      */ 
/*      */     
/*  178 */     nl = this.document.getElementsByTagNameNS("http://www.w3.org/2004/xbl", "import");
/*      */     
/*  180 */     Element[] imports = new Element[nl.getLength()];
/*      */     
/*  182 */     for (int i = 0; i < imports.length; i++) {
/*  183 */       imports[i] = (Element)nl.item(i);
/*      */     }
/*      */ 
/*      */     
/*  187 */     AbstractDocument doc = (AbstractDocument)this.document;
/*  188 */     XBLEventSupport es = (XBLEventSupport)doc.initializeEventSupport();
/*  189 */     es.addImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", this.docRemovedListener, true);
/*      */ 
/*      */ 
/*      */     
/*  193 */     es.addImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeInserted", this.docInsertedListener, true);
/*      */ 
/*      */ 
/*      */     
/*  197 */     es.addImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMSubtreeModified", this.docSubtreeListener, true);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  203 */     for (int i = 0; i < defs.length; i++) {
/*  204 */       if (defs[i].getAttributeNS(null, "ref").length() != 0) {
/*  205 */         addDefinitionRef(defs[i]);
/*      */       } else {
/*  207 */         String ns = defs[i].getElementNamespaceURI();
/*  208 */         String ln = defs[i].getElementLocalName();
/*  209 */         addDefinition(ns, ln, defs[i], null);
/*      */       } 
/*      */     } 
/*      */ 
/*      */     
/*  214 */     for (int i = 0; i < imports.length; i++) {
/*  215 */       addImport(imports[i]);
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*  220 */     this.isProcessing = true;
/*  221 */     bind(this.document.getDocumentElement());
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void stopProcessing() {
/*  228 */     if (!this.isProcessing) {
/*      */       return;
/*      */     }
/*  231 */     this.isProcessing = false;
/*      */ 
/*      */     
/*  234 */     AbstractDocument doc = (AbstractDocument)this.document;
/*  235 */     XBLEventSupport es = (XBLEventSupport)doc.initializeEventSupport();
/*  236 */     es.removeImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", this.docRemovedListener, true);
/*      */ 
/*      */ 
/*      */     
/*  240 */     es.removeImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeInserted", this.docInsertedListener, true);
/*      */ 
/*      */ 
/*      */     
/*  244 */     es.removeImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMSubtreeModified", this.docSubtreeListener, true);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  250 */     int nSlots = this.imports.values().size();
/*  251 */     ImportRecord[] arrayOfImportRecord = new ImportRecord[nSlots];
/*  252 */     this.imports.values().toArray(arrayOfImportRecord);
/*  253 */     for (int i = 0; i < arrayOfImportRecord.length; i++) {
/*  254 */       ImportRecord ir = arrayOfImportRecord[i];
/*  255 */       if (ir.importElement.getLocalName().equals("definition")) {
/*  256 */         removeDefinitionRef(ir.importElement);
/*      */       } else {
/*  258 */         removeImport(ir.importElement);
/*      */       } 
/*      */     } 
/*      */ 
/*      */     
/*  263 */     Object[] defRecs = this.definitions.getValuesArray();
/*  264 */     this.definitions.clear();
/*  265 */     for (int i = 0; i < defRecs.length; i++) {
/*  266 */       DefinitionRecord defRec = (DefinitionRecord)defRecs[i];
/*  267 */       TreeSet defs = (TreeSet)this.definitionLists.get(defRec.namespaceURI, defRec.localName);
/*      */       
/*  269 */       if (defs != null) {
/*  270 */         while (!defs.isEmpty()) {
/*  271 */           defRec = (DefinitionRecord)defs.first();
/*  272 */           defs.remove(defRec);
/*  273 */           removeDefinition(defRec);
/*      */         } 
/*  275 */         this.definitionLists.put(defRec.namespaceURI, defRec.localName, null);
/*      */       } 
/*      */     } 
/*  278 */     this.definitionLists = new DoublyIndexedTable();
/*  279 */     this.contentManagers.clear();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  286 */   public boolean isProcessing() { return this.isProcessing; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addDefinitionRef(Element defRef) {
/*  294 */     String ref = defRef.getAttributeNS(null, "ref");
/*  295 */     Element e = this.ctx.getReferencedElement(defRef, ref);
/*  296 */     if (!"http://www.w3.org/2004/xbl".equals(e.getNamespaceURI()) || !"definition".equals(e.getLocalName()))
/*      */     {
/*  298 */       throw new BridgeException(this.ctx, defRef, "uri.badTarget", new Object[] { ref });
/*      */     }
/*      */ 
/*      */     
/*  302 */     ImportRecord ir = new ImportRecord(this, defRef, e);
/*  303 */     this.imports.put(defRef, ir);
/*      */     
/*  305 */     NodeEventTarget et = (NodeEventTarget)defRef;
/*  306 */     et.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMAttrModified", this.refAttrListener, false, null);
/*      */ 
/*      */ 
/*      */     
/*  310 */     XBLOMDefinitionElement d = (XBLOMDefinitionElement)defRef;
/*  311 */     String ns = d.getElementNamespaceURI();
/*  312 */     String ln = d.getElementLocalName();
/*  313 */     addDefinition(ns, ln, (XBLOMDefinitionElement)e, defRef);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void removeDefinitionRef(Element defRef) {
/*  321 */     ImportRecord ir = (ImportRecord)this.imports.get(defRef);
/*  322 */     NodeEventTarget et = (NodeEventTarget)defRef;
/*  323 */     et.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMAttrModified", this.refAttrListener, false);
/*      */ 
/*      */     
/*  326 */     DefinitionRecord defRec = (DefinitionRecord)this.definitions.get(ir.node, defRef);
/*      */     
/*  328 */     removeDefinition(defRec);
/*  329 */     this.imports.remove(defRef);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addImport(Element imp) {
/*  336 */     String bindings = imp.getAttributeNS(null, "bindings");
/*  337 */     Node n = this.ctx.getReferencedNode(imp, bindings);
/*  338 */     if (n.getNodeType() == 1 && (!"http://www.w3.org/2004/xbl".equals(n.getNamespaceURI()) || !"xbl".equals(n.getLocalName())))
/*      */     {
/*      */       
/*  341 */       throw new BridgeException(this.ctx, imp, "uri.badTarget", new Object[] { n });
/*      */     }
/*      */ 
/*      */     
/*  345 */     ImportRecord ir = new ImportRecord(this, imp, n);
/*  346 */     this.imports.put(imp, ir);
/*      */     
/*  348 */     NodeEventTarget et = (NodeEventTarget)imp;
/*  349 */     et.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMAttrModified", this.importAttrListener, false, null);
/*      */ 
/*      */ 
/*      */     
/*  353 */     et = (NodeEventTarget)n;
/*  354 */     et.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeInserted", ir.importInsertedListener, false, null);
/*      */ 
/*      */     
/*  357 */     et.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", ir.importRemovedListener, false, null);
/*      */ 
/*      */     
/*  360 */     et.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMSubtreeModified", ir.importSubtreeListener, false, null);
/*      */ 
/*      */     
/*  363 */     addImportedDefinitions(imp, n);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addImportedDefinitions(Element imp, Node n) {
/*  370 */     if (n instanceof XBLOMDefinitionElement) {
/*  371 */       XBLOMDefinitionElement def = (XBLOMDefinitionElement)n;
/*  372 */       String ns = def.getElementNamespaceURI();
/*  373 */       String ln = def.getElementLocalName();
/*  374 */       addDefinition(ns, ln, def, imp);
/*      */     } else {
/*  376 */       n = n.getFirstChild();
/*  377 */       while (n != null) {
/*  378 */         addImportedDefinitions(imp, n);
/*  379 */         n = n.getNextSibling();
/*      */       } 
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void removeImport(Element imp) {
/*  388 */     ImportRecord ir = (ImportRecord)this.imports.get(imp);
/*  389 */     NodeEventTarget et = (NodeEventTarget)ir.node;
/*  390 */     et.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeInserted", ir.importInsertedListener, false);
/*      */ 
/*      */     
/*  393 */     et.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", ir.importRemovedListener, false);
/*      */ 
/*      */     
/*  396 */     et.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMSubtreeModified", ir.importSubtreeListener, false);
/*      */ 
/*      */ 
/*      */     
/*  400 */     et = (NodeEventTarget)imp;
/*  401 */     et.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMAttrModified", this.importAttrListener, false);
/*      */ 
/*      */ 
/*      */     
/*  405 */     Object[] defRecs = this.definitions.getValuesArray();
/*  406 */     for (int i = 0; i < defRecs.length; i++) {
/*  407 */       DefinitionRecord defRec = (DefinitionRecord)defRecs[i];
/*  408 */       if (defRec.importElement == imp) {
/*  409 */         removeDefinition(defRec);
/*      */       }
/*      */     } 
/*  412 */     this.imports.remove(imp);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addDefinition(String namespaceURI, String localName, XBLOMDefinitionElement def, Element imp) {
/*  433 */     ImportRecord ir = (ImportRecord)this.imports.get(imp);
/*  434 */     DefinitionRecord oldDefRec = null;
/*      */     
/*  436 */     TreeSet defs = (TreeSet)this.definitionLists.get(namespaceURI, localName);
/*  437 */     if (defs == null) {
/*  438 */       defs = new TreeSet();
/*  439 */       this.definitionLists.put(namespaceURI, localName, defs);
/*  440 */     } else if (defs.size() > 0) {
/*  441 */       oldDefRec = (DefinitionRecord)defs.first();
/*      */     } 
/*  443 */     XBLOMTemplateElement template = null;
/*  444 */     for (Node n = def.getFirstChild(); n != null; n = n.getNextSibling()) {
/*  445 */       if (n instanceof XBLOMTemplateElement) {
/*  446 */         template = (XBLOMTemplateElement)n;
/*      */         break;
/*      */       } 
/*      */     } 
/*  450 */     DefinitionRecord defRec = new DefinitionRecord(this, namespaceURI, localName, def, template, imp);
/*      */     
/*  452 */     defs.add(defRec);
/*  453 */     this.definitions.put(def, imp, defRec);
/*  454 */     addDefinitionElementListeners(def, ir);
/*  455 */     if (defs.first() != defRec) {
/*      */       return;
/*      */     }
/*  458 */     if (oldDefRec != null) {
/*  459 */       XBLOMDefinitionElement oldDef = oldDefRec.definition;
/*  460 */       XBLOMTemplateElement oldTemplate = oldDefRec.template;
/*  461 */       if (oldTemplate != null) {
/*  462 */         removeTemplateElementListeners(oldTemplate, ir);
/*      */       }
/*  464 */       removeDefinitionElementListeners(oldDef, ir);
/*      */     } 
/*  466 */     if (template != null) {
/*  467 */       addTemplateElementListeners(template, ir);
/*      */     }
/*  469 */     if (this.isProcessing) {
/*  470 */       rebind(namespaceURI, localName, this.document.getDocumentElement());
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addDefinitionElementListeners(XBLOMDefinitionElement def, ImportRecord ir) {
/*  479 */     XBLEventSupport es = (XBLEventSupport)def.initializeEventSupport();
/*  480 */     es.addImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMAttrModified", ir.defAttrListener, false);
/*      */ 
/*      */ 
/*      */     
/*  484 */     es.addImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeInserted", ir.defNodeInsertedListener, false);
/*      */ 
/*      */ 
/*      */     
/*  488 */     es.addImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", ir.defNodeRemovedListener, false);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void addTemplateElementListeners(XBLOMTemplateElement template, ImportRecord ir) {
/*  499 */     XBLEventSupport es = (XBLEventSupport)template.initializeEventSupport();
/*      */     
/*  501 */     es.addImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMAttrModified", ir.templateMutationListener, false);
/*      */ 
/*      */ 
/*      */     
/*  505 */     es.addImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeInserted", ir.templateMutationListener, false);
/*      */ 
/*      */ 
/*      */     
/*  509 */     es.addImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", ir.templateMutationListener, false);
/*      */ 
/*      */ 
/*      */     
/*  513 */     es.addImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMCharacterDataModified", ir.templateMutationListener, false);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void removeDefinition(DefinitionRecord defRec) {
/*  526 */     TreeSet defs = (TreeSet)this.definitionLists.get(defRec.namespaceURI, defRec.localName);
/*      */     
/*  528 */     if (defs == null) {
/*      */       return;
/*      */     }
/*  531 */     Element imp = defRec.importElement;
/*  532 */     ImportRecord ir = (ImportRecord)this.imports.get(imp);
/*  533 */     DefinitionRecord activeDefRec = (DefinitionRecord)defs.first();
/*  534 */     defs.remove(defRec);
/*  535 */     this.definitions.remove(defRec.definition, imp);
/*  536 */     removeDefinitionElementListeners(defRec.definition, ir);
/*  537 */     if (defRec != activeDefRec) {
/*      */       return;
/*      */     }
/*  540 */     if (defRec.template != null) {
/*  541 */       removeTemplateElementListeners(defRec.template, ir);
/*      */     }
/*  543 */     rebind(defRec.namespaceURI, defRec.localName, this.document.getDocumentElement());
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void removeDefinitionElementListeners(XBLOMDefinitionElement def, ImportRecord ir) {
/*  553 */     XBLEventSupport es = (XBLEventSupport)def.initializeEventSupport();
/*  554 */     es.removeImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMAttrModified", ir.defAttrListener, false);
/*      */ 
/*      */ 
/*      */     
/*  558 */     es.removeImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeInserted", ir.defNodeInsertedListener, false);
/*      */ 
/*      */ 
/*      */     
/*  562 */     es.removeImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", ir.defNodeRemovedListener, false);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void removeTemplateElementListeners(XBLOMTemplateElement template, ImportRecord ir) {
/*  574 */     XBLEventSupport es = (XBLEventSupport)template.initializeEventSupport();
/*      */     
/*  576 */     es.removeImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMAttrModified", ir.templateMutationListener, false);
/*      */ 
/*      */ 
/*      */     
/*  580 */     es.removeImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeInserted", ir.templateMutationListener, false);
/*      */ 
/*      */ 
/*      */     
/*  584 */     es.removeImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMNodeRemoved", ir.templateMutationListener, false);
/*      */ 
/*      */ 
/*      */     
/*  588 */     es.removeImplementationEventListenerNS("http://www.w3.org/2001/xml-events", "DOMCharacterDataModified", ir.templateMutationListener, false);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected DefinitionRecord getActiveDefinition(String namespaceURI, String localName) {
/*  600 */     TreeSet defs = (TreeSet)this.definitionLists.get(namespaceURI, localName);
/*  601 */     if (defs == null || defs.size() == 0) {
/*  602 */       return null;
/*      */     }
/*  604 */     return (DefinitionRecord)defs.first();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void unbind(Element e) {
/*  611 */     if (e instanceof BindableElement) {
/*  612 */       setActiveDefinition((BindableElement)e, null);
/*      */     } else {
/*  614 */       NodeList nl = getXblScopedChildNodes(e);
/*  615 */       for (int i = 0; i < nl.getLength(); i++) {
/*  616 */         Node n = nl.item(i);
/*  617 */         if (n.getNodeType() == 1) {
/*  618 */           unbind((Element)n);
/*      */         }
/*      */       } 
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void bind(Element e) {
/*  628 */     AbstractDocument doc = (AbstractDocument)e.getOwnerDocument();
/*  629 */     if (doc != this.document) {
/*  630 */       XBLManager xm = doc.getXBLManager();
/*  631 */       if (xm instanceof DefaultXBLManager) {
/*  632 */         ((DefaultXBLManager)xm).bind(e);
/*      */         
/*      */         return;
/*      */       } 
/*      */     } 
/*  637 */     if (e instanceof BindableElement) {
/*  638 */       DefinitionRecord defRec = getActiveDefinition(e.getNamespaceURI(), e.getLocalName());
/*      */ 
/*      */       
/*  641 */       setActiveDefinition((BindableElement)e, defRec);
/*      */     } else {
/*  643 */       NodeList nl = getXblScopedChildNodes(e);
/*  644 */       for (int i = 0; i < nl.getLength(); i++) {
/*  645 */         Node n = nl.item(i);
/*  646 */         if (n.getNodeType() == 1) {
/*  647 */           bind((Element)n);
/*      */         }
/*      */       } 
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void rebind(String namespaceURI, String localName, Element e) {
/*  658 */     AbstractDocument doc = (AbstractDocument)e.getOwnerDocument();
/*  659 */     if (doc != this.document) {
/*  660 */       XBLManager xm = doc.getXBLManager();
/*  661 */       if (xm instanceof DefaultXBLManager) {
/*  662 */         ((DefaultXBLManager)xm).rebind(namespaceURI, localName, e);
/*      */         
/*      */         return;
/*      */       } 
/*      */     } 
/*  667 */     if (e instanceof BindableElement && namespaceURI.equals(e.getNamespaceURI()) && localName.equals(e.getLocalName())) {
/*      */ 
/*      */       
/*  670 */       DefinitionRecord defRec = getActiveDefinition(e.getNamespaceURI(), e.getLocalName());
/*      */ 
/*      */       
/*  673 */       setActiveDefinition((BindableElement)e, defRec);
/*      */     } else {
/*  675 */       NodeList nl = getXblScopedChildNodes(e);
/*  676 */       for (int i = 0; i < nl.getLength(); i++) {
/*  677 */         Node n = nl.item(i);
/*  678 */         if (n.getNodeType() == 1) {
/*  679 */           rebind(namespaceURI, localName, (Element)n);
/*      */         }
/*      */       } 
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void setActiveDefinition(BindableElement elt, DefinitionRecord defRec) {
/*  691 */     XBLRecord rec = getRecord(elt);
/*  692 */     rec.definitionElement = (defRec == null) ? null : defRec.definition;
/*  693 */     if (defRec != null && defRec.definition != null && defRec.template != null) {
/*      */ 
/*      */       
/*  696 */       setXblShadowTree(elt, cloneTemplate(defRec.template));
/*      */     } else {
/*  698 */       setXblShadowTree(elt, null);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void setXblShadowTree(BindableElement elt, XBLOMShadowTreeElement newShadow) {
/*  707 */     XBLOMShadowTreeElement oldShadow = (XBLOMShadowTreeElement)getXblShadowTree(elt);
/*      */     
/*  709 */     if (oldShadow != null) {
/*  710 */       fireShadowTreeEvent(elt, "unbinding", oldShadow);
/*  711 */       ContentManager cm = getContentManager(oldShadow);
/*  712 */       if (cm != null) {
/*  713 */         cm.dispose();
/*      */       }
/*  715 */       elt.setShadowTree(null);
/*  716 */       XBLRecord rec = getRecord(oldShadow);
/*  717 */       rec.boundElement = null;
/*  718 */       oldShadow.removeEventListenerNS("http://www.w3.org/2001/xml-events", "DOMSubtreeModified", this.docSubtreeListener, false);
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/*  723 */     if (newShadow != null) {
/*  724 */       newShadow.addEventListenerNS("http://www.w3.org/2001/xml-events", "DOMSubtreeModified", this.docSubtreeListener, false, null);
/*      */ 
/*      */ 
/*      */       
/*  728 */       fireShadowTreeEvent(elt, "prebind", newShadow);
/*  729 */       elt.setShadowTree(newShadow);
/*  730 */       XBLRecord rec = getRecord(newShadow);
/*  731 */       rec.boundElement = elt;
/*  732 */       AbstractDocument doc = (AbstractDocument)elt.getOwnerDocument();
/*      */       
/*  734 */       XBLManager xm = doc.getXBLManager();
/*  735 */       ContentManager cm = new ContentManager(newShadow, xm);
/*  736 */       setContentManager(newShadow, cm);
/*      */     } 
/*  738 */     invalidateChildNodes(elt);
/*  739 */     if (newShadow != null) {
/*  740 */       NodeList nl = getXblScopedChildNodes(elt);
/*  741 */       for (int i = 0; i < nl.getLength(); i++) {
/*  742 */         Node n = nl.item(i);
/*  743 */         if (n.getNodeType() == 1) {
/*  744 */           bind((Element)n);
/*      */         }
/*      */       } 
/*  747 */       dispatchBindingChangedEvent(elt, newShadow);
/*  748 */       fireShadowTreeEvent(elt, "bound", newShadow);
/*      */     } else {
/*  750 */       dispatchBindingChangedEvent(elt, newShadow);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void fireShadowTreeEvent(BindableElement elt, String type, XBLShadowTreeElement e) {
/*  760 */     DocumentEvent de = (DocumentEvent)elt.getOwnerDocument();
/*  761 */     ShadowTreeEvent evt = (ShadowTreeEvent)de.createEvent("ShadowTreeEvent");
/*      */     
/*  763 */     evt.initShadowTreeEventNS("http://www.w3.org/2004/xbl", type, true, false, e);
/*  764 */     elt.dispatchEvent(evt);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected XBLOMShadowTreeElement cloneTemplate(XBLOMTemplateElement template) {
/*  772 */     XBLOMShadowTreeElement clone = (XBLOMShadowTreeElement)template.getOwnerDocument().createElementNS("http://www.w3.org/2004/xbl", "shadowTree");
/*      */ 
/*      */ 
/*      */     
/*  776 */     NamedNodeMap attrs = template.getAttributes();
/*  777 */     for (int i = 0; i < attrs.getLength(); i++) {
/*  778 */       Attr attr = (Attr)attrs.item(i);
/*  779 */       if (attr instanceof org.apache.batik.dom.AbstractAttrNS) {
/*  780 */         clone.setAttributeNodeNS(attr);
/*      */       } else {
/*  782 */         clone.setAttributeNode(attr);
/*      */       } 
/*      */     } 
/*  785 */     Node n = template.getFirstChild();
/*  786 */     for (; n != null; 
/*  787 */       n = n.getNextSibling()) {
/*  788 */       clone.appendChild(n.cloneNode(true));
/*      */     }
/*  790 */     return clone;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Node getXblParentNode(Node n) {
/*  797 */     XBLOMContentElement xBLOMContentElement = getXblContentElement(n);
/*  798 */     Node parent = (xBLOMContentElement == null) ? n.getParentNode() : xBLOMContentElement.getParentNode();
/*      */ 
/*      */     
/*  801 */     if (parent instanceof XBLOMContentElement) {
/*  802 */       parent = parent.getParentNode();
/*      */     }
/*  804 */     if (parent instanceof XBLOMShadowTreeElement) {
/*  805 */       parent = getXblBoundElement(parent);
/*      */     }
/*  807 */     return parent;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public NodeList getXblChildNodes(Node n) {
/*  814 */     XBLRecord rec = getRecord(n);
/*  815 */     if (rec.childNodes == null) {
/*  816 */       rec.childNodes = new XblChildNodes(this, rec);
/*      */     }
/*  818 */     return rec.childNodes;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public NodeList getXblScopedChildNodes(Node n) {
/*  826 */     XBLRecord rec = getRecord(n);
/*  827 */     if (rec.scopedChildNodes == null) {
/*  828 */       rec.scopedChildNodes = new XblScopedChildNodes(this, rec);
/*      */     }
/*  830 */     return rec.scopedChildNodes;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Node getXblFirstChild(Node n) {
/*  837 */     NodeList nl = getXblChildNodes(n);
/*  838 */     return nl.item(0);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Node getXblLastChild(Node n) {
/*  845 */     NodeList nl = getXblChildNodes(n);
/*  846 */     return nl.item(nl.getLength() - 1);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Node getXblPreviousSibling(Node n) {
/*  854 */     Node p = getXblParentNode(n);
/*  855 */     if (p == null || (getRecord(p)).childNodes == null) {
/*  856 */       return n.getPreviousSibling();
/*      */     }
/*  858 */     XBLRecord rec = getRecord(n);
/*  859 */     if (!rec.linksValid) {
/*  860 */       updateLinks(n);
/*      */     }
/*  862 */     return rec.previousSibling;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Node getXblNextSibling(Node n) {
/*  870 */     Node p = getXblParentNode(n);
/*  871 */     if (p == null || (getRecord(p)).childNodes == null) {
/*  872 */       return n.getNextSibling();
/*      */     }
/*  874 */     XBLRecord rec = getRecord(n);
/*  875 */     if (!rec.linksValid) {
/*  876 */       updateLinks(n);
/*      */     }
/*  878 */     return rec.nextSibling;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Element getXblFirstElementChild(Node n) {
/*  885 */     n = getXblFirstChild(n);
/*  886 */     while (n != null && n.getNodeType() != 1) {
/*  887 */       n = getXblNextSibling(n);
/*      */     }
/*  889 */     return (Element)n;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Element getXblLastElementChild(Node n) {
/*  896 */     n = getXblLastChild(n);
/*  897 */     while (n != null && n.getNodeType() != 1) {
/*  898 */       n = getXblPreviousSibling(n);
/*      */     }
/*  900 */     return (Element)n;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Element getXblPreviousElementSibling(Node n) {
/*      */     do {
/*  909 */       n = getXblPreviousSibling(n);
/*  910 */     } while (n != null && n.getNodeType() != 1);
/*  911 */     return (Element)n;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Element getXblNextElementSibling(Node n) {
/*      */     do {
/*  920 */       n = getXblNextSibling(n);
/*  921 */     } while (n != null && n.getNodeType() != 1);
/*  922 */     return (Element)n;
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   public Element getXblBoundElement(Node n) {
/*      */     Node node;
/*  929 */     while (n != null && !(n instanceof XBLShadowTreeElement)) {
/*  930 */       XBLOMContentElement xBLOMContentElement, content = getXblContentElement(n);
/*  931 */       if (content != null) {
/*  932 */         xBLOMContentElement = content;
/*      */       }
/*  934 */       node = xBLOMContentElement.getParentNode();
/*      */     } 
/*  936 */     if (node == null) {
/*  937 */       return null;
/*      */     }
/*  939 */     return (getRecord(node)).boundElement;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Element getXblShadowTree(Node n) {
/*  946 */     if (n instanceof BindableElement) {
/*  947 */       BindableElement elt = (BindableElement)n;
/*  948 */       return elt.getShadowTree();
/*      */     } 
/*  950 */     return null;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public NodeList getXblDefinitions(Node n) {
/*  957 */     String namespaceURI = n.getNamespaceURI();
/*  958 */     String localName = n.getLocalName();
/*  959 */     return new NodeList(this, namespaceURI, localName) { private final String val$namespaceURI;
/*      */         public Node item(int i) {
/*  961 */           TreeSet defs = (TreeSet)this.this$0.definitionLists.get(this.val$namespaceURI, this.val$localName);
/*  962 */           if (defs != null && defs.size() != 0 && i == 0) {
/*  963 */             DefaultXBLManager.DefinitionRecord defRec = (DefaultXBLManager.DefinitionRecord)defs.first();
/*  964 */             return defRec.definition;
/*      */           } 
/*  966 */           return null;
/*      */         } private final String val$localName; private final DefaultXBLManager this$0;
/*      */         public int getLength() {
/*  969 */           Set defs = (TreeSet)this.this$0.definitionLists.get(this.val$namespaceURI, this.val$localName);
/*  970 */           return (defs != null && defs.size() != 0) ? 1 : 0;
/*      */         } }
/*      */       ;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected XBLRecord getRecord(Node n) {
/*  979 */     XBLManagerData xmd = (XBLManagerData)n;
/*  980 */     XBLRecord rec = (XBLRecord)xmd.getManagerData();
/*  981 */     if (rec == null) {
/*  982 */       rec = new XBLRecord(this);
/*  983 */       rec.node = n;
/*  984 */       xmd.setManagerData(rec);
/*      */     } 
/*  986 */     return rec;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void updateLinks(Node n) {
/*  994 */     XBLRecord rec = getRecord(n);
/*  995 */     rec.previousSibling = null;
/*  996 */     rec.nextSibling = null;
/*  997 */     rec.linksValid = true;
/*  998 */     Node p = getXblParentNode(n);
/*  999 */     if (p != null) {
/* 1000 */       NodeList xcn = getXblChildNodes(p);
/* 1001 */       if (xcn instanceof XblChildNodes) {
/* 1002 */         ((XblChildNodes)xcn).update();
/*      */       }
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1012 */   public XBLOMContentElement getXblContentElement(Node n) { return (getRecord(n)).contentElement; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static int computeBubbleLimit(Node from, Node to) {
/* 1022 */     ArrayList fromList = new ArrayList(10);
/* 1023 */     ArrayList toList = new ArrayList(10);
/* 1024 */     while (from != null) {
/* 1025 */       fromList.add(from);
/* 1026 */       from = ((NodeXBL)from).getXblParentNode();
/*      */     } 
/* 1028 */     while (to != null) {
/* 1029 */       toList.add(to);
/* 1030 */       to = ((NodeXBL)to).getXblParentNode();
/*      */     } 
/* 1032 */     int fromSize = fromList.size();
/* 1033 */     int toSize = toList.size();
/* 1034 */     for (int i = 0; i < fromSize && i < toSize; i++) {
/* 1035 */       Node n1 = (Node)fromList.get(fromSize - i - 1);
/* 1036 */       Node n2 = (Node)toList.get(toSize - i - 1);
/* 1037 */       if (n1 != n2) {
/* 1038 */         Node prevBoundElement = ((NodeXBL)n1).getXblBoundElement();
/* 1039 */         while (i > 0 && prevBoundElement != fromList.get(fromSize - i - true)) {
/* 1040 */           i--;
/*      */         }
/* 1042 */         return fromSize - i - 1;
/*      */       } 
/*      */     } 
/* 1045 */     return 1;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public ContentManager getContentManager(Node n) {
/* 1053 */     Node b = getXblBoundElement(n);
/* 1054 */     if (b != null) {
/* 1055 */       Element s = getXblShadowTree(b);
/* 1056 */       if (s != null) {
/*      */         ContentManager contentManager;
/* 1058 */         Document doc = b.getOwnerDocument();
/* 1059 */         if (doc != this.document) {
/* 1060 */           DefaultXBLManager xm = (DefaultXBLManager)((AbstractDocument)doc).getXBLManager();
/*      */           
/* 1062 */           contentManager = (ContentManager)xm.contentManagers.get(s);
/*      */         } else {
/* 1064 */           contentManager = (ContentManager)this.contentManagers.get(s);
/*      */         } 
/* 1066 */         return contentManager;
/*      */       } 
/*      */     } 
/* 1069 */     return null;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   void setContentManager(Element shadow, ContentManager cm) {
/* 1076 */     if (cm == null) {
/* 1077 */       this.contentManagers.remove(shadow);
/*      */     } else {
/* 1079 */       this.contentManagers.put(shadow, cm);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void invalidateChildNodes(Node n) {
/* 1088 */     XBLRecord rec = getRecord(n);
/* 1089 */     if (rec.childNodes != null) {
/* 1090 */       rec.childNodes.invalidate();
/*      */     }
/* 1092 */     if (rec.scopedChildNodes != null) {
/* 1093 */       rec.scopedChildNodes.invalidate();
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1103 */   public void addContentSelectionChangedListener(ContentSelectionChangedListener l) { this.contentSelectionChangedListenerList.add(ContentSelectionChangedListener.class, l); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1113 */   public void removeContentSelectionChangedListener(ContentSelectionChangedListener l) { this.contentSelectionChangedListenerList.remove(ContentSelectionChangedListener.class, l); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1121 */   protected Object[] getContentSelectionChangedListeners() { return this.contentSelectionChangedListenerList.getListenerList(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   void shadowTreeSelectedContentChanged(Set deselected, Set selected) {
/* 1129 */     Iterator i = deselected.iterator();
/* 1130 */     while (i.hasNext()) {
/* 1131 */       Node n = (Node)i.next();
/* 1132 */       if (n.getNodeType() == 1) {
/* 1133 */         unbind((Element)n);
/*      */       }
/*      */     } 
/* 1136 */     i = selected.iterator();
/* 1137 */     while (i.hasNext()) {
/* 1138 */       Node n = (Node)i.next();
/* 1139 */       if (n.getNodeType() == 1) {
/* 1140 */         bind((Element)n);
/*      */       }
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1149 */   public void addBindingListener(BindingListener l) { this.bindingListenerList.add(BindingListener.class, l); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 1156 */   public void removeBindingListener(BindingListener l) { this.bindingListenerList.remove(BindingListener.class, l); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void dispatchBindingChangedEvent(Element bindableElement, Element shadowTree) {
/* 1166 */     Object[] ls = this.bindingListenerList.getListenerList();
/* 1167 */     for (int i = ls.length - 2; i >= 0; i -= 2) {
/* 1168 */       BindingListener l = (BindingListener)ls[i + 1];
/* 1169 */       l.bindingChanged(bindableElement, shadowTree);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected boolean isActiveDefinition(XBLOMDefinitionElement def, Element imp) {
/* 1179 */     DefinitionRecord defRec = (DefinitionRecord)this.definitions.get(def, imp);
/* 1180 */     if (defRec == null) {
/* 1181 */       return false;
/*      */     }
/* 1183 */     return (defRec == getActiveDefinition(defRec.namespaceURI, defRec.localName));
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class DefinitionRecord
/*      */     implements Comparable
/*      */   {
/*      */     public String namespaceURI;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public String localName;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public XBLOMDefinitionElement definition;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public XBLOMTemplateElement template;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public Element importElement;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     private final DefaultXBLManager this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public DefinitionRecord(DefaultXBLManager this$0, String ns, String ln, XBLOMDefinitionElement def, XBLOMTemplateElement t, Element imp) {
/* 1224 */       this.this$0 = this$0;
/* 1225 */       this.namespaceURI = ns;
/* 1226 */       this.localName = ln;
/* 1227 */       this.definition = def;
/* 1228 */       this.template = t;
/* 1229 */       this.importElement = imp;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1236 */     public boolean equals(Object other) { return (compareTo(other) == 0); }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public int compareTo(Object other) {
/*      */       AbstractNode abstractNode2, abstractNode1;
/* 1243 */       DefinitionRecord rec = (DefinitionRecord)other;
/*      */       
/* 1245 */       if (this.importElement == null) {
/* 1246 */         abstractNode1 = this.definition;
/* 1247 */         if (rec.importElement == null) {
/* 1248 */           abstractNode2 = rec.definition;
/*      */         } else {
/* 1250 */           abstractNode2 = (AbstractNode)rec.importElement;
/*      */         } 
/* 1252 */       } else if (rec.importElement == null) {
/* 1253 */         abstractNode1 = (AbstractNode)this.importElement;
/* 1254 */         abstractNode2 = rec.definition;
/* 1255 */       } else if (this.definition.getOwnerDocument() == rec.definition.getOwnerDocument()) {
/*      */         
/* 1257 */         abstractNode1 = this.definition;
/* 1258 */         abstractNode2 = rec.definition;
/*      */       } else {
/* 1260 */         abstractNode1 = (AbstractNode)this.importElement;
/* 1261 */         abstractNode2 = (AbstractNode)rec.importElement;
/*      */       } 
/* 1263 */       short comp = abstractNode1.compareDocumentPosition(abstractNode2);
/* 1264 */       if ((comp & 0x2) != 0) {
/* 1265 */         return -1;
/*      */       }
/* 1267 */       if ((comp & 0x4) != 0) {
/* 1268 */         return 1;
/*      */       }
/* 1270 */       return 0;
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class ImportRecord
/*      */   {
/*      */     public Element importElement;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public Node node;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public DefaultXBLManager.DefNodeInsertedListener defNodeInsertedListener;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public DefaultXBLManager.DefNodeRemovedListener defNodeRemovedListener;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public DefaultXBLManager.DefAttrListener defAttrListener;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public DefaultXBLManager.ImportInsertedListener importInsertedListener;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public DefaultXBLManager.ImportRemovedListener importRemovedListener;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public DefaultXBLManager.ImportSubtreeListener importSubtreeListener;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public DefaultXBLManager.TemplateMutationListener templateMutationListener;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     private final DefaultXBLManager this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public ImportRecord(DefaultXBLManager this$0, Element imp, Node n) {
/* 1331 */       this.this$0 = this$0;
/* 1332 */       this.importElement = imp;
/* 1333 */       this.node = n;
/* 1334 */       this.defNodeInsertedListener = new DefaultXBLManager.DefNodeInsertedListener(this$0, imp);
/* 1335 */       this.defNodeRemovedListener = new DefaultXBLManager.DefNodeRemovedListener(this$0, imp);
/* 1336 */       this.defAttrListener = new DefaultXBLManager.DefAttrListener(this$0, imp);
/* 1337 */       this.importInsertedListener = new DefaultXBLManager.ImportInsertedListener(this$0, imp);
/* 1338 */       this.importRemovedListener = new DefaultXBLManager.ImportRemovedListener(this$0);
/* 1339 */       this.importSubtreeListener = new DefaultXBLManager.ImportSubtreeListener(this$0, imp, this.importRemovedListener);
/*      */       
/* 1341 */       this.templateMutationListener = new DefaultXBLManager.TemplateMutationListener(this$0, imp);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class ImportInsertedListener
/*      */     implements EventListener
/*      */   {
/*      */     protected Element importElement;
/*      */ 
/*      */     
/*      */     private final DefaultXBLManager this$0;
/*      */ 
/*      */ 
/*      */     
/*      */     public ImportInsertedListener(DefaultXBLManager this$0, Element importElement) {
/* 1358 */       this.this$0 = this$0;
/* 1359 */       this.importElement = importElement;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1366 */       EventTarget target = evt.getTarget();
/* 1367 */       if (target instanceof XBLOMDefinitionElement) {
/* 1368 */         XBLOMDefinitionElement def = (XBLOMDefinitionElement)target;
/* 1369 */         this.this$0.addDefinition(def.getElementNamespaceURI(), def.getElementLocalName(), def, this.importElement);
/*      */       } 
/*      */     }
/*      */   }
/*      */   
/*      */   protected class ImportRemovedListener
/*      */     implements EventListener {
/*      */     protected LinkedList toBeRemoved;
/*      */     private final DefaultXBLManager this$0;
/*      */     
/*      */     protected ImportRemovedListener(DefaultXBLManager this$0) {
/* 1380 */       this.this$0 = this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1385 */       this.toBeRemoved = new LinkedList();
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1391 */     public void handleEvent(Event evt) { this.toBeRemoved.add(evt.getTarget()); }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class ImportSubtreeListener
/*      */     implements EventListener
/*      */   {
/*      */     protected Element importElement;
/*      */ 
/*      */ 
/*      */     
/*      */     protected DefaultXBLManager.ImportRemovedListener importRemovedListener;
/*      */ 
/*      */ 
/*      */     
/*      */     private final DefaultXBLManager this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public ImportSubtreeListener(DefaultXBLManager this$0, Element imp, DefaultXBLManager.ImportRemovedListener irl) {
/* 1414 */       this.this$0 = this$0;
/* 1415 */       this.importElement = imp;
/* 1416 */       this.importRemovedListener = irl;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1423 */       Object[] defs = this.importRemovedListener.toBeRemoved.toArray();
/* 1424 */       this.importRemovedListener.toBeRemoved.clear();
/* 1425 */       for (int i = 0; i < defs.length; i++) {
/* 1426 */         XBLOMDefinitionElement def = (XBLOMDefinitionElement)defs[i];
/* 1427 */         DefaultXBLManager.DefinitionRecord defRec = (DefaultXBLManager.DefinitionRecord)this.this$0.definitions.get(def, this.importElement);
/*      */         
/* 1429 */         this.this$0.removeDefinition(defRec);
/*      */       } 
/*      */     }
/*      */   }
/*      */   
/*      */   protected class DocInsertedListener implements EventListener {
/*      */     private final DefaultXBLManager this$0;
/*      */     
/* 1437 */     protected DocInsertedListener(DefaultXBLManager this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1443 */       EventTarget target = evt.getTarget();
/* 1444 */       if (target instanceof XBLOMDefinitionElement) {
/*      */         
/* 1446 */         if (this.this$0.getXblBoundElement((Node)target) == null) {
/* 1447 */           XBLOMDefinitionElement def = (XBLOMDefinitionElement)target;
/*      */           
/* 1449 */           if (def.getAttributeNS(null, "ref").length() == 0) {
/*      */             
/* 1451 */             this.this$0.addDefinition(def.getElementNamespaceURI(), def.getElementLocalName(), def, null);
/*      */           
/*      */           }
/*      */           else {
/*      */             
/* 1456 */             this.this$0.addDefinitionRef(def);
/*      */           } 
/*      */         } 
/* 1459 */       } else if (target instanceof org.apache.batik.dom.svg12.XBLOMImportElement) {
/*      */         
/* 1461 */         if (this.this$0.getXblBoundElement((Node)target) == null) {
/* 1462 */           this.this$0.addImport((Element)target);
/*      */         }
/*      */       } else {
/* 1465 */         evt = EventSupport.getUltimateOriginalEvent(evt);
/* 1466 */         target = evt.getTarget();
/* 1467 */         Node parent = this.this$0.getXblParentNode((Node)target);
/* 1468 */         if (parent != null) {
/* 1469 */           this.this$0.invalidateChildNodes(parent);
/*      */         }
/* 1471 */         if (target instanceof BindableElement) {
/*      */ 
/*      */ 
/*      */ 
/*      */           
/* 1476 */           Node n = ((Node)target).getParentNode();
/* 1477 */           for (; n != null; 
/* 1478 */             n = n.getParentNode()) {
/* 1479 */             if (n instanceof BindableElement && (this.this$0.getRecord(n)).definitionElement != null) {
/*      */               return;
/*      */             }
/*      */           } 
/*      */           
/* 1484 */           this.this$0.bind((Element)target);
/*      */         } 
/*      */       } 
/*      */     } }
/*      */   protected class DocRemovedListener implements EventListener { protected LinkedList defsToBeRemoved; protected LinkedList importsToBeRemoved;
/*      */     protected LinkedList nodesToBeInvalidated;
/*      */     private final DefaultXBLManager this$0;
/*      */     
/*      */     protected DocRemovedListener(DefaultXBLManager this$0) {
/* 1493 */       this.this$0 = this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1498 */       this.defsToBeRemoved = new LinkedList();
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1503 */       this.importsToBeRemoved = new LinkedList();
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1508 */       this.nodesToBeInvalidated = new LinkedList();
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1514 */       EventTarget target = evt.getTarget();
/* 1515 */       if (target instanceof XBLOMDefinitionElement) {
/*      */         
/* 1517 */         if (this.this$0.getXblBoundElement((Node)target) == null) {
/* 1518 */           this.defsToBeRemoved.add(target);
/*      */         }
/* 1520 */       } else if (target instanceof org.apache.batik.dom.svg12.XBLOMImportElement) {
/*      */         
/* 1522 */         if (this.this$0.getXblBoundElement((Node)target) == null) {
/* 1523 */           this.importsToBeRemoved.add(target);
/*      */         }
/*      */       } 
/*      */       
/* 1527 */       Node parent = this.this$0.getXblParentNode((Node)target);
/* 1528 */       if (parent != null)
/* 1529 */         this.nodesToBeInvalidated.add(parent); 
/*      */     } }
/*      */ 
/*      */   
/*      */   protected class DocSubtreeListener
/*      */     implements EventListener {
/*      */     private final DefaultXBLManager this$0;
/*      */     
/* 1537 */     protected DocSubtreeListener(DefaultXBLManager this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1543 */       Object[] defs = this.this$0.docRemovedListener.defsToBeRemoved.toArray();
/* 1544 */       this.this$0.docRemovedListener.defsToBeRemoved.clear();
/* 1545 */       for (int i = 0; i < defs.length; i++) {
/* 1546 */         XBLOMDefinitionElement def = (XBLOMDefinitionElement)defs[i];
/* 1547 */         if (def.getAttributeNS(null, "ref").length() == 0) {
/* 1548 */           DefaultXBLManager.DefinitionRecord defRec = (DefaultXBLManager.DefinitionRecord)this.this$0.definitions.get(def, null);
/*      */           
/* 1550 */           this.this$0.removeDefinition(defRec);
/*      */         } else {
/* 1552 */           this.this$0.removeDefinitionRef(def);
/*      */         } 
/*      */       } 
/*      */       
/* 1556 */       Object[] imps = this.this$0.docRemovedListener.importsToBeRemoved.toArray();
/* 1557 */       this.this$0.docRemovedListener.importsToBeRemoved.clear();
/* 1558 */       for (int i = 0; i < imps.length; i++) {
/* 1559 */         this.this$0.removeImport((Element)imps[i]);
/*      */       }
/*      */       
/* 1562 */       Object[] nodes = this.this$0.docRemovedListener.nodesToBeInvalidated.toArray();
/* 1563 */       this.this$0.docRemovedListener.nodesToBeInvalidated.clear();
/* 1564 */       for (int i = 0; i < nodes.length; i++) {
/* 1565 */         this.this$0.invalidateChildNodes((Node)nodes[i]);
/*      */       }
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class TemplateMutationListener
/*      */     implements EventListener
/*      */   {
/*      */     protected Element importElement;
/*      */ 
/*      */     
/*      */     private final DefaultXBLManager this$0;
/*      */ 
/*      */ 
/*      */     
/*      */     public TemplateMutationListener(DefaultXBLManager this$0, Element imp) {
/* 1583 */       this.this$0 = this$0;
/* 1584 */       this.importElement = imp;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1591 */       Node n = (Node)evt.getTarget();
/* 1592 */       while (n != null && !(n instanceof XBLOMDefinitionElement)) {
/* 1593 */         n = n.getParentNode();
/*      */       }
/*      */       
/* 1596 */       DefaultXBLManager.DefinitionRecord defRec = (DefaultXBLManager.DefinitionRecord)this.this$0.definitions.get(n, this.importElement);
/*      */       
/* 1598 */       if (defRec == null) {
/*      */         return;
/*      */       }
/*      */       
/* 1602 */       this.this$0.rebind(defRec.namespaceURI, defRec.localName, this.this$0.document.getDocumentElement());
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class DefAttrListener
/*      */     implements EventListener
/*      */   {
/*      */     protected Element importElement;
/*      */ 
/*      */     
/*      */     private final DefaultXBLManager this$0;
/*      */ 
/*      */ 
/*      */     
/*      */     public DefAttrListener(DefaultXBLManager this$0, Element imp) {
/* 1620 */       this.this$0 = this$0;
/* 1621 */       this.importElement = imp;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1628 */       EventTarget target = evt.getTarget();
/* 1629 */       if (!(target instanceof XBLOMDefinitionElement)) {
/*      */         return;
/*      */       }
/*      */       
/* 1633 */       XBLOMDefinitionElement def = (XBLOMDefinitionElement)target;
/* 1634 */       if (!this.this$0.isActiveDefinition(def, this.importElement)) {
/*      */         return;
/*      */       }
/*      */       
/* 1638 */       MutationEvent mevt = (MutationEvent)evt;
/* 1639 */       String attrName = mevt.getAttrName();
/* 1640 */       if (attrName.equals("element")) {
/* 1641 */         DefaultXBLManager.DefinitionRecord defRec = (DefaultXBLManager.DefinitionRecord)this.this$0.definitions.get(def, this.importElement);
/*      */         
/* 1643 */         this.this$0.removeDefinition(defRec);
/*      */         
/* 1645 */         this.this$0.addDefinition(def.getElementNamespaceURI(), def.getElementLocalName(), def, this.importElement);
/*      */ 
/*      */       
/*      */       }
/* 1649 */       else if (attrName.equals("ref") && 
/* 1650 */         mevt.getNewValue().length() != 0) {
/* 1651 */         DefaultXBLManager.DefinitionRecord defRec = (DefaultXBLManager.DefinitionRecord)this.this$0.definitions.get(def, this.importElement);
/*      */         
/* 1653 */         this.this$0.removeDefinition(defRec);
/* 1654 */         this.this$0.addDefinitionRef(def);
/*      */       } 
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class DefNodeInsertedListener
/*      */     implements EventListener
/*      */   {
/*      */     protected Element importElement;
/*      */ 
/*      */     
/*      */     private final DefaultXBLManager this$0;
/*      */ 
/*      */ 
/*      */     
/*      */     public DefNodeInsertedListener(DefaultXBLManager this$0, Element imp) {
/* 1673 */       this.this$0 = this$0;
/* 1674 */       this.importElement = imp;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1681 */       MutationEvent mevt = (MutationEvent)evt;
/* 1682 */       Node parent = mevt.getRelatedNode();
/* 1683 */       if (!(parent instanceof XBLOMDefinitionElement)) {
/*      */         return;
/*      */       }
/*      */       
/* 1687 */       EventTarget target = evt.getTarget();
/* 1688 */       if (!(target instanceof XBLOMTemplateElement)) {
/*      */         return;
/*      */       }
/* 1691 */       XBLOMTemplateElement template = (XBLOMTemplateElement)target;
/*      */       
/* 1693 */       DefaultXBLManager.DefinitionRecord defRec = (DefaultXBLManager.DefinitionRecord)this.this$0.definitions.get(parent, this.importElement);
/*      */       
/* 1695 */       if (defRec == null) {
/*      */         return;
/*      */       }
/*      */       
/* 1699 */       DefaultXBLManager.ImportRecord ir = (DefaultXBLManager.ImportRecord)this.this$0.imports.get(this.importElement);
/*      */       
/* 1701 */       if (defRec.template != null) {
/* 1702 */         Node n = parent.getFirstChild();
/* 1703 */         for (; n != null; 
/* 1704 */           n = n.getNextSibling()) {
/* 1705 */           if (n == template) {
/* 1706 */             this.this$0.removeTemplateElementListeners(defRec.template, ir);
/* 1707 */             defRec.template = template; break;
/*      */           } 
/* 1709 */           if (n == defRec.template) {
/*      */             return;
/*      */           }
/*      */         } 
/*      */       } else {
/* 1714 */         defRec.template = template;
/*      */       } 
/* 1716 */       this.this$0.addTemplateElementListeners(template, ir);
/* 1717 */       this.this$0.rebind(defRec.namespaceURI, defRec.localName, this.this$0.document.getDocumentElement());
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class DefNodeRemovedListener
/*      */     implements EventListener
/*      */   {
/*      */     protected Element importElement;
/*      */ 
/*      */     
/*      */     private final DefaultXBLManager this$0;
/*      */ 
/*      */ 
/*      */     
/*      */     public DefNodeRemovedListener(DefaultXBLManager this$0, Element imp) {
/* 1735 */       this.this$0 = this$0;
/* 1736 */       this.importElement = imp;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1743 */       MutationEvent mevt = (MutationEvent)evt;
/* 1744 */       Node parent = mevt.getRelatedNode();
/* 1745 */       if (!(parent instanceof XBLOMDefinitionElement)) {
/*      */         return;
/*      */       }
/*      */       
/* 1749 */       EventTarget target = evt.getTarget();
/* 1750 */       if (!(target instanceof XBLOMTemplateElement)) {
/*      */         return;
/*      */       }
/* 1753 */       XBLOMTemplateElement template = (XBLOMTemplateElement)target;
/*      */       
/* 1755 */       DefaultXBLManager.DefinitionRecord defRec = (DefaultXBLManager.DefinitionRecord)this.this$0.definitions.get(parent, this.importElement);
/*      */       
/* 1757 */       if (defRec == null || defRec.template != template) {
/*      */         return;
/*      */       }
/*      */       
/* 1761 */       DefaultXBLManager.ImportRecord ir = (DefaultXBLManager.ImportRecord)this.this$0.imports.get(this.importElement);
/*      */       
/* 1763 */       this.this$0.removeTemplateElementListeners(template, ir);
/* 1764 */       defRec.template = null;
/*      */       
/* 1766 */       Node n = template.getNextSibling();
/* 1767 */       for (; n != null; 
/* 1768 */         n = n.getNextSibling()) {
/* 1769 */         if (n instanceof XBLOMTemplateElement) {
/* 1770 */           defRec.template = (XBLOMTemplateElement)n;
/*      */           
/*      */           break;
/*      */         } 
/*      */       } 
/* 1775 */       this.this$0.addTemplateElementListeners(defRec.template, ir);
/* 1776 */       this.this$0.rebind(defRec.namespaceURI, defRec.localName, this.this$0.document.getDocumentElement());
/*      */     }
/*      */   }
/*      */   
/*      */   protected class ImportAttrListener
/*      */     implements EventListener {
/*      */     private final DefaultXBLManager this$0;
/*      */     
/* 1784 */     protected ImportAttrListener(DefaultXBLManager this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1790 */       EventTarget target = evt.getTarget();
/* 1791 */       if (target != evt.getCurrentTarget()) {
/*      */         return;
/*      */       }
/*      */       
/* 1795 */       MutationEvent mevt = (MutationEvent)evt;
/* 1796 */       if (mevt.getAttrName().equals("bindings")) {
/* 1797 */         Element imp = (Element)target;
/* 1798 */         this.this$0.removeImport(imp);
/* 1799 */         this.this$0.addImport(imp);
/*      */       } 
/*      */     }
/*      */   }
/*      */   
/*      */   protected class RefAttrListener implements EventListener {
/*      */     private final DefaultXBLManager this$0;
/*      */     
/* 1807 */     protected RefAttrListener(DefaultXBLManager this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void handleEvent(Event evt) {
/* 1813 */       EventTarget target = evt.getTarget();
/* 1814 */       if (target != evt.getCurrentTarget()) {
/*      */         return;
/*      */       }
/*      */       
/* 1818 */       MutationEvent mevt = (MutationEvent)evt;
/* 1819 */       if (mevt.getAttrName().equals("ref")) {
/* 1820 */         Element defRef = (Element)target;
/* 1821 */         this.this$0.removeDefinitionRef(defRef);
/* 1822 */         if (mevt.getNewValue().length() == 0) {
/* 1823 */           XBLOMDefinitionElement def = (XBLOMDefinitionElement)defRef;
/*      */           
/* 1825 */           String ns = def.getElementNamespaceURI();
/* 1826 */           String ln = def.getElementLocalName();
/* 1827 */           this.this$0.addDefinition(ns, ln, (XBLOMDefinitionElement)defRef, null);
/*      */         } else {
/*      */           
/* 1830 */           this.this$0.addDefinitionRef(defRef);
/*      */         } 
/*      */       } 
/*      */     } }
/*      */   protected class XBLRecord { public Node node; public DefaultXBLManager.XblChildNodes childNodes;
/*      */     public DefaultXBLManager.XblScopedChildNodes scopedChildNodes;
/*      */     public XBLOMContentElement contentElement;
/*      */     public XBLOMDefinitionElement definitionElement;
/*      */     
/* 1839 */     protected XBLRecord(DefaultXBLManager this$0) { this.this$0 = this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public BindableElement boundElement;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public boolean linksValid;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public Node nextSibling;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public Node previousSibling;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     private final DefaultXBLManager this$0; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected class XblChildNodes
/*      */     implements NodeList
/*      */   {
/*      */     protected DefaultXBLManager.XBLRecord record;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected List nodes;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected int size;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     private final DefaultXBLManager this$0;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public XblChildNodes(DefaultXBLManager this$0, DefaultXBLManager.XBLRecord rec) {
/* 1912 */       this.this$0 = this$0;
/* 1913 */       this.record = rec;
/* 1914 */       this.nodes = new ArrayList();
/* 1915 */       this.size = -1;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected void update() {
/* 1922 */       this.size = 0;
/* 1923 */       Node shadowTree = this.this$0.getXblShadowTree(this.record.node);
/* 1924 */       Node last = null;
/* 1925 */       Node m = (shadowTree == null) ? this.record.node.getFirstChild() : shadowTree.getFirstChild();
/*      */       
/* 1927 */       while (m != null) {
/* 1928 */         last = collectXblChildNodes(m, last);
/* 1929 */         m = m.getNextSibling();
/*      */       } 
/* 1931 */       if (last != null) {
/* 1932 */         DefaultXBLManager.XBLRecord rec = this.this$0.getRecord(last);
/* 1933 */         rec.nextSibling = null;
/* 1934 */         rec.linksValid = true;
/*      */       } 
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected Node collectXblChildNodes(Node n, Node prev) {
/* 1942 */       boolean isChild = false;
/* 1943 */       if (n.getNodeType() == 1) {
/* 1944 */         if (!"http://www.w3.org/2004/xbl".equals(n.getNamespaceURI())) {
/* 1945 */           isChild = true;
/* 1946 */         } else if (n instanceof XBLOMContentElement) {
/* 1947 */           ContentManager cm = this.this$0.getContentManager(n);
/* 1948 */           if (cm != null) {
/* 1949 */             NodeList selected = cm.getSelectedContent((XBLOMContentElement)n);
/*      */             
/* 1951 */             for (int i = 0; i < selected.getLength(); i++) {
/* 1952 */               prev = collectXblChildNodes(selected.item(i), prev);
/*      */             }
/*      */           } 
/*      */         } 
/*      */       } else {
/*      */         
/* 1958 */         isChild = true;
/*      */       } 
/* 1960 */       if (isChild) {
/* 1961 */         this.nodes.add(n);
/* 1962 */         this.size++;
/* 1963 */         if (prev != null) {
/* 1964 */           DefaultXBLManager.XBLRecord rec = this.this$0.getRecord(prev);
/* 1965 */           rec.nextSibling = n;
/* 1966 */           rec.linksValid = true;
/*      */         } 
/* 1968 */         DefaultXBLManager.XBLRecord rec = this.this$0.getRecord(n);
/* 1969 */         rec.previousSibling = prev;
/* 1970 */         rec.linksValid = true;
/* 1971 */         prev = n;
/*      */       } 
/* 1973 */       return prev;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public void invalidate() {
/* 1982 */       for (int i = 0; i < this.size; i++) {
/* 1983 */         DefaultXBLManager.XBLRecord rec = this.this$0.getRecord((Node)this.nodes.get(i));
/* 1984 */         rec.previousSibling = null;
/* 1985 */         rec.nextSibling = null;
/* 1986 */         rec.linksValid = false;
/*      */       } 
/* 1988 */       this.nodes.clear();
/* 1989 */       this.size = -1;
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public Node getFirstNode() {
/* 1996 */       if (this.size == -1) {
/* 1997 */         update();
/*      */       }
/* 1999 */       return (this.size == 0) ? null : (Node)this.nodes.get(0);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public Node getLastNode() {
/* 2006 */       if (this.size == -1) {
/* 2007 */         update();
/*      */       }
/* 2009 */       return (this.size == 0) ? null : (Node)this.nodes.get(this.nodes.size() - 1);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public Node item(int index) {
/* 2016 */       if (this.size == -1) {
/* 2017 */         update();
/*      */       }
/* 2019 */       if (index < 0 || index >= this.size) {
/* 2020 */         return null;
/*      */       }
/* 2022 */       return (Node)this.nodes.get(index);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public int getLength() {
/* 2029 */       if (this.size == -1) {
/* 2030 */         update();
/*      */       }
/* 2032 */       return this.size;
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   protected class XblScopedChildNodes
/*      */     extends XblChildNodes
/*      */   {
/*      */     private final DefaultXBLManager this$0;
/*      */ 
/*      */     
/*      */     public XblScopedChildNodes(DefaultXBLManager this$0, DefaultXBLManager.XBLRecord rec) {
/* 2045 */       super(this$0, rec);
/*      */       this.this$0 = this$0;
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*      */     protected void update() {
/* 2052 */       this.size = 0;
/* 2053 */       Node shadowTree = this.this$0.getXblShadowTree(this.record.node);
/* 2054 */       Node n = (shadowTree == null) ? this.record.node.getFirstChild() : shadowTree.getFirstChild();
/*      */       
/* 2056 */       while (n != null) {
/* 2057 */         collectXblScopedChildNodes(n);
/* 2058 */         n = n.getNextSibling();
/*      */       } 
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     protected void collectXblScopedChildNodes(Node n) {
/* 2066 */       boolean isChild = false;
/* 2067 */       if (n.getNodeType() == 1) {
/* 2068 */         if (!n.getNamespaceURI().equals("http://www.w3.org/2004/xbl")) {
/* 2069 */           isChild = true;
/* 2070 */         } else if (n instanceof XBLOMContentElement) {
/* 2071 */           ContentManager cm = this.this$0.getContentManager(n);
/* 2072 */           if (cm != null) {
/* 2073 */             NodeList selected = cm.getSelectedContent((XBLOMContentElement)n);
/*      */             
/* 2075 */             for (int i = 0; i < selected.getLength(); i++) {
/* 2076 */               collectXblScopedChildNodes(selected.item(i));
/*      */             }
/*      */           } 
/*      */         } 
/*      */       } else {
/* 2081 */         isChild = true;
/*      */       } 
/* 2083 */       if (isChild) {
/* 2084 */         this.nodes.add(n);
/* 2085 */         this.size++;
/*      */       } 
/*      */     }
/*      */   }
/*      */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\svg12\DefaultXBLManager.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */