/*     */ package org.apache.batik.bridge.svg12;
/*     */ 
/*     */ import java.awt.Point;
/*     */ import java.awt.geom.Point2D;
/*     */ import org.apache.batik.bridge.BridgeContext;
/*     */ import org.apache.batik.bridge.BridgeEventSupport;
/*     */ import org.apache.batik.bridge.FocusManager;
/*     */ import org.apache.batik.bridge.UserAgent;
/*     */ import org.apache.batik.dom.events.DOMKeyboardEvent;
/*     */ import org.apache.batik.dom.events.DOMMouseEvent;
/*     */ import org.apache.batik.dom.events.DOMTextEvent;
/*     */ import org.apache.batik.dom.events.NodeEventTarget;
/*     */ import org.apache.batik.dom.svg12.SVGOMWheelEvent;
/*     */ import org.apache.batik.dom.util.DOMUtilities;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.apache.batik.gvt.event.EventDispatcher;
/*     */ import org.apache.batik.gvt.event.GraphicsNodeKeyEvent;
/*     */ import org.apache.batik.gvt.event.GraphicsNodeMouseEvent;
/*     */ import org.apache.batik.gvt.event.GraphicsNodeMouseWheelEvent;
/*     */ import org.apache.batik.gvt.event.GraphicsNodeMouseWheelListener;
/*     */ import org.w3c.dom.Document;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.events.DocumentEvent;
/*     */ import org.w3c.dom.events.EventTarget;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class SVG12BridgeEventSupport
/*     */   extends BridgeEventSupport
/*     */ {
/*     */   public static void addGVTListener(BridgeContext ctx, Document doc) {
/*  68 */     UserAgent ua = ctx.getUserAgent();
/*  69 */     if (ua != null) {
/*  70 */       EventDispatcher dispatcher = ua.getEventDispatcher();
/*  71 */       if (dispatcher != null) {
/*  72 */         Listener listener = new Listener(ctx, ua);
/*  73 */         dispatcher.addGraphicsNodeMouseListener(listener);
/*  74 */         dispatcher.addGraphicsNodeMouseWheelListener(listener);
/*  75 */         dispatcher.addGraphicsNodeKeyListener(listener);
/*     */ 
/*     */         
/*  78 */         BridgeEventSupport.GVTUnloadListener gVTUnloadListener = new BridgeEventSupport.GVTUnloadListener(dispatcher, listener);
/*  79 */         NodeEventTarget target = (NodeEventTarget)doc;
/*  80 */         target.addEventListenerNS("http://www.w3.org/2001/xml-events", "SVGUnload", gVTUnloadListener, false, null);
/*     */ 
/*     */ 
/*     */         
/*  84 */         BridgeEventSupport.storeEventListenerNS(ctx, target, "http://www.w3.org/2001/xml-events", "SVGUnload", gVTUnloadListener, false);
/*     */       } 
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected static class Listener
/*     */     extends BridgeEventSupport.Listener
/*     */     implements GraphicsNodeMouseWheelListener
/*     */   {
/*     */     protected SVG12BridgeContext ctx12;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public Listener(BridgeContext ctx, UserAgent u) {
/* 106 */       super(ctx, u);
/* 107 */       this.ctx12 = (SVG12BridgeContext)ctx;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void keyPressed(GraphicsNodeKeyEvent evt) {
/* 118 */       if (!this.isDown) {
/* 119 */         this.isDown = true;
/* 120 */         dispatchKeyboardEvent("keydown", evt);
/*     */       } 
/* 122 */       if (evt.getKeyChar() == Character.MAX_VALUE)
/*     */       {
/*     */         
/* 125 */         dispatchTextEvent(evt);
/*     */       }
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void keyReleased(GraphicsNodeKeyEvent evt) {
/* 134 */       dispatchKeyboardEvent("keyup", evt);
/* 135 */       this.isDown = false;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 143 */     public void keyTyped(GraphicsNodeKeyEvent evt) { dispatchTextEvent(evt); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     protected void dispatchKeyboardEvent(String eventType, GraphicsNodeKeyEvent evt) {
/* 151 */       FocusManager fmgr = this.context.getFocusManager();
/* 152 */       if (fmgr == null) {
/*     */         return;
/*     */       }
/*     */       
/* 156 */       Element targetElement = (Element)fmgr.getCurrentEventTarget();
/* 157 */       if (targetElement == null) {
/* 158 */         targetElement = this.context.getDocument().getDocumentElement();
/*     */       }
/* 160 */       DocumentEvent d = (DocumentEvent)targetElement.getOwnerDocument();
/* 161 */       DOMKeyboardEvent keyEvt = (DOMKeyboardEvent)d.createEvent("KeyboardEvent");
/*     */       
/* 163 */       String modifiers = DOMUtilities.getModifiersList(evt.getLockState(), evt.getModifiers());
/*     */ 
/*     */       
/* 166 */       keyEvt.initKeyboardEventNS("http://www.w3.org/2001/xml-events", eventType, true, true, null, mapKeyCodeToIdentifier(evt.getKeyCode()), mapKeyLocation(evt.getKeyLocation()), modifiers);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/*     */       try {
/* 176 */         ((EventTarget)targetElement).dispatchEvent(keyEvt);
/*     */       } catch (RuntimeException e) {
/* 178 */         this.ua.displayError(e);
/*     */       } 
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     protected void dispatchTextEvent(GraphicsNodeKeyEvent evt) {
/* 186 */       FocusManager fmgr = this.context.getFocusManager();
/* 187 */       if (fmgr == null) {
/*     */         return;
/*     */       }
/*     */       
/* 191 */       Element targetElement = (Element)fmgr.getCurrentEventTarget();
/* 192 */       if (targetElement == null) {
/* 193 */         targetElement = this.context.getDocument().getDocumentElement();
/*     */       }
/* 195 */       DocumentEvent d = (DocumentEvent)targetElement.getOwnerDocument();
/* 196 */       DOMTextEvent textEvt = (DOMTextEvent)d.createEvent("TextEvent");
/* 197 */       textEvt.initTextEventNS("http://www.w3.org/2001/xml-events", "textInput", true, true, null, String.valueOf(evt.getKeyChar()));
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/*     */       try {
/* 205 */         ((EventTarget)targetElement).dispatchEvent(textEvt);
/*     */       } catch (RuntimeException e) {
/* 207 */         this.ua.displayError(e);
/*     */       } 
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 215 */     protected int mapKeyLocation(int location) { return location - 1; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 221 */     protected static String[][] IDENTIFIER_KEY_CODES = new String[256][];
/*     */     static  {
/* 223 */       putIdentifierKeyCode("U+0030", 48);
/*     */       
/* 225 */       putIdentifierKeyCode("U+0031", 49);
/*     */       
/* 227 */       putIdentifierKeyCode("U+0032", 50);
/*     */       
/* 229 */       putIdentifierKeyCode("U+0033", 51);
/*     */       
/* 231 */       putIdentifierKeyCode("U+0034", 52);
/*     */       
/* 233 */       putIdentifierKeyCode("U+0035", 53);
/*     */       
/* 235 */       putIdentifierKeyCode("U+0036", 54);
/*     */       
/* 237 */       putIdentifierKeyCode("U+0037", 55);
/*     */       
/* 239 */       putIdentifierKeyCode("U+0038", 56);
/*     */       
/* 241 */       putIdentifierKeyCode("U+0039", 57);
/*     */       
/* 243 */       putIdentifierKeyCode("Accept", 30);
/*     */       
/* 245 */       putIdentifierKeyCode("Again", 65481);
/*     */       
/* 247 */       putIdentifierKeyCode("U+0041", 65);
/*     */       
/* 249 */       putIdentifierKeyCode("AllCandidates", 256);
/*     */       
/* 251 */       putIdentifierKeyCode("Alphanumeric", 240);
/*     */       
/* 253 */       putIdentifierKeyCode("AltGraph", 65406);
/*     */       
/* 255 */       putIdentifierKeyCode("Alt", 18);
/*     */       
/* 257 */       putIdentifierKeyCode("U+0026", 150);
/*     */       
/* 259 */       putIdentifierKeyCode("U+0027", 222);
/*     */       
/* 261 */       putIdentifierKeyCode("U+002A", 151);
/*     */       
/* 263 */       putIdentifierKeyCode("U+0040", 512);
/*     */       
/* 265 */       putIdentifierKeyCode("U+005C", 92);
/*     */       
/* 267 */       putIdentifierKeyCode("U+0008", 8);
/*     */       
/* 269 */       putIdentifierKeyCode("U+0042", 66);
/*     */       
/* 271 */       putIdentifierKeyCode("U+0018", 3);
/*     */       
/* 273 */       putIdentifierKeyCode("CapsLock", 20);
/*     */       
/* 275 */       putIdentifierKeyCode("U+005E", 514);
/*     */       
/* 277 */       putIdentifierKeyCode("U+0043", 67);
/*     */       
/* 279 */       putIdentifierKeyCode("Clear", 12);
/*     */       
/* 281 */       putIdentifierKeyCode("CodeInput", 258);
/*     */       
/* 283 */       putIdentifierKeyCode("U+003A", 513);
/*     */       
/* 285 */       putIdentifierKeyCode("U+0301", 129);
/*     */       
/* 287 */       putIdentifierKeyCode("U+0306", 133);
/*     */       
/* 289 */       putIdentifierKeyCode("U+030C", 138);
/*     */       
/* 291 */       putIdentifierKeyCode("U+0327", 139);
/*     */       
/* 293 */       putIdentifierKeyCode("U+0302", 130);
/*     */       
/* 295 */       putIdentifierKeyCode("U+0308", 135);
/*     */       
/* 297 */       putIdentifierKeyCode("U+0307", 134);
/*     */       
/* 299 */       putIdentifierKeyCode("U+030B", 137);
/*     */       
/* 301 */       putIdentifierKeyCode("U+0300", 128);
/*     */       
/* 303 */       putIdentifierKeyCode("U+0345", 141);
/*     */       
/* 305 */       putIdentifierKeyCode("U+0304", 132);
/*     */       
/* 307 */       putIdentifierKeyCode("U+0328", 140);
/*     */       
/* 309 */       putIdentifierKeyCode("U+030A", 136);
/*     */       
/* 311 */       putIdentifierKeyCode("U+0303", 131);
/*     */       
/* 313 */       putIdentifierKeyCode("U+002C", 44);
/*     */       
/* 315 */       putIdentifierKeyCode("Compose", 65312);
/*     */       
/* 317 */       putIdentifierKeyCode("Control", 17);
/*     */       
/* 319 */       putIdentifierKeyCode("Convert", 28);
/*     */       
/* 321 */       putIdentifierKeyCode("Copy", 65485);
/*     */       
/* 323 */       putIdentifierKeyCode("Cut", 65489);
/*     */       
/* 325 */       putIdentifierKeyCode("U+007F", 127);
/*     */       
/* 327 */       putIdentifierKeyCode("U+0044", 68);
/*     */       
/* 329 */       putIdentifierKeyCode("U+0024", 515);
/*     */       
/* 331 */       putIdentifierKeyCode("Down", 40);
/*     */       
/* 333 */       putIdentifierKeyCode("U+0045", 69);
/*     */       
/* 335 */       putIdentifierKeyCode("End", 35);
/*     */       
/* 337 */       putIdentifierKeyCode("Enter", 10);
/*     */       
/* 339 */       putIdentifierKeyCode("U+003D", 61);
/*     */       
/* 341 */       putIdentifierKeyCode("U+001B", 27);
/*     */       
/* 343 */       putIdentifierKeyCode("U+20AC", 516);
/*     */       
/* 345 */       putIdentifierKeyCode("U+0021", 517);
/*     */       
/* 347 */       putIdentifierKeyCode("F10", 121);
/*     */       
/* 349 */       putIdentifierKeyCode("F11", 122);
/*     */       
/* 351 */       putIdentifierKeyCode("F12", 123);
/*     */       
/* 353 */       putIdentifierKeyCode("F13", 61440);
/*     */       
/* 355 */       putIdentifierKeyCode("F14", 61441);
/*     */       
/* 357 */       putIdentifierKeyCode("F15", 61442);
/*     */       
/* 359 */       putIdentifierKeyCode("F16", 61443);
/*     */       
/* 361 */       putIdentifierKeyCode("F17", 61444);
/*     */       
/* 363 */       putIdentifierKeyCode("F18", 61445);
/*     */       
/* 365 */       putIdentifierKeyCode("F19", 61446);
/*     */       
/* 367 */       putIdentifierKeyCode("F1", 112);
/*     */       
/* 369 */       putIdentifierKeyCode("F20", 61447);
/*     */       
/* 371 */       putIdentifierKeyCode("F21", 61448);
/*     */       
/* 373 */       putIdentifierKeyCode("F22", 61449);
/*     */       
/* 375 */       putIdentifierKeyCode("F23", 61450);
/*     */       
/* 377 */       putIdentifierKeyCode("F24", 61451);
/*     */       
/* 379 */       putIdentifierKeyCode("F2", 113);
/*     */       
/* 381 */       putIdentifierKeyCode("F3", 114);
/*     */       
/* 383 */       putIdentifierKeyCode("F4", 115);
/*     */       
/* 385 */       putIdentifierKeyCode("F5", 116);
/*     */       
/* 387 */       putIdentifierKeyCode("F6", 117);
/*     */       
/* 389 */       putIdentifierKeyCode("F7", 118);
/*     */       
/* 391 */       putIdentifierKeyCode("F8", 119);
/*     */       
/* 393 */       putIdentifierKeyCode("F9", 120);
/*     */       
/* 395 */       putIdentifierKeyCode("FinalMode", 24);
/*     */       
/* 397 */       putIdentifierKeyCode("Find", 65488);
/*     */       
/* 399 */       putIdentifierKeyCode("U+0046", 70);
/*     */       
/* 401 */       putIdentifierKeyCode("U+002E", 46);
/*     */       
/* 403 */       putIdentifierKeyCode("FullWidth", 243);
/*     */       
/* 405 */       putIdentifierKeyCode("U+0047", 71);
/*     */       
/* 407 */       putIdentifierKeyCode("U+0060", 192);
/*     */       
/* 409 */       putIdentifierKeyCode("U+003E", 160);
/*     */       
/* 411 */       putIdentifierKeyCode("HalfWidth", 244);
/*     */       
/* 413 */       putIdentifierKeyCode("U+0023", 520);
/*     */       
/* 415 */       putIdentifierKeyCode("Help", 156);
/*     */       
/* 417 */       putIdentifierKeyCode("Hiragana", 242);
/*     */       
/* 419 */       putIdentifierKeyCode("U+0048", 72);
/*     */       
/* 421 */       putIdentifierKeyCode("Home", 36);
/*     */       
/* 423 */       putIdentifierKeyCode("U+0049", 73);
/*     */       
/* 425 */       putIdentifierKeyCode("Insert", 155);
/*     */       
/* 427 */       putIdentifierKeyCode("U+00A1", 518);
/*     */       
/* 429 */       putIdentifierKeyCode("JapaneseHiragana", 260);
/*     */       
/* 431 */       putIdentifierKeyCode("JapaneseKatakana", 259);
/*     */       
/* 433 */       putIdentifierKeyCode("JapaneseRomaji", 261);
/*     */       
/* 435 */       putIdentifierKeyCode("U+004A", 74);
/*     */       
/* 437 */       putIdentifierKeyCode("KanaMode", 262);
/*     */       
/* 439 */       putIdentifierKeyCode("KanjiMode", 25);
/*     */       
/* 441 */       putIdentifierKeyCode("Katakana", 241);
/*     */       
/* 443 */       putIdentifierKeyCode("U+004B", 75);
/*     */       
/* 445 */       putIdentifierKeyCode("U+007B", 161);
/*     */       
/* 447 */       putIdentifierKeyCode("Left", 37);
/*     */       
/* 449 */       putIdentifierKeyCode("U+0028", 519);
/*     */       
/* 451 */       putIdentifierKeyCode("U+005B", 91);
/*     */       
/* 453 */       putIdentifierKeyCode("U+003C", 153);
/*     */       
/* 455 */       putIdentifierKeyCode("U+004C", 76);
/*     */       
/* 457 */       putIdentifierKeyCode("Meta", 157);
/*     */       
/* 459 */       putIdentifierKeyCode("Meta", 157);
/*     */       
/* 461 */       putIdentifierKeyCode("U+002D", 45);
/*     */       
/* 463 */       putIdentifierKeyCode("U+004D", 77);
/*     */       
/* 465 */       putIdentifierKeyCode("ModeChange", 31);
/*     */       
/* 467 */       putIdentifierKeyCode("U+004E", 78);
/*     */       
/* 469 */       putIdentifierKeyCode("Nonconvert", 29);
/*     */       
/* 471 */       putIdentifierKeyCode("NumLock", 144);
/*     */       
/* 473 */       putIdentifierKeyCode("NumLock", 144);
/*     */       
/* 475 */       putIdentifierKeyCode("U+004F", 79);
/*     */       
/* 477 */       putIdentifierKeyCode("PageDown", 34);
/*     */       
/* 479 */       putIdentifierKeyCode("PageUp", 33);
/*     */       
/* 481 */       putIdentifierKeyCode("Paste", 65487);
/*     */       
/* 483 */       putIdentifierKeyCode("Pause", 19);
/*     */       
/* 485 */       putIdentifierKeyCode("U+0050", 80);
/*     */       
/* 487 */       putIdentifierKeyCode("U+002B", 521);
/*     */       
/* 489 */       putIdentifierKeyCode("PreviousCandidate", 257);
/*     */       
/* 491 */       putIdentifierKeyCode("PrintScreen", 154);
/*     */       
/* 493 */       putIdentifierKeyCode("Props", 65482);
/*     */       
/* 495 */       putIdentifierKeyCode("U+0051", 81);
/*     */       
/* 497 */       putIdentifierKeyCode("U+0022", 152);
/*     */       
/* 499 */       putIdentifierKeyCode("U+007D", 162);
/*     */       
/* 501 */       putIdentifierKeyCode("Right", 39);
/*     */       
/* 503 */       putIdentifierKeyCode("U+0029", 522);
/*     */       
/* 505 */       putIdentifierKeyCode("U+005D", 93);
/*     */       
/* 507 */       putIdentifierKeyCode("U+0052", 82);
/*     */       
/* 509 */       putIdentifierKeyCode("RomanCharacters", 245);
/*     */       
/* 511 */       putIdentifierKeyCode("Scroll", 145);
/*     */       
/* 513 */       putIdentifierKeyCode("Scroll", 145);
/*     */       
/* 515 */       putIdentifierKeyCode("U+003B", 59);
/*     */       
/* 517 */       putIdentifierKeyCode("U+309A", 143);
/*     */       
/* 519 */       putIdentifierKeyCode("Shift", 16);
/*     */       
/* 521 */       putIdentifierKeyCode("Shift", 16);
/*     */       
/* 523 */       putIdentifierKeyCode("U+0053", 83);
/*     */       
/* 525 */       putIdentifierKeyCode("U+002F", 47);
/*     */       
/* 527 */       putIdentifierKeyCode("U+0020", 32);
/*     */       
/* 529 */       putIdentifierKeyCode("Stop", 65480);
/*     */       
/* 531 */       putIdentifierKeyCode("U+0009", 9);
/*     */       
/* 533 */       putIdentifierKeyCode("U+0054", 84);
/*     */       
/* 535 */       putIdentifierKeyCode("U+0055", 85);
/*     */       
/* 537 */       putIdentifierKeyCode("U+005F", 523);
/*     */       
/* 539 */       putIdentifierKeyCode("Undo", 65483);
/*     */       
/* 541 */       putIdentifierKeyCode("Unidentified", 0);
/*     */       
/* 543 */       putIdentifierKeyCode("Up", 38);
/*     */       
/* 545 */       putIdentifierKeyCode("U+0056", 86);
/*     */       
/* 547 */       putIdentifierKeyCode("U+3099", 142);
/*     */       
/* 549 */       putIdentifierKeyCode("U+0057", 87);
/*     */       
/* 551 */       putIdentifierKeyCode("U+0058", 88);
/*     */       
/* 553 */       putIdentifierKeyCode("U+0059", 89);
/*     */       
/* 555 */       putIdentifierKeyCode("U+005A", 90);
/*     */ 
/*     */       
/* 558 */       putIdentifierKeyCode("U+0030", 96);
/*     */       
/* 560 */       putIdentifierKeyCode("U+0031", 97);
/*     */       
/* 562 */       putIdentifierKeyCode("U+0032", 98);
/*     */       
/* 564 */       putIdentifierKeyCode("U+0033", 99);
/*     */       
/* 566 */       putIdentifierKeyCode("U+0034", 100);
/*     */       
/* 568 */       putIdentifierKeyCode("U+0035", 101);
/*     */       
/* 570 */       putIdentifierKeyCode("U+0036", 102);
/*     */       
/* 572 */       putIdentifierKeyCode("U+0037", 103);
/*     */       
/* 574 */       putIdentifierKeyCode("U+0038", 104);
/*     */       
/* 576 */       putIdentifierKeyCode("U+0039", 105);
/*     */       
/* 578 */       putIdentifierKeyCode("U+002A", 106);
/*     */       
/* 580 */       putIdentifierKeyCode("Down", 225);
/*     */       
/* 582 */       putIdentifierKeyCode("U+002E", 110);
/*     */       
/* 584 */       putIdentifierKeyCode("Left", 226);
/*     */       
/* 586 */       putIdentifierKeyCode("U+002D", 109);
/*     */       
/* 588 */       putIdentifierKeyCode("U+002B", 107);
/*     */       
/* 590 */       putIdentifierKeyCode("Right", 227);
/*     */       
/* 592 */       putIdentifierKeyCode("U+002F", 111);
/*     */       
/* 594 */       putIdentifierKeyCode("Up", 224);
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     protected static void putIdentifierKeyCode(String keyIdentifier, int keyCode) {
/* 604 */       if (IDENTIFIER_KEY_CODES[keyCode / 256] == null) {
/* 605 */         IDENTIFIER_KEY_CODES[keyCode / 256] = new String[256];
/*     */       }
/* 607 */       IDENTIFIER_KEY_CODES[keyCode / 256][keyCode % 256] = keyIdentifier;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     protected String mapKeyCodeToIdentifier(int keyCode) {
/* 614 */       String[] a = IDENTIFIER_KEY_CODES[keyCode / 256];
/* 615 */       if (a == null) {
/* 616 */         return "Unidentified";
/*     */       }
/* 618 */       return a[keyCode % 256];
/*     */     }
/*     */ 
/*     */ 
/*     */     
/*     */     public void mouseWheelMoved(GraphicsNodeMouseWheelEvent evt) {
/* 624 */       Document doc = this.context.getPrimaryBridgeContext().getDocument();
/* 625 */       Element targetElement = doc.getDocumentElement();
/* 626 */       DocumentEvent d = (DocumentEvent)doc;
/* 627 */       SVGOMWheelEvent wheelEvt = (SVGOMWheelEvent)d.createEvent("WheelEvent");
/*     */       
/* 629 */       wheelEvt.initWheelEventNS("http://www.w3.org/2001/xml-events", "wheel", true, true, null, evt.getWheelDelta());
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/*     */       try {
/* 637 */         ((EventTarget)targetElement).dispatchEvent(wheelEvt);
/*     */       } catch (RuntimeException e) {
/* 639 */         this.ua.displayError(e);
/*     */       } 
/*     */     }
/*     */ 
/*     */ 
/*     */     
/*     */     public void mouseEntered(GraphicsNodeMouseEvent evt) {
/* 646 */       Point clientXY = evt.getClientPoint();
/* 647 */       GraphicsNode node = evt.getGraphicsNode();
/* 648 */       Element targetElement = getEventTarget(node, new Point2D.Float(evt.getX(), evt.getY()));
/*     */       
/* 650 */       Element relatedElement = getRelatedElement(evt);
/* 651 */       int n = 0;
/* 652 */       if (relatedElement != null && targetElement != null) {
/* 653 */         n = DefaultXBLManager.computeBubbleLimit(targetElement, relatedElement);
/*     */       }
/*     */       
/* 656 */       dispatchMouseEvent("mouseover", targetElement, relatedElement, clientXY, evt, true, n);
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void mouseExited(GraphicsNodeMouseEvent evt) {
/* 666 */       Point clientXY = evt.getClientPoint();
/*     */       
/* 668 */       GraphicsNode node = evt.getRelatedNode();
/* 669 */       Element targetElement = getEventTarget(node, clientXY);
/* 670 */       if (this.lastTargetElement != null) {
/* 671 */         int n = 0;
/* 672 */         if (targetElement != null)
/*     */         {
/* 674 */           n = DefaultXBLManager.computeBubbleLimit(this.lastTargetElement, targetElement);
/*     */         }
/*     */         
/* 677 */         dispatchMouseEvent("mouseout", this.lastTargetElement, targetElement, clientXY, evt, true, n);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         
/* 684 */         this.lastTargetElement = null;
/*     */       } 
/*     */     }
/*     */     
/*     */     public void mouseMoved(GraphicsNodeMouseEvent evt) {
/* 689 */       Point clientXY = evt.getClientPoint();
/* 690 */       GraphicsNode node = evt.getGraphicsNode();
/* 691 */       Element targetElement = getEventTarget(node, clientXY);
/* 692 */       Element holdLTE = this.lastTargetElement;
/* 693 */       if (holdLTE != targetElement) {
/* 694 */         if (holdLTE != null) {
/* 695 */           int n = 0;
/* 696 */           if (targetElement != null) {
/* 697 */             n = DefaultXBLManager.computeBubbleLimit(holdLTE, targetElement);
/*     */           }
/*     */           
/* 700 */           dispatchMouseEvent("mouseout", holdLTE, targetElement, clientXY, evt, true, n);
/*     */         } 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         
/* 708 */         if (targetElement != null) {
/* 709 */           int n = 0;
/* 710 */           if (holdLTE != null) {
/* 711 */             n = DefaultXBLManager.computeBubbleLimit(targetElement, holdLTE);
/*     */           }
/*     */           
/* 714 */           dispatchMouseEvent("mouseover", targetElement, holdLTE, clientXY, evt, true, n);
/*     */         } 
/*     */       } 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 723 */       dispatchMouseEvent("mousemove", targetElement, null, clientXY, evt, false, 0);
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 750 */     protected void dispatchMouseEvent(String eventType, Element targetElement, Element relatedElement, Point clientXY, GraphicsNodeMouseEvent evt, boolean cancelable) { dispatchMouseEvent(eventType, targetElement, relatedElement, clientXY, evt, cancelable, 0); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     protected void dispatchMouseEvent(String eventType, Element targetElement, Element relatedElement, Point clientXY, GraphicsNodeMouseEvent evt, boolean cancelable, int bubbleLimit) {
/* 774 */       if (this.ctx12.mouseCaptureTarget != null) {
/* 775 */         NodeEventTarget net = null;
/* 776 */         if (targetElement != null) {
/* 777 */           net = (NodeEventTarget)targetElement;
/* 778 */           while (net != null && net != this.ctx12.mouseCaptureTarget) {
/* 779 */             net = net.getParentNodeEventTarget();
/*     */           }
/*     */         } 
/* 782 */         if (net == null) {
/* 783 */           if (this.ctx12.mouseCaptureSendAll) {
/* 784 */             targetElement = (Element)this.ctx12.mouseCaptureTarget;
/*     */           } else {
/* 786 */             targetElement = null;
/*     */           } 
/*     */         }
/*     */       } 
/*     */       
/* 791 */       if (targetElement != null) {
/* 792 */         Point screenXY = evt.getScreenPoint();
/*     */         
/* 794 */         DocumentEvent d = (DocumentEvent)targetElement.getOwnerDocument();
/*     */         
/* 796 */         DOMMouseEvent mouseEvt = (DOMMouseEvent)d.createEvent("MouseEvents");
/*     */         
/* 798 */         String modifiers = DOMUtilities.getModifiersList(evt.getLockState(), evt.getModifiers());
/*     */ 
/*     */         
/* 801 */         mouseEvt.initMouseEventNS("http://www.w3.org/2001/xml-events", eventType, true, cancelable, null, evt.getClickCount(), screenXY.x, screenXY.y, clientXY.x, clientXY.y, (short)(evt.getButton() - 1), (EventTarget)relatedElement, modifiers);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         
/* 815 */         mouseEvt.setBubbleLimit(bubbleLimit);
/*     */         
/*     */         try {
/* 818 */           ((EventTarget)targetElement).dispatchEvent(mouseEvt);
/*     */         } catch (RuntimeException e) {
/* 820 */           this.ua.displayError(e);
/*     */         } finally {
/* 822 */           this.lastTargetElement = targetElement;
/*     */         } 
/*     */       } 
/*     */       
/* 826 */       if (this.ctx12.mouseCaptureTarget != null && this.ctx12.mouseCaptureAutoRelease && "mouseup".equals(eventType))
/*     */       {
/*     */         
/* 829 */         this.ctx12.stopMouseCapture();
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\svg12\SVG12BridgeEventSupport.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */