/*     */ package org.apache.batik.bridge.svg12;
/*     */ 
/*     */ import java.awt.Color;
/*     */ import java.awt.RenderingHints;
/*     */ import java.awt.Shape;
/*     */ import java.awt.font.TextAttribute;
/*     */ import java.awt.geom.AffineTransform;
/*     */ import java.awt.geom.Point2D;
/*     */ import java.text.AttributedCharacterIterator;
/*     */ import java.text.AttributedString;
/*     */ import java.util.ArrayList;
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.LinkedList;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import org.apache.batik.bridge.Bridge;
/*     */ import org.apache.batik.bridge.BridgeContext;
/*     */ import org.apache.batik.bridge.CSSUtilities;
/*     */ import org.apache.batik.bridge.CursorManager;
/*     */ import org.apache.batik.bridge.GVTBuilder;
/*     */ import org.apache.batik.bridge.SVGAElementBridge;
/*     */ import org.apache.batik.bridge.SVGTextElementBridge;
/*     */ import org.apache.batik.bridge.SVGUtilities;
/*     */ import org.apache.batik.bridge.TextUtilities;
/*     */ import org.apache.batik.bridge.UserAgent;
/*     */ import org.apache.batik.css.engine.CSSEngine;
/*     */ import org.apache.batik.css.engine.value.ComputedValue;
/*     */ import org.apache.batik.css.engine.value.Value;
/*     */ import org.apache.batik.css.engine.value.ValueConstants;
/*     */ import org.apache.batik.css.engine.value.svg12.LineHeightValue;
/*     */ import org.apache.batik.css.engine.value.svg12.SVG12ValueConstants;
/*     */ import org.apache.batik.dom.AbstractNode;
/*     */ import org.apache.batik.dom.events.NodeEventTarget;
/*     */ import org.apache.batik.dom.svg.SVGOMElement;
/*     */ import org.apache.batik.dom.svg12.XBLEventSupport;
/*     */ import org.apache.batik.dom.util.XLinkSupport;
/*     */ import org.apache.batik.dom.util.XMLSupport;
/*     */ import org.apache.batik.gvt.CompositeGraphicsNode;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.apache.batik.gvt.TextNode;
/*     */ import org.apache.batik.gvt.flow.BlockInfo;
/*     */ import org.apache.batik.gvt.flow.FlowTextNode;
/*     */ import org.apache.batik.gvt.flow.RegionInfo;
/*     */ import org.apache.batik.gvt.flow.TextLineBreaks;
/*     */ import org.apache.batik.gvt.text.GVTAttributedCharacterIterator;
/*     */ import org.apache.batik.gvt.text.TextPaintInfo;
/*     */ import org.apache.batik.gvt.text.TextPath;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.Node;
/*     */ import org.w3c.dom.events.Event;
/*     */ import org.w3c.dom.events.EventListener;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SVGFlowRootElementBridge
/*     */   extends SVG12TextElementBridge
/*     */ {
/*  90 */   public static final AttributedCharacterIterator.Attribute FLOW_PARAGRAPH = GVTAttributedCharacterIterator.TextAttribute.FLOW_PARAGRAPH;
/*     */ 
/*     */ 
/*     */   
/*  94 */   public static final AttributedCharacterIterator.Attribute FLOW_EMPTY_PARAGRAPH = GVTAttributedCharacterIterator.TextAttribute.FLOW_EMPTY_PARAGRAPH;
/*     */ 
/*     */ 
/*     */   
/*  98 */   public static final AttributedCharacterIterator.Attribute FLOW_LINE_BREAK = GVTAttributedCharacterIterator.TextAttribute.FLOW_LINE_BREAK;
/*     */ 
/*     */ 
/*     */   
/* 102 */   public static final AttributedCharacterIterator.Attribute FLOW_REGIONS = GVTAttributedCharacterIterator.TextAttribute.FLOW_REGIONS;
/*     */ 
/*     */ 
/*     */   
/* 106 */   public static final AttributedCharacterIterator.Attribute LINE_HEIGHT = GVTAttributedCharacterIterator.TextAttribute.LINE_HEIGHT;
/*     */ 
/*     */ 
/*     */   
/* 110 */   public static final GVTAttributedCharacterIterator.TextAttribute TEXTPATH = GVTAttributedCharacterIterator.TextAttribute.TEXTPATH;
/*     */ 
/*     */ 
/*     */   
/* 114 */   public static final GVTAttributedCharacterIterator.TextAttribute ANCHOR_TYPE = GVTAttributedCharacterIterator.TextAttribute.ANCHOR_TYPE;
/*     */ 
/*     */ 
/*     */   
/* 118 */   public static final GVTAttributedCharacterIterator.TextAttribute LETTER_SPACING = GVTAttributedCharacterIterator.TextAttribute.LETTER_SPACING;
/*     */ 
/*     */ 
/*     */   
/* 122 */   public static final GVTAttributedCharacterIterator.TextAttribute WORD_SPACING = GVTAttributedCharacterIterator.TextAttribute.WORD_SPACING;
/*     */ 
/*     */ 
/*     */   
/* 126 */   public static final GVTAttributedCharacterIterator.TextAttribute KERNING = GVTAttributedCharacterIterator.TextAttribute.KERNING;
/*     */   
/*     */   protected Map flowRegionNodes;
/*     */   
/*     */   protected TextNode textNode;
/*     */   
/*     */   protected RegionChangeListener regionChangeListener;
/*     */   
/*     */   protected int startLen;
/*     */   
/* 136 */   protected TextNode getTextNode() { return this.textNode; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 152 */   public String getNamespaceURI() { return "http://www.w3.org/2000/svg"; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 159 */   public String getLocalName() { return "flowRoot"; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 166 */   public Bridge getInstance() { return new SVGFlowRootElementBridge(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 173 */   public boolean isComposite() { return false; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public GraphicsNode createGraphicsNode(BridgeContext ctx, Element e) {
/* 185 */     if (!SVGUtilities.matchUserAgent(e, ctx.getUserAgent())) {
/* 186 */       return null;
/*     */     }
/*     */     
/* 189 */     CompositeGraphicsNode cgn = new CompositeGraphicsNode();
/*     */ 
/*     */     
/* 192 */     String s = e.getAttributeNS(null, "transform");
/* 193 */     if (s.length() != 0) {
/* 194 */       cgn.setTransform(SVGUtilities.convertTransform(e, "transform", s, ctx));
/*     */     }
/*     */ 
/*     */ 
/*     */     
/* 199 */     cgn.setVisible(CSSUtilities.convertVisibility(e));
/*     */ 
/*     */     
/* 202 */     RenderingHints hints = null;
/* 203 */     hints = CSSUtilities.convertColorRendering(e, hints);
/* 204 */     hints = CSSUtilities.convertTextRendering(e, hints);
/* 205 */     if (hints != null) {
/* 206 */       cgn.setRenderingHints(hints);
/*     */     }
/*     */ 
/*     */     
/* 210 */     CompositeGraphicsNode cgn2 = new CompositeGraphicsNode();
/* 211 */     cgn.add(cgn2);
/*     */ 
/*     */     
/* 214 */     FlowTextNode tn = (FlowTextNode)instantiateGraphicsNode();
/* 215 */     tn.setLocation(getLocation(ctx, e));
/*     */ 
/*     */     
/* 218 */     if (ctx.getTextPainter() != null) {
/* 219 */       tn.setTextPainter(ctx.getTextPainter());
/*     */     }
/* 221 */     this.textNode = tn;
/* 222 */     cgn.add(tn);
/*     */     
/* 224 */     associateSVGContext(ctx, e, cgn);
/*     */ 
/*     */     
/* 227 */     Node child = getFirstChild(e);
/* 228 */     while (child != null) {
/* 229 */       if (child.getNodeType() == 1) {
/* 230 */         addContextToChild(ctx, (Element)child);
/*     */       }
/* 232 */       child = getNextSibling(child);
/*     */     } 
/*     */     
/* 235 */     return cgn;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 242 */   protected GraphicsNode instantiateGraphicsNode() { return new FlowTextNode(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 254 */   protected Point2D getLocation(BridgeContext ctx, Element e) { return new Point2D.Float(0.0F, 0.0F); }
/*     */ 
/*     */   
/*     */   protected boolean isTextElement(Element e) {
/* 258 */     if (!"http://www.w3.org/2000/svg".equals(e.getNamespaceURI()))
/* 259 */       return false; 
/* 260 */     String nodeName = e.getLocalName();
/* 261 */     return (nodeName.equals("flowDiv") || nodeName.equals("flowLine") || nodeName.equals("flowPara") || nodeName.equals("flowRegionBreak") || nodeName.equals("flowSpan"));
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected boolean isTextChild(Element e) {
/* 269 */     if (!"http://www.w3.org/2000/svg".equals(e.getNamespaceURI()))
/* 270 */       return false; 
/* 271 */     String nodeName = e.getLocalName();
/* 272 */     return (nodeName.equals("a") || nodeName.equals("flowLine") || nodeName.equals("flowPara") || nodeName.equals("flowRegionBreak") || nodeName.equals("flowSpan"));
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void buildGraphicsNode(BridgeContext ctx, Element e, GraphicsNode node) {
/* 290 */     CompositeGraphicsNode cgn = (CompositeGraphicsNode)node;
/*     */ 
/*     */     
/* 293 */     boolean isStatic = !ctx.isDynamic();
/* 294 */     if (isStatic) {
/* 295 */       this.flowRegionNodes = new HashMap();
/*     */     } else {
/* 297 */       this.regionChangeListener = new RegionChangeListener(this);
/*     */     } 
/* 299 */     CompositeGraphicsNode cgn2 = (CompositeGraphicsNode)cgn.get(0);
/* 300 */     GVTBuilder builder = ctx.getGVTBuilder();
/* 301 */     for (Node n = getFirstChild(e); n != null; n = getNextSibling(n)) {
/* 302 */       if (n instanceof org.apache.batik.dom.svg12.SVGOMFlowRegionElement) {
/* 303 */         Node m = getFirstChild(n);
/* 304 */         for (; m != null; 
/* 305 */           m = getNextSibling(m)) {
/* 306 */           if (m.getNodeType() == 1) {
/*     */ 
/*     */             
/* 309 */             GraphicsNode gn = builder.build(ctx, (Element)m);
/* 310 */             if (gn != null) {
/* 311 */               cgn2.add(gn);
/* 312 */               if (isStatic) {
/* 313 */                 this.flowRegionNodes.put(m, gn);
/*     */               }
/*     */             } 
/*     */           } 
/*     */         } 
/* 318 */         if (!isStatic) {
/* 319 */           AbstractNode an = (AbstractNode)n;
/* 320 */           XBLEventSupport es = (XBLEventSupport)an.initializeEventSupport();
/*     */           
/* 322 */           es.addImplementationEventListenerNS("http://www.w3.org/2000/svg", "shapechange", this.regionChangeListener, false);
/*     */         } 
/*     */       } 
/*     */     } 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 330 */     GraphicsNode tn = (GraphicsNode)cgn.get(1);
/* 331 */     super.buildGraphicsNode(ctx, e, tn);
/*     */ 
/*     */     
/* 334 */     this.flowRegionNodes = null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 340 */   protected void computeLaidoutText(BridgeContext ctx, Element e, GraphicsNode node) { super.computeLaidoutText(ctx, getFlowDivElement(e), node); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void addContextToChild(BridgeContext ctx, Element e) {
/* 356 */     if ("http://www.w3.org/2000/svg".equals(e.getNamespaceURI())) {
/* 357 */       String ln = e.getLocalName();
/* 358 */       if (ln.equals("flowDiv") || ln.equals("flowLine") || ln.equals("flowPara") || ln.equals("flowSpan"))
/*     */       {
/*     */ 
/*     */         
/* 362 */         ((SVGOMElement)e).setSVGContext(new FlowContentBridge(this, ctx, this, e));
/*     */       }
/*     */     } 
/*     */ 
/*     */ 
/*     */     
/* 368 */     Node child = getFirstChild(e);
/* 369 */     while (child != null) {
/* 370 */       if (child.getNodeType() == 1) {
/* 371 */         addContextToChild(ctx, (Element)child);
/*     */       }
/* 373 */       child = getNextSibling(child);
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void removeContextFromChild(BridgeContext ctx, Element e) {
/* 388 */     if ("http://www.w3.org/2000/svg".equals(e.getNamespaceURI())) {
/* 389 */       String ln = e.getLocalName();
/* 390 */       if (ln.equals("flowDiv") || ln.equals("flowLine") || ln.equals("flowPara") || ln.equals("flowSpan"))
/*     */       {
/*     */ 
/*     */         
/* 394 */         ((SVGTextElementBridge.AbstractTextChildBridgeUpdateHandler)((SVGOMElement)e).getSVGContext()).dispose();
/*     */       }
/*     */     } 
/*     */ 
/*     */     
/* 399 */     Node child = getFirstChild(e);
/* 400 */     while (child != null) {
/* 401 */       if (child.getNodeType() == 1) {
/* 402 */         removeContextFromChild(ctx, (Element)child);
/*     */       }
/* 404 */       child = getNextSibling(child);
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected AttributedString buildAttributedString(BridgeContext ctx, Element element) {
/* 417 */     if (element == null) return null;
/*     */     
/* 419 */     List rgns = getRegions(ctx, element);
/* 420 */     AttributedString ret = getFlowDiv(ctx, element);
/* 421 */     if (ret == null) return ret; 
/* 422 */     ret.addAttribute(FLOW_REGIONS, rgns, 0, 1);
/* 423 */     TextLineBreaks.findLineBrk(ret);
/*     */     
/* 425 */     return ret;
/*     */   }
/*     */   
/*     */   protected void dumpACIWord(AttributedString as) {
/* 429 */     if (as == null)
/*     */       return; 
/* 431 */     StringBuffer chars = new StringBuffer();
/* 432 */     StringBuffer brkStr = new StringBuffer();
/* 433 */     AttributedCharacterIterator aci = as.getIterator();
/* 434 */     AttributedCharacterIterator.Attribute WORD_LIMIT = TextLineBreaks.WORD_LIMIT;
/*     */ 
/*     */     
/* 437 */     char ch = aci.current();
/* 438 */     for (; ch != Character.MAX_VALUE; 
/* 439 */       ch = aci.next()) {
/*     */       
/* 441 */       chars.append(ch).append(' ').append(' ');
/* 442 */       int w = ((Integer)aci.getAttribute(WORD_LIMIT)).intValue();
/* 443 */       brkStr.append(w).append(' ');
/* 444 */       if (w < 10)
/*     */       {
/* 446 */         brkStr.append(' ');
/*     */       }
/*     */     } 
/* 449 */     System.out.println(chars.toString());
/* 450 */     System.out.println(brkStr.toString());
/*     */   }
/*     */   
/*     */   protected Element getFlowDivElement(Element elem) {
/* 454 */     String eNS = elem.getNamespaceURI();
/* 455 */     if (!eNS.equals("http://www.w3.org/2000/svg")) return null;
/*     */     
/* 457 */     String nodeName = elem.getLocalName();
/* 458 */     if (nodeName.equals("flowDiv")) return elem;
/*     */     
/* 460 */     if (!nodeName.equals("flowRoot")) return null;
/*     */     
/* 462 */     Node n = getFirstChild(elem);
/* 463 */     for (; n != null; n = getNextSibling(n)) {
/* 464 */       if (n.getNodeType() == 1) {
/*     */         
/* 466 */         String nNS = n.getNamespaceURI();
/* 467 */         if ("http://www.w3.org/2000/svg".equals(nNS))
/*     */         
/* 469 */         { Element e = (Element)n;
/* 470 */           String ln = e.getLocalName();
/* 471 */           if (ln.equals("flowDiv"))
/* 472 */             return e;  } 
/*     */       } 
/* 474 */     }  return null;
/*     */   }
/*     */   
/*     */   protected AttributedString getFlowDiv(BridgeContext ctx, Element element) {
/* 478 */     Element flowDiv = getFlowDivElement(element);
/* 479 */     if (flowDiv == null) return null;
/*     */     
/* 481 */     return gatherFlowPara(ctx, flowDiv);
/*     */   }
/*     */ 
/*     */   
/*     */   protected AttributedString gatherFlowPara(BridgeContext ctx, Element div) {
/* 486 */     TextPaintInfo divTPI = new TextPaintInfo();
/*     */     
/* 488 */     divTPI.visible = true;
/* 489 */     divTPI.fillPaint = Color.black;
/* 490 */     this.elemTPI.put(div, divTPI);
/*     */     
/* 492 */     SVGTextElementBridge.AttributedStringBuffer asb = new SVGTextElementBridge.AttributedStringBuffer();
/* 493 */     List paraEnds = new ArrayList();
/* 494 */     List paraElems = new ArrayList();
/* 495 */     List lnLocs = new ArrayList();
/* 496 */     Node n = getFirstChild(div);
/* 497 */     for (; n != null; 
/* 498 */       n = getNextSibling(n)) {
/*     */       
/* 500 */       if (n.getNodeType() == 1 && getNamespaceURI().equals(n.getNamespaceURI())) {
/*     */ 
/*     */ 
/*     */         
/* 504 */         Element e = (Element)n;
/*     */         
/* 506 */         String ln = e.getLocalName();
/* 507 */         if (ln.equals("flowPara")) {
/* 508 */           fillAttributedStringBuffer(ctx, e, true, null, null, asb, lnLocs);
/*     */ 
/*     */           
/* 511 */           paraElems.add(e);
/* 512 */           paraEnds.add(new Integer(asb.length()));
/* 513 */         } else if (ln.equals("flowRegionBreak")) {
/* 514 */           fillAttributedStringBuffer(ctx, e, true, null, null, asb, lnLocs);
/*     */ 
/*     */           
/* 517 */           paraElems.add(e);
/* 518 */           paraEnds.add(new Integer(asb.length()));
/*     */         } 
/*     */       } 
/* 521 */     }  divTPI.startChar = 0;
/* 522 */     divTPI.endChar = asb.length() - 1;
/*     */ 
/*     */ 
/*     */     
/* 526 */     AttributedString ret = asb.toAttributedString();
/* 527 */     if (ret == null) {
/* 528 */       return null;
/*     */     }
/*     */ 
/*     */     
/* 532 */     int prevLN = 0;
/* 533 */     Iterator lnIter = lnLocs.iterator();
/* 534 */     while (lnIter.hasNext()) {
/* 535 */       int nextLN = ((Integer)lnIter.next()).intValue();
/* 536 */       if (nextLN == prevLN) {
/*     */         continue;
/*     */       }
/* 539 */       ret.addAttribute(FLOW_LINE_BREAK, new Object(), prevLN, nextLN);
/*     */ 
/*     */       
/* 542 */       prevLN = nextLN;
/*     */     } 
/*     */     
/* 545 */     int start = 0;
/*     */     
/* 547 */     List emptyPara = null;
/* 548 */     for (int i = 0; i < paraElems.size(); i++, start = end) {
/* 549 */       Element elem = (Element)paraElems.get(i);
/* 550 */       int end = ((Integer)paraEnds.get(i)).intValue();
/* 551 */       if (start == end) {
/* 552 */         if (emptyPara == null)
/* 553 */           emptyPara = new LinkedList(); 
/* 554 */         emptyPara.add(makeBlockInfo(ctx, elem));
/*     */       }
/*     */       else {
/*     */         
/* 558 */         ret.addAttribute(FLOW_PARAGRAPH, makeBlockInfo(ctx, elem), start, end);
/*     */         
/* 560 */         if (emptyPara != null) {
/* 561 */           ret.addAttribute(FLOW_EMPTY_PARAGRAPH, emptyPara, start, end);
/* 562 */           emptyPara = null;
/*     */         } 
/*     */       } 
/*     */     } 
/* 566 */     return ret;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected List getRegions(BridgeContext ctx, Element element) {
/* 574 */     element = (Element)element.getParentNode();
/* 575 */     List ret = new LinkedList();
/* 576 */     Node n = getFirstChild(element);
/* 577 */     for (; n != null; n = getNextSibling(n)) {
/*     */       
/* 579 */       if (n.getNodeType() == 1 && 
/* 580 */         "http://www.w3.org/2000/svg".equals(n.getNamespaceURI())) {
/*     */ 
/*     */         
/* 583 */         Element e = (Element)n;
/* 584 */         String ln = e.getLocalName();
/* 585 */         if ("flowRegion".equals(ln)) {
/*     */ 
/*     */           
/* 588 */           float verticalAlignment = 0.0F;
/*     */           
/* 590 */           gatherRegionInfo(ctx, e, verticalAlignment, ret);
/*     */         } 
/*     */       } 
/* 593 */     }  return ret;
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   protected void gatherRegionInfo(BridgeContext ctx, Element rgn, float verticalAlign, List regions) {
/* 599 */     boolean isStatic = !ctx.isDynamic();
/* 600 */     for (Node n = getFirstChild(rgn); n != null; n = getNextSibling(n)) {
/*     */       
/* 602 */       if (n.getNodeType() == 1) {
/*     */ 
/*     */ 
/*     */         
/* 606 */         GraphicsNode gn = isStatic ? (GraphicsNode)this.flowRegionNodes.get(n) : ctx.getGraphicsNode(n);
/*     */         
/* 608 */         Shape s = gn.getOutline();
/* 609 */         if (s != null) {
/*     */ 
/*     */ 
/*     */           
/* 613 */           AffineTransform at = gn.getTransform();
/* 614 */           if (at != null) {
/* 615 */             s = at.createTransformedShape(s);
/*     */           }
/* 617 */           regions.add(new RegionInfo(s, verticalAlign));
/*     */         } 
/*     */       } 
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void fillAttributedStringBuffer(BridgeContext ctx, Element element, boolean top, Integer bidiLevel, Map initialAttributes, SVGTextElementBridge.AttributedStringBuffer asb, List lnLocs) {
/* 635 */     if (!SVGUtilities.matchUserAgent(element, ctx.getUserAgent()) || !CSSUtilities.convertDisplay(element)) {
/*     */       return;
/*     */     }
/*     */ 
/*     */     
/* 640 */     String s = XMLSupport.getXMLSpace(element);
/* 641 */     boolean preserve = s.equals("preserve");
/*     */     
/* 643 */     Element nodeElement = element;
/* 644 */     int elementStartChar = asb.length();
/*     */     
/* 646 */     if (top) {
/* 647 */       this.endLimit = this.startLen = asb.length();
/*     */     }
/* 649 */     if (preserve) {
/* 650 */       this.endLimit = this.startLen;
/*     */     }
/*     */     
/* 653 */     Map map = (initialAttributes == null) ? new HashMap() : new HashMap(initialAttributes);
/*     */ 
/*     */     
/* 656 */     initialAttributes = getAttributeMap(ctx, element, null, bidiLevel, map);
/*     */     
/* 658 */     Object o = map.get(TextAttribute.BIDI_EMBEDDING);
/* 659 */     Integer subBidiLevel = bidiLevel;
/* 660 */     if (o != null) {
/* 661 */       subBidiLevel = (Integer)o;
/*     */     }
/*     */     
/* 664 */     int lineBreak = -1;
/* 665 */     if (lnLocs.size() != 0) {
/* 666 */       lineBreak = ((Integer)lnLocs.get(lnLocs.size() - 1)).intValue();
/*     */     }
/*     */     
/* 669 */     Node n = getFirstChild(element);
/* 670 */     for (; n != null; 
/* 671 */       n = getNextSibling(n)) {
/*     */       String ln; boolean bool;
/* 673 */       if (preserve) {
/* 674 */         bool = false;
/*     */       } else {
/* 676 */         int len = asb.length();
/* 677 */         if (len == this.startLen) {
/* 678 */           bool = true;
/*     */         } else {
/* 680 */           bool = (asb.getLastChar() == 32);
/* 681 */           int idx = lnLocs.size() - 1;
/* 682 */           if (!bool && idx >= 0) {
/* 683 */             Integer i = (Integer)lnLocs.get(idx);
/* 684 */             if (i.intValue() == len) {
/* 685 */               bool = true;
/*     */             }
/*     */           } 
/*     */         } 
/*     */       } 
/*     */       
/* 691 */       switch (n.getNodeType()) {
/*     */         
/*     */         case 1:
/* 694 */           if (!"http://www.w3.org/2000/svg".equals(n.getNamespaceURI())) {
/*     */             break;
/*     */           }
/* 697 */           nodeElement = (Element)n;
/*     */           
/* 699 */           ln = n.getLocalName();
/*     */           
/* 701 */           if (ln.equals("flowLine")) {
/* 702 */             int before = asb.length();
/* 703 */             fillAttributedStringBuffer(ctx, nodeElement, false, subBidiLevel, initialAttributes, asb, lnLocs);
/*     */ 
/*     */ 
/*     */ 
/*     */             
/* 708 */             lineBreak = asb.length();
/* 709 */             lnLocs.add(new Integer(lineBreak));
/* 710 */             if (before != lineBreak)
/* 711 */               initialAttributes = null;  break;
/*     */           } 
/* 713 */           if (ln.equals("flowSpan") || ln.equals("altGlyph")) {
/*     */             
/* 715 */             int before = asb.length();
/* 716 */             fillAttributedStringBuffer(ctx, nodeElement, false, subBidiLevel, initialAttributes, asb, lnLocs);
/*     */ 
/*     */             
/* 719 */             if (asb.length() != before)
/* 720 */               initialAttributes = null;  break;
/*     */           } 
/* 722 */           if (ln.equals("a")) {
/* 723 */             if (ctx.isInteractive()) {
/* 724 */               NodeEventTarget target = (NodeEventTarget)nodeElement;
/* 725 */               UserAgent ua = ctx.getUserAgent();
/*     */               
/* 727 */               SVGAElementBridge.CursorHolder ch = new SVGAElementBridge.CursorHolder(CursorManager.DEFAULT_CURSOR);
/*     */               
/* 729 */               target.addEventListenerNS("http://www.w3.org/2001/xml-events", "click", new SVGAElementBridge.AnchorListener(ua, ch), false, null);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */               
/* 735 */               target.addEventListenerNS("http://www.w3.org/2001/xml-events", "mouseover", new SVGAElementBridge.CursorMouseOverListener(ua, ch), false, null);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */               
/* 741 */               target.addEventListenerNS("http://www.w3.org/2001/xml-events", "mouseout", new SVGAElementBridge.CursorMouseOutListener(ua, ch), false, null);
/*     */             } 
/*     */ 
/*     */ 
/*     */ 
/*     */             
/* 747 */             int before = asb.length();
/* 748 */             fillAttributedStringBuffer(ctx, nodeElement, false, subBidiLevel, initialAttributes, asb, lnLocs);
/*     */ 
/*     */             
/* 751 */             if (asb.length() != before)
/* 752 */               initialAttributes = null;  break;
/*     */           } 
/* 754 */           if (ln.equals("tref")) {
/* 755 */             String uriStr = XLinkSupport.getXLinkHref((Element)n);
/* 756 */             Element ref = ctx.getReferencedElement((Element)n, uriStr);
/* 757 */             s = TextUtilities.getElementContent(ref);
/* 758 */             s = normalizeString(s, preserve, bool);
/* 759 */             if (s.length() != 0) {
/* 760 */               int trefStart = asb.length();
/* 761 */               Map m = new HashMap();
/* 762 */               getAttributeMap(ctx, nodeElement, null, bidiLevel, m);
/* 763 */               asb.append(s, m);
/* 764 */               int trefEnd = asb.length() - 1;
/*     */               
/* 766 */               TextPaintInfo tpi = (TextPaintInfo)this.elemTPI.get(nodeElement);
/* 767 */               tpi.startChar = trefStart;
/* 768 */               tpi.endChar = trefEnd;
/*     */             } 
/*     */           } 
/*     */           break;
/*     */         
/*     */         case 3:
/*     */         case 4:
/* 775 */           s = n.getNodeValue();
/* 776 */           s = normalizeString(s, preserve, bool);
/* 777 */           if (s.length() != 0) {
/* 778 */             asb.append(s, map);
/* 779 */             if (preserve) {
/* 780 */               this.endLimit = asb.length();
/*     */             }
/* 782 */             initialAttributes = null;
/*     */           } 
/*     */           break;
/*     */       } 
/*     */     } 
/* 787 */     if (top) {
/* 788 */       boolean strippedSome = false;
/* 789 */       while (this.endLimit < asb.length() && asb.getLastChar() == 32) {
/* 790 */         int idx = lnLocs.size() - 1;
/* 791 */         int len = asb.length();
/* 792 */         if (idx >= 0) {
/* 793 */           Integer i = (Integer)lnLocs.get(idx);
/* 794 */           if (i.intValue() >= len) {
/* 795 */             i = new Integer(len - 1);
/* 796 */             lnLocs.set(idx, i);
/* 797 */             idx--;
/* 798 */             while (idx >= 0) {
/* 799 */               i = (Integer)lnLocs.get(idx);
/* 800 */               if (i.intValue() < len - 1)
/*     */                 break; 
/* 802 */               lnLocs.remove(idx);
/* 803 */               idx--;
/*     */             } 
/*     */           } 
/*     */         } 
/* 807 */         asb.stripLast();
/* 808 */         strippedSome = true;
/*     */       } 
/* 810 */       if (strippedSome) {
/* 811 */         Iterator iter = this.elemTPI.values().iterator();
/* 812 */         while (iter.hasNext()) {
/* 813 */           TextPaintInfo tpi = (TextPaintInfo)iter.next();
/* 814 */           if (tpi.endChar >= asb.length()) {
/* 815 */             tpi.endChar = asb.length() - 1;
/* 816 */             if (tpi.startChar > tpi.endChar) {
/* 817 */               tpi.startChar = tpi.endChar;
/*     */             }
/*     */           } 
/*     */         } 
/*     */       } 
/*     */     } 
/* 823 */     int elementEndChar = asb.length() - 1;
/* 824 */     TextPaintInfo tpi = (TextPaintInfo)this.elemTPI.get(element);
/* 825 */     tpi.startChar = elementStartChar;
/* 826 */     tpi.endChar = elementEndChar;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected Map getAttributeMap(BridgeContext ctx, Element element, TextPath textPath, Integer bidiLevel, Map result) {
/* 834 */     Map inheritingMap = super.getAttributeMap(ctx, element, textPath, bidiLevel, result);
/*     */ 
/*     */     
/* 837 */     float fontSize = TextUtilities.convertFontSize(element).floatValue();
/* 838 */     float lineHeight = getLineHeight(ctx, element, fontSize);
/* 839 */     result.put(LINE_HEIGHT, new Float(lineHeight));
/*     */     
/* 841 */     return inheritingMap;
/*     */   }
/*     */   
/*     */   protected void checkMap(Map attrs) {
/* 845 */     if (attrs.containsKey(TEXTPATH)) {
/*     */       return;
/*     */     }
/*     */     
/* 849 */     if (attrs.containsKey(ANCHOR_TYPE)) {
/*     */       return;
/*     */     }
/*     */     
/* 853 */     if (attrs.containsKey(LETTER_SPACING)) {
/*     */       return;
/*     */     }
/*     */     
/* 857 */     if (attrs.containsKey(WORD_SPACING)) {
/*     */       return;
/*     */     }
/*     */     
/* 861 */     if (attrs.containsKey(KERNING)) {
/*     */       return;
/*     */     }
/*     */   }
/*     */   
/* 866 */   int marginTopIndex = -1;
/* 867 */   int marginRightIndex = -1;
/* 868 */   int marginBottomIndex = -1;
/* 869 */   int marginLeftIndex = -1;
/* 870 */   int indentIndex = -1;
/* 871 */   int textAlignIndex = -1;
/* 872 */   int lineHeightIndex = -1;
/*     */   
/*     */   protected void initCSSPropertyIndexes(Element e) {
/* 875 */     CSSEngine eng = CSSUtilities.getCSSEngine(e);
/* 876 */     this.marginTopIndex = eng.getPropertyIndex("margin-top");
/* 877 */     this.marginRightIndex = eng.getPropertyIndex("margin-right");
/* 878 */     this.marginBottomIndex = eng.getPropertyIndex("margin-bottom");
/* 879 */     this.marginLeftIndex = eng.getPropertyIndex("margin-left");
/* 880 */     this.indentIndex = eng.getPropertyIndex("indent");
/* 881 */     this.textAlignIndex = eng.getPropertyIndex("text-align");
/* 882 */     this.lineHeightIndex = eng.getPropertyIndex("line-height");
/*     */   }
/*     */   public BlockInfo makeBlockInfo(BridgeContext ctx, Element element) {
/*     */     byte b;
/* 886 */     if (this.marginTopIndex == -1) initCSSPropertyIndexes(element);
/*     */ 
/*     */     
/* 889 */     Value v = CSSUtilities.getComputedStyle(element, this.marginTopIndex);
/* 890 */     float top = v.getFloatValue();
/*     */     
/* 892 */     v = CSSUtilities.getComputedStyle(element, this.marginRightIndex);
/* 893 */     float right = v.getFloatValue();
/*     */     
/* 895 */     v = CSSUtilities.getComputedStyle(element, this.marginBottomIndex);
/* 896 */     float bottom = v.getFloatValue();
/*     */     
/* 898 */     v = CSSUtilities.getComputedStyle(element, this.marginLeftIndex);
/* 899 */     float left = v.getFloatValue();
/*     */     
/* 901 */     v = CSSUtilities.getComputedStyle(element, this.indentIndex);
/* 902 */     float indent = v.getFloatValue();
/*     */     
/* 904 */     v = CSSUtilities.getComputedStyle(element, this.textAlignIndex);
/* 905 */     if (v == ValueConstants.INHERIT_VALUE) {
/* 906 */       v = CSSUtilities.getComputedStyle(element, 11);
/*     */       
/* 908 */       if (v == ValueConstants.LTR_VALUE) {
/* 909 */         v = SVG12ValueConstants.START_VALUE;
/*     */       } else {
/* 911 */         v = SVG12ValueConstants.END_VALUE;
/*     */       } 
/*     */     } 
/* 914 */     if (v == SVG12ValueConstants.START_VALUE) {
/* 915 */       b = 0;
/* 916 */     } else if (v == SVG12ValueConstants.MIDDLE_VALUE) {
/* 917 */       b = 1;
/* 918 */     } else if (v == SVG12ValueConstants.END_VALUE) {
/* 919 */       b = 2;
/*     */     } else {
/* 921 */       b = 3;
/*     */     } 
/* 923 */     Map fontAttrs = new HashMap(20);
/* 924 */     List fontList = getFontList(ctx, element, fontAttrs);
/* 925 */     Float fs = (Float)fontAttrs.get(TextAttribute.SIZE);
/* 926 */     float fontSize = fs.floatValue();
/* 927 */     float lineHeight = getLineHeight(ctx, element, fontSize);
/*     */     
/* 929 */     String ln = element.getLocalName();
/*     */     
/* 931 */     boolean rgnBr = ln.equals("flowRegionBreak");
/* 932 */     return new BlockInfo(top, right, bottom, left, indent, b, lineHeight, fontList, fontAttrs, rgnBr);
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   protected float getLineHeight(BridgeContext ctx, Element element, float fontSize) {
/* 938 */     if (this.lineHeightIndex == -1) initCSSPropertyIndexes(element);
/*     */     
/* 940 */     Value v = CSSUtilities.getComputedStyle(element, this.lineHeightIndex);
/* 941 */     if (v == ValueConstants.INHERIT_VALUE || v == SVG12ValueConstants.NORMAL_VALUE)
/*     */     {
/* 943 */       return fontSize * 1.1F;
/*     */     }
/*     */     
/* 946 */     float lineHeight = v.getFloatValue();
/* 947 */     if (v instanceof ComputedValue)
/* 948 */       v = ((ComputedValue)v).getComputedValue(); 
/* 949 */     if (v instanceof LineHeightValue && ((LineHeightValue)v).getFontSizeRelative())
/*     */     {
/* 951 */       lineHeight *= fontSize; } 
/* 952 */     return lineHeight;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected class FlowContentBridge
/*     */     extends SVGTextElementBridge.AbstractTextChildTextContent
/*     */   {
/*     */     private final SVGFlowRootElementBridge this$0;
/*     */ 
/*     */ 
/*     */     
/*     */     public FlowContentBridge(SVGFlowRootElementBridge this$0, BridgeContext ctx, SVGTextElementBridge parent, Element e) {
/* 966 */       super(this$0, ctx, parent, e);
/*     */       this.this$0 = this$0;
/*     */     } }
/*     */   
/*     */   protected class RegionChangeListener implements EventListener {
/*     */     private final SVGFlowRootElementBridge this$0;
/*     */     
/* 973 */     protected RegionChangeListener(SVGFlowRootElementBridge this$0) { this.this$0 = this$0; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public void handleEvent(Event evt) {
/* 980 */       this.this$0.laidoutText = null;
/* 981 */       this.this$0.computeLaidoutText(this.this$0.ctx, this.this$0.e, this.this$0.getTextNode());
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\svg12\SVGFlowRootElementBridge.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */