/*     */ package org.apache.batik.bridge.svg12;
/*     */ 
/*     */ import java.awt.Color;
/*     */ import java.awt.Paint;
/*     */ import java.util.HashMap;
/*     */ import java.util.Map;
/*     */ import org.apache.batik.bridge.AnimatableGenericSVGBridge;
/*     */ import org.apache.batik.bridge.BridgeContext;
/*     */ import org.apache.batik.bridge.BridgeException;
/*     */ import org.apache.batik.bridge.CSSUtilities;
/*     */ import org.apache.batik.bridge.PaintBridge;
/*     */ import org.apache.batik.bridge.PaintServer;
/*     */ import org.apache.batik.css.engine.CSSEngine;
/*     */ import org.apache.batik.css.engine.CSSStylableElement;
/*     */ import org.apache.batik.css.engine.StyleMap;
/*     */ import org.apache.batik.css.engine.value.Value;
/*     */ import org.apache.batik.css.engine.value.svg.ICCColor;
/*     */ import org.apache.batik.dom.svg.SVGOMDocument;
/*     */ import org.apache.batik.dom.util.XLinkSupport;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.apache.batik.util.ParsedURL;
/*     */ import org.w3c.dom.Element;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SVGSolidColorElementBridge
/*     */   extends AnimatableGenericSVGBridge
/*     */   implements PaintBridge
/*     */ {
/*  65 */   public String getNamespaceURI() { return "http://www.w3.org/2000/svg"; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  72 */   public String getLocalName() { return "solidColor"; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Paint createPaint(BridgeContext ctx, Element paintElement, Element paintedElement, GraphicsNode paintedNode, float opacity) {
/*  90 */     opacity = extractOpacity(paintElement, opacity, ctx);
/*     */     
/*  92 */     return extractColor(paintElement, opacity, ctx);
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   protected static float extractOpacity(Element paintElement, float opacity, BridgeContext ctx) {
/*  98 */     Map refs = new HashMap();
/*  99 */     CSSEngine eng = CSSUtilities.getCSSEngine(paintElement);
/* 100 */     int pidx = eng.getPropertyIndex("solid-opacity");
/*     */ 
/*     */     
/*     */     while (true) {
/* 104 */       Value opacityVal = CSSUtilities.getComputedStyle(paintElement, pidx);
/*     */ 
/*     */ 
/*     */       
/* 108 */       StyleMap sm = ((CSSStylableElement)paintElement).getComputedStyleMap(null);
/*     */       
/* 110 */       if (!sm.isNullCascaded(pidx)) {
/*     */         
/* 112 */         float attr = PaintServer.convertOpacity(opacityVal);
/* 113 */         return opacity * attr;
/*     */       } 
/*     */       
/* 116 */       String uri = XLinkSupport.getXLinkHref(paintElement);
/* 117 */       if (uri.length() == 0) {
/* 118 */         return opacity;
/*     */       }
/*     */       
/* 121 */       SVGOMDocument doc = (SVGOMDocument)paintElement.getOwnerDocument();
/* 122 */       ParsedURL purl = new ParsedURL(doc.getURL(), uri);
/*     */ 
/*     */       
/* 125 */       if (refs.containsKey(purl)) {
/* 126 */         throw new BridgeException(ctx, paintElement, "xlink.href.circularDependencies", new Object[] { uri });
/*     */       }
/*     */ 
/*     */ 
/*     */       
/* 131 */       refs.put(purl, purl);
/* 132 */       paintElement = ctx.getReferencedElement(paintElement, uri);
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   protected static Color extractColor(Element paintElement, float opacity, BridgeContext ctx) {
/* 139 */     Map refs = new HashMap();
/* 140 */     CSSEngine eng = CSSUtilities.getCSSEngine(paintElement);
/* 141 */     int pidx = eng.getPropertyIndex("solid-color");
/*     */ 
/*     */     
/*     */     while (true) {
/* 145 */       Value colorDef = CSSUtilities.getComputedStyle(paintElement, pidx);
/*     */ 
/*     */ 
/*     */       
/* 149 */       StyleMap sm = ((CSSStylableElement)paintElement).getComputedStyleMap(null);
/*     */       
/* 151 */       if (!sm.isNullCascaded(pidx)) {
/*     */         
/* 153 */         if (colorDef.getCssValueType() == 1)
/*     */         {
/* 155 */           return PaintServer.convertColor(colorDef, opacity);
/*     */         }
/* 157 */         return PaintServer.convertRGBICCColor(paintElement, colorDef.item(0), (ICCColor)colorDef.item(1), opacity, ctx);
/*     */       } 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 165 */       String uri = XLinkSupport.getXLinkHref(paintElement);
/* 166 */       if (uri.length() == 0)
/*     */       {
/* 168 */         return new Color(0.0F, 0.0F, 0.0F, opacity);
/*     */       }
/*     */       
/* 171 */       SVGOMDocument doc = (SVGOMDocument)paintElement.getOwnerDocument();
/* 172 */       ParsedURL purl = new ParsedURL(doc.getURL(), uri);
/*     */ 
/*     */       
/* 175 */       if (refs.containsKey(purl)) {
/* 176 */         throw new BridgeException(ctx, paintElement, "xlink.href.circularDependencies", new Object[] { uri });
/*     */       }
/*     */ 
/*     */ 
/*     */       
/* 181 */       refs.put(purl, purl);
/* 182 */       paintElement = ctx.getReferencedElement(paintElement, uri);
/*     */     } 
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\svg12\SVGSolidColorElementBridge.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */