/*     */ package org.apache.batik.bridge.svg12;
/*     */ 
/*     */ import org.apache.batik.bridge.AbstractGraphicsNodeBridge;
/*     */ import org.apache.batik.bridge.Bridge;
/*     */ import org.apache.batik.bridge.BridgeContext;
/*     */ import org.apache.batik.bridge.GVTBuilder;
/*     */ import org.apache.batik.dom.AbstractDocument;
/*     */ import org.apache.batik.dom.svg12.XBLOMContentElement;
/*     */ import org.apache.batik.gvt.CompositeGraphicsNode;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.Node;
/*     */ import org.w3c.dom.NodeList;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class XBLContentElementBridge
/*     */   extends AbstractGraphicsNodeBridge
/*     */ {
/*     */   protected ContentChangedListener contentChangedListener;
/*     */   protected ContentManager contentManager;
/*     */   
/*  64 */   public String getLocalName() { return "content"; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  71 */   public String getNamespaceURI() { return "http://www.w3.org/2004/xbl"; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  78 */   public Bridge getInstance() { return new XBLContentElementBridge(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  89 */   public GraphicsNode createGraphicsNode(BridgeContext ctx, Element e) { return buildCompositeGraphicsNode(ctx, e, null); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public CompositeGraphicsNode buildCompositeGraphicsNode(BridgeContext ctx, Element e, CompositeGraphicsNode cgn) {
/* 106 */     XBLOMContentElement content = (XBLOMContentElement)e;
/* 107 */     AbstractDocument doc = (AbstractDocument)e.getOwnerDocument();
/* 108 */     DefaultXBLManager xm = (DefaultXBLManager)doc.getXBLManager();
/* 109 */     this.contentManager = xm.getContentManager(e);
/*     */     
/* 111 */     if (cgn == null) {
/* 112 */       cgn = new CompositeGraphicsNode();
/* 113 */       associateSVGContext(ctx, e, cgn);
/*     */     } else {
/* 115 */       int s = cgn.size();
/* 116 */       for (int i = 0; i < s; i++) {
/* 117 */         cgn.remove(0);
/*     */       }
/*     */     } 
/*     */     
/* 121 */     GVTBuilder builder = ctx.getGVTBuilder();
/* 122 */     NodeList nl = this.contentManager.getSelectedContent(content);
/* 123 */     if (nl != null) {
/* 124 */       for (int i = 0; i < nl.getLength(); i++) {
/* 125 */         Node n = nl.item(i);
/* 126 */         if (n.getNodeType() == 1) {
/* 127 */           GraphicsNode gn = builder.build(ctx, (Element)n);
/* 128 */           if (gn != null) {
/* 129 */             cgn.add(gn);
/*     */           }
/*     */         } 
/*     */       } 
/*     */     }
/*     */     
/* 135 */     if (ctx.isDynamic() && 
/* 136 */       this.contentChangedListener == null) {
/*     */       
/* 138 */       this.contentChangedListener = new ContentChangedListener(this);
/* 139 */       this.contentManager.addContentSelectionChangedListener(content, this.contentChangedListener);
/*     */     } 
/*     */ 
/*     */ 
/*     */     
/* 144 */     return cgn;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 152 */   protected GraphicsNode instantiateGraphicsNode() { return null; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 166 */   public void buildGraphicsNode(BridgeContext ctx, Element e, GraphicsNode node) { initializeDynamicSupport(ctx, e, node); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 174 */   public boolean getDisplay(Element e) { return true; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 182 */   public boolean isComposite() { return false; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void dispose() {
/* 190 */     super.dispose();
/*     */     
/* 192 */     if (this.contentChangedListener != null)
/* 193 */       this.contentManager.removeContentSelectionChangedListener((XBLOMContentElement)this.e, this.contentChangedListener); 
/*     */   }
/*     */   
/*     */   protected class ContentChangedListener
/*     */     implements ContentSelectionChangedListener
/*     */   {
/*     */     private final XBLContentElementBridge this$0;
/*     */     
/* 201 */     protected ContentChangedListener(XBLContentElementBridge this$0) { this.this$0 = this$0; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 210 */     public void contentSelectionChanged(ContentSelectionChangedEvent csce) { this.this$0.buildCompositeGraphicsNode(this.this$0.ctx, this.this$0.e, (CompositeGraphicsNode)this.this$0.node); }
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-bridge.jar!\org\apache\batik\bridge\svg12\XBLContentElementBridge.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */